/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.xrtygis.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.core.util.StringUtil;
import com.tofly.common.ftp.config.FtpUtil;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.xrtygis.config.DictConfig;
import com.tofly.xrtygis.config.FilePathBean;
import com.tofly.xrtygis.custom.util.service.FileInfoService;
import com.tofly.xrtygis.em.Constant;
import com.tofly.xrtygis.entity.Dmatable;
import com.tofly.xrtygis.entity.DeviceArchiveManage;
import com.tofly.xrtygis.service.DmatableService;
import com.tofly.xrtygis.service.TfDeviceArchiveManageService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 设备档案管理
 *
 * @author admin
 * @date Fri Mar 11 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfdevicearchivemanage")
@Api(tags="设备档案管理接口")
public class TfDeviceArchiveManageController {
    private final static String TABLE_NAME = "TF_DEVICE_ARCHIVE_MANAGE";
    private final TfDeviceArchiveManageService tfDeviceArchiveManageService;
    private final DmatableService dmatableService;

    private final FileInfoService fileInfoService;
    private final FilePathBean filePathBean;

    private final DictConfig dictConfig;

    /**
    * 分页查询
    * @param page 分页对象
    * @param tfDeviceArchiveManage 设备档案管理
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getDeviceArchiveManagePage(Page page, @ModelAttribute DeviceArchiveManage tfDeviceArchiveManage) {
        try{
            String waterIds = tfDeviceArchiveManage.getWaterIds();
            List<Long> waterIdList = new ArrayList<>();
            if(StringUtil.isNotEmpty(waterIds)){
                waterIdList = Arrays.stream(waterIds.split(Constant.CSV)).map(s -> Long.parseLong(s.trim()))
                        .collect(Collectors.toList());
            }
            return ResultRespone.success(tfDeviceArchiveManageService.page(page, new QueryWrapper<DeviceArchiveManage>().lambda()
                    .in(StringUtil.isNotEmpty(waterIdList),DeviceArchiveManage::getWaterId,waterIdList)
                    .setEntity(tfDeviceArchiveManage)));
        }catch (Exception e){
            e.printStackTrace();
            return ResultRespone.failed();
        }
    }


    /**
     * 通过id查询设备档案管理
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
        try{
            return ResultRespone.success(tfDeviceArchiveManageService.getById(id));
        }catch (Exception e){
            e.printStackTrace();
            return ResultRespone.failed();
        }
    }

    /**
     * 新增设备档案管理
     * @param tfDeviceArchiveManage 设备档案管理
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增设备档案管理" )
    @ApiOperation(value = "新增设备档案管理")
    @PostMapping
    public ResultRespone save(@RequestPart DeviceArchiveManage tfDeviceArchiveManage,
                              @RequestPart(required = false) List<MultipartFile> fileList) {
        try{
            if (!fileList.isEmpty()) {
                //文件保存
                StringBuilder fileids = fileInfoService.saveFileList(fileList, filePathBean.getCostNotesFile(), TABLE_NAME);
                tfDeviceArchiveManage.setViewJson(fileids.toString());
            }
            boolean save = tfDeviceArchiveManageService.save(tfDeviceArchiveManage);
            // 判断是否为计量表 为计量表则同步
            String deviceType = tfDeviceArchiveManage.getDeviceType();
            if(dictConfig.getFlow().equals(deviceType)){
                Dmatable dmatable = new Dmatable();
                dmatable.setId(tfDeviceArchiveManage.getId());
                dmatable.setDeviceId(tfDeviceArchiveManage.getId());
                dmatable.setMinFlow((double)0);
                dmatable.setId(tfDeviceArchiveManage.getId());
                dmatableService.save(dmatable);
            }
            return ResultRespone.success(save);
        }catch (Exception e){
            e.printStackTrace();
            return ResultRespone.failed();
        }
    }

    /**
     * 修改设备档案管理
     * @param tfDeviceArchiveManage 设备档案管理
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改设备档案管理" )
    @ApiOperation(value = "修改设备档案管理")
    @PutMapping
    public ResultRespone updateById(@RequestPart DeviceArchiveManage tfDeviceArchiveManage,
                                    @RequestPart(required = false) List<MultipartFile> fileList) {
        if (!fileList.isEmpty()) {
            //文件保存
            StringBuilder fileids = fileInfoService.saveFileList(fileList, filePathBean.getCostNotesFile(), TABLE_NAME);
            tfDeviceArchiveManage.setViewJson(fileids.toString());
        }
        // 判断是否为计量表 为计量表则同步
        String deviceType = tfDeviceArchiveManage.getDeviceType();
        Dmatable one = dmatableService.getOne(new QueryWrapper<Dmatable>().lambda().eq(Dmatable::getDeviceId, tfDeviceArchiveManage.getId()));
        if("0".equals(deviceType)){
            Dmatable dmatable = new Dmatable();
            dmatable.setId(tfDeviceArchiveManage.getId());
            dmatable.setDeviceId(tfDeviceArchiveManage.getId());
            dmatable.setId(tfDeviceArchiveManage.getId());
            dmatable.setMinFlow((double)0);
            if(null==one){
                dmatableService.save(dmatable);
            }
        }else {
            // 删除区域表(不为计量表)
            if(null!=one && StringUtil.isNotEmpty(deviceType)){
                dmatableService.removeById(one.getId());
            }

        }

        try{
            return ResultRespone.success(tfDeviceArchiveManageService.updateById(tfDeviceArchiveManage));
        }catch (Exception e){
            e.printStackTrace();
            return ResultRespone.failed();
        }
    }


    /**
     * 通过id批量删除设备档案管理
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除设备档案管理" )
    @ApiOperation(value = "通过ID批量删除设备档案管理")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        try{
            List<Long> idList = Arrays.stream(ids.split(",")).map(k -> Long.parseLong(k.trim())).collect(Collectors.toList());
            // 删除计量表同步的设备
            dmatableService.remove(new QueryWrapper<Dmatable>().lambda().in(Dmatable::getDeviceId,idList));
            return ResultRespone.success(tfDeviceArchiveManageService.removeByIds(idList));
        }catch (Exception e){
            e.printStackTrace();
            return ResultRespone.failed();
        }
    }
}
