/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.xrtygis.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.xrtygis.custom.entity.dto.WisdomDisplayDto;
import com.tofly.xrtygis.entity.TfLineInfo;
import com.tofly.xrtygis.service.TfLineInfoService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;
/**
 * 线路信息表
 *
 * @author JF
 * @date Tue Aug 01 00:00:00 CST 2023
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tflineinfo")
@Api(tags="线路信息表接口")
public class TfLineInfoController {

    private final  TfLineInfoService tfLineInfoService;

    /**
     *
     * @param
     * @description 线路支路树
     */
    @GetMapping("/lineTree")
    @ApiOperation(value = "线路支路树")
    public ResultRespone lineTree(){
        try {
            return ResultRespone.success(tfLineInfoService.lineTree());
        }catch (Exception e){
            e.printStackTrace();
            return ResultRespone.failed(e.getMessage());
        }
    }

    /**
    * 分页查询
    * @param page 分页对象
    * @param tfLineInfo 线路信息表
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getTfLineInfoPage(Page page, @ModelAttribute TfLineInfo tfLineInfo) {
      return ResultRespone.success(tfLineInfoService.page(page, Wrappers.query(tfLineInfo)));
    }


    /**
     * 通过id查询线路信息表
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(tfLineInfoService.getById(id));
    }

    /**
     * 新增线路信息表
     * @param tfLineInfo 线路信息表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增线路信息表" )
    @ApiOperation(value = "新增线路信息表")
    @PostMapping
    public ResultRespone save(@RequestBody TfLineInfo tfLineInfo) {
      return ResultRespone.success(tfLineInfoService.save(tfLineInfo));
    }

    /**
     * 修改线路信息表
     * @param tfLineInfo 线路信息表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改线路信息表" )
    @ApiOperation(value = "修改线路信息表")
    @PutMapping
    public ResultRespone updateById(@RequestBody TfLineInfo tfLineInfo) {
      return ResultRespone.success(tfLineInfoService.updateById(tfLineInfo));
    }

    /**
     * 通过id删除线路信息表
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除线路信息表" )
    @ApiOperation(value = "通过ID删除线路信息表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(tfLineInfoService.removeById(id));
    }

    /**
     * 通过id批量删除线路信息表
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除线路信息表" )
    @ApiOperation(value = "通过ID批量删除线路信息表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(tfLineInfoService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
