package com.tofly.xrtygis.custom.controller.home;

import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.xrtygis.commons.Constant;
import com.tofly.xrtygis.em.ExcelExportFileTypeEnum;
import com.tofly.xrtygis.entity.vo.home.monitor.InstantaneousInfoVo;
import com.tofly.xrtygis.entity.vo.home.monitor.SiteListVo;
import com.tofly.xrtygis.entity.vo.home.monitor.WaterTimeFlowVo;
import com.tofly.xrtygis.entity.vo.home.monitor.WaterTimeTotalVo;
import com.tofly.xrtygis.service.home.OnlineMonitorService;
import com.tofly.xrtygis.util.ExcelUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/onlineMonitor")
@Api(tags = "在线监控接口信息")
@Slf4j
public class OnlineMonitorController {
    @Autowired
    private OnlineMonitorService onlineMonitorService;

    @ApiOperation(value = "获取监测设备列表")
    @GetMapping("/getSiteList")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "type", value = "设备类型,不传默认全部", required = false, dataType = "String"),
            @ApiImplicitParam(name = "siteName", value = "设备名称模糊查询,不传默认全部", required = false, dataType = "String"),
            @ApiImplicitParam(name = "state", value = "设备状态,多个以,分隔查询,不传默认全部", required = false, dataType = "String"),
    })
    public ResultRespone<List<SiteListVo>> getSiteList(String type, String siteName,String state) {
        ResultRespone<List<SiteListVo>> result = new ResultRespone<>();
        List<SiteListVo> siteList = onlineMonitorService.getSiteList(type, siteName,state);
        result.setResult(siteList);
        return result;
    }

    @ApiOperation(value = "根据指标类型,id获取监测信息详情")
    @GetMapping("/getMonitorInfo")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "indexType", value = "指标类型,不传默认 1 累计流量类型", required = false, dataType = "String"),
            @ApiImplicitParam(name = "code", value = "指标编码,不传默认 全部", required = false, dataType = "String")
    })
    public ResultRespone<List<InstantaneousInfoVo>> getMonitorInfo(String indexType,String code) {
        ResultRespone<List<InstantaneousInfoVo>> result = new ResultRespone<>();
        List<InstantaneousInfoVo> monitorInfo = onlineMonitorService.getMonitorInfo(indexType, code);
        result.setResult(monitorInfo);
        return result;
    }

    @ApiOperation(value = "获取分水用时列表")
    @GetMapping("/getWaterTimeTotal")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "code", value = "设备编码,多个以,分隔 不传默认为所有", required = false, dataType = "String"),
            @ApiImplicitParam(name = "areaId", value = "设备区域归属id", required = false, dataType = "Long"),
            @ApiImplicitParam(name = "collectId", value = "南北校区：0南校区，1北校区", required = false, dataType = "String"),
            @ApiImplicitParam(name = "startDate", value = "开始时间 格式 yyyy-MM-dd 不传默认为当天开始时间", required = false, dataType = "String"),
            @ApiImplicitParam(name = "endDate", value = "结束时间 格式 yyyy-MM-dd 不传默认为当天结束时间", required = false, dataType = "String"),
            @ApiImplicitParam(name = "startTime", value = "开始时间 格式 HH:mm:ss 不传默认为00:00:00", required = false, dataType = "String"),
            @ApiImplicitParam(name = "endTime", value = "结束时间 格式  HH:mm:ss 不传默认为23:59:59", required = false, dataType = "String"),
    })
    public ResultRespone<List<WaterTimeTotalVo>> getWaterTimeTotal(String code, Long areaId,String collectId,String startDate, String endDate, String startTime, String endTime) {
        ResultRespone<List<WaterTimeTotalVo>> result = new ResultRespone<>();
        List<WaterTimeTotalVo> waterTimeTotal = onlineMonitorService.getWaterTimeTotal(code, areaId,collectId,startDate, endDate,startTime,endTime);
        result.setResult(waterTimeTotal);
        return result;
    }

    @ApiOperation(value = "分水用时用详情")
    @GetMapping("/getWaterTimeFlowDetail")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "code", value = "设备编码,多个以,分隔 不传默认为所有", required = false, dataType = "String"),
            @ApiImplicitParam(name = "areaId", value = "设备区域归属id", required = false, dataType = "Long"),
            @ApiImplicitParam(name = "collectId", value = "南北校区：0南校区，1北校区", required = false, dataType = "String"),
            @ApiImplicitParam(name = "startDate", value = "开始时间 格式 yyyy-MM-dd 不传默认为当天开始时间", required = false, dataType = "String"),
            @ApiImplicitParam(name = "endDate", value = "结束时间 格式 yyyy-MM-dd 不传默认为当天结束时间", required = false, dataType = "String"),
            @ApiImplicitParam(name = "startTime", value = "开始时间 格式 HH:mm:ss 不传默认为00:00:00", required = false, dataType = "String"),
            @ApiImplicitParam(name = "endTime", value = "结束时间 格式  HH:mm:ss 不传默认为23:59:59", required = false, dataType = "String"),
            @ApiImplicitParam(name = "flag", value = "是否分页 不传默认不分页", required = false, dataType = "String"),
            @ApiImplicitParam(name = "export", value = "是否导出 不传默认不导出", required = false, dataType = "String"),
            @ApiImplicitParam(name = "isTotal", value = "是否统计 不传默认不统计", required = false, dataType = "String"),
            @ApiImplicitParam(name = "dataGroup",value = "数据分组 传入分钟 不传默认5分钟",required = false,dataType = "integer")
    })
    public ResultRespone<Object> getWaterTimeFlowDetail(Page page, String code,Long areaId,String collectId, String startDate, String endDate, String startTime,String endTime, String flag, String export,String isTotal ,Integer dataGroup,HttpServletResponse response) {
        ResultRespone<Object> result = new ResultRespone<>();
        if(StringUtils.isEmpty(export)){
            result.setResult(onlineMonitorService.getWaterTimeFlowDetail(page,code, areaId,collectId,startDate, endDate,startTime, endTime,flag,isTotal,dataGroup));
            return result;
        }else {
            ExcelExportFileTypeEnum excelExportFileTypeEnum = null;
            // 从模型类型枚举根据templateServiceType获取策略对应模板数据
            Map<String, Object> dataMap = new HashMap<>(12);
            if(!StringUtils.isEmpty(isTotal)){
                excelExportFileTypeEnum = ExcelExportFileTypeEnum.EXPORT_WATER_TIME_TOTAL;
                dataMap.put("startTime",startTime);
                dataMap.put("endTime",endTime);
            }else {
                excelExportFileTypeEnum = ExcelExportFileTypeEnum.EXPORT_WATER_TIME;
            }

            Object waterTimeFlowDetail = onlineMonitorService.getWaterTimeFlowDetail(page, code, areaId,collectId,startDate, endDate, startTime,endTime,flag,isTotal,dataGroup);
            dataMap.put("list", waterTimeFlowDetail);
            ExcelUtil.export(dataMap, excelExportFileTypeEnum, Constant.FILE_PATH_TEST_EXPORT_EXCEL, response);
            return null;
        }
    }
}
