package com.tofly.xrtygis.custom.controller.home;

import com.tofly.common.core.entity.ResultRespone;

import com.tofly.xrtygis.entity.vo.home.workbench.*;
import com.tofly.xrtygis.service.home.WorkbenchService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@RequestMapping("/workbench")
@Api(tags = "工作台接口信息")
@Slf4j
public class WorkbenchController {
    @Autowired
    private WorkbenchService workbenchService;

    @ApiOperation(value = "获取今日夜间最小流量报警点数/年度供水总量/同比上一年涨的趋势百分比")
    @GetMapping("/getModuleTitle")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "minimumFlowDay", value = "夜间最小流量报警点数查询日期 格式yyyy-MM-dd 不传默认当天", required = false, dataType = "String"),
            @ApiImplicitParam(name = "day", value = "年度供水总量,趋势查询日期 格式yyyy-MM-dd 不传默认当年月日同比", required = false, dataType = "String")
    })
    public ResultRespone<ModuleTitleVo> getModuleTitle(String minimumFlowDay, String day) {
        ResultRespone<ModuleTitleVo> result = new ResultRespone<>();
        ModuleTitleVo moduleTitle = workbenchService.getModuleTitle(minimumFlowDay, day);
        result.setResult(moduleTitle);
        return result;
    }

    @ApiOperation(value = "获取指定年月份供水总量排行榜")
    @GetMapping("/getWaterSupplyRanking")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "yearMonth", value = "供水总量排行榜查询日期 格式yyyy-MM 不传默认当月", required = false, dataType = "String")
    })
    public ResultRespone<WaterSupplyRankingVo> getWaterSupplyRanking(String yearMonth) {
        ResultRespone<WaterSupplyRankingVo> result = new ResultRespone<>();
        WaterSupplyRankingVo waterSupplyRanking = workbenchService.getWaterSupplyRanking(yearMonth);
        result.setResult(waterSupplyRanking);
        return result;
    }

    @ApiOperation(value = "获取指定年月日夜间最小流量监测排行榜")
    @GetMapping("/getMinimumFlowRanking")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "day", value = "夜间最小流量监测排行榜查询日期 格式yyyy-MM-dd 不传默认当日", required = false, dataType = "String")
    })
    public ResultRespone<MinimumFlowRankingVo> getMinimumFlowRanking(String day) {
        ResultRespone<MinimumFlowRankingVo> result = new ResultRespone<>();
        MinimumFlowRankingVo minimumFlowRanking = workbenchService.getMinimumFlowRanking(day);
        result.setResult(minimumFlowRanking);
        return result;
    }

    @ApiOperation(value = "获取指定年累计供水量统计")
    @GetMapping("/getWaterSupplyAccumulative")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "yearMonth", value = "累计供水量统计查询日期 格式yyyy-mm 不传默认当年月", required = false, dataType = "String")
    })
    public ResultRespone<List<WaterSupplyAccumulativeVo>> getWaterSupplyAccumulative(String yearMonth) {
        ResultRespone<List<WaterSupplyAccumulativeVo>> result = new ResultRespone<>();
        List<WaterSupplyAccumulativeVo> waterSupplyAccumulative = workbenchService.getWaterSupplyAccumulative(yearMonth);
        result.setResult(waterSupplyAccumulative);
        return result;
    }

    @ApiOperation(value = "获取指定年每月供水量统计")
    @GetMapping("/getWaterSupplyMonth")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "yearMonth", value = "每月供水量统计查询日期 格式yyyy-MM 不传默认当年月", required = false, dataType = "String")
    })
    public ResultRespone<List<WaterSupplyMonthVo>> getWaterSupplyMonth(String yearMonth) {
        ResultRespone<List<WaterSupplyMonthVo>> result = new ResultRespone<>();
        List<WaterSupplyMonthVo> waterSupplyMonth = workbenchService.getWaterSupplyMonth(yearMonth);
        result.setResult(waterSupplyMonth);
        return result;
    }

    @ApiOperation(value = "获取指定年月日近15天监测点夜间最小流量状态")
    @GetMapping("/getMinimumFlowState")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "day", value = "近15天监测点夜间最小流量状态查询日期 格式yyyy-MM-dd 不传默认当天", required = false, dataType = "String")
    })
    public ResultRespone<List<MinimumFlowStateVo>> getMinimumFlowState(String day) {
        ResultRespone<List<MinimumFlowStateVo>> result = new ResultRespone<>();
        List<MinimumFlowStateVo> minimumFlowState = workbenchService.getMinimumFlowState(day);
        result.setResult(minimumFlowState);
        return result;
    }

}
