package com.tofly.xrtygis.custom.mapper;


import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.xrtygis.entity.vo.home.monitor.*;
import org.apache.ibatis.annotations.Param;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author 20126
 */
public interface OnlineMonitorMapper {
    /**
     * 获取基础监测设备列表信息
     *
     * @param type
     * @param siteName
     * @return
     */
    List<SiteListVo> getSiteListVo(@Param("type") String type, @Param("siteName") String siteName);

    /**
     * 获取指标实时数据
     *
     * @return
     */
    List<RealDataVo> getRealDataList();

    /**
     * 根据指标获取夜间流量阈值参考值
     *
     * @param code
     * @return
     */
    BigDecimal getIndexMinFow(@Param("code") String code);

    /**
     * 流量监测图(0-24小时)
     *
     * @param code
     * @param startTime
     * @param endTime
     * @return
     */
    List<InstantaneousInfoChildVo> getMonitorInfo(@Param("code") String code,
                                                  @Param("startTime") Date startTime,
                                                  @Param("endTime") Date endTime);

    /**
     * 分水用时统计列表
     *
     * @param codeList
     * @param dateList
     * @return
     */
    List<WaterTimeTotalVo> getWaterTimeTotal(@Param("codeList") List<String> codeList,
                                             @Param("areaId") Long areaId,
                                             @Param("collectId") String collectId,
                                             @Param("dateList") List<Map<String, Date>> dateList,
                                             @Param("dictType") String dictType);

    /**
     * 分水用时用详情
     *
     * @param codeList
     * @param dateList
     * @return
     */
    List<WaterTimeFlowVo> getWaterTimeFlowDetail(@Param("codeList") List<String> codeList,
                                                 @Param("dateList") List<Map<String, Date>> dateList,
                                                 @Param("areaId") Long areaId,
                                                 @Param("collectId") String collectId,
                                                 @Param("dataGroup") Integer dataGroup,
                                                 @Param("dictType") String dictType);

    /**
     * 分水用时用详情 分页
     *
     * @param codeList
     * @param dateList
     * @return
     */
    Page<WaterTimeFlowVo> getWaterTimeFlowDetail(Page page, @Param("codeList") List<String> codeList,
                                                 @Param("dateList") List<Map<String, Date>> dateList,
                                                 @Param("areaId") Long areaId,
                                                 @Param("collectId") String collectId,
                                                 @Param("dataGroup") Integer dataGroup,
                                                 @Param("dictType") String dictType);

    /**
     * 分水用时详情统计信息
     *
     * @param codeList
     * @param dateList
     * @return
     */
    List<WaterTimeFlowTotalDetailVo> getWaterTimeFlowTotalDetail(@Param("codeList") List<String> codeList,
                                                                 @Param("areaId") Long areaId,
                                                                 @Param("collectId") String collectId,
                                                                 @Param("dateList") List<Map<String, Date>> dateList,
                                                                 @Param("dictType") String dictType);
}
