package com.tofly.xrtygis.custom.mapper;


import com.tofly.xrtygis.entity.vo.home.workbench.*;
import org.apache.ibatis.annotations.Param;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

/**
 * @author 20126
 */
public interface WorkbenchMapper {
    /**
     * 当前啥时间范围内夜间最小流量报警点数
     *
     * @param startTime
     * @param endTime
     * @return
     */
    Integer getMinimumFlowAlarmNum(@Param("startTime") Date startTime, @Param("endTime") Date endTime);

    /**
     * 获取 年度供水总量/同比年度供水总量/同比上一年涨的趋势百分比
     * @param startYear
     * @param endYear
     * @param startOldYear
     * @param endOldYear
     * @return
     */
    ModuleTitleVo getModuleTitleInfo(@Param("startYear")Date startYear,@Param("endYear")Date endYear,@Param("startOldYear")Date startOldYear,@Param("endOldYear")Date endOldYear, @Param("dictNameList") List<String> dictNameList);

    /**
     * 获取指定年月份供水总量排行榜
     * @param startTime
     * @param endTime
     * @return
     */
    List<WaterSupplyRankingChildVo> getWaterSupplyRanking(@Param("startTime") Date startTime, @Param("endTime") Date endTime);

    /**
     * 获取指定年月日总供水量
     * @param startTime
     * @param endTime
     * @return
     */
    BigDecimal getTotalWaterSupply(@Param("startTime") Date startTime, @Param("endTime") Date endTime);
    /**
     * 获取指定年月日夜间最小流量监测排行榜
     * @param startTime
     * @param endTime
     * @return
     */
    List<MinimumFlowRankingChildVo> getMinimumFlowRanking(@Param("startTime") Date startTime, @Param("endTime") Date endTime);

    /**
     * 累计供水量统计
     * @param startTime
     * @param endTime
     * @return
     */
    List<WaterSupplyAccumulativeChildVo> getWaterSupplyAccumulative(@Param("startTime") Date startTime, @Param("endTime") Date endTime, @Param("dictNameList") List<String> dictNameList);

    /**
     * 累计月供水量统计
     * @param startTime
     * @param endTime
     * @return
     */
    List<WaterSupplyMonthChildVo> getWaterSupplyMonth(@Param("startTime") Date startTime, @Param("endTime") Date endTime, @Param("dictNameList") List<String> dictNameList);

    /**
     * 获取指定年月日近15天监测点夜间最小流量状态
     * @param state
     * @param startTime
     * @param endTime
     * @return
     */
    List<MinimumFlowStateChildVo> getMinimumFlowState(@Param("state")String state,@Param("startTime") Date startTime, @Param("endTime") Date endTime);

    List<String> getDictNameList(Long pid);
}
