package com.tofly.xrtygis.entity.dto;


import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.baomidou.mybatisplus.annotation.*;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiModel;
import com.fasterxml.jackson.annotation.JsonFormat;
import org.codehaus.jackson.map.annotate.JsonSerialize;
import org.codehaus.jackson.map.ser.std.ToStringSerializer;
import org.springframework.format.annotation.DateTimeFormat;
import lombok.Data;

import java.math.BigDecimal;
import java.util.*;

import org.springframework.web.multipart.MultipartFile;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;

import java.io.Serializable;

import lombok.EqualsAndHashCode;

import javax.validation.constraints.*;

/**
 * 设备档案管理(DeviceArchiveManage)表实体类
 *
 * @author HaiQiu
 * @since 2023-03-02 09:22:04
 */
@Data
@Accessors(chain = true)
@NoArgsConstructor
@EqualsAndHashCode(callSuper = false)
@ApiModel(value = "DeviceArchiveManage对象", description = "设备档案管理")
public class DeviceArchiveManageDto implements Serializable {

    private static final long serialVersionUID = 931098294055536554L;

    /**
     * 所属水厂ID
     */
    @Digits(integer = 15, fraction = 0, message = "所属水厂ID长度不能超过15")
    @ApiModelProperty(value = "所属水厂ID")
    private Long waterId;

    /**
     * 设备编号
     */
    @Size(max = 33, message = "设备编号长度不能超过33")
    @ApiModelProperty(value = "设备编号")
    private String code;

    /**
     * 设备名称
     */
    @Size(max = 66, message = "设备名称长度不能超过66")
    @ApiModelProperty(value = "设备名称")
    private String name;

    /**
     * 类型（1水厂,2管网）
     */
    @Size(max = 0, message = "类型（1水厂,2管网）长度不能超过0")
    @ApiModelProperty(value = "类型（1水厂,2管网）")
    private String type;

    /**
     * 制造日期
     */
    @ApiModelProperty(value = "制造日期")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date makeDate;

    /**
     * 设备类型 53400
     */
    @Size(max = 0, message = "设备类型 53400长度不能超过0")
    @ApiModelProperty(value = "设备类型 53400")
    private String deviceType;

    /**
     * 制造厂家全称
     */
    @Size(max = 66, message = "制造厂家全称长度不能超过66")
    @ApiModelProperty(value = "制造厂家全称")
    private String manufacturerDesc;

    /**
     * 制造厂家
     */
    @Size(max = 66, message = "制造厂家长度不能超过66")
    @ApiModelProperty(value = "制造厂家")
    private String manufacturer;

    /**
     * 型号规格
     */
    @Size(max = 33, message = "型号规格长度不能超过33")
    @ApiModelProperty(value = "型号规格")
    private String modelSpecification;

    /**
     * 状态（0故障,1运行中,2空闲,3检修中）
     */
    @Size(max = 0, message = "状态（0故障,1运行中,2空闲,3检修中）长度不能超过0")
    @ApiModelProperty(value = "状态（0故障,1运行中,2空闲,3检修中）")
    private String status;

    /**
     * 经度
     */
    @ApiModelProperty(value = "经度")
    private Double longitude;
    /**
     * 纬度
     */
    @ApiModelProperty(value = "纬度")
    private Double latitude;

    /**
     * 负责人
     */
    @Digits(integer = 15, fraction = 0, message = "负责人长度不能超过15")
    @ApiModelProperty(value = "负责人")
    private Long leader;

    /**
     * 联系电话
     */
    @Size(max = 33, message = "联系电话长度不能超过33")
    @ApiModelProperty(value = "联系电话")
    private String phone;

    /**
     * 描述
     */
    @Size(max = 333, message = "描述长度不能超过333")
    @ApiModelProperty(value = "描述")
    private String describe;

    @Digits(integer = 15, fraction = 0, message = "${column.comment}长度不能超过15")
    private Long createUser;

    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date createTime;

    @Digits(integer = 15, fraction = 0, message = "${column.comment}长度不能超过15")
    @TableId(type = IdType.ASSIGN_ID)
    private Long id;

    /**
     * 地址
     */
    @Size(max = 333, message = "地址长度不能超过333")
    @ApiModelProperty(value = "地址")
    private String address;

    /**
     * 视点Json对象
     */
    @Size(max = 333, message = "视点Json对象长度不能超过333")
    @ApiModelProperty(value = "视点Json对象")
    private String viewJson;

    /**
     * 口径(mm),15,20,25,40,50,80,100,200
     */
    @Size(max = 16, message = "口径(mm),15,20,25,40,50,80,100,200长度不能超过16")
    @ApiModelProperty(value = "口径(mm),15,20,25,40,50,80,100,200")
    private String bore;

    /**
     * 压力设备id(归属)
     */
    @Digits(integer = 15, fraction = 0, message = "压力设备id(归属)长度不能超过15")
    @ApiModelProperty(value = "压力设备id(归属)")
    private Long pressureDeviceId;

    /**
     * 属于区域,字典码表pcid54000
     */
    @Size(max = 3, message = "属于区域,字典码表pcid54000长度不能超过3")
    @ApiModelProperty(value = "属于区域,字典码表pcid54000")
    private String areaId;

    /**
     * 检测类型
     */
    @Size(max = 85, message = "检测类型长度不能超过85")
    @ApiModelProperty(value = "检测类型")
    private String detectionType;

    /**
     * 出厂编码
     */
    @Size(max = 85, message = "出厂编码长度不能超过85")
    @ApiModelProperty(value = "出厂编码")
    private String factoryCode;

    /**
     * 巡检养护周期
     */
    @Size(max = 85, message = "巡检养护周期长度不能超过85")
    @ApiModelProperty(value = "巡检养护周期")
    private String nursingTime;

    /**
     * 使用寿命
     */
    @Size(max = 85, message = "使用寿命长度不能超过85")
    @ApiModelProperty(value = "使用寿命")
    private String lifeSpan;

    /**
     * 购买价格
     */
    @Size(max = 85, message = "购买价格长度不能超过85")
    @ApiModelProperty(value = "购买价格")
    private String purchasePrice;

    /**
     * 采购价格
     */
    @Size(max = 85, message = "采购价格长度不能超过85")
    @ApiModelProperty(value = "采购价格")
    private String procurePrice;

    /**
     * 安装时间
     */
    @ApiModelProperty(value = "安装时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date installTime;

    /**
     * 购买时间
     */
    @ApiModelProperty(value = "购买时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date purchaseTime;

    /**
     * 所属院区(0南校区，1北校区) or 字典获取
     */
    @Size(max = 85, message = "所属院区(0南校区，1北校区) or 字典获取长度不能超过85")
    @ApiModelProperty(value = "所属院区(0南校区，1北校区) or 字典获取")
    private String belongToCollege;

    /**
     * 所属对象(0学院，1商户) or 字典获取
     */
    @Size(max = 85, message = "所属对象(0学院，1商户) or 字典获取长度不能超过85")
    @ApiModelProperty(value = "所属对象(0学院，1商户) or 字典获取")
    private String belongToObject;

    /**
     * 功能用途
     */
    @Size(max = 300, message = "功能用途长度不能超过300")
    @ApiModelProperty(value = "功能用途")
    private String description;

    /**
     * 所在楼栋ID
     */
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @JsonSerialize(using = ToStringSerializer.class)
    @Digits(integer = 20, fraction = 0, message = "所在楼栋ID长度不能超过20")
    @ApiModelProperty(value = "所在楼栋ID")
    private Long buildingCollegeId;

}

