package com.tofly.xrtygis.service.home.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.tofly.xrtygis.commons.Constant;
import com.tofly.xrtygis.config.DictConfig;
import com.tofly.xrtygis.custom.mapper.OnlineMonitorMapper;
import com.tofly.xrtygis.custom.util.DateTimeUtil;
import com.tofly.xrtygis.em.ExcelExportFileTypeEnum;
import com.tofly.xrtygis.entity.Dmatable;
import com.tofly.xrtygis.entity.vo.home.monitor.*;
import com.tofly.xrtygis.service.DmatableService;
import com.tofly.xrtygis.service.home.OnlineMonitorService;
import com.tofly.xrtygis.util.ExcelUtil;
import lombok.AllArgsConstructor;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.*;

import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

@Service
@AllArgsConstructor
public class OnlineMonitorServiceImpl implements OnlineMonitorService {

    private final OnlineMonitorMapper onlineMonitorMapper;

    private final DmatableService dmatableService;

    private final DictConfig dictConfig;

    @Override
    public List<SiteListVo> getSiteList(String type, String siteName, String state) {
        // 获取基础监测设备信息
        List<SiteListVo> siteListVo = onlineMonitorMapper.getSiteListVo(type, siteName);
        // 获取所有监测指标实时监测数据
        List<RealDataVo> realDataList = onlineMonitorMapper.getRealDataList();
        // 按照设备id,指标id 生成map
        Map<Long, Map<Long, List<RealDataVo>>> realDateMap = realDataList.stream().collect(Collectors.groupingBy(RealDataVo::getDeviceId, Collectors.groupingBy(RealDataVo::getIndexId)));
        // 获取所有计量表夜间最小设置阈值
        List<Dmatable> dmaTableList = dmatableService.list(new QueryWrapper<Dmatable>().lambda().select(Dmatable::getDeviceId, Dmatable::getMinFlow));
        // 按照设备id 生成map获取夜间最小流量报警阈值
        Map<Long, Double> longDoubleMap = new HashMap<>(12);
        if (null != dmaTableList && dmaTableList.size() > 0) {
            longDoubleMap = dmaTableList.stream().collect(Collectors.toMap(Dmatable::getDeviceId, Dmatable::getMinFlow, (k1, k2) -> k2));

        }

        // 获取实时监测数据/是否报警    -(若设备为计量表类型 则查询计量表上下阈值判断夜间最小流量(2-4点)重新赋值阈值)
        Map<Long, Double> finalLongDoubleMap = longDoubleMap;
        siteListVo.forEach(info -> {
            Long siteId = info.getSiteId();
            // 默认设备不报警
            AtomicReference<String> flag = new AtomicReference<>("0");
            // 最大指标监测时间
            AtomicReference<Date> time = new AtomicReference<>(null);

            // 获取对应设备的指标基础信息
            List<IndexInfoVo> indexInfoVoList = info.getIndexInfoVoList();
            // 获取对应设备的指标实时监测信息
            Map<Long, List<RealDataVo>> longListMap = realDateMap.get(siteId);
            // 判断是否为计量表类型
            String type1 = info.getType();
            indexInfoVoList.forEach(indexInfo -> {
                if (null != longListMap) {
                    // 指标监测值
                    List<RealDataVo> realDataVos = longListMap.get(indexInfo.getIndexId());
                    if (!CollectionUtils.isEmpty(realDataVos)) {
                        RealDataVo realDataVo = realDataVos.get(0);
                        BigDecimal flow = realDataVo.getFlow();
                        indexInfo.setFlow(flow);
                        Date scadaTime = realDataVo.getScadaTime();
                        indexInfo.setTime(scadaTime);
                        // 默认不报警
                        indexInfo.setFlag("0");
                        // 外层监测更新时间比较
                        if (null != time.get()) {
                            if (scadaTime.compareTo(time.get()) > 0) {
                                time.set(scadaTime);
                            }
                        } else {
                            time.set(scadaTime);
                        }
                        // 判断设备是否离线 24小时
                        LocalDateTime localDateTime = LocalDateTime.now();
                        Date minusHours = DateTimeUtil.convertLDTToDate(localDateTime.minusHours(24));
                        if (scadaTime.getTime() < minusHours.getTime()) {
                            indexInfo.setFlag("2");
                            flag.set("2");
                        } else {
                            // 类型为0计量表
                            if (type1 != null && "0".endsWith(type1)) {
                                Double aDouble = finalLongDoubleMap.get(siteId);
                                // 0-4点才统计异常
                                LocalDateTime todayBeginTime = DateTimeUtil.getTodayBeginTime();
                                LocalDateTime plusHours = todayBeginTime.plusHours(4);
                                LocalDateTime now = LocalDateTime.now();

                                if (null != aDouble) {
                                    BigDecimal bigDecimal = BigDecimal.valueOf(aDouble);
                                    indexInfo.setNormDown(bigDecimal);
                                    if (flow.compareTo(bigDecimal) > 0) {
                                        if (now.compareTo(todayBeginTime) >= 0 && now.compareTo(plusHours) <= 0) {
                                            flag.set("1");
                                            indexInfo.setFlag("1");
                                        }

                                    }
                                }
                            } else {
                                BigDecimal normDown = indexInfo.getNormDown();
                                BigDecimal normUp = indexInfo.getNormUp();
                                if (normUp != null && normDown != null && flow != null) {
                                    if (flow.compareTo(normDown) > 0 && flow.compareTo(normUp) <= 0) {
                                        indexInfo.setFlag("0");
                                    }
                                } else {
                                    flag.set("1");
                                    indexInfo.setFlag("1");
                                }
                            }
                        }
                    }
                }

            });
            info.setTime(time.get());
            info.setFlag(flag.get());

        });
        List<SiteListVo> result = new ArrayList<>();
        List<String> stateList = new ArrayList<>();
        if (!StringUtils.isEmpty(state)) {
            stateList = Arrays.stream(state.split(Constant.CSV)).collect(Collectors.toList());
        }

        if (stateList.size() > 0) {
            List<String> finalStateList = stateList;
            siteListVo.forEach(site -> {
                if (finalStateList.contains(site.getFlag())) {
                    result.add(site);
                }
            });
        } else {
            result.addAll(siteListVo);
        }


        return result;
    }

    @Override
    public List<InstantaneousInfoVo> getMonitorInfo(String type, String code) {
        List<InstantaneousInfoVo> voList = new ArrayList<>();
        InstantaneousInfoVo vo1 = new InstantaneousInfoVo();
        InstantaneousInfoVo vo2 = new InstantaneousInfoVo();
        LocalDateTime startLocalDate = DateTimeUtil.getTodayBeginTime().minusDays(1);
        LocalDateTime endLocalDate = DateTimeUtil.getTodayEndTime().minusDays(1);
        LocalDateTime startLodLocalDate = startLocalDate.minusDays(1);
        LocalDateTime endLodLocalDate = endLocalDate.minusDays(1);
        if (StringUtils.isEmpty(type) || "1".equals(type)) {
            // 流量为夜间
            BigDecimal indexMinFow = onlineMonitorMapper.getIndexMinFow(code);
            vo1.setMinFlow(indexMinFow);
            vo2.setMinFlow(indexMinFow);

            // 获取该指标在时间范围内实时数据以小时展示
            List<InstantaneousInfoChildVo> monitorInfo1 = onlineMonitorMapper.getMonitorInfo(code, DateTimeUtil.convertLDTToDate(startLocalDate), DateTimeUtil.convertLDTToDate(endLocalDate));
            List<InstantaneousInfoChildVo> monitorInfo2 = onlineMonitorMapper.getMonitorInfo(code, DateTimeUtil.convertLDTToDate(startLodLocalDate), DateTimeUtil.convertLDTToDate(endLodLocalDate));

            vo1.setDimension("0");
            vo1.setMapList(monitorInfo1);
            vo2.setDimension("1");
            vo2.setMapList(monitorInfo2);
        }
        voList.add(vo1);
        voList.add(vo2);
        return voList;
    }

    @Override
    public List<WaterTimeTotalVo> getWaterTimeTotal(String code, Long areaId, String collectId, String startDate, String endDate, String startTime, String endTime) {
        String sTime = startDate + " " + startTime;
        String eTime = endDate + " " + endTime;
        LocalDateTime sDateTime = null;
        LocalDateTime eDateTime = null;
        if (StringUtils.isEmpty(sTime) || StringUtils.isEmpty(eTime)) {
            sDateTime = DateTimeUtil.getTodayBeginTime();
            eDateTime = DateTimeUtil.getTodayEndTime();
        } else {
            sDateTime = DateTimeUtil.fromString2LocalDateTime(sTime, DateTimeUtil.FULL_FORMAT);
            eDateTime = DateTimeUtil.fromString2LocalDateTime(eTime, DateTimeUtil.FULL_FORMAT);
        }

        List<Map<String, Date>> mapDateList = DateTimeUtil.getsAllDatesInTheDateTimeRange(sDateTime, eDateTime);

        List<String> codeList = new ArrayList<>();
        if (!StringUtils.isEmpty(code)) {
            codeList = Arrays.stream(code.split(Constant.CSV))
                    .collect(Collectors.toList());
        }
        return onlineMonitorMapper.getWaterTimeTotal(codeList, areaId, collectId, mapDateList, dictConfig.getMomentFlowAllIndex());
    }

    @Override
    public Object getWaterTimeFlowDetail(Page page, String code, Long areaId, String collectId, String startDate, String endDate, String startTime, String endTime, String flag, String isTotal, Integer dataGroup) {
        if (null == dataGroup || 0 == dataGroup) {
            dataGroup = 5;
        }
        String sTime = startDate + " " + startTime;
        String eTime = endDate + " " + endTime;
        LocalDateTime sDateTime = null;
        LocalDateTime eDateTime = null;
        if (StringUtils.isEmpty(sTime) || StringUtils.isEmpty(eTime)) {
            sDateTime = DateTimeUtil.getTodayBeginTime();
            eDateTime = DateTimeUtil.getTodayEndTime();
        } else {
            sDateTime = DateTimeUtil.fromString2LocalDateTime(sTime, DateTimeUtil.FULL_FORMAT);
            eDateTime = DateTimeUtil.fromString2LocalDateTime(eTime, DateTimeUtil.FULL_FORMAT);
        }

        List<Map<String, Date>> mapDateList = DateTimeUtil.getsAllDatesInTheDateTimeRange(sDateTime, eDateTime);


        List<String> codeList = new ArrayList<>();
        if (!StringUtils.isEmpty(code)) {
            codeList = Arrays.stream(code.split(Constant.CSV))
                    .collect(Collectors.toList());
        }
        if (StringUtils.isEmpty(flag)) {
            // 判断是否统计导出
            if (!StringUtils.isEmpty(isTotal)) {
                return onlineMonitorMapper.getWaterTimeFlowTotalDetail(codeList, areaId, collectId, mapDateList, dictConfig.getMomentFlowAllIndex());
            } else {
                return onlineMonitorMapper.getWaterTimeFlowDetail(codeList, mapDateList, areaId, collectId, dataGroup, dictConfig.getMomentFlowAllIndex());
            }


        } else {
            return onlineMonitorMapper.getWaterTimeFlowDetail(page, codeList, mapDateList, areaId, collectId, dataGroup, dictConfig.getMomentFlowAllIndex());
        }

    }
}
