package com.tofly.xrtygis.service.home.impl;

import com.tofly.xrtygis.commons.Constant;
import com.tofly.xrtygis.custom.mapper.WorkbenchMapper;
import com.tofly.xrtygis.custom.util.DateTimeUtil;
import com.tofly.xrtygis.entity.vo.home.workbench.*;
import com.tofly.xrtygis.service.home.WorkbenchService;
import lombok.AllArgsConstructor;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.temporal.TemporalAdjusters;
import java.util.*;
import java.util.stream.Collectors;

@Service
@AllArgsConstructor
public class WorkbenchServiceImpl implements WorkbenchService {

    private final WorkbenchMapper workbenchMapper;

    @Override
    public ModuleTitleVo getModuleTitle(String minimumFlowDay, String day) {
        ModuleTitleVo vo = new ModuleTitleVo();
        Date startFlow = null;
        Date endFlow = null;
        Date startYear = null;
        Date endYear = null;
        Date startOldYear = null;
        Date endOldYear = null;
        // 时间处理 天开始 天结束
        if (StringUtils.isEmpty(minimumFlowDay)) {
            LocalDateTime todayBeginTime = DateTimeUtil.getTodayBeginTime();
            int year1 = todayBeginTime.getYear();
            int monthValue = todayBeginTime.getMonthValue();
            int dayOfMonth = todayBeginTime.getDayOfMonth();
            startFlow = DateTimeUtil.convertLDTToDate(todayBeginTime);
            endFlow = DateTimeUtil.convertLDTToDate(DateTimeUtil.getTodayEndTime());
            vo.setMinimumFlowDay(year1 + Constant.DATE_SEPARATION + monthValue + Constant.DATE_SEPARATION + dayOfMonth);
        } else {
            String[] split = minimumFlowDay.split(Constant.DATE_SEPARATION);
            LocalDateTime localDate = LocalDateTime.of(Integer.parseInt(split[0]), Integer.parseInt(split[1]), Integer.parseInt(split[2]),0,0,0);
            startFlow = DateTimeUtil.convertLDTToDate(localDate);
            Date date = DateTimeUtil.convertLDTToDate(localDate);
            endFlow = DateTimeUtil.convertLDTToDate(DateTimeUtil.getAssignTodayEndTime(DateTimeUtil.fromLong2LocalDateTime(date.getTime())));
            vo.setMinimumFlowDay(minimumFlowDay);
        }
        // 时间处理 年月日开始 年月日结束 / 上一年月日开始 上一年月日结束
        if (StringUtils.isEmpty(day)) {
            // 年月日结束
            LocalDateTime localDateEnd = DateTimeUtil.getTodayEndTime();
            // 年月日开始
            LocalDateTime localDateStart = DateTimeUtil.fromLong2LocalDateTime(DateTimeUtil.localDateToDate(DateTimeUtil.getFirstDayOfCurrentYear()).getTime());

            vo.setYear(localDateEnd.getYear() + "");
            // 上一年月日结束
            LocalDateTime endOldYearLocalDate = localDateEnd.minusYears(1);

            // 上一年月日开始
            LocalDateTime startOldYearLocalDate = localDateStart.minusYears(1);
            vo.setComparedYear(endOldYearLocalDate.getYear() + "");
            startYear = DateTimeUtil.convertLDTToDate(localDateStart);
            endYear = DateTimeUtil.convertLDTToDate(localDateEnd);
            startOldYear = DateTimeUtil.convertLDTToDate(startOldYearLocalDate);
            endOldYear = DateTimeUtil.convertLDTToDate(endOldYearLocalDate);
        } else {
            String[] split = day.split(Constant.DATE_SEPARATION);
            // 年月日结束
            LocalDateTime localDateEnd = LocalDateTime.of(Integer.parseInt(split[0]), Integer.parseInt(split[1]), Integer.parseInt(split[2]),23,59,59);
            // 年月日开始
            LocalDateTime localDateStart = DateTimeUtil.fromLong2LocalDateTime(DateTimeUtil.localDateToDate(DateTimeUtil.getFirstDayOfCurrentYear()).getTime());

            vo.setYear(localDateEnd.getYear() + "");
            // 上一年月日结束
            LocalDateTime endOldYearLocalDate = localDateEnd.minusYears(1);
            // 上一年月日开始
            LocalDateTime startOldYearLocalDate = localDateStart.minusYears(1);
            vo.setComparedYear(endOldYearLocalDate.getYear() + "");
            startYear = DateTimeUtil.convertLDTToDate(localDateStart);
            endYear = DateTimeUtil.convertLDTToDate(localDateEnd);
            startOldYear = DateTimeUtil.convertLDTToDate(startOldYearLocalDate);
            endOldYear = DateTimeUtil.convertLDTToDate(endOldYearLocalDate);
        }
        // 结果查询
        List<String> dictNameList = workbenchMapper.getDictNameList(75L);
        Integer minimumFlowAlarmNum = workbenchMapper.getMinimumFlowAlarmNum(startFlow, endFlow);
        ModuleTitleVo moduleTitleInfo = workbenchMapper.getModuleTitleInfo(startYear, endYear, startOldYear, endOldYear, dictNameList);
        vo.setMinimumFlowAlarmNum(minimumFlowAlarmNum);
        vo.setWaterSupply(moduleTitleInfo.getWaterSupply());
        vo.setComparedSupply(moduleTitleInfo.getComparedSupply());
        vo.setTrendUpRate(moduleTitleInfo.getTrendUpRate());
        return vo;
    }

    @Override
    public WaterSupplyRankingVo getWaterSupplyRanking(String yearMonth) {
        WaterSupplyRankingVo vo = new WaterSupplyRankingVo();
        Date startDate = null;
        Date endDate = null;
        // 时间处理 月开始 月结束
        if (StringUtils.isEmpty(yearMonth)) {
            // 默认当月
            LocalDate firstDayOfCurrentMonth1 = DateTimeUtil.getFirstDayOfCurrentMonth();

            int year = firstDayOfCurrentMonth1.getYear();
            int monthValue = firstDayOfCurrentMonth1.getMonthValue();
            startDate = DateTimeUtil.localDateToDate(DateTimeUtil.getFirstDayOfCurrentMonth());
            endDate = DateTimeUtil.convertLDTToDate(DateTimeUtil.getTodayEndTime());
            vo.setYearMonth(year + Constant.DATE_SEPARATION + monthValue);
        } else {
            String[] split = yearMonth.split(Constant.DATE_SEPARATION);
            LocalDateTime localDate = LocalDateTime.of(Integer.parseInt(split[0]), Integer.parseInt(split[1]), 1,0,0,0);
            startDate = DateTimeUtil.convertLDTToDate(localDate);
            Date date = DateTimeUtil.convertLDTToDate(localDate.with(TemporalAdjusters.lastDayOfMonth()));
            endDate = DateTimeUtil.convertLDTToDate(DateTimeUtil.getAssignTodayEndTime(DateTimeUtil.fromLong2LocalDateTime(date.getTime())));
            vo.setYearMonth(yearMonth);
        }
        // 查询昨日供水量排行
        Date sDate = DateTimeUtil.convertLDTToDate(DateTimeUtil.getTodayBeginTime().minusDays(1));
        Date eDate = DateTimeUtil.convertLDTToDate(DateTimeUtil.getTodayEndTime().minusDays(1));
        List<WaterSupplyRankingChildVo> yesterdayWaterSupplyRanking = workbenchMapper.getWaterSupplyRanking(sDate, eDate);
        // 查询月份
        List<WaterSupplyRankingChildVo> waterSupplyRanking = workbenchMapper.getWaterSupplyRanking(startDate, endDate);
        // 查询昨日总供水量
        BigDecimal yesterdayTotalWaterSupply = workbenchMapper.getTotalWaterSupply(sDate, eDate);
        // 查询月份总供水量
        BigDecimal monthTotalWaterSupply = workbenchMapper.getTotalWaterSupply(startDate, endDate);

        // 处理
        yesterdayWaterSupplyRanking.forEach(info->{
            BigDecimal flow = info.getFlow();
            if(null!=yesterdayTotalWaterSupply&&yesterdayTotalWaterSupply.compareTo(BigDecimal.ZERO)>=1){
                info.setRate(flow.divide(yesterdayTotalWaterSupply,4,BigDecimal.ROUND_HALF_UP).multiply(BigDecimal.valueOf(100)));
            }else {
                info.setRate(BigDecimal.ZERO);
            }
        });
        waterSupplyRanking.forEach(info->{
            BigDecimal flow = info.getFlow();
            if(null!=monthTotalWaterSupply&&monthTotalWaterSupply.compareTo(BigDecimal.ZERO)>=1){
                info.setRate(flow.divide(monthTotalWaterSupply,4,BigDecimal.ROUND_HALF_UP).multiply(BigDecimal.valueOf(100)));
            }else {
                info.setRate(BigDecimal.ZERO);
            }
        });
        vo.setDay(DateTimeUtil.getLocalDate(sDate,DateTimeUtil.DAY_FORMAT));
        vo.setDayMapList(yesterdayWaterSupplyRanking);
        vo.setMapList(waterSupplyRanking);
        return vo;
    }

    @Override
    public MinimumFlowRankingVo getMinimumFlowRanking(String day) {
        MinimumFlowRankingVo vo = new MinimumFlowRankingVo();
        Date startFlow = null;
        Date endFlow = null;
        // 时间处理 年月日开始 年月日结束
        if (StringUtils.isEmpty(day)) {
            LocalDateTime todayBeginTime = DateTimeUtil.getTodayBeginTime();
            int year1 = todayBeginTime.getYear();
            int monthValue = todayBeginTime.getMonthValue();
            int dayOfMonth = todayBeginTime.getDayOfMonth();
            startFlow = DateTimeUtil.convertLDTToDate(todayBeginTime);
            endFlow = DateTimeUtil.convertLDTToDate(DateTimeUtil.getTodayEndTime());
            vo.setDay(year1 + Constant.DATE_SEPARATION + monthValue + Constant.DATE_SEPARATION + dayOfMonth);
        } else {
            String[] split = day.split(Constant.DATE_SEPARATION);
            LocalDateTime localDate = LocalDateTime.of(Integer.parseInt(split[0]), Integer.parseInt(split[1]), Integer.parseInt(split[2]),0,0,0);
            startFlow = DateTimeUtil.convertLDTToDate(localDate);
            Date date = DateTimeUtil.convertLDTToDate(localDate);
            endFlow = DateTimeUtil.convertLDTToDate(DateTimeUtil.getAssignTodayEndTime(DateTimeUtil.fromLong2LocalDateTime(date.getTime())));
            vo.setDay(day);
        }
        // 查询
        List<MinimumFlowRankingChildVo> minimumFlowRanking = workbenchMapper.getMinimumFlowRanking(startFlow, endFlow);
        vo.setMapList(minimumFlowRanking);
        return vo;
    }

    @Override
    public List<WaterSupplyAccumulativeVo> getWaterSupplyAccumulative(String yearMonth) {
        List<WaterSupplyAccumulativeVo> voList = new ArrayList<>();
        WaterSupplyAccumulativeVo vo1 = new WaterSupplyAccumulativeVo();
        WaterSupplyAccumulativeVo vo2 = new WaterSupplyAccumulativeVo();
        Date startYear = null;
        Date endYear = null;
        Date startOldYear = null;
        Date endOldYear = null;
        // 时间处理 年月日开始 年月日结束 / 上一年月日开始 上一年月日结束
        if (StringUtils.isEmpty(yearMonth)) {
            // 年月日结束
            Date date = DateTimeUtil.localDateToDate(DateTimeUtil.getLastDayOfCurrentMonth());
            LocalDateTime localDateEnd = DateTimeUtil.getAssignTodayEndTime(DateTimeUtil.fromLong2LocalDateTime(date.getTime()));
            // 年月日开始
            LocalDateTime localDateStart = DateTimeUtil.fromLong2LocalDateTime(DateTimeUtil.localDateToDate(DateTimeUtil.getFirstDayOfCurrentYear()).getTime());
            vo1.setYear(localDateEnd.getYear() + "");
            vo1.setDimension("0");
            // 上一年月日结束
            LocalDateTime endOldYearLocalDate = localDateEnd.minusYears(1);
            // 上一年月日开始
            LocalDateTime startOldYearLocalDate = localDateStart.minusYears(1);
            vo2.setYear(endOldYearLocalDate.getYear() + "");
            vo2.setDimension("1");
            startYear = DateTimeUtil.convertLDTToDate(localDateStart.plusMinutes(5));
            endYear = DateTimeUtil.convertLDTToDate(localDateEnd);
            startOldYear = DateTimeUtil.convertLDTToDate(startOldYearLocalDate.plusMinutes(5));
            endOldYear = DateTimeUtil.convertLDTToDate(endOldYearLocalDate);
        } else {
            String[] split = yearMonth.split(Constant.DATE_SEPARATION);
            // 年月日结束
            LocalDateTime localDateEnd =  LocalDateTime.of(Integer.parseInt(split[0]), Integer.parseInt(split[1]), 1,23,59,59);
            // 年月日开始
            LocalDateTime localDateStart = localDateEnd.with(TemporalAdjusters.firstDayOfYear());
            vo1.setYear(localDateEnd.getYear() + "");
            vo1.setDimension("0");
            // 上一年月日结束
            LocalDateTime endOldYearLocalDate = localDateEnd.minusYears(1);
            // 上一年月日开始
            LocalDateTime startOldYearLocalDate = localDateStart.minusYears(1);
            vo2.setYear(endOldYearLocalDate.getYear() + "");
            vo2.setDimension("1");
            startYear = DateTimeUtil.convertLDTToDate(localDateStart.plusMinutes(5));
            endYear = DateTimeUtil.convertLDTToDate(localDateEnd);
            startOldYear = DateTimeUtil.convertLDTToDate(startOldYearLocalDate.plusMinutes(5));
            endOldYear = DateTimeUtil.convertLDTToDate(endOldYearLocalDate);
        }
        // 查询
        List<String> dictNameList = workbenchMapper.getDictNameList(75L);
        List<WaterSupplyAccumulativeChildVo> childVoList1 = workbenchMapper.getWaterSupplyAccumulative(startYear, endYear, dictNameList);
        Map<String, WaterSupplyAccumulativeChildVo> collect1 = childVoList1.stream().collect(Collectors.toMap(WaterSupplyAccumulativeChildVo::getMonth, m -> m, (k1, k2) -> k2));
        List<WaterSupplyAccumulativeChildVo> childVoList2 = workbenchMapper.getWaterSupplyAccumulative(startOldYear, endOldYear, dictNameList);
        Map<String, WaterSupplyAccumulativeChildVo> collect2 = childVoList2.stream().collect(Collectors.toMap(WaterSupplyAccumulativeChildVo::getMonth, m -> m, (k1, k2) -> k2));

        // 获取时间范围内月份时间列表
        List<String> childMonthList1 = DateTimeUtil.getsAllMonthsInTheDateRange(DateTimeUtil.UDateToLocalDate(startYear), DateTimeUtil.UDateToLocalDate(endYear));
        List<String> childMonthList2 = DateTimeUtil.getsAllMonthsInTheDateRange(DateTimeUtil.UDateToLocalDate(startOldYear), DateTimeUtil.UDateToLocalDate(endOldYear));
        childMonthList1.forEach(
                month ->{
                    if(!collect1.containsKey(month)){
                        WaterSupplyAccumulativeChildVo vo = new WaterSupplyAccumulativeChildVo();
                        vo.setMonth(month);
                        vo.setWaterSupply(BigDecimal.ZERO);
                        childVoList1.add(vo);
                    }
                }
        );
        childMonthList2.forEach(
                month ->{
                    if(!collect2.containsKey(month)){
                        WaterSupplyAccumulativeChildVo vo = new WaterSupplyAccumulativeChildVo();
                        vo.setMonth(month);
                        vo.setWaterSupply(BigDecimal.ZERO);
                        childVoList2.add(vo);
                    }
                }
        );
        List<WaterSupplyAccumulativeChildVo> sort1 = childVoList1.stream().sorted(Comparator.comparing(WaterSupplyAccumulativeChildVo::getMonth)).collect(Collectors.toList());
        vo1.setMapList(sort1);
        voList.add(vo1);
        List<WaterSupplyAccumulativeChildVo> sort2 = childVoList2.stream().sorted(Comparator.comparing(WaterSupplyAccumulativeChildVo::getMonth)).collect(Collectors.toList());
        vo2.setMapList(sort2);
        voList.add(vo2);
        return voList;
    }

    @Override
    public List<WaterSupplyMonthVo> getWaterSupplyMonth(String yearMonth) {
        List<WaterSupplyMonthVo> voList = new ArrayList<>();
        WaterSupplyMonthVo vo1 = new WaterSupplyMonthVo();
        WaterSupplyMonthVo vo2 = new WaterSupplyMonthVo();
        Date startYear = null;
        Date endYear = null;
        Date startOldYear = null;
        Date endOldYear = null;
        // 时间处理 年月日开始 年月日结束 / 上一年月日开始 上一年月日结束
        if (StringUtils.isEmpty(yearMonth)) {
            // 年月日结束
            Date date = DateTimeUtil.localDateToDate(DateTimeUtil.getLastDayOfCurrentMonth());
            LocalDateTime localDateEnd = DateTimeUtil.getAssignTodayEndTime(DateTimeUtil.fromLong2LocalDateTime(date.getTime()));
            // 年月日开始
            LocalDateTime localDateStart = localDateEnd.with(TemporalAdjusters.firstDayOfYear());
            vo1.setYear(localDateEnd.getYear() + "");
            vo1.setDimension("0");
            // 上一年月日结束
            LocalDateTime endOldYearLocalDate = localDateEnd.minusYears(1);
            // 上一年月日开始
            LocalDateTime startOldYearLocalDate = localDateStart.minusYears(1);
            vo2.setYear(endOldYearLocalDate.getYear() + "");
            vo2.setDimension("1");
            startYear = DateTimeUtil.convertLDTToDate(localDateStart);
            endYear = DateTimeUtil.convertLDTToDate(localDateEnd);
            startOldYear = DateTimeUtil.convertLDTToDate(startOldYearLocalDate);
            endOldYear = DateTimeUtil.convertLDTToDate(endOldYearLocalDate);
        } else {
            String[] split = yearMonth.split(Constant.DATE_SEPARATION);
            LocalDateTime firstDayOfCurrentMonth = LocalDateTime.of(Integer.parseInt(split[0]), Integer.parseInt(split[1]), 1,23,59,59);
            // 年月日结束
            LocalDateTime localDateEnd = firstDayOfCurrentMonth.minusMonths(1);
            // 年月日开始
            LocalDateTime localDateStart = localDateEnd.with(TemporalAdjusters.firstDayOfYear());
            vo1.setYear(localDateEnd.getYear() + "");
            vo1.setDimension("0");
            // 上一年月日结束
            LocalDateTime endOldYearLocalDate = localDateEnd.minusYears(1);
            // 上一年月日开始
            LocalDateTime startOldYearLocalDate = localDateStart.minusYears(1);
            vo2.setYear(endOldYearLocalDate.getYear() + "");
            vo2.setDimension("1");
            startYear = DateTimeUtil.convertLDTToDate(localDateStart);
            endYear = DateTimeUtil.convertLDTToDate(localDateEnd);
            startOldYear = DateTimeUtil.convertLDTToDate(startOldYearLocalDate);
            endOldYear = DateTimeUtil.convertLDTToDate(endOldYearLocalDate);
        }


        // 查询
        List<String> dictNameList = workbenchMapper.getDictNameList(75L);
        List<WaterSupplyMonthChildVo> childVoList1 = workbenchMapper.getWaterSupplyMonth(startYear, endYear, dictNameList);
        Map<String, WaterSupplyMonthChildVo> collect1 = childVoList1.stream().collect(Collectors.toMap(WaterSupplyMonthChildVo::getMonth, m -> m, (k1, k2) -> k2));
        List<WaterSupplyMonthChildVo> childVoList2 = workbenchMapper.getWaterSupplyMonth(startOldYear, endOldYear, dictNameList);
        Map<String, WaterSupplyMonthChildVo> collect2 = childVoList2.stream().collect(Collectors.toMap(WaterSupplyMonthChildVo::getMonth, m -> m, (k1, k2) -> k2));

        // 获取时间范围内月份时间列表
        List<String> childMonthList1 = DateTimeUtil.getsAllMonthsInTheDateRange(DateTimeUtil.UDateToLocalDate(startYear), DateTimeUtil.UDateToLocalDate(endYear));
        List<String> childMonthList2 = DateTimeUtil.getsAllMonthsInTheDateRange(DateTimeUtil.UDateToLocalDate(startOldYear), DateTimeUtil.UDateToLocalDate(endOldYear));
        childMonthList1.forEach(
                month ->{
                    if(!collect1.containsKey(month)){
                        WaterSupplyMonthChildVo vo = new WaterSupplyMonthChildVo();
                        vo.setMonth(month);
                        vo.setWaterSupply(BigDecimal.ZERO);
                        childVoList1.add(vo);
                    }
                }
        );
        childMonthList2.forEach(
                month ->{
                    if(!collect2.containsKey(month)){
                        WaterSupplyMonthChildVo vo = new WaterSupplyMonthChildVo();
                        vo.setMonth(month);
                        vo.setWaterSupply(BigDecimal.ZERO);
                        childVoList2.add(vo);
                    }
                }
        );
        List<WaterSupplyMonthChildVo> sort1 = childVoList1.stream().sorted(Comparator.comparing(WaterSupplyMonthChildVo::getMonth)).collect(Collectors.toList());
        vo1.setMapList(sort1);
        voList.add(vo1);
        List<WaterSupplyMonthChildVo> sort2 = childVoList2.stream().sorted(Comparator.comparing(WaterSupplyMonthChildVo::getMonth)).collect(Collectors.toList());
        vo2.setMapList(sort2);
        voList.add(vo2);
        return voList;
    }

    @Override
    public List<MinimumFlowStateVo> getMinimumFlowState(String day) {
        List<MinimumFlowStateVo> voList = new ArrayList<>();
        MinimumFlowStateVo vo1 = new MinimumFlowStateVo();
        MinimumFlowStateVo vo2 = new MinimumFlowStateVo();
        Date startYear = null;
        Date endYear = null;

        // 时间处理 年月日开始 年月日结束
        if (StringUtils.isEmpty(day)) {
            LocalDateTime localDate = DateTimeUtil.getTodayBeginTime();
            LocalDateTime minusDays = localDate.minusDays(14);
            startYear = DateTimeUtil.convertLDTToDate(minusDays);
            endYear = DateTimeUtil.convertLDTToDate(localDate);
        } else {
            String[] split = day.split(Constant.DATE_SEPARATION);
            LocalDateTime localDate = LocalDateTime.of(Integer.parseInt(split[0]), Integer.parseInt(split[1]), Integer.parseInt(split[2]),23,59,59);
            LocalDateTime minusDays = localDate.minusDays(14);
            startYear = DateTimeUtil.convertLDTToDate(minusDays);
            endYear = DateTimeUtil.convertLDTToDate(localDate);

        }
        // 查询
        vo1.setState("0");
        List<MinimumFlowStateChildVo> stateChildVoList1= workbenchMapper.getMinimumFlowState("0", startYear, endYear);
        Map<String, MinimumFlowStateChildVo> collect1 = stateChildVoList1.stream().collect(Collectors.toMap(MinimumFlowStateChildVo::getDay, m -> m, (k1, k2) -> k2));
        vo2.setState("1");
        List<MinimumFlowStateChildVo> stateChildVoList2 = workbenchMapper.getMinimumFlowState("1", startYear, endYear);
        Map<String, MinimumFlowStateChildVo> collect2 = stateChildVoList2.stream().collect(Collectors.toMap(MinimumFlowStateChildVo::getDay, m -> m, (k1, k2) -> k2));

        // 获取时间范围内天数时间列表
        List<String> stringList = DateTimeUtil.getsAllDatesInTheDateRange(DateTimeUtil.UDateToLocalDate(startYear), DateTimeUtil.UDateToLocalDate(endYear));
        stringList.forEach(
                monthDay ->{
                    if(!collect1.containsKey(monthDay)){
                        MinimumFlowStateChildVo vo = new MinimumFlowStateChildVo();
                        vo.setDay(monthDay);
                        vo.setNum(0);
                        stateChildVoList1.add(vo);
                    }
                }
        );
        stringList.forEach(
                monthDay ->{
                    if(!collect2.containsKey(monthDay)){
                        MinimumFlowStateChildVo vo = new MinimumFlowStateChildVo();
                        vo.setDay(monthDay);
                        vo.setNum(0);
                        stateChildVoList2.add(vo);
                    }
                }
        );

        vo1.setMapList(stateChildVoList1.stream().sorted(Comparator.comparing(MinimumFlowStateChildVo::getDay)).collect(Collectors.toList()));
        vo2.setMapList(stateChildVoList2.stream().sorted(Comparator.comparing(MinimumFlowStateChildVo::getDay)).collect(Collectors.toList()));
        voList.add(vo1);
        voList.add(vo2);
        return voList;
    }


}
