/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.xrtygis.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.core.util.DateUtil;
import com.tofly.common.core.util.StringUtil;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.pojo.message.Messagepush;
import com.tofly.pojo.message.Messagerecipient;
import com.tofly.xrtygis.config.DictConfig;
import com.tofly.xrtygis.custom.entity.dma.ConstomTableList;
import com.tofly.xrtygis.custom.entity.enums.CompanyName;
import com.tofly.xrtygis.custom.entity.enums.GisMessage;
import com.tofly.xrtygis.custom.entity.vo.MinFlowErrorAnalyseVo;
import com.tofly.xrtygis.custom.util.Date7Utils;
import com.tofly.xrtygis.custom.util.DateTimeUtil;
import com.tofly.xrtygis.em.Constant;
import com.tofly.xrtygis.em.ExcelExportFileTypeEnum;
import com.tofly.xrtygis.entity.DeviceArchiveManage;
import com.tofly.xrtygis.entity.Dmanightflow;
import com.tofly.xrtygis.entity.Dmatable;
import com.tofly.xrtygis.entity.TfDeviceArchiveManage;
import com.tofly.xrtygis.entity.dto.DmanightflowCountDto;
import com.tofly.xrtygis.service.DevicemanagerService;
import com.tofly.xrtygis.service.DmanightflowService;
import com.tofly.xrtygis.service.DmatableService;
import com.tofly.xrtygis.service.TfDeviceArchiveManageService;
import com.tofly.xrtygis.strategy.ExcelStrategyContext;
import com.tofly.xrtygis.util.ExcelUtil;
import com.xxl.job.core.biz.model.ReturnT;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import com.tofly.xrtygis.mapper.DmanightflowMapper;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

/**
 * 夜间最小流量分析表
 *
 * @author wangjieliu
 * @date Tue May 25 00:00:00 CST 2021
 */
@Service("dmanightflowService")
@Slf4j
public class DmanightflowServiceImpl extends ServiceImpl<DmanightflowMapper, Dmanightflow> implements DmanightflowService {
    private final Logger logger = LoggerFactory.getLogger(DmanightflowServiceImpl.class);

    //查询累计流量的字典类型字段
    @Autowired
    private DictConfig dictConfig;

    @Autowired
    private ExcelStrategyContext excelStrategyContext;
    @Autowired
    private DmatableService dmatableService;
    @Autowired
    private TfDeviceArchiveManageService deviceArchiveManageService;

    @Override
    public IPage<Map> getMinFlow(HttpServletResponse response, String tableIds, String startTime, String endTime, String flag, String templateFlag, String isState, Page page) {
        //判断时间
        if (StringUtil.isEmpty(startTime) || StringUtil.isEmpty(endTime)) {
            throw new ToflyDeniedException("请选择月份时间");
        }
        if (StringUtil.isEmpty(tableIds)) {
            throw new ToflyDeniedException("请传入仪表ID");
        }
        //格式化时间
        Date startDate = DateUtil.formatDate(startTime, DateTimeUtil.FULL_FORMAT);
        Date endDate = DateUtil.formatDate(endTime, DateTimeUtil.FULL_FORMAT);
        //计算3个月后的时间
        Date addDate = Date7Utils.addNextPeriod(3, startDate);
        //比较时间是否大于3个月之后的时间
        if (endDate.after(addDate)) {
            //如果超过了3个月，则最多取到3个月的数据
            endDate = addDate;
        }
        if ("1".equals(flag)) {
            return baseMapper.getMinFlowCollet(tableIds, startDate, endDate, isState, page);
        } else if ("2".equals(flag)) {
            //需要导出的
            if ("1".equals(templateFlag)) {
                List<Map> mapList = baseMapper.getMinFlowDetail(tableIds, startDate, endDate, isState);
                //修改时间戳
//                mapList.forEach(k->{
//                    k.put("uday",k.get("uday").toString().substring(0,10));
//                });
                // 根据templateServiceType,templateFlag获取对应模板数据
                ExcelExportFileTypeEnum excelExportFileTypeEnum = ExcelExportFileTypeEnum.getExcelExportFileTypeEnum("nightFlowServiceImpl", templateFlag);
                if (null == excelExportFileTypeEnum) {
                    return null;
                }
                //将查询的结果转json字符串的格式，因为导出的时候只能传一个字符串；toJSONStringWithDateFormat可以转换时间格式字符串
                String resultListToString = JSONArray.toJSONStringWithDateFormat(mapList, "yyyy-MM-dd");
                // 从模型类型枚举根据templateServiceType获取策略对应模板数据
                List<Map<String, Object>> dataList = excelStrategyContext.getExportData("nightFlowServiceImpl", templateFlag, resultListToString);
                ExcelUtil.export(dataList.get(0), excelExportFileTypeEnum, Constant.FILE_PATH_TEST_EXPORT_EXCEL, response);
                return null;
            } else {
                IPage<Map> iPage = baseMapper.getMinFlowDetailPage(tableIds, startDate, endDate, page, isState);
                List<Map> mapList = iPage.getRecords();
                //修改时间戳
                mapList.forEach(k -> {
                    k.put("uday", k.get("uday").toString().substring(0, 10));
                });
                iPage.setRecords(mapList);
                return iPage;
            }
        }
        return null;
    }

    @Override
    public List<MinFlowErrorAnalyseVo> minFLowErrorAnalyse(String tableIds, String startDay, String endDay) {
        List<MinFlowErrorAnalyseVo> minFlowErrorAnalyseVos = baseMapper.minFLowErrorAnalyse(Arrays.stream(tableIds.split(",")).map(Long::parseLong).collect(Collectors.toList()), startDay, endDay);
        if (CollUtil.isEmpty(minFlowErrorAnalyseVos)) {
            return minFlowErrorAnalyseVos;
        }
        minFlowErrorAnalyseVos.forEach(info -> {
            List<Long> belowTableIdList = new ArrayList<>();
            getChildDeviceList(info.getTableId(), belowTableIdList);
            if (CollUtil.isEmpty(belowTableIdList)) {
                belowTableIdList.add(0L);
            }
            List<MinFlowErrorAnalyseVo> belowVos = baseMapper.minFLowErrorAnalyse(belowTableIdList, info.getScadaTime(), info.getScadaTime());
            info.setBelowErrorCount(0);
            if (CollUtil.isNotEmpty(belowVos)) {
                info.setBelowErrorCount(belowVos.stream().map(MinFlowErrorAnalyseVo::getThisErrorCount).collect(Collectors.toList()).stream().mapToInt(Integer::valueOf).sum());
            }
            info.setAllErrorCount(info.getThisErrorCount() + info.getBelowErrorCount());
        });
        return minFlowErrorAnalyseVos;
    }

    @Override
    public List<Dmanightflow> minFLowAnalyse(Long tableId, String queryType, String startDay, String endDay) {
        List<Long> tableIdList = new ArrayList<>();
        if (queryType.equals("1")) {
            tableIdList.add(tableId);
        } else {
            getChildDeviceList(tableId, tableIdList);
        }
        if (CollUtil.isEmpty(tableIdList)) {
            return null;
        } else {
            return this.list(new QueryWrapper<Dmanightflow>()
                    .in("TABLE_ID", tableIdList)
                    .between("to_char(MINTIME , 'yyyy-MM-dd')", startDay, endDay));
        }
    }

    @Override
    public ConstomTableList getTableTree(String isFlow) {
        //查询所有区域观察表
        List<Dmatable> dmatableList = dmatableService.list(Wrappers.<Dmatable>lambdaQuery().eq("1".equals(isFlow), Dmatable::getIsflow, "1")
                .eq("2".equals(isFlow), Dmatable::getIspress, "1"));
        // 自定义一个存放顶级公司名称的对象
        ConstomTableList dmatable = new ConstomTableList();
        dmatable.setId(0L);
        dmatable.setSchName(CompanyName.CD_TY_SW.getName());
        List<Map> mapList = JSONObject.parseArray(JSONObject.toJSONString(dmatableList), Map.class);
        //存放区域观察表
        dmatable.setTableList(mapList);
        return new ConstomTableList();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean nightMinimumFlowAnalysis(Date date) {
        // 获取当前日期观察表(2:00 ~ 4:00)流量
        LocalDateTime now = DateTimeUtil.fromLong2LocalDateTime(date.getTime());
        Date startTime = DateTimeUtil.convertLDTToDate(now.plusHours(0));
        Date endTime = DateTimeUtil.convertLDTToDate(now.plusHours(5));
        List<Dmanightflow> flowList = baseMapper.getFlowList(startTime, endTime, dictConfig.getNightFlowAllIndex());
        List<Dmanightflow> pressureList = baseMapper.getPressureList(startTime, endTime);
        // 清空时间端范围内数据
        this.remove(new QueryWrapper<Dmanightflow>().lambda().between(Dmanightflow::getMintime, startTime, endTime));
        flowList.forEach(info -> {
            // 保证 流量与压力 采集时间在 5分钟之内
            LocalDateTime time1 = DateTimeUtil.fromLong2LocalDateTime(info.getMintime().getTime());
            Long tableId = info.getTableId();
            pressureList.forEach(pressure -> {
                Long pressureTableId = pressure.getTableId();
                LocalDateTime time2 = DateTimeUtil.fromLong2LocalDateTime(pressure.getMintime().getTime());
                LocalDateTime startTime1 = time1.minusMinutes(5);
                LocalDateTime endTime1 = time1.plusMinutes(5);
                if (tableId.equals(pressureTableId)) {
                    if (startTime1.isBefore(time2) || time2.isBefore(endTime1)) {
                        info.setMinpval(pressure.getMinpval());
                    }

                }

            });
            Date time = new Date();
            info.setUday(time);
            info.setUtime(time);
            info.setUmonth(DateTimeUtil.getLocalDate(time, DateTimeUtil.MONTHS_FORMAT));
        });
        if (flowList.size() > 0) {
            // 获取异常信息发送短信给所有人
            List<Dmanightflow> collect = flowList.stream().filter(i -> "1".equals(i.getUsfl())).collect(Collectors.toList());
            AtomicReference<Long> messagePushId = new AtomicReference<>(baseMapper.getPlanTempMaxId("TF_YWPN_MESSAGEPUSH_W"));
            AtomicReference<Long> messageRecipientId = new AtomicReference<>(baseMapper.getPlanTempMaxId("TF_YWPN_MESSAGERECIPIENT_W"));
            // 获取所有用户
            List<Long> userIdList = baseMapper.getUserIdList();
            List<Messagepush> messagePushList = new ArrayList<>();
            List<Messagerecipient> messageRecipientList = new ArrayList<>();
            collect.forEach(data -> {
                // 消息组合
                String tableName = data.getTableName();
                Date mintime = data.getMintime();
                Double minval = data.getMinval();
                Double tval = data.getTval();
                String mess = tableName + "设备在" + DateTimeUtil.getLocalDate(mintime, DateTimeUtil.FULL_FORMAT) + ",夜间最小流量(监测值为: " + minval + " m³/h,阈值: " + tval + "m³/h) 超标:" + (minval - tval) + "m³/h";
                userIdList.forEach(userId -> {
                    Messagepush messagepush = new Messagepush();
                    Long message1 = messagePushId.get();
                    Long recipient1 = messageRecipientId.get();
                    // 消息主题/消息类型/消息表名/消息表中id/路由地址跳转
                    messagepush.setId(message1);
                    messagepush.setTopic(GisMessage.YJZXLL_ERROR.getTopic());
                    messagepush.setType(GisMessage.YJZXLL_ERROR.getType());
                    messagepush.setTableName(GisMessage.YJZXLL_ERROR.getTableName());
                    messagepush.setMessage(mess);
                    messagepush.setTableId(data.getId());
                    // 跳转到工单页面
                    messagepush.setRouteIp("");
                    messagepush.setRouteIpApp("");
                    messagepush.setCreateUser(userId);
                    messagepush.setCreateTime(DateTimeUtil.convertLDTToDate(now));
                    // 消息与接收人信息处理
                    Messagerecipient messagerecipient = new Messagerecipient();
                    messagerecipient.setId(recipient1);
                    messagerecipient.setMessageId(message1);
                    messagerecipient.setRecipient(userId);
                    messagerecipient.setFlag("0");
                    messagePushList.add(messagepush);
                    messageRecipientList.add(messagerecipient);
                    Long idAndSet = message1 + 1;
                    Long andSet = recipient1 + 1;
                    messagePushId.set(idAndSet);
                    messageRecipientId.set(andSet);
                });
            });

            if (messagePushList.size() > 0) {
//                baseMapper.addMessageList(messagePushList);
//                baseMapper.addMessageRecipientList(messageRecipientList);
                ExecutorService executorService = Executors.newSingleThreadExecutor();
                executorService.execute(() -> {
                    //为了防止SQL语句超出长度出错，分成几次插入
                    if (messagePushList.size() <= 2000) {
                        baseMapper.addMessageList(messagePushList);
                    } else {
                        int times = messagePushList.size() / 2000;
                        for (int i = 0; i <= times; i++) {
                            baseMapper.addMessageList(messagePushList.subList(i * 2000, Math.min((i + 1) * 2000, messagePushList.size())));
                        }
                    }
                });
                logger.info("新增消息推送数据完成");

                ExecutorService executorService2 = Executors.newSingleThreadExecutor();
                executorService2.execute(() -> {
                    //为了防止SQL语句超出长度出错，分成几次插入
                    if (messageRecipientList.size() <= 2000) {
                        baseMapper.addMessageRecipientList(messageRecipientList);
                    } else {
                        int times = messageRecipientList.size() / 2000;
                        for (int i = 0; i <= times; i++) {
                            baseMapper.addMessageRecipientList(messageRecipientList.subList(i * 2000, Math.min((i + 1) * 2000, messageRecipientList.size())));
                        }
                    }
                });
                logger.info("新增消息推送数据完成");
            }
            return this.saveBatch(new HashSet<>(flowList));
        } else {
            return true;
        }

    }

    @Override
    public Object getDmanightflowPageCount(Page page, DmanightflowCountDto dmanightflow) {
        filterParams(dmanightflow);
        Page<Map<String, Object>> maps = baseMapper.getDmanightflowPage(page, dmanightflow);
        return maps;
    }

    @Override
    public Object getDmanightflowCountStats(Page page, DmanightflowCountDto dmanightflow) {
        filterParams(dmanightflow);
        List<Map<String, Object>> maps = baseMapper.getDmanightflowCountStats(page, dmanightflow);
        return maps;
    }

    /**
     * 过滤参数
     * @param dmanightflow
     */
    private static void filterParams(DmanightflowCountDto dmanightflow) {
        if (dmanightflow == null || (dmanightflow.getStart() == null || dmanightflow.getEnd() == null)) {
            throw new RuntimeException("开始时间和结束时间参数不能为空");
        }
        if (!StringUtils.hasText(dmanightflow.getUsfl())){
            dmanightflow.setUsfl("0");
        }
    }

    /**
     *
     * @description 查找所有下级设备ID
     */
    public void getChildDeviceList(Long tableId, List<Long> belowTableIdList) {
        List<DeviceArchiveManage> deviceArchiveManageAll = deviceArchiveManageService.list(new QueryWrapper<DeviceArchiveManage>().lambda()
                .eq(DeviceArchiveManage::getParentDeviceId, tableId));
        if (CollUtil.isNotEmpty(deviceArchiveManageAll)) {
            belowTableIdList.addAll(deviceArchiveManageAll.stream().map(DeviceArchiveManage::getId).collect(Collectors.toList()));
            deviceArchiveManageAll.forEach(info -> getChildDeviceList(info.getId(), belowTableIdList));
        }
    }
}
