/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.xrtygis.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.xrtygis.entity.DeviceArchiveManage;
import com.tofly.xrtygis.entity.TfDeviceArchiveManage;
import com.tofly.xrtygis.entity.TfLineInfo;
import com.tofly.xrtygis.service.TfDeviceArchiveManageService;
import com.tofly.xrtygis.service.TfLineInfoService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.tofly.xrtygis.mapper.TfLineInfoMapper;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 线路信息表
 *
 * @author JF
 * @date Tue Aug 01 00:00:00 CST 2023
 */
@Service("tfLineInfoService")
public class TfLineInfoServiceImpl extends ServiceImpl<TfLineInfoMapper, TfLineInfo> implements TfLineInfoService {

    @Autowired
    private TfDeviceArchiveManageService deviceArchiveManageService;

    @Override
    public List<TfLineInfo> lineTree() {
        List<TfLineInfo> parents = this.list(new QueryWrapper<TfLineInfo>().lambda()
                .isNull(TfLineInfo::getParentId));
        List<TfLineInfo> all = this.list();
        for (TfLineInfo parent : parents) {
            List<TfLineInfo> children = this.list(new QueryWrapper<TfLineInfo>().lambda()
                    .eq(TfLineInfo::getParentId,parent.getId()));
//            List<DeviceArchiveManage> deviceArchiveManages = deviceArchiveManageService.list(new QueryWrapper<DeviceArchiveManage>().lambda()
//                    .eq(DeviceArchiveManage::getBelongLineId,parent.getId()));
//            if (CollUtil.isNotEmpty(deviceArchiveManages)) {
//                parent.setDeviceIds(deviceArchiveManages.stream().map(DeviceArchiveManage::getId).collect(Collectors.toList()));
//            }
            parent.setChildren(getRegionList(children, all));
        }
        TfLineInfo waterLine = parents.get(0);
        waterLine.setDeviceIds(deviceArchiveManageService.list(new QueryWrapper<DeviceArchiveManage>().lambda()
                .eq(DeviceArchiveManage::getDetectionType, "0"))
                .stream().map(DeviceArchiveManage::getId).collect(Collectors.toList()));
        TfLineInfo elecLine = parents.get(1);
        elecLine.setDeviceIds(deviceArchiveManageService.list(new QueryWrapper<DeviceArchiveManage>().lambda()
                .eq(DeviceArchiveManage::getDetectionType, "1"))
                .stream().map(DeviceArchiveManage::getId).collect(Collectors.toList()));
        return parents;
    }

    /**
     *
     * @param children 需要封装的列表集
     * @param all 所有列表集
     * @return regionVosList 包含子集的列表集
     * @description 递归市区
     */
    public List<TfLineInfo> getRegionList(List<TfLineInfo> children, List<TfLineInfo> all) {
        List<TfLineInfo> list = new ArrayList<>();
        List<TfLineInfo> childrenFinal;
        if (CollectionUtils.isNotEmpty(children)) {
            for (TfLineInfo vo : children) {
                if (vo.getParentId() == null) {
                    continue;
                }
                List<Long> deviceIdList = new ArrayList<>();
                List<TfLineInfo> lineInfoListFinal = new ArrayList<>();
                lineInfoListFinal.add(vo);
                getDeviceAllList(lineInfoListFinal, all, deviceIdList);
                if (CollUtil.isNotEmpty(deviceIdList)) {
                    vo.setDeviceIds(deviceIdList.stream().distinct().collect(Collectors.toList()));
                } else {
                    vo.setDeviceIds(deviceIdList);
                }
                if (vo.getName().equals("南校区")) {
                    vo.setDeviceIds(deviceArchiveManageService.list(new QueryWrapper<DeviceArchiveManage>().lambda()
                            .eq(DeviceArchiveManage::getBelongToCollege, "0"))
                            .stream().map(DeviceArchiveManage::getId).collect(Collectors.toList()));
                }
                if (vo.getName().equals("北校区")) {
                    vo.setDeviceIds(deviceArchiveManageService.list(new QueryWrapper<DeviceArchiveManage>().lambda()
                                    .eq(DeviceArchiveManage::getBelongToCollege, "1"))
                            .stream().map(DeviceArchiveManage::getId).collect(Collectors.toList()));
                }
                childrenFinal = all.stream().filter(e -> Objects.equals(e.getParentId(), vo.getId())).collect(Collectors.toList());
                if (CollUtil.isEmpty(childrenFinal)) {
                    vo.setChildren(null);
                } else {
                    vo.setChildren(getRegionList(childrenFinal, all));
                }
                list.add(vo);
            }
        }
        return list;
    }

    /**
     *
     * @param children 需要封装的列表集
     * @param all 所有列表集
     * @return regionVosList 包含子集的列表集
     * @description 递归市区
     */
    public void getDeviceAllList(List<TfLineInfo> children, List<TfLineInfo> all, List<Long> deviceIdAll) {
        List<TfLineInfo> childrenFinal;
        for (TfLineInfo vo : children) {
            if (vo.getParentId() == null) {
                continue;
            }
            List<DeviceArchiveManage> deviceArchiveManages = deviceArchiveManageService.list(new QueryWrapper<DeviceArchiveManage>().lambda()
                    .eq(DeviceArchiveManage::getBelongLineId,vo.getId()));
            if (CollUtil.isNotEmpty(deviceArchiveManages)) {
                deviceIdAll.addAll(deviceArchiveManages.stream().map(DeviceArchiveManage::getId).collect(Collectors.toList()));
            }
            childrenFinal = all.stream().filter(e -> Objects.equals(e.getParentId(), vo.getId())).collect(Collectors.toList());
            if (CollUtil.isNotEmpty(childrenFinal)) {
                getDeviceAllList(childrenFinal, all, deviceIdAll);
            }
        }
    }
}
