package com.tofly.building.controller;


import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.haiqiu.tools.word.utils.WordUtils;
import com.tofly.building.entity.BuildingReport;
import com.tofly.building.entity.dto.BuildingReportDto;
import com.tofly.building.entity.vo.BuildReportVo;
import com.tofly.building.service.BuildingReportService;
import io.swagger.annotations.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;

import javax.annotation.Resource;
import java.io.InputStream;
import java.io.Serializable;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static org.apache.ibatis.io.Resources.getResourceAsStream;

/**
 * 楼栋用电用水报告(BuildingReport)表控制层
 *
 * @author HaiQiu
 * @since 2023-03-17 09:16:33
 */
@Api(tags = "楼栋用电用水报告")
@RestController
@RequestMapping("buildingReport")
public class BuildingReportController {
    /**
     * 服务对象
     */
    @Resource
    private BuildingReportService buildingReportService;

    /**
     * 分页查询数据
     *
     * @param page           分页对象
     * @param buildingReport 查询实体
     * @return 所有数据
     */
    @ToFlyAppLog(title = "楼栋用电用水报告-分页查询数据")
    @ApiOperation(value = "分页查询数据")
    @GetMapping("/page")
    public ResultRespone<Page<BuildingReport>> pageList(Page page, @ModelAttribute BuildingReport buildingReport) {
        return ResultRespone.success(buildingReportService.pageList(page, buildingReport), "查询成功");
    }

    /**
     * 查询所有数据
     *
     * @param buildingReport 查询实体
     * @return 所有数据
     */
    @ToFlyAppLog(title = "楼栋用电用水报告-查询所有数据")
    @ApiOperation(value = "查询所有数据")
    @GetMapping("/list")
    public ResultRespone<List<BuildingReport>> allList(@ModelAttribute BuildingReport buildingReport) {
        return ResultRespone.success(buildingReportService.allList(buildingReport), "查询成功");
    }

    /**
     * 通过主键查询单条数据
     *
     * @param id 主键
     * @return 单条数据
     */
    @ToFlyAppLog(title = "楼栋用电用水报告-通过主键查询单条数据")
    @ApiOperation(value = "通过主键查询单条数据")
    @ApiImplicitParam(name = "id", value = "数据ID")
    @GetMapping("{id}")
    public ResultRespone<BuildingReport> get(@PathVariable Serializable id) {
        return ResultRespone.success(buildingReportService.get(id), "查询成功");
    }

    /**
     * 新增数据
     *
     * @param buildingReportDto 实体
     * @param files             上传文件附件
     * @return 新增结果
     */
    @ToFlyAppLog(title = "楼栋用电用水报告-新增数据")
    @ApiOperation(value = "新增数据")
    @PostMapping
    public ResultRespone insert(@RequestPart @Validated BuildingReportDto buildingReportDto,
                                @RequestPart(required = false) @ApiParam("附件列表") List<MultipartFile> files) {
        buildingReportService.insert(buildingReportDto, files);
        return ResultRespone.success(null, "新增成功");
    }

    /**
     * 修改数据
     *
     * @param buildingReportDto 实体
     * @param files             上传文件附件
     * @return 修改结果
     */
    @ToFlyAppLog(title = "楼栋用电用水报告-修改数据")
    @ApiOperation(value = "修改数据")
    @PutMapping
    public ResultRespone edit(@RequestPart @Validated BuildingReportDto buildingReportDto,
                              @RequestPart(required = false) @ApiParam("附件列表") List<MultipartFile> files) {
        buildingReportService.edit(buildingReportDto, files);
        return ResultRespone.success(null, "修改成功");
    }

    /**
     * 删除数据
     *
     * @param idList 主键结合
     * @return 删除结果
     */
    @ToFlyAppLog(title = "楼栋用电用水报告-删除数据")
    @ApiOperation(value = "删除数据")
    @ApiImplicitParam(name = "idList", value = "数据ID数组集合")
    @DeleteMapping
    public ResultRespone delete(@RequestParam("idList") List<Long> idList) {
        buildingReportService.delete(idList);
        return ResultRespone.success(null, "删除成功");
    }

    /**
     * 数据导入Excel模板下载
     */
    @ToFlyAppLog(title = "楼栋用电用水报告-模版下载")
    @ApiOperation(value = "模版下载")
    @GetMapping("/template")
    public void template() {
        buildingReportService.template();
    }

    /**
     * Excel数据导入
     *
     * @param file 导入的Excel数据文件
     * @return 导入结果
     */
    @ToFlyAppLog(title = "楼栋用电用水报告-Excel数据导入")
    @ApiOperation(value = "数据导入")
    @ApiImplicitParam(name = "file", value = "Excel文件", required = true)
    @PostMapping("/import")
    public ResultRespone importData(@RequestParam(name = "file") MultipartFile file) {
        buildingReportService.importData(file);
        return ResultRespone.success(null, "导入成功");
    }


    /**
     * 数据导出
     *
     * @param page           分页
     * @param buildingReport 查询模型
     * @param isAll          是否导出全部数据，true是，false否
     */
    @ToFlyAppLog(title = "楼栋用电用水报告-数据导出")
    @ApiOperation(value = "数据导出")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "isAll", value = "是否导出全部数据，true是，false否（默认否）"),
    })
    @GetMapping("/export")
    public void export(Page page, BuildingReport buildingReport, boolean isAll) {
        buildingReportService.export(page, buildingReport, isAll);
    }

    /**
     * 生成能耗报告
     *
     * @param time 自定时间
     */
    @ApiOperation(value = "生成能耗报告")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "time", value = "自定义导出时间 yyyy-MM", dataType = "String", required = true)
    })
    @ToFlyAppLog(title = "生成能耗报告")
    @GetMapping("/calReport")
    public ResultRespone calReport(String time) {
        return ResultRespone.success(buildingReportService.calReport(time), "成功");
    }


    /**
     * 能耗报告PDF在线下载预览
     *
     * @param time 自定义时间
     * @param sendMail 邮箱接收方
     */
    @ApiOperation(value = "能耗报告PDF在线下载预览")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "time", value = "能耗报告PDF在线下载预览:yyyy-MM", dataType = "String", required = true),
            @ApiImplicitParam(name = "sendMail", value = "邮箱接收方（不传用默认邮箱地址）", dataType = "String")
    })
    @ToFlyAppLog(title = "能耗报告PDF在线下载预览")
    @GetMapping("/getReport")
    public void getReport(String time, String sendMail) {
        buildingReportService.getReport(time, sendMail);
    }
}

