package com.tofly.building.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.building.entity.BuildingReportMouth;
import com.tofly.building.entity.dto.BuildingReportMouthDto;
import com.tofly.building.service.BuildingReportMouthService;
import io.swagger.annotations.*;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;

import javax.annotation.Resource;
import java.io.Serializable;
import java.util.List;

/**
 * 楼栋用电用水报告月份表(BuildingReportMouth)表控制层
 *
 * @author HaiQiu
 * @since 2023-03-20 09:51:11
 */
@Api(tags = "楼栋用电用水报告月份表")
@RestController
@RequestMapping("buildingReportMouth")
public class BuildingReportMouthController {
    /**
     * 服务对象
     */
    @Resource
    private BuildingReportMouthService buildingReportMouthService;

    /**
     * 分页查询数据
     *
     * @param page                分页对象
     * @param buildingReportMouth 查询实体
     * @return 所有数据
     */
    @ToFlyAppLog(title = "楼栋用电用水报告月份表-分页查询数据")
    @ApiOperation(value = "分页查询数据")
    @GetMapping("/page")
    public ResultRespone<Page<BuildingReportMouth>> pageList(Page page, @ModelAttribute BuildingReportMouth buildingReportMouth) {
        return ResultRespone.success(buildingReportMouthService.pageList(page, buildingReportMouth), "查询成功");
    }

    /**
     * 查询所有数据
     *
     * @param buildingReportMouth 查询实体
     * @return 所有数据
     */
    @ToFlyAppLog(title = "楼栋用电用水报告月份表-查询所有数据")
    @ApiOperation(value = "查询所有数据")
    @GetMapping("/list")
    public ResultRespone<List<BuildingReportMouth>> allList(@ModelAttribute BuildingReportMouth buildingReportMouth) {
        return ResultRespone.success(buildingReportMouthService.allList(buildingReportMouth), "查询成功");
    }

    /**
     * 通过主键查询单条数据
     *
     * @param id 主键
     * @return 单条数据
     */
    @ToFlyAppLog(title = "楼栋用电用水报告月份表-通过主键查询单条数据")
    @ApiOperation(value = "通过主键查询单条数据")
    @ApiImplicitParam(name = "id", value = "数据ID")
    @GetMapping("{id}")
    public ResultRespone<BuildingReportMouth> get(@PathVariable Serializable id) {
        return ResultRespone.success(buildingReportMouthService.get(id), "查询成功");
    }

    /**
     * 新增数据
     *
     * @param buildingReportMouthDto 实体
     * @param files                  上传文件附件
     * @return 新增结果
     */
    @ToFlyAppLog(title = "楼栋用电用水报告月份表-新增数据")
    @ApiOperation(value = "新增数据")
    @PostMapping
    public ResultRespone insert(@RequestPart @Validated BuildingReportMouthDto buildingReportMouthDto,
                                @RequestPart(required = false) @ApiParam("附件列表") List<MultipartFile> files) {
        buildingReportMouthService.insert(buildingReportMouthDto, files);
        return ResultRespone.success(null, "新增成功");
    }

    /**
     * 修改数据
     *
     * @param buildingReportMouthDto 实体
     * @param files                  上传文件附件
     * @return 修改结果
     */
    @ToFlyAppLog(title = "楼栋用电用水报告月份表-修改数据")
    @ApiOperation(value = "修改数据")
    @PutMapping
    public ResultRespone edit(@RequestPart @Validated BuildingReportMouthDto buildingReportMouthDto,
                              @RequestPart(required = false) @ApiParam("附件列表") List<MultipartFile> files) {
        buildingReportMouthService.edit(buildingReportMouthDto, files);
        return ResultRespone.success(null, "修改成功");
    }

    /**
     * 删除数据
     *
     * @param idList 主键结合
     * @return 删除结果
     */
    @ToFlyAppLog(title = "楼栋用电用水报告月份表-删除数据")
    @ApiOperation(value = "删除数据")
    @ApiImplicitParam(name = "idList", value = "数据ID数组集合")
    @DeleteMapping
    public ResultRespone delete(@RequestParam("idList") List<Long> idList) {
        buildingReportMouthService.delete(idList);
        return ResultRespone.success(null, "删除成功");
    }

    /**
     * 数据导入Excel模板下载
     */
    @ToFlyAppLog(title = "楼栋用电用水报告月份表-模版下载")
    @ApiOperation(value = "模版下载")
    @GetMapping("/template")
    public void template() {
        buildingReportMouthService.template();
    }

    /**
     * Excel数据导入
     *
     * @param file 导入的Excel数据文件
     * @return 导入结果
     */
    @ToFlyAppLog(title = "楼栋用电用水报告月份表-Excel数据导入")
    @ApiOperation(value = "数据导入")
    @ApiImplicitParam(name = "file", value = "Excel文件", required = true)
    @PostMapping("/import")
    public ResultRespone importData(@RequestParam(name = "file") MultipartFile file) {
        buildingReportMouthService.importData(file);
        return ResultRespone.success(null, "导入成功");
    }


    /**
     * 数据导出
     *
     * @param page                分页
     * @param buildingReportMouth 查询模型
     * @param isAll               是否导出全部数据，true是，false否
     */
    @ToFlyAppLog(title = "楼栋用电用水报告月份表-数据导出")
    @ApiOperation(value = "数据导出")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "isAll", value = "是否导出全部数据，true是，false否（默认否）"),
    })
    @GetMapping("/export")
    public void export(Page page, BuildingReportMouth buildingReportMouth, boolean isAll) {
        buildingReportMouthService.export(page, buildingReportMouth, isAll);
    }
}

