package com.tofly.building.controller;

import com.tofly.building.entity.BuildingReport;
import com.tofly.building.entity.ToEmail;
import com.tofly.building.entity.dto.SendReportMailDto;
import com.tofly.building.service.BuildingReportService;
import com.tofly.building.service.CustomMailService;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;

/**
 * @author haiqiu
 * @date 2023/4/6
 */
@Api(tags = "邮件在线发送")
@RestController
@RequestMapping("mail")
public class MailTestController {
    @Autowired
    private CustomMailService customMailService;

    @Autowired
    private BuildingReportService buildingReportService;

    /**
     * 发送字符串邮件
     * @param toEmail 发送邮件内容
     * @return 消息提示
     */
    @ToFlyAppLog(title = "发送邮件-发送字符串邮件")
    @GetMapping("/str")
    public ResultRespone sendMailString(@ModelAttribute ToEmail toEmail) {
        return ResultRespone.success(customMailService.sendMailString(toEmail));
    }

    /**
     * 发送html邮件
     * @param toEmail 发送html邮件
     * @return 消息提示
     */
    @ToFlyAppLog(title = "发送邮件-发送html邮件")
    @GetMapping("/html")
    public ResultRespone sendMailHtml(@ModelAttribute ToEmail toEmail) {
        return ResultRespone.success(customMailService.sendMailHtml(toEmail));
    }

    /**
     * 发送能耗报告邮件
     * @param sendReportMailDto
     * @return 消息提示
     */
    @ToFlyAppLog(title = "发送邮件-能耗报告")
    @ApiOperation(value = "发送邮件-能耗报告")
    @PostMapping("/sendReportMail")
    public ResultRespone sendReportMail(@RequestBody SendReportMailDto sendReportMailDto) {
        buildingReportService.sendMailCustom(sendReportMailDto.getTimeStr(), sendReportMailDto.getReports(), sendReportMailDto.getSendMail());
        return ResultRespone.success();
    }
}
