package com.tofly.building.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.haiqiu.tools.excel.utils.ExcelUtils;
import com.tofly.building.entity.BuildingReport;
import com.tofly.building.entity.ScadaDeviceArchiveManage;
import com.tofly.building.entity.dto.ScadaDeviceArchiveManageDto;
import com.tofly.building.entity.vo.BuildReportVo;
import com.tofly.building.entity.vo.SafeElectricityVo;
import com.tofly.building.entity.vo.ScadaScadaDeviceArchiveManageVo;
import com.tofly.building.service.BuildingReportService;
import com.tofly.building.service.ScadaDeviceArchiveManageService;
import com.tofly.common.core.entity.ResultRespone;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * @author HaiQiu
 * @since 2023/3/6
 */
@Api(tags = "scada在线监控管理")
@RestController
@RequestMapping("moment")
public class ScadaMomentController {

    @Autowired
    private ScadaDeviceArchiveManageService scadaDeviceArchiveManageService;

    @Autowired
    private BuildingReportService buildingReportService;


    /**
     * 在线监控实时查询
     *
     * @param page      分页
     * @param manageDto 查询参数
     * @return 分页数据
     */
    @ApiOperation(value = "分页查询实时数据")
    @GetMapping("/page/monitor")
    public ResultRespone<Page<ScadaScadaDeviceArchiveManageVo>> pageList(Page page, @ModelAttribute ScadaDeviceArchiveManageDto manageDto) {
        return ResultRespone.success(scadaDeviceArchiveManageService.pageList(page, manageDto));
    }


    /**
     * 查询所有实时数据
     *
     * @param manageDto 查询所有实时数据
     * @return 所有数据
     */
    @ApiOperation(value = "查询所有实时数据")
    @PostMapping("/list/monitor")
    public ResultRespone<List<ScadaDeviceArchiveManage>> allListMonitor(@ModelAttribute ScadaDeviceArchiveManageDto manageDto) {
        return ResultRespone.success(scadaDeviceArchiveManageService.allListMonitor(manageDto));
    }


    /**
     * 查询所有统计数据
     *
     * @param manageDto 查询所有统计数据
     * @return 所有数据
     */
    @ApiOperation(value = "查询所有统计数据")
    @PostMapping("/list/statistics")
    public ResultRespone<List<ScadaDeviceArchiveManage>> allListStatistics(@ModelAttribute ScadaDeviceArchiveManageDto manageDto) {
        return ResultRespone.success(scadaDeviceArchiveManageService.allListStatistics(manageDto));
    }

    /**
     * 查询范围时间scada统计实时数据，通过实时查询历史表统计得到的数据
     * @param manageDto 查询范围时间scada统计数据
     * @return 查询范围时间scada统计数据
     */
    @ApiOperation(value = "查询范围时间scada统计实时数据")
    @PostMapping("/list/time/statistics")
    public ResultRespone selectScadaStatsByTime(@ModelAttribute ScadaDeviceArchiveManageDto manageDto) {
        return ResultRespone.success(scadaDeviceArchiveManageService.selectScadaStatsByTime(manageDto));
    }


    /**
     * 根据条件获取指标集合
     * @param manageDto 参数
     * @return 根据条件获取指标集合
     */
    @ApiOperation(value = "根据条件获取指标集合")
    @PostMapping("/list/codes/allIndex")
    public ResultRespone selectScadaCodes(@ModelAttribute ScadaDeviceArchiveManageDto manageDto) {
        return ResultRespone.success(scadaDeviceArchiveManageService.selectScadaCodes(manageDto));
    }


    /**
     * 查询分页历史数据
     *
     * @param manageDto 查询分页历史数据
     * @return 查询分页历史数据
     */
    @ApiOperation(value = "查询分页历史数据")
    @PostMapping("/page/history")
    public ResultRespone<Page<ScadaDeviceArchiveManage>> pageListHistory(Page page, @ModelAttribute ScadaDeviceArchiveManageDto manageDto) {
        return ResultRespone.success(scadaDeviceArchiveManageService.pageListHistory(page, manageDto));
    }

    /**
     * 查询所有历史数据
     *
     * @param manageDto 查询所有历史数据
     * @return 查询所有历史数据
     */
    @ApiOperation(value = "查询所有历史数据")
    @PostMapping("/list/history")
    public ResultRespone<List<ScadaDeviceArchiveManage>> allListHistory(@ModelAttribute ScadaDeviceArchiveManageDto manageDto) {
        return ResultRespone.success(scadaDeviceArchiveManageService.allListHistory(manageDto));
    }

    /**
     * 查询所有报警数据
     *
     * @param manageDto 查询所有报警数据
     * @return 查询所有报警数据
     */
    @ApiOperation(value = "查询所有报警数据")
    @PostMapping("/list/alarm")
    public ResultRespone<List<ScadaDeviceArchiveManage>> allListAlarm(@ModelAttribute ScadaDeviceArchiveManageDto manageDto) {
        return ResultRespone.success(scadaDeviceArchiveManageService.allListAlarm(manageDto));
    }

    /**
     * 安全用电评比
     *
     * @param manageDto 安全用电评比
     * @return 安全用电评比
     */
    @ApiOperation(value = "安全用电评比")
    @PostMapping("/list/safeElectricity")
    public ResultRespone<List<SafeElectricityVo>> safeElectricity(@ModelAttribute ScadaDeviceArchiveManageDto manageDto) {
        return ResultRespone.success(scadaDeviceArchiveManageService.safeElectricity(manageDto));
    }


    /**
     * 安全用电评比
     *
     * @param manageDto 安全用电评比 导出
     * @return 安全用电评比 导出
     */
    @ApiOperation(value = "安全用电评比，导出excel")
    @GetMapping("/export/safeElectricity")
    public void exportSafeElectricity(@ModelAttribute ScadaDeviceArchiveManageDto manageDto) {
        List<SafeElectricityVo> safeElectricityVos = scadaDeviceArchiveManageService.safeElectricity(manageDto);
        ExcelUtils.getListToExcel(SafeElectricityVo.class, safeElectricityVos, "安全用电评比");
    }


    /**
     * 实时能耗分析
     *
     * @param manageDto 查询参数
     * @return 实时能耗分析
     */
    @ApiOperation(value = "实时能耗|整体能耗分析")
    @GetMapping("/real/analyse")
    public ResultRespone<Map<String, Object>> analyse(@ModelAttribute ScadaDeviceArchiveManageDto manageDto) {
        return ResultRespone.success(scadaDeviceArchiveManageService.analyse(manageDto));
    }

    /**
     * 查询范围时间统计用水用电数据
     *
     * @param scadaDeviceArchiveManageDto 查询wrapper
     * @return 查询范围时间统计数据
     */
    @ApiOperation(value = "查询范围时间统计用水用电数据")
    @GetMapping("/unit/statistics")
    public ResultRespone<List<BuildReportVo>> selectScadaByBuildId(@ModelAttribute ScadaDeviceArchiveManageDto scadaDeviceArchiveManageDto) {
        return ResultRespone.success(buildingReportService.selectScadaByBuildId(scadaDeviceArchiveManageDto));
    }

    /**
     * 查询范围时间统计用水用电合计数据
     *
     * @param scadaDeviceArchiveManageDto 查询wrapper
     * @return 查询范围时间统计合计数据
     */
    @ApiOperation(value = "查询范围时间统计用水用电合计数据")
    @GetMapping("/sum/statistics")
    public ResultRespone<List<BuildingReport>> selectScadaStatByBuildId(@ModelAttribute ScadaDeviceArchiveManageDto scadaDeviceArchiveManageDto) {
        return ResultRespone.success(buildingReportService.selectScadaStatByBuildId(scadaDeviceArchiveManageDto));
    }

}
