package com.tofly.building.entity;


import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.baomidou.mybatisplus.annotation.*;
import com.tofly.common.constant.SqlCondition;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiModel;
import com.fasterxml.jackson.annotation.JsonFormat;
import org.codehaus.jackson.map.annotate.JsonSerialize;
import org.codehaus.jackson.map.ser.std.ToStringSerializer;
import org.springframework.format.annotation.DateTimeFormat;
import lombok.Data;

import java.util.*;

import org.springframework.web.multipart.MultipartFile;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;

import java.io.Serializable;

import lombok.EqualsAndHashCode;
import com.haiqiu.tools.excel.annotation.Excel;

/**
 * 校内商户管理(BuildingMerchant)表实体类
 *
 * @author HaiQiu
 * @since 2023-02-28 14:49:06
 */
@Data
@Accessors(chain = true)
@NoArgsConstructor
@EqualsAndHashCode(callSuper = false)
@TableName(value = "TF_BUILDING_MERCHANT", resultMap = "BuildingMerchantMap")
@ApiModel(value = "BuildingMerchant对象", description = "校内商户管理")
public class BuildingMerchant extends Model<BuildingMerchant> {

    private static final long serialVersionUID = 465766168983797197L;

    /**
     * 主键ID
     */
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @JsonSerialize(using = ToStringSerializer.class)
    @Excel(name = "主键ID")
    @ApiModelProperty(value = "主键ID")
    @TableId(type = IdType.ASSIGN_ID)
    private Long id;
    /**
     * 校内商户
     */
    @Excel(name = "校内商户")
    @TableField(condition = SqlCondition.ORACLE_LIKE)
    @ApiModelProperty(value = "校内商户")
    private String name;
    /**
     * 所属院区(0南校区，1北校区) or 字典获取
     */
    @Excel(name = "所属院区")
    @ApiModelProperty(value = "所属院区(0南校区，1北校区) or 字典获取")
    private String belongToCollege;
    /**
     * 商户类型（楼栋（单选码值，0实体店，1无人售卖，2其他）） or 字典获取
     */
    @Excel(name = "商户类型")
    @ApiModelProperty(value = "商户类型（楼栋（单选码值，0实体店，1无人售卖，2其他）） or 字典获取")
    private String buildingType;
    /**
     * 所在楼栋ID
     */
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @JsonSerialize(using = ToStringSerializer.class)
    @Excel(name = "所在楼栋ID")
    @ApiModelProperty(value = "所在楼栋ID")
    private Long buildingCollegeId;
    @TableField(exist = false)
    @ApiModelProperty(value = "所在楼栋")
    @Excel(name = "所在楼栋")
    private String buildingCollegeIdName;
    /**
     * 所属部门ID
     */
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @JsonSerialize(using = ToStringSerializer.class)
    @Excel(name = "所属部门ID")
    @ApiModelProperty(value = "所属部门ID")
    private Long deptId;
    @TableField(exist = false)
    @ApiModelProperty(value = "所属部门")
    @Excel(name = "所属部门")
    private String deptIdName;
    /**
     * 坐标位置（经度纬度）
     */
    @Excel(name = "坐标位置（经度纬度）")
    @ApiModelProperty(value = "坐标位置（经度纬度）")
    private String coordinate;
    /**
     * 附件
     */
    @Excel(name = "附件")
    @ApiModelProperty(value = "附件")
    private String annex;
    /**
     * 排序
     */
    @Excel(name = "排序")
    @ApiModelProperty(value = "排序")
    private Long sort;
    /**
     * 备注
     */
    @Excel(name = "备注")
    @ApiModelProperty(value = "备注")
    private String remark;
    /**
     * 删除，0未删除，1已删除
     */
    @Excel(name = "删除，0未删除，1已删除")
    @TableLogic
    @ApiModelProperty(value = "删除，0未删除，1已删除")
    private Boolean deleted;
    /**
     * 创建人
     */
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @JsonSerialize(using = ToStringSerializer.class)
    @TableField(fill = FieldFill.INSERT)
    @Excel(name = "创建人")
    @ApiModelProperty(value = "创建人")
    private Long createUser;
    @TableField(exist = false)
    @ApiModelProperty(value = "创建人", hidden = true)
    @Excel(name = "创建人")
    private String createUserName;
    /**
     * 创建时间
     */
    @Excel(name = "创建时间")
    @TableField(fill = FieldFill.INSERT)
    @ApiModelProperty(value = "创建时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date createTime;
    /**
     * 更新人
     */
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @JsonSerialize(using = ToStringSerializer.class)
    @TableField(fill = FieldFill.INSERT_UPDATE)
    @Excel(name = "更新人")
    @ApiModelProperty(value = "更新人")
    private Long updateUser;
    @TableField(exist = false)
    @ApiModelProperty(value = "更新人", hidden = true)
    @Excel(name = "更新人")
    private String updateUserName;
    /**
     * 更新时间
     */
    @Excel(name = "更新时间")
    @TableField(fill = FieldFill.INSERT_UPDATE)
    @ApiModelProperty(value = "更新时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date updateTime;
}

