package com.tofly.building.entity;


import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.baomidou.mybatisplus.annotation.*;
import com.tofly.building.entity.vo.BuildReportVo;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiModel;
import com.fasterxml.jackson.annotation.JsonFormat;
import org.codehaus.jackson.map.annotate.JsonSerialize;
import org.codehaus.jackson.map.ser.std.ToStringSerializer;
import org.springframework.format.annotation.DateTimeFormat;
import lombok.Data;

import java.util.*;

import org.springframework.web.multipart.MultipartFile;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;

import java.io.Serializable;

import lombok.EqualsAndHashCode;
import com.haiqiu.tools.excel.annotation.Excel;

/**
 * 楼栋用电用水报告(BuildingReport)表实体类
 *
 * @author HaiQiu
 * @since 2023-03-17 09:16:32
 */
@Data
@Accessors(chain = true)
@NoArgsConstructor
@EqualsAndHashCode(callSuper = false)
@TableName(value = "TF_BUILDING_REPORT", resultMap = "BuildingReportMap")
@ApiModel(value = "BuildingReport对象", description = "楼栋用电用水报告")
public class BuildingReport extends Model<BuildingReport> implements Serializable {

    private static final long serialVersionUID = 515917415749966813L;

    /**
     * 主键ID
     */
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @JsonSerialize(using = ToStringSerializer.class)
    @Excel(name = "主键ID")
    @ApiModelProperty(value = "主键ID")
    @TableId(type = IdType.ASSIGN_ID)
    private Long id;
    /**
     * 生成时间
     */
    @Excel(name = "生成时间")
    @TableField(fill = FieldFill.INSERT)
    @ApiModelProperty(value = "生成时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date createTime;
    /**
     * 接送者邮箱
     */
    @Excel(name = "接送者邮箱")
    @ApiModelProperty(value = "接送者邮箱")
    private String sendMail;
    /**
     * 发送时间
     */
    @Excel(name = "发送时间")
    @ApiModelProperty(value = "发送时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date sendTime;
    /**
     * 报告名称
     */
    @Excel(name = "报告名称")
    @ApiModelProperty(value = "报告名称")
    private String name;
    /**
     * 附件
     */
    @Excel(name = "附件")
    @ApiModelProperty(value = "附件")
    private String annex;
    /**
     * 排序
     */
    @Excel(name = "排序")
    @ApiModelProperty(value = "排序")
    private Long sort;
    /**
     * 备注
     */
    @Excel(name = "备注")
    @ApiModelProperty(value = "备注")
    private String remark;
    /**
     * 删除，0未删除，1已删除
     */
    @Excel(name = "删除，0未删除，1已删除")
//    @TableLogic
    @ApiModelProperty(value = "删除，0未删除，1已删除")
    private Boolean deleted;
    /**
     * 时间
     */
    @Excel(name = "时间")
    @ApiModelProperty(value = "时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date dataTime;
    /**
     * 数据名称
     */
    @Excel(name = "数据名称")
    @ApiModelProperty(value = "数据名称")
    private String title;
    /**
     * 现在合计
     */
    @Excel(name = "现在合计")
    @ApiModelProperty(value = "现在合计")
    private String nowSum;
    /**
     * 与上月相比
     */
    @Excel(name = "与上月相比")
    @ApiModelProperty(value = "与上月相比")
    private String lastMonth;
    /**
     * 与去年同期相比
     */
    @Excel(name = "与去年同期相比")
    @ApiModelProperty(value = "与去年同期相比")
    private String lastYear;
    /**
     * 漏损率
     */
    @Excel(name = "漏损率/折合标煤（吨）")
    @ApiModelProperty(value = "漏损率/折合标煤（吨）")
    private String leakage;
    /**
     * 内容存储
     */
    @Excel(name = "内容存储")
    @ApiModelProperty(value = "内容存储")
    private String content;
    /**
     * 0用水，1用电
     */
    @Excel(name = "0用水，1用电")
    @ApiModelProperty(value = "0用水，1用电")
    private String type;

    /**
     * 0用水，1用电
     */
    @Excel(name = "0用水，1用电")
    @TableField(exist = false)
    @ApiModelProperty(value = "0用水，1用电")
    private String typeName;

    /**
     * 所属院区(0南校区，1北校区) or 字典获取
     */
    @Excel(name = "所属院区")
    @TableField(exist = false)
    @ApiModelProperty(value = "所属院区(0南校区，1北校区) or 字典获取")
    private String belongToCollege;

    /**
     * 区域 属院区(0南校区，1北校区) or 字典获取
     */
    @TableField(exist = false)
    @ApiModelProperty(value = "区域 属院区(0南校区，1北校区) or 字典获取")
    private String area;

    /**
     * 子集数据
     */
    @TableField(exist = false)
    @ApiModelProperty(value = "子集数据")
    private List<BuildReportVo> buildReportVos;

    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @JsonSerialize(using = ToStringSerializer.class)
    @Excel(name = "月份表ID")
    @ApiModelProperty(value = "月份表ID")
    private Long reportMouthId;

    /**
     * 所在楼栋ID
     */
    @ApiModelProperty(value = "所在楼栋ID")
    @TableField(exist = false)
    private Long buildingCollegeId;

    /**
     * 楼栋类型
     */
    @ApiModelProperty(value = "楼栋类型")
    @TableField(exist = false)
    private String buildingType;
}

