package com.tofly.building.entity;


import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.baomidou.mybatisplus.annotation.*;
import com.tofly.scada.entity.ScadaReport;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiModel;
import com.fasterxml.jackson.annotation.JsonFormat;
import org.springframework.format.annotation.DateTimeFormat;
import lombok.Data;

import java.math.BigDecimal;
import java.util.*;

import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;

import java.io.Serializable;

import lombok.EqualsAndHashCode;
import com.haiqiu.tools.excel.annotation.Excel;

/**
 * 指标配置(Allocation)表实体类
 *
 * @author HaiQiu
 * @since 2023-03-06 10:33:03
 */
@Data
@Accessors(chain = true)
@NoArgsConstructor
@EqualsAndHashCode(callSuper = false)
@KeySequence("TF_INDEX_ALLOCATION")
@TableName(value = "TF_INDEX_ALLOCATION", resultMap = "AllocationMap")
@ApiModel(value = "Allocation对象", description = "指标配置")
public class ScadaAllocation extends Model<ScadaAllocation> implements Serializable {

    private static final long serialVersionUID = -16632550043431298L;

    /**
     * 所属设备ID
     */
    @Excel(name = "所属设备ID")
    @ApiModelProperty(value = "所属设备ID")
    private Long deviceId;
    /**
     * 指标类型（1出口,2入口,3加药）
     */
    @Excel(name = "指标类型（1出口,2入口,3加药）")
    @ApiModelProperty(value = "指标类型（1出口,2入口,3加药）")
    private String model;
    /**
     * 变量编码
     */
    @Excel(name = "变量编码")
    @ApiModelProperty(value = "变量编码")
    private String variableCode;
    /**
     * 变量名称
     */
    @Excel(name = "变量名称")
    @ApiModelProperty(value = "变量名称")
    private String variableName;
    /**
     * 显示名称
     */
    @Excel(name = "显示名称")
    @ApiModelProperty(value = "显示名称")
    private String displayName;
    /**
     * 单位
     */
    @Excel(name = "单位")
    @ApiModelProperty(value = "单位")
    private String unit;
    /**
     * 报警上限
     */
    @Excel(name = "报警上限")
    @ApiModelProperty(value = "报警上限")
    private BigDecimal alarmUpperLimit;
    /**
     * 报警下限
     */
    @Excel(name = "报警下限")
    @ApiModelProperty(value = "报警下限")
    private BigDecimal alarmLowerLimit;
    /**
     * 是否报警（0未报警,1已报警）
     */
    @Excel(name = "是否报警（0未报警,1已报警）")
    @ApiModelProperty(value = "是否报警（0未报警,1已报警）")
    private String isAlarm;
    /**
     * 是否展示（0否,1是）
     */
    @ApiModelProperty(value = "是否展示（0否,1是）")
    private String isDisplay;
    /**
     * 排序号
     */
    @Excel(name = "排序号")
    @ApiModelProperty(value = "排序号")
    private String sortNumber;
    /**
     * 精度
     */
    @Excel(name = "精度")
    @ApiModelProperty(value = "精度")
    private Long accuracy;
    /**
     * 设备档案编号
     */
    @Excel(name = "设备档案编号")
    @ApiModelProperty(value = "设备档案编号")
    private String archivesCode;
    /**
     * 描述
     */
    @Excel(name = "描述")
    @ApiModelProperty(value = "描述")
    private String describe;
    @TableField(fill = FieldFill.INSERT)
    @Excel(name = "${column.comment}")
    private Long createUser;
    @TableField(exist = false)
    @ApiModelProperty(value = "${column.comment}")
    @Excel(name = "${column.comment}")
    private String createUserName;
    @Excel(name = "${column.comment}")
    @TableField(fill = FieldFill.INSERT)
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date createTime;
    @Excel(name = "${column.comment}")
    @TableId(type = IdType.INPUT)
    private Long id;
    /**
     * 指标归属(字典码表关联 父级字典id60000)
     */
    @Excel(name = "指标归属(字典码表关联 父级字典id60000)")
    @ApiModelProperty(value = "指标归属(字典码表关联 父级字典id60000)")
    private String type;
    /**
     * 绑定报警类型ID
     */
    @Excel(name = "绑定报警类型ID")
    @ApiModelProperty(value = "绑定报警类型ID")
    private String reportId;

    /*********************************scada信息*********************************/

    @ApiModelProperty(value = "指标编码")
    @TableField(exist = false)
    private String code;

    @ApiModelProperty(value = "指标值")
    @TableField(exist = false)
    private String value;

    @ApiModelProperty(value = "指标采集时间")
    @TableField(exist = false)
    private Date scadaTime;

    @ApiModelProperty(value = "指标实时采集统计时间")
    @TableField(exist = false)
    private String dateTimeStr;

    @ApiModelProperty(value = "指标更新时间")
    @TableField(exist = false)
    private Date updateTime;

    /**
     * 统计类型：0小时统计 1天统计 2月统计 3年统计
     */
    @ApiModelProperty(value = "统计类型：0小时统计 1天统计 2月统计 3年统计")
    @TableField(exist = false)
    private Integer statisticsType;

    /**
     * 统计开始时间
     */
    @ApiModelProperty(value = "统计开始时间")
    @TableField(exist = false)
    private Date startTime;

    /**
     * 统计结束时间
     */
    @ApiModelProperty(value = "统计结束时间")
    @TableField(exist = false)
    private Date endTime;

    /**
     * 最大值
     */
    @ApiModelProperty(value = "最大值")
    @TableField(exist = false)
    private String max;

    /**
     * max-min
     */
    @ApiModelProperty(value = "max-min")
    private String diff;

    /**
     * 最小值
     */
    @ApiModelProperty(value = "最小值")
    @TableField(exist = false)
    private String min;

    /**
     * 平均值
     */
    @ApiModelProperty(value = "平均值")
    @TableField(exist = false)
    private String ave;

    /**
     * 叠加值
     */
    @ApiModelProperty(value = "叠加值")
    @TableField(exist = false)
    private String fold;

    /**
     * 指标归属(字典码表关联 例如:01 瞬时流量)
     */
    @ApiModelProperty(value = "指标归属(字典码表关联 例如:01 瞬时流量)")
    @TableField(exist = false)
    private String dataType;

    /**
     * 合格率
     */
    @ApiModelProperty(value = "合格率")
    @TableField(exist = false)
    private String pass;

    /**
     * 超标次数
     */
    @ApiModelProperty(value = "超标次数")
    @TableField(exist = false)
    private String errorCount;

    /**
     * 不合格超标时长
     */
    @ApiModelProperty(value = "不合格超标时长")
    @TableField(exist = false)
    private String errorTimeTotal;

    /**
     * 超标总时长
     */
    @ApiModelProperty(value = "超标总时长")
    @TableField(exist = false)
    private String timeTotal;


    /*********************************scada报警*********************************/
    @ApiModelProperty(value = "scada报警")
    @TableField(exist = false)
    private List<ScadaReport> scadaReports;

    @ApiModelProperty(value = "scada报警数据ID")
    @TableField(exist = false)
    private Long scadaAlarmId;

    @ApiModelProperty(value = "是否报警,0否，1是")
    @TableField(exist = false)
    private String reportAlarm;

    @ApiModelProperty(value = "scada报警次数")
    @TableField(exist = false)
    private Long scadaAlarmCount;


    /*********************************附加状态*********************************/
    @ApiModelProperty(value = "是否报警")
    @TableField(exist = false)
    private boolean isReport;

    @ApiModelProperty(value = "是否离线")
    @TableField(exist = false)
    private boolean isOffline;

    @ApiModelProperty(value = "是否故障")
    @TableField(exist = false)
    private boolean isTitch;

}

