package com.tofly.building.entity.dto;


import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.baomidou.mybatisplus.annotation.*;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiModel;
import com.fasterxml.jackson.annotation.JsonFormat;
import org.codehaus.jackson.map.annotate.JsonSerialize;
import org.codehaus.jackson.map.ser.std.ToStringSerializer;
import org.springframework.format.annotation.DateTimeFormat;
import lombok.Data;

import java.util.*;

import org.springframework.web.multipart.MultipartFile;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;

import java.io.Serializable;

import lombok.EqualsAndHashCode;

import javax.validation.constraints.*;

/**
 * 学院机构管理(BuildingCollege)表实体类
 *
 * @author HaiQiu
 * @since 2023-02-28 11:26:58
 */
@Data
@Accessors(chain = true)
@NoArgsConstructor
@EqualsAndHashCode(callSuper = false)
@ApiModel(value = "BuildingCollege对象", description = "学院机构管理")
public class BuildingCollegeDto implements Serializable {

    private static final long serialVersionUID = 346678350639913996L;

    /**
     * 主键ID
     */
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @JsonSerialize(using = ToStringSerializer.class)
    @Digits(integer = 20, fraction = 0, message = "主键ID长度不能超过20")
    @ApiModelProperty(value = "主键ID")
    @TableId(type = IdType.ASSIGN_ID)
    private Long id;

    /**
     * 学院机构名称
     */
    @Size(max = 85, message = "学院机构名称长度不能超过85")
    @NotBlank(message = "学院机构名称不能为空")
    @ApiModelProperty(value = "学院机构名称")
    private String name;

    /**
     * 所属上级ID
     */
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @JsonSerialize(using = ToStringSerializer.class)
    @Digits(integer = 20, fraction = 0, message = "所属上级ID长度不能超过20")
    @ApiModelProperty(value = "所属上级ID")
    private Long pid;

    /**
     * 所在楼栋ID
     */
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @JsonSerialize(using = ToStringSerializer.class)
    @Digits(integer = 20, fraction = 0, message = "所在楼栋ID长度不能超过20")
    @NotNull(message = "所在楼栋ID不能为空")
    @ApiModelProperty(value = "所在楼栋ID")
    private Long buildingCollegeId;

    /**
     * 所属院区(0南校区，1北校区) or 字典获取
     */
    @Size(max = 85, message = "所属院区(0南校区，1北校区) or 字典获取长度不能超过85")
    @NotBlank(message = "所属院区(0南校区，1北校区) or 字典获取不能为空")
    @ApiModelProperty(value = "所属院区(0南校区，1北校区) or 字典获取")
    private String belongToCollege;

    /**
     * 排序
     */
    @Digits(integer = 15, fraction = 0, message = "排序长度不能超过15")
    @ApiModelProperty(value = "排序")
    private Long sort;

    /**
     * 备注
     */
    @Size(max = 300, message = "备注长度不能超过300")
    @ApiModelProperty(value = "备注")
    private String remark;

}

