package com.tofly.building.entity.dto;


import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.baomidou.mybatisplus.annotation.*;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiModel;
import com.fasterxml.jackson.annotation.JsonFormat;
import org.codehaus.jackson.map.annotate.JsonSerialize;
import org.codehaus.jackson.map.ser.std.ToStringSerializer;
import org.springframework.format.annotation.DateTimeFormat;
import lombok.Data;

import java.util.*;

import org.springframework.web.multipart.MultipartFile;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;

import java.io.Serializable;

import lombok.EqualsAndHashCode;

import javax.validation.constraints.*;

/**
 * 楼栋建筑管理(BuildingErcet)表实体类
 *
 * @author HaiQiu
 * @since 2023-02-28 11:26:59
 */
@Data
@Accessors(chain = true)
@NoArgsConstructor
@EqualsAndHashCode(callSuper = false)
@ApiModel(value = "BuildingErcet对象", description = "楼栋建筑管理")
public class BuildingErcetDto implements Serializable {

    private static final long serialVersionUID = -51481300610398806L;

    /**
     * 主键ID
     */
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @JsonSerialize(using = ToStringSerializer.class)
    @Digits(integer = 20, fraction = 0, message = "主键ID长度不能超过20")
    @ApiModelProperty(value = "主键ID")
    @TableId(type = IdType.ASSIGN_ID)
    private Long id;

    /**
     * 楼栋（房间）名称
     */
    @Size(max = 85, message = "楼栋（房间）名称长度不能超过85")
    @NotBlank(message = "楼栋（房间）名称不能为空")
    @ApiModelProperty(value = "楼栋（房间）名称")
    private String name;

    /**
     * 所属院区(0南校区，1北校区) or 字典获取
     */
    @Size(max = 85, message = "所属院区(0南校区，1北校区) or 字典获取长度不能超过85")
    @ApiModelProperty(value = "所属院区(0南校区，1北校区) or 字典获取")
    private String belongToCollege;

    /**
     * 楼栋类型（楼栋（单选码值，0教学楼，1办公楼，2宿舍楼，3商业楼，4食堂.5其他）） or 字典获取
     */
    @Size(max = 85, message = "楼栋类型（楼栋（单选码值，0教学楼，1办公楼，2宿舍楼，3商业楼，4食堂.5其他）） or 字典获取长度不能超过85")
    @NotBlank(message = "楼栋类型（楼栋（单选码值，0教学楼，1办公楼，2宿舍楼，3商业楼，4食堂.5其他）） or 字典获取不能为空")
    @ApiModelProperty(value = "楼栋类型（楼栋（单选码值，0教学楼，1办公楼，2宿舍楼，3商业楼，4食堂.5其他）） or 字典获取")
    private String buildingType;

    /**
     * 属性（楼栋（单选码值，楼栋是1级、楼栋2级、房间是3级）） or 字典获取
     */
    @Size(max = 85, message = "属性（楼栋（单选码值，楼栋是1级、楼栋2级、房间是3级）） or 字典获取长度不能超过85")
    @NotBlank(message = "属性（楼栋（单选码值，楼栋是1级、楼栋2级、房间是3级）） or 字典获取不能为空")
    @ApiModelProperty(value = "属性（楼栋（单选码值，楼栋是1级、楼栋2级、房间是3级）） or 字典获取")
    private String attributeVar;

    /**
     * 所属上级ID
     */
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @JsonSerialize(using = ToStringSerializer.class)
    @Digits(integer = 20, fraction = 0, message = "所属上级ID长度不能超过20")
    @ApiModelProperty(value = "所属上级ID")
    private Long pid;

    /**
     * 建筑特征
     */
    @Size(max = 300, message = "建筑特征长度不能超过300")
    @ApiModelProperty(value = "建筑特征")
    private String characteristic;

    /**
     * 功能用途
     */
    @Size(max = 300, message = "功能用途长度不能超过300")
    @ApiModelProperty(value = "功能用途")
    private String description;

    /**
     * 建筑面积
     */
    @Size(max = 85, message = "建筑面积长度不能超过85")
    @ApiModelProperty(value = "建筑面积")
    private String area;

    /**
     * 人数
     */
    @Size(max = 85, message = "人数长度不能超过85")
    @ApiModelProperty(value = "人数")
    private String personCount;

    /**
     * 坐标位置（经度纬度）
     */
    @Size(max = 85, message = "坐标位置（经度纬度）长度不能超过85")
    @ApiModelProperty(value = "坐标位置（经度纬度）")
    private String coordinate;

    /**
     * 附件
     */
    @Size(max = 1333, message = "附件长度不能超过1333")
    @ApiModelProperty(value = "附件")
    private String annex;

    /**
     * 排序
     */
    @Digits(integer = 15, fraction = 0, message = "排序长度不能超过15")
    @ApiModelProperty(value = "排序")
    private Long sort;

    /**
     * 备注
     */
    @Size(max = 300, message = "备注长度不能超过300")
    @ApiModelProperty(value = "备注")
    private String remark;


}

