package com.tofly.building.entity.dto;


import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.baomidou.mybatisplus.annotation.*;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiModel;
import com.fasterxml.jackson.annotation.JsonFormat;
import org.codehaus.jackson.map.annotate.JsonSerialize;
import org.codehaus.jackson.map.ser.std.ToStringSerializer;
import org.springframework.format.annotation.DateTimeFormat;
import lombok.Data;

import java.util.*;

import org.springframework.web.multipart.MultipartFile;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;

import java.io.Serializable;

import lombok.EqualsAndHashCode;

import javax.validation.constraints.*;

/**
 * 校内商户管理(BuildingMerchant)表实体类
 *
 * @author HaiQiu
 * @since 2023-02-28 11:27:00
 */
@Data
@Accessors(chain = true)
@NoArgsConstructor
@EqualsAndHashCode(callSuper = false)
@ApiModel(value = "BuildingMerchant对象", description = "校内商户管理")
public class BuildingMerchantDto implements Serializable {

    private static final long serialVersionUID = 732711455251954649L;

    /**
     * 主键ID
     */
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @JsonSerialize(using = ToStringSerializer.class)
    @Digits(integer = 20, fraction = 0, message = "主键ID长度不能超过20")
    @ApiModelProperty(value = "主键ID")
    @TableId(type = IdType.ASSIGN_ID)
    private Long id;

    /**
     * 校内商户
     */
    @Size(max = 85, message = "校内商户长度不能超过85")
    @NotBlank(message = "校内商户不能为空")
    @ApiModelProperty(value = "校内商户")
    private String name;

    /**
     * 所属院区(0南校区，1北校区) or 字典获取
     */
    @Size(max = 85, message = "所属院区(0南校区，1北校区) or 字典获取长度不能超过85")
    @NotBlank(message = "所属院区(0南校区，1北校区) or 字典获取不能为空")
    @ApiModelProperty(value = "所属院区(0南校区，1北校区) or 字典获取")
    private String belongToCollege;

    /**
     * 商户类型（楼栋（单选码值，0实体店，1无人售卖，2其他）） or 字典获取
     */
    @Size(max = 85, message = "商户类型（楼栋（单选码值，0实体店，1无人售卖，2其他）） or 字典获取长度不能超过85")
    @NotBlank(message = "商户类型（楼栋（单选码值，0实体店，1无人售卖，2其他）） or 字典获取不能为空")
    @ApiModelProperty(value = "商户类型（楼栋（单选码值，0实体店，1无人售卖，2其他）） or 字典获取")
    private String buildingType;

    /**
     * 所在楼栋ID
     */
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @JsonSerialize(using = ToStringSerializer.class)
    @Digits(integer = 20, fraction = 0, message = "所在楼栋ID长度不能超过20")
    @NotNull(message = "所在楼栋ID不能为空")
    @ApiModelProperty(value = "所在楼栋ID")
    private Long buildingCollegeId;

    /**
     * 所属部门ID
     */
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @JsonSerialize(using = ToStringSerializer.class)
    @Digits(integer = 20, fraction = 0, message = "所属部门ID长度不能超过20")
    @ApiModelProperty(value = "所属部门ID")
    private Long deptId;

    /**
     * 坐标位置（经度纬度）
     */
    @Size(max = 85, message = "坐标位置（经度纬度）长度不能超过85")
    @ApiModelProperty(value = "坐标位置（经度纬度）")
    private String coordinate;

    /**
     * 附件
     */
    @Size(max = 1333, message = "附件长度不能超过1333")
    @ApiModelProperty(value = "附件")
    private String annex;

    /**
     * 排序
     */
    @Digits(integer = 15, fraction = 0, message = "排序长度不能超过15")
    @ApiModelProperty(value = "排序")
    private Long sort;

    /**
     * 备注
     */
    @Size(max = 300, message = "备注长度不能超过300")
    @ApiModelProperty(value = "备注")
    private String remark;

}

