package com.tofly.building.entity.dto;


import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.baomidou.mybatisplus.annotation.*;
import com.haiqiu.tools.excel.annotation.Excel;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiModel;
import com.fasterxml.jackson.annotation.JsonFormat;
import org.codehaus.jackson.map.annotate.JsonSerialize;
import org.codehaus.jackson.map.ser.std.ToStringSerializer;
import org.springframework.format.annotation.DateTimeFormat;
import lombok.Data;

import java.util.*;

import org.springframework.web.multipart.MultipartFile;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;

import java.io.Serializable;

import lombok.EqualsAndHashCode;

import javax.validation.constraints.*;

/**
 * 楼栋用电用水报告(BuildingReport)表实体类
 *
 * @author HaiQiu
 * @since 2023-03-17 09:16:34
 */
@Data
@Accessors(chain = true)
@NoArgsConstructor
@EqualsAndHashCode(callSuper = false)
@ApiModel(value = "BuildingReport对象", description = "楼栋用电用水报告")
public class BuildingReportDto implements Serializable {

    private static final long serialVersionUID = -29653056986298742L;

    /**
     * 主键ID
     */
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @JsonSerialize(using = ToStringSerializer.class)
    @Digits(integer = 20, fraction = 0, message = "主键ID长度不能超过20")
    @ApiModelProperty(value = "主键ID")
    @TableId(type = IdType.ASSIGN_ID)
    private Long id;

    /**
     * 生成时间
     */
    @ApiModelProperty(value = "生成时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date createTime;

    /**
     * 接送者邮箱
     */
    @Size(max = 85, message = "接送者邮箱长度不能超过85")
    @ApiModelProperty(value = "接送者邮箱")
    private String sendMail;

    /**
     * 发送时间
     */
    @ApiModelProperty(value = "发送时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date sendTime;

    /**
     * 报告名称
     */
    @Size(max = 85, message = "报告名称长度不能超过85")
    @ApiModelProperty(value = "报告名称")
    private String name;

    /**
     * 附件
     */
    @Size(max = 333, message = "附件长度不能超过333")
    @ApiModelProperty(value = "附件")
    private String annex;

    /**
     * 排序
     */
    @Digits(integer = 15, fraction = 0, message = "排序长度不能超过15")
    @ApiModelProperty(value = "排序")
    private Long sort;

    /**
     * 备注
     */
    @Size(max = 300, message = "备注长度不能超过300")
    @ApiModelProperty(value = "备注")
    private String remark;

    /**
     * 删除，0未删除，1已删除
     */
    @ApiModelProperty(value = "删除，0未删除，1已删除")
    private Boolean deleted;

    /**
     * 时间
     */
    @ApiModelProperty(value = "时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date dataTime;

    /**
     * 数据名称
     */
    @Size(max = 85, message = "数据名称长度不能超过85")
    @ApiModelProperty(value = "数据名称")
    private String title;

    /**
     * 现在合计
     */
    @Size(max = 85, message = "现在合计长度不能超过85")
    @ApiModelProperty(value = "现在合计")
    private String nowSum;

    /**
     * 与上月相比
     */
    @Size(max = 85, message = "与上月相比长度不能超过85")
    @ApiModelProperty(value = "与上月相比")
    private String lastMonth;

    /**
     * 与去年同期相比
     */
    @Size(max = 85, message = "与去年同期相比长度不能超过85")
    @ApiModelProperty(value = "与去年同期相比")
    private String lastYear;

    /**
     * 漏损率
     */
    @Size(max = 85, message = "漏损率长度不能超过85")
    @ApiModelProperty(value = "漏损率")
    private String leakage;

    /**
     * 内容存储
     */
    @Size(max = 1000, message = "内容存储长度不能超过1000")
    @ApiModelProperty(value = "内容存储")
    private String content;

    /**
     * 0用水，1用电
     */
    @Size(max = 85, message = "0用水，1用电长度不能超过85")
    @ApiModelProperty(value = "0用水，1用电")
    private String type;

    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @JsonSerialize(using = ToStringSerializer.class)
    @ApiModelProperty(value = "月份表ID")
    private Long reportMouthId;

}

