package com.tofly.building.service;


import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import com.tofly.building.entity.BuildingErcet;
import com.tofly.building.entity.dto.BuildingErcetDto;

import com.tofly.building.entity.vo.BuildingErcetVo;
import org.springframework.web.multipart.MultipartFile;

import java.io.Serializable;
import java.util.List;
import java.util.Map;

/**
 * 楼栋建筑管理(BuildingErcet)表服务接口
 *
 * @author HaiQiu
 * @since 2023-02-28 11:26:59
 */
public interface BuildingErcetService extends IService<BuildingErcet> {

    /**
     * 根据ID查询
     *
     * @param id id
     * @return 数据
     */
    BuildingErcet get(Serializable id);

    /**
     * 新增
     *
     * @param buildingErcetDto 实体
     * @param files            上传文件附件
     */
    void insert(BuildingErcetDto buildingErcetDto, List<MultipartFile> files);

    /**
     * 修改
     *
     * @param buildingErcetDto 实体
     * @param files            上传文件附件
     */
    void edit(BuildingErcetDto buildingErcetDto, List<MultipartFile> files);

    /**
     * 批量删除
     *
     * @param idList 数据ID集合
     */
    void delete(List<Long> idList);

    /**
     * 生成模板下载
     */
    void template();

    /**
     * 数据导入
     *
     * @param file 文件
     */
    void importData(MultipartFile file);

    /**
     * 分页查询
     *
     * @param page          分页
     * @param buildingErcet 查询wrapper
     * @return 数据集合
     */
    Page<BuildingErcet> pageList(Page page, BuildingErcet buildingErcet);

    /**
     * 数据导出
     *
     * @param page          分页
     * @param buildingErcet 查询模型
     * @param isAll         是否导出全部数据，true是，false否（默认否）
     */
    void export(Page page, BuildingErcet buildingErcet, boolean isAll);

    /**
     * 查询所有数据
     *
     * @param buildingErcet 查询条件
     * @return 查询所有数据
     */
    List<BuildingErcet> allList(BuildingErcet buildingErcet);

    /**
     * 查询树形数据
     *
     * @param buildingErcet 查询条件
     * @return 查询树形数据
     */
    Map<String, List<BuildingErcetVo>> tree(BuildingErcet buildingErcet);
}

