package com.tofly.building.service;


import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import com.tofly.building.entity.BuildingMerchant;
import com.tofly.building.entity.dto.BuildingMerchantDto;

import org.springframework.web.multipart.MultipartFile;

import java.io.Serializable;
import java.util.List;

/**
 * 校内商户管理(BuildingMerchant)表服务接口
 *
 * @author HaiQiu
 * @since 2023-02-28 11:27:00
 */
public interface BuildingMerchantService extends IService<BuildingMerchant> {

    /**
     * 根据ID查询
     *
     * @param id id
     * @return 数据
     */
    BuildingMerchant get(Serializable id);

    /**
     * 新增
     *
     * @param buildingMerchantDto 实体
     * @param files               上传文件附件
     */
    void insert(BuildingMerchantDto buildingMerchantDto, List<MultipartFile> files);

    /**
     * 修改
     *
     * @param buildingMerchantDto 实体
     * @param files               上传文件附件
     */
    void edit(BuildingMerchantDto buildingMerchantDto, List<MultipartFile> files);

    /**
     * 批量删除
     *
     * @param idList 数据ID集合
     */
    void delete(List<Long> idList);

    /**
     * 生成模板下载
     */
    void template();

    /**
     * 数据导入
     *
     * @param file 文件
     */
    void importData(MultipartFile file);

    /**
     * 分页查询
     *
     * @param page             分页
     * @param buildingMerchant 查询wrapper
     * @return 数据集合
     */
    Page<BuildingMerchant> pageList(Page page, BuildingMerchant buildingMerchant);

    /**
     * 数据导出
     *
     * @param page             分页
     * @param buildingMerchant 查询模型
     * @param isAll            是否导出全部数据，true是，false否（默认否）
     */
    void export(Page page, BuildingMerchant buildingMerchant, boolean isAll);

    /**
     * 查询所有数据
     *
     * @param buildingMerchant 查询条件
     * @return 查询所有数据
     */
    List<BuildingMerchant> allList(BuildingMerchant buildingMerchant);
}

