package com.tofly.building.service;


import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import com.tofly.building.entity.BuildingReport;
import com.tofly.building.entity.dto.BuildingReportDto;
import com.tofly.building.entity.dto.ScadaDeviceArchiveManageDto;
import com.tofly.building.entity.vo.BuildReportVo;
import org.springframework.web.multipart.MultipartFile;

import java.io.Serializable;
import java.util.List;

/**
 * 楼栋用电用水报告(BuildingReport)表服务接口
 *
 * @author HaiQiu
 * @since 2023-03-17 09:16:33
 */
public interface BuildingReportService extends IService<BuildingReport> {

    /**
     * 根据ID查询
     *
     * @param id id
     * @return 数据
     */
    BuildingReport get(Serializable id);

    /**
     * 新增
     *
     * @param buildingReportDto 实体
     * @param files             上传文件附件
     */
    void insert(BuildingReportDto buildingReportDto, List<MultipartFile> files);

    /**
     * 修改
     *
     * @param buildingReportDto 实体
     * @param files             上传文件附件
     */
    void edit(BuildingReportDto buildingReportDto, List<MultipartFile> files);

    /**
     * 批量删除
     *
     * @param idList 数据ID集合
     */
    void delete(List<Long> idList);

    /**
     * 生成模板下载
     */
    void template();

    /**
     * 数据导入
     *
     * @param file 文件
     */
    void importData(MultipartFile file);

    /**
     * 分页查询
     *
     * @param page           分页
     * @param buildingReport 查询wrapper
     * @return 数据集合
     */
    Page<BuildingReport> pageList(Page page, BuildingReport buildingReport);

    /**
     * 数据导出
     *
     * @param page           分页
     * @param buildingReport 查询模型
     * @param isAll          是否导出全部数据，true是，false否（默认否）
     */
    void export(Page page, BuildingReport buildingReport, boolean isAll);

    /**
     * 查询所有数据
     *
     * @param buildingReport 查询条件
     * @return 查询所有数据
     */
    List<BuildingReport> allList(BuildingReport buildingReport);

    /**
     * 生成能耗报告
     * @param time 自定时间 yyyy-MM
     */
    List<BuildingReport>  calReport(String time);

    /**
     * 能耗报告文档生成
     * @param time 自定义时间
     * @param sendMail 邮箱接收方
     */
    void getReport(String time, String sendMail);

    /**
     * 查询范围时间统计数据
     * @param scadaDeviceArchiveManageDto 查询wrapper
     * @return 查询范围时间统计数据
     */
    List<BuildReportVo> selectScadaByBuildId(ScadaDeviceArchiveManageDto scadaDeviceArchiveManageDto);

    /**
     * 查询范围时间统计合计数据
     * @param scadaDeviceArchiveManageDto 查询wrapper
     * @return 查询范围时间统计合计数据
     */
    List<BuildingReport> selectScadaStatByBuildId(ScadaDeviceArchiveManageDto scadaDeviceArchiveManageDto);

    /**
     * 手动发送邮件
     * @param timeStr 时间标题
     * @param reports 楼栋用电用水报告列表
     * @param sendMail 邮件接收方
     * @return 查询范围时间统计合计数据
     */
    void sendMailCustom(String timeStr, List<BuildingReport> reports, String sendMail);

    /**
     * 获取昨日能耗数据（Json字符串）
     */
    String getEnergyData();

}

