package com.tofly.building.service.impl;


import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.core.toolkit.support.SFunction;
import com.haiqiu.tools.excel.utils.ExcelUtils;
import com.tofly.building.utils.OrderItem;
import org.springframework.util.CollectionUtils;
import org.springframework.beans.BeanUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.building.mapper.BuildingCollegeMapper;
import com.tofly.building.entity.BuildingCollege;
import com.tofly.building.entity.dto.BuildingCollegeDto;
import com.tofly.building.service.BuildingCollegeService;
import org.springframework.stereotype.Service;

import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import java.util.Collections;
import java.util.List;
import java.io.Serializable;

/**
 * 学院机构管理(BuildingCollege)表服务实现类
 *
 * @author HaiQiu
 * @since 2023-02-28 11:26:58
 */
@Service("buildingCollegeService")
public class BuildingCollegeServiceImpl extends ServiceImpl<BuildingCollegeMapper, BuildingCollege> implements BuildingCollegeService {


    /**
     * 根据ID查询
     *
     * @param id id
     * @return 数据
     */
    @Override
    public BuildingCollege get(Serializable id) {
        return getById(id);
    }


    /**
     * 新增
     *
     * @param buildingCollegeDto 实体
     */
    @Override
    public void insert(BuildingCollegeDto buildingCollegeDto) {
        BuildingCollege buildingCollege = convert(buildingCollegeDto);
        if (!save(buildingCollege)) {
            throw new RuntimeException("新增失败");
        }
    }

    /**
     * 修改
     *
     * @param buildingCollegeDto 实体
     */
    @Override
    public void edit(BuildingCollegeDto buildingCollegeDto) {
        BuildingCollege buildingCollege = convert(buildingCollegeDto);
        if (!updateById(buildingCollege)) {
            throw new RuntimeException("修改失败");
        }
    }

    /**
     * 实体转换
     *
     * @param buildingCollegeDto dto实体
     */
    private BuildingCollege convert(BuildingCollegeDto buildingCollegeDto) {
        BuildingCollege buildingCollege = new BuildingCollege();
        BeanUtils.copyProperties(buildingCollegeDto, buildingCollege);
        return buildingCollege;
    }

    /**
     * 批量删除
     *
     * @param idList 数据ID集合
     */
    @Override
    public void delete(List<Long> idList) {
        if (!removeByIds(idList)) {
            throw new RuntimeException("删除失败");
        }
    }

    /**
     * 生成模板下载
     */
    @Override
    public void template() {
        BuildingCollege buildingCollege = new BuildingCollege();
        ExcelUtils.createTemplate("学院机构管理", "学院机构管理",
                "学院机构管理", BuildingCollege.class, Collections.singletonList(buildingCollege));
    }

    /**
     * 数据导入
     *
     * @param file 文件
     */
    @Override
    public void importData(MultipartFile file) {
        List<BuildingCollege> excelToList = ExcelUtils.getExcelToList("学院机构管理", file, BuildingCollege.class);
        if (CollectionUtils.isEmpty(excelToList)) {
            throw new RuntimeException("数据解析为空");
        }
        saveBatch(excelToList);
    }

    /**
     * 分页查询
     *
     * @param page            分页
     * @param buildingCollege 查询wrapper
     * @return 数据集合
     */
    @Override
    public Page<BuildingCollege> pageList(Page page, BuildingCollege buildingCollege) {
        return page(page, Wrappers.lambdaQuery(buildingCollege)).addOrder(page.getOrders());
    }

    /**
     * 数据导出
     *
     * @param page            分页
     * @param buildingCollege 查询模型
     * @param isAll           是否导出全部数据，true是，false否（默认否）
     */
    @Override
    public void export(Page page, BuildingCollege buildingCollege, boolean isAll) {
        Page<BuildingCollege> pageData = new Page<>();
        if (!isAll) {
            pageData = page(page, Wrappers.lambdaQuery(buildingCollege));
        } else {
            pageData.setRecords(list(Wrappers.lambdaQuery(buildingCollege)));
        }
        //导出数据
        ExcelUtils.getListToExcel(BuildingCollege.class, pageData.getRecords(), "学院机构管理");
    }

    /**
     * 查询所有数据
     *
     * @param buildingCollege 查询条件
     * @param orderItem       排序字段
     * @return 查询所有数据
     */
    @Override
    public List<BuildingCollege> allList(BuildingCollege buildingCollege, OrderItem orderItem) {
        return list(Wrappers.lambdaQuery(buildingCollege)
                .last(StringUtils.hasText(orderItem.getOrderBySql()),orderItem.getOrderBySql()));
    }
}

