package com.tofly.building.service.impl;


import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.haiqiu.tools.excel.utils.ExcelUtils;
import com.tofly.building.entity.vo.BuildingErcetVo;
import com.tofly.common.service.IFileInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.CollectionUtils;
import org.springframework.beans.BeanUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.building.mapper.BuildingErcetMapper;
import com.tofly.building.entity.BuildingErcet;
import com.tofly.building.entity.dto.BuildingErcetDto;
import com.tofly.building.service.BuildingErcetService;
import org.springframework.stereotype.Service;

import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.io.Serializable;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 楼栋建筑管理(BuildingErcet)表服务实现类
 *
 * @author HaiQiu
 * @since 2023-02-28 11:26:59
 */
@Service("buildingErcetService")
public class BuildingErcetServiceImpl extends ServiceImpl<BuildingErcetMapper, BuildingErcet> implements BuildingErcetService {


    @Value("${buildingErcet.ftp.uploads:/scyz/project/buildingErcet}")
    private String path;
    @Autowired
    private IFileInfoService fileInfoService;

    /**
     * 根据ID查询
     *
     * @param id id
     * @return 数据
     */
    @Override
    public BuildingErcet get(Serializable id) {
        return getById(id);
    }


    /**
     * 新增
     *
     * @param buildingErcetDto 实体
     * @param files            上传文件附件
     */
    @Override
    public void insert(BuildingErcetDto buildingErcetDto, List<MultipartFile> files) {
        BuildingErcet buildingErcet = convert(buildingErcetDto);
        buildingErcet.setAnnex(fileInfoService.uploadFiles(files, buildingErcet.getAnnex()
                , buildingErcet.getAnnex(), path, "TF_BUILDING_ERCET"));
        if (!save(buildingErcet)) {
            throw new RuntimeException("新增失败");
        }
    }

    /**
     * 修改
     *
     * @param buildingErcetDto 实体
     * @param files            上传文件附件
     */
    @Override
    public void edit(BuildingErcetDto buildingErcetDto, List<MultipartFile> files) {
        BuildingErcet buildingErcet = convert(buildingErcetDto);
        buildingErcet.setAnnex(fileInfoService.uploadFiles(files, buildingErcet.getAnnex()
                , buildingErcet.getAnnex(), path, "TF_BUILDING_ERCET"));
        if (!updateById(buildingErcet)) {
            throw new RuntimeException("修改失败");
        }
    }

    /**
     * 实体转换
     *
     * @param buildingErcetDto dto实体
     */
    private BuildingErcet convert(BuildingErcetDto buildingErcetDto) {
        BuildingErcet buildingErcet = new BuildingErcet();
        BeanUtils.copyProperties(buildingErcetDto, buildingErcet);
        return buildingErcet;
    }

    /**
     * 批量删除
     *
     * @param idList 数据ID集合
     */
    @Override
    public void delete(List<Long> idList) {
        if (count(Wrappers.<BuildingErcet>lambdaQuery().in(BuildingErcet::getPid,idList)) !=0){
            throw new RuntimeException("选中删除项包含子级数据，请先删除子级数据");
        }
        if (!removeByIds(idList)) {
            throw new RuntimeException("删除失败");
        }
    }

    /**
     * 生成模板下载
     */
    @Override
    public void template() {
        BuildingErcet buildingErcet = new BuildingErcet();
        ExcelUtils.createTemplate("楼栋建筑管理", "楼栋建筑管理",
                "楼栋建筑管理", BuildingErcet.class, Collections.singletonList(buildingErcet));
    }

    /**
     * 数据导入
     *
     * @param file 文件
     */
    @Override
    public void importData(MultipartFile file) {
        List<BuildingErcet> excelToList = ExcelUtils.getExcelToList("楼栋建筑管理", file, BuildingErcet.class);
        if (CollectionUtils.isEmpty(excelToList)) {
            throw new RuntimeException("数据解析为空");
        }
        saveBatch(excelToList);
    }

    /**
     * 分页查询
     *
     * @param page          分页
     * @param buildingErcet 查询wrapper
     * @return 数据集合
     */
    @Override
    public Page<BuildingErcet> pageList(Page page, BuildingErcet buildingErcet) {
        return page(page, Wrappers.lambdaQuery(buildingErcet)).addOrder(page.getOrders());
    }

    /**
     * 数据导出
     *
     * @param page          分页
     * @param buildingErcet 查询模型
     * @param isAll         是否导出全部数据，true是，false否（默认否）
     */
    @Override
    public void export(Page page, BuildingErcet buildingErcet, boolean isAll) {
        Page<BuildingErcet> pageData = new Page<>();
        if (!isAll) {
            pageData = page(page, Wrappers.lambdaQuery(buildingErcet));
        } else {
            pageData.setRecords(list(Wrappers.lambdaQuery(buildingErcet)));
        }
        //导出数据
        ExcelUtils.getListToExcel(BuildingErcet.class, pageData.getRecords(), "楼栋建筑管理");
    }

    /**
     * 查询所有数据
     *
     * @param buildingErcet 查询条件
     * @return 查询所有数据
     */
    @Override
    public List<BuildingErcet> allList(BuildingErcet buildingErcet) {
        return list(Wrappers.lambdaQuery(buildingErcet));
    }


    /**
     * 查询树形数据
     *
     * @param buildingErcet 查询条件
     * @return 查询树形数据
     */
    @Override
    public Map<String, List<BuildingErcetVo>> tree(BuildingErcet buildingErcet) {
        List<BuildingErcet> buildingErcets = allList(buildingErcet);
        if (CollectionUtils.isEmpty(buildingErcets)) {
            return null;
        }
        List<BuildingErcetVo> voList = buildingErcets.stream().map(b -> {
            BuildingErcetVo vo = new BuildingErcetVo();
            BeanUtils.copyProperties(b, vo);
            return vo;
        }).collect(Collectors.toList());
        List<BuildingErcetVo> treeData = getTreeData(voList);
        //转换数据根据校区分组
        return treeData.stream().collect(Collectors.groupingBy(BuildingErcet::getBelongToCollege));
    }

    private List<BuildingErcetVo> getTreeData(List<BuildingErcetVo> voList) {
        //父节点
        long pid = 0;
        //返回新数组节点
        List<BuildingErcetVo> finalList = new ArrayList<>();
        List<BuildingErcetVo> list = allList(null).stream().map(b -> {
            BuildingErcetVo vo = new BuildingErcetVo();
            BeanUtils.copyProperties(b, vo);
            return vo;
        }).collect(Collectors.toList());
        for (BuildingErcetVo vo : voList) {
//            if (vo.getPid() == null || vo.getPid().equals(pid) ) {
//                finalList.add(getChildrenVo(vo, voList));
//            }
            finalList.add(getChildrenVo(vo, list));
        }
        return finalList;
    }

    private BuildingErcetVo getChildrenVo(BuildingErcetVo vo, List<BuildingErcetVo> voList) {
        vo.setChildren(new ArrayList<>());
        for (BuildingErcetVo ercetVo : voList) {
            if (vo.getChildren() == null) {
                vo.setChildren(new ArrayList<>());
            }
            if (vo.getId().equals(ercetVo.getPid())) {
                vo.getChildren().add(getChildrenVo(ercetVo, voList));
            }
        }
        return vo;
    }
}

