package com.tofly.building.service.impl;


import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.haiqiu.tools.excel.utils.ExcelUtils;
import com.tofly.common.service.IFileInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.CollectionUtils;
import org.springframework.beans.BeanUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.building.mapper.BuildingMerchantMapper;
import com.tofly.building.entity.BuildingMerchant;
import com.tofly.building.entity.dto.BuildingMerchantDto;
import com.tofly.building.service.BuildingMerchantService;
import org.springframework.stereotype.Service;

import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.io.Serializable;

/**
 * 校内商户管理(BuildingMerchant)表服务实现类
 *
 * @author HaiQiu
 * @since 2023-02-28 11:27:00
 */
@Service("buildingMerchantService")
public class BuildingMerchantServiceImpl extends ServiceImpl<BuildingMerchantMapper, BuildingMerchant> implements BuildingMerchantService {


    @Value("${buildingMerchant.ftp.uploads:/scyz/project/buildingMerchant}")
    private String path;
    @Autowired
    private IFileInfoService fileInfoService;

    /**
     * 根据ID查询
     *
     * @param id id
     * @return 数据
     */
    @Override
    public BuildingMerchant get(Serializable id) {
        return getById(id);
    }


    /**
     * 新增
     *
     * @param buildingMerchantDto 实体
     * @param files               上传文件附件
     */
    @Override
    public void insert(BuildingMerchantDto buildingMerchantDto, List<MultipartFile> files) {
        BuildingMerchant buildingMerchant = convert(buildingMerchantDto);
        buildingMerchant.setAnnex(fileInfoService.uploadFiles(files, buildingMerchant.getAnnex()
                , buildingMerchant.getAnnex(), path, "TF_BUILDING_MERCHANT"));
        if (!save(buildingMerchant)) {
            throw new RuntimeException("新增失败");
        }
    }

    /**
     * 修改
     *
     * @param buildingMerchantDto 实体
     * @param files               上传文件附件
     */
    @Override
    public void edit(BuildingMerchantDto buildingMerchantDto, List<MultipartFile> files) {
        BuildingMerchant buildingMerchant = convert(buildingMerchantDto);
        buildingMerchant.setAnnex(fileInfoService.uploadFiles(files, buildingMerchant.getAnnex()
                , buildingMerchant.getAnnex(), path, "TF_BUILDING_MERCHANT"));
        if (!updateById(buildingMerchant)) {
            throw new RuntimeException("修改失败");
        }
    }

    /**
     * 实体转换
     *
     * @param buildingMerchantDto dto实体
     */
    private BuildingMerchant convert(BuildingMerchantDto buildingMerchantDto) {
        BuildingMerchant buildingMerchant = new BuildingMerchant();
        BeanUtils.copyProperties(buildingMerchantDto, buildingMerchant);
        return buildingMerchant;
    }

    /**
     * 批量删除
     *
     * @param idList 数据ID集合
     */
    @Override
    public void delete(List<Long> idList) {
        if (!removeByIds(idList)) {
            throw new RuntimeException("删除失败");
        }
    }

    /**
     * 生成模板下载
     */
    @Override
    public void template() {
        BuildingMerchant buildingMerchant = new BuildingMerchant();
        ExcelUtils.createTemplate("校内商户管理", "校内商户管理",
                "校内商户管理", BuildingMerchant.class, Collections.singletonList(buildingMerchant));
    }

    /**
     * 数据导入
     *
     * @param file 文件
     */
    @Override
    public void importData(MultipartFile file) {
        List<BuildingMerchant> excelToList = ExcelUtils.getExcelToList("校内商户管理", file, BuildingMerchant.class);
        if (CollectionUtils.isEmpty(excelToList)) {
            throw new RuntimeException("数据解析为空");
        }
        saveBatch(excelToList);
    }

    /**
     * 分页查询
     *
     * @param page             分页
     * @param buildingMerchant 查询wrapper
     * @return 数据集合
     */
    @Override
    public Page<BuildingMerchant> pageList(Page page, BuildingMerchant buildingMerchant) {

        return page(page, Wrappers.lambdaQuery(buildingMerchant)).addOrder(page.getOrders());
    }

    /**
     * 数据导出
     *
     * @param page             分页
     * @param buildingMerchant 查询模型
     * @param isAll            是否导出全部数据，true是，false否（默认否）
     */
    @Override
    public void export(Page page, BuildingMerchant buildingMerchant, boolean isAll) {
        Page<BuildingMerchant> pageData = new Page<>();
        if (!isAll) {
            pageData = page(page, Wrappers.lambdaQuery(buildingMerchant));
        } else {
            pageData.setRecords(list(Wrappers.lambdaQuery(buildingMerchant)));
        }
        //导出数据
        ExcelUtils.getListToExcel(BuildingMerchant.class, pageData.getRecords(), "校内商户管理");
    }

    /**
     * 查询所有数据
     *
     * @param buildingMerchant 查询条件
     * @return 查询所有数据
     */
    @Override
    public List<BuildingMerchant> allList(BuildingMerchant buildingMerchant) {
        return list(Wrappers.lambdaQuery(buildingMerchant));
    }
}

