package com.tofly.building.service.impl;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.haiqiu.tools.excel.utils.ExcelUtils;
import com.tofly.common.service.IFileInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.CollectionUtils;
import org.springframework.beans.BeanUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.building.mapper.BuildingReportMouthMapper;
import com.tofly.building.entity.BuildingReportMouth;
import com.tofly.building.entity.dto.BuildingReportMouthDto;
import com.tofly.building.service.BuildingReportMouthService;
import org.springframework.stereotype.Service;

import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.io.Serializable;

/**
 * 楼栋用电用水报告月份表(BuildingReportMouth)表服务实现类
 *
 * @author HaiQiu
 * @since 2023-03-20 09:51:11
 */
@Service("buildingReportMouthService")
public class BuildingReportMouthServiceImpl extends ServiceImpl<BuildingReportMouthMapper, BuildingReportMouth> implements BuildingReportMouthService {


    @Value("${buildingReportMouth.ftp.uploads:/dcsw/project/buildingReportMouth}")
    private String path;
    @Autowired
    private IFileInfoService fileInfoService;

    /**
     * 根据ID查询
     *
     * @param id id
     * @return 数据
     */
    @Override
    public BuildingReportMouth get(Serializable id) {
        return getById(id);
    }


    /**
     * 新增
     *
     * @param buildingReportMouthDto 实体
     * @param files                  上传文件附件
     */
    @Override
    public void insert(BuildingReportMouthDto buildingReportMouthDto, List<MultipartFile> files) {
        BuildingReportMouth buildingReportMouth = convert(buildingReportMouthDto);
        buildingReportMouth.setAnnex(fileInfoService.uploadFiles(files, buildingReportMouth.getAnnex()
                , buildingReportMouth.getAnnex(), path, "TF_BUILDING_REPORT_MOUTH"));
        if (!save(buildingReportMouth)) {
            throw new RuntimeException("新增失败");
        }
    }

    /**
     * 修改
     *
     * @param buildingReportMouthDto 实体
     * @param files                  上传文件附件
     */
    @Override
    public void edit(BuildingReportMouthDto buildingReportMouthDto, List<MultipartFile> files) {
        BuildingReportMouth buildingReportMouth = convert(buildingReportMouthDto);
        buildingReportMouth.setAnnex(fileInfoService.uploadFiles(files, buildingReportMouth.getAnnex()
                , buildingReportMouth.getAnnex(), path, "TF_BUILDING_REPORT_MOUTH"));
        if (!updateById(buildingReportMouth)) {
            throw new RuntimeException("修改失败");
        }
    }

    /**
     * 实体转换
     *
     * @param buildingReportMouthDto dto实体
     */
    private BuildingReportMouth convert(BuildingReportMouthDto buildingReportMouthDto) {
        BuildingReportMouth buildingReportMouth = new BuildingReportMouth();
        BeanUtils.copyProperties(buildingReportMouthDto, buildingReportMouth);
        return buildingReportMouth;
    }

    /**
     * 批量删除
     *
     * @param idList 数据ID集合
     */
    @Override
    public void delete(List<Long> idList) {
        if (!removeByIds(idList)) {
            throw new RuntimeException("删除失败");
        }
    }

    /**
     * 生成模板下载
     */
    @Override
    public void template() {
        BuildingReportMouth buildingReportMouth = new BuildingReportMouth();
        ExcelUtils.createTemplate("楼栋用电用水报告月份表", "楼栋用电用水报告月份表",
                "楼栋用电用水报告月份表", BuildingReportMouth.class, Collections.singletonList(buildingReportMouth));
    }

    /**
     * 数据导入
     *
     * @param file 文件
     */
    @Override
    public void importData(MultipartFile file) {
        List<BuildingReportMouth> excelToList = ExcelUtils.getExcelToList("楼栋用电用水报告月份表", file, BuildingReportMouth.class);
        if (CollectionUtils.isEmpty(excelToList)) {
            throw new RuntimeException("数据解析为空");
        }
        saveBatch(excelToList);
    }

    /**
     * 分页查询
     *
     * @param page                分页
     * @param buildingReportMouth 查询wrapper
     * @return 数据集合
     */
    @Override
    public Page<BuildingReportMouth> pageList(Page page, BuildingReportMouth buildingReportMouth) {
        LambdaQueryWrapper<BuildingReportMouth> wrapper = Wrappers.lambdaQuery(buildingReportMouth);
        if (StringUtils.hasText(buildingReportMouth.getReportTime())){
            wrapper.apply("TO_CHAR(DATA_TIME,'yyyy-MM') = " + "'"+ buildingReportMouth.getReportTime()
                            .replaceAll(" ","")+ "'");
        }
        return page(page, wrapper).addOrder(page.getOrders());
    }

    /**
     * 数据导出
     *
     * @param page                分页
     * @param buildingReportMouth 查询模型
     * @param isAll               是否导出全部数据，true是，false否（默认否）
     */
    @Override
    public void export(Page page, BuildingReportMouth buildingReportMouth, boolean isAll) {
        Page<BuildingReportMouth> pageData = new Page<>();
        if (!isAll) {
            pageData = page(page, Wrappers.lambdaQuery(buildingReportMouth));
        } else {
            pageData.setRecords(list(Wrappers.lambdaQuery(buildingReportMouth)));
        }
        //导出数据
        ExcelUtils.getListToExcel(BuildingReportMouth.class, pageData.getRecords(), "楼栋用电用水报告月份表");
    }

    /**
     * 查询所有数据
     *
     * @param buildingReportMouth 查询条件
     * @return 查询所有数据
     */
    @Override
    public List<BuildingReportMouth> allList(BuildingReportMouth buildingReportMouth) {
        return list(Wrappers.lambdaQuery(buildingReportMouth)
                .apply(StringUtils.hasText(buildingReportMouth.getReportTime())
                        ,"DATA_TIME = TO_DATE({0},'yyyy-MM') ", buildingReportMouth.getReportTime()));
    }
}

