package com.tofly.building.service.impl;


import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.deepoove.poi.XWPFTemplate;
import com.deepoove.poi.config.Configure;
import com.deepoove.poi.plugin.table.LoopRowTableRenderPolicy;
import com.deepoove.poi.util.PoitlIOUtils;
import com.haiqiu.tools.common.utils.HttpServletUtils;
import com.haiqiu.tools.excel.utils.ExcelUtils;
import com.tofly.building.entity.BuildingErcet;
import com.tofly.building.entity.BuildingReport;
import com.tofly.building.entity.BuildingReportMouth;
import com.tofly.building.entity.ToEmail;
import com.tofly.building.entity.dto.BuildingReportDto;
import com.tofly.building.entity.dto.ScadaDeviceArchiveManageDto;
import com.tofly.building.entity.vo.BuildReportVo;
import com.tofly.building.mapper.BuildingReportMapper;
import com.tofly.building.service.BuildingErcetService;
import com.tofly.building.service.BuildingReportMouthService;
import com.tofly.building.service.BuildingReportService;
import com.tofly.building.service.CustomMailService;
import com.tofly.common.ftp.config.FtpUtil;
import com.tofly.common.service.IFileInfoService;
import com.tofly.scada.config.DictConfig;
import com.tofly.scada.entity.vo.EnergyValueVo;
import com.tofly.scada.entity.vo.TimeValueVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.ClassPathResource;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.net.URLEncoder;
import java.nio.file.Files;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 楼栋用电用水报告(BuildingReport)表服务实现类
 *
 * @author HaiQiu
 * @since 2023-03-17 09:16:34
 */
@Slf4j
@Service("buildingReportService")
public class BuildingReportServiceImpl extends ServiceImpl<BuildingReportMapper, BuildingReport> implements BuildingReportService {


    public static final String DOCX = ".docx";
    public static final String PDF = ".pdf";

    @Autowired
    private DictConfig dictConfig;

    @Autowired
    private CustomMailService customMailService;

    @Value("${buildingReport.ftp.uploads:/dcsw/project/buildingReport}")
    private String path;

    @Value("${buildingReport.report.uploads.name:四川中医药高等专科学校}")
    private String titleName;

    @Value("${spring.mail.sendTo}")
    private String sendTo;

    @Autowired
    private IFileInfoService fileInfoService;

    @Autowired
    private BuildingReportMouthService reportMouthService;

    @Autowired
    private BuildingErcetService buildingErcetService;

    /**
     * 根据ID查询
     *
     * @param id id
     * @return 数据
     */
    @Override
    public BuildingReport get(Serializable id) {
        return getById(id);
    }


    /**
     * 新增
     *
     * @param buildingReportDto 实体
     * @param files             上传文件附件
     */
    @Override
    public void insert(BuildingReportDto buildingReportDto, List<MultipartFile> files) {
        BuildingReport buildingReport = convert(buildingReportDto);
        buildingReport.setAnnex(fileInfoService.uploadFiles(files, buildingReport.getAnnex()
                , buildingReport.getAnnex(), path, "TF_BUILDING_REPORT"));
        if (!save(buildingReport)) {
            throw new RuntimeException("新增失败");
        }
    }

    /**
     * 修改
     *
     * @param buildingReportDto 实体
     * @param files             上传文件附件
     */
    @Override
    public void edit(BuildingReportDto buildingReportDto, List<MultipartFile> files) {
        BuildingReport buildingReport = convert(buildingReportDto);
        buildingReport.setAnnex(fileInfoService.uploadFiles(files, buildingReport.getAnnex()
                , buildingReport.getAnnex(), path, "TF_BUILDING_REPORT"));
        if (!updateById(buildingReport)) {
            throw new RuntimeException("修改失败");
        }
    }

    /**
     * 实体转换
     *
     * @param buildingReportDto dto实体
     */
    private BuildingReport convert(BuildingReportDto buildingReportDto) {
        BuildingReport buildingReport = new BuildingReport();
        BeanUtils.copyProperties(buildingReportDto, buildingReport);
        return buildingReport;
    }

    /**
     * 批量删除
     *
     * @param idList 数据ID集合
     */
    @Override
    public void delete(List<Long> idList) {
        if (!removeByIds(idList)) {
            throw new RuntimeException("删除失败");
        }
    }

    /**
     * 生成模板下载
     */
    @Override
    public void template() {
        BuildingReport buildingReport = new BuildingReport();
        ExcelUtils.createTemplate("楼栋用电用水报告", "楼栋用电用水报告",
                "楼栋用电用水报告", BuildingReport.class, Collections.singletonList(buildingReport));
    }

    /**
     * 数据导入
     *
     * @param file 文件
     */
    @Override
    public void importData(MultipartFile file) {
        List<BuildingReport> excelToList = ExcelUtils.getExcelToList("楼栋用电用水报告", file, BuildingReport.class);
        if (CollectionUtils.isEmpty(excelToList)) {
            throw new RuntimeException("数据解析为空");
        }
        saveBatch(excelToList);
    }

    /**
     * 分页查询
     *
     * @param page           分页
     * @param buildingReport 查询wrapper
     * @return 数据集合
     */
    @Override
    public Page<BuildingReport> pageList(Page page, BuildingReport buildingReport) {

        return page(page, Wrappers.lambdaQuery(buildingReport)).addOrder(page.getOrders());
    }

    /**
     * 数据导出
     *
     * @param page           分页
     * @param buildingReport 查询模型
     * @param isAll          是否导出全部数据，true是，false否（默认否）
     */
    @Override
    public void export(Page page, BuildingReport buildingReport, boolean isAll) {
        Page<BuildingReport> pageData = new Page<>();
        if (!isAll) {
            pageData = page(page, Wrappers.lambdaQuery(buildingReport));
        } else {
            pageData.setRecords(list(Wrappers.lambdaQuery(buildingReport)));
        }
        //导出数据
        ExcelUtils.getListToExcel(BuildingReport.class, pageData.getRecords(), "楼栋用电用水报告");
    }

    /**
     * 查询所有数据
     *
     * @param buildingReport 查询条件
     * @return 查询所有数据
     */
    @Override
    public List<BuildingReport> allList(BuildingReport buildingReport) {
        return list(Wrappers.lambdaQuery(buildingReport));
    }


//    @Scheduled(cron = "0 0 1 1 * ? ")
//    public void calReport() {
//        Calendar calendar = Calendar.getInstance();
//        calendar.setTime(new Date());
//        calendar.add(Calendar.MONTH, -1);
//        calReport(calendar.getTime());
//    }

    @Override
    public List<BuildingReport> calReport(String time) {
        Date timeDate = DateUtil.parse(time, "yyyy-MM");
        List<BuildingReportMouth> reportMouthList = reportMouthService.list(Wrappers.<BuildingReportMouth>lambdaQuery()
                .apply("TO_CHAR(DATA_TIME,'yyyy-MM') = " + "'"+
                        time.replaceAll(" ","")+ "'"));
        if (CollUtil.isNotEmpty(reportMouthList)) {
            reportMouthList.forEach(info ->{
                if (org.apache.commons.lang3.StringUtils.isNotEmpty(info.getAnnex())) {
                    FtpUtil.deleteFile(info.getAnnex());
                }
            });
        }
        List<BuildingReport> reportList = list(Wrappers.<BuildingReport>lambdaQuery()
                .apply("TO_CHAR(DATA_TIME,'yyyy-MM') = " + "'"+
                        time.replaceAll(" ","")+ "'"));
        if (CollUtil.isNotEmpty(reportList)) {
            reportList.forEach(info ->{
                if (org.apache.commons.lang3.StringUtils.isNotEmpty(info.getAnnex())) {
                    FtpUtil.deleteFile(info.getAnnex());
                }
            });
        }
        //删除总报告
        reportMouthService.remove(Wrappers.<BuildingReportMouth>lambdaQuery()
                .apply("TO_CHAR(DATA_TIME,'yyyy-MM') = " + "'"+
                        time.replaceAll(" ","")+ "'"));
        //删除用水用电报告
        remove(Wrappers.<BuildingReport>lambdaQuery()
                .apply("TO_CHAR(DATA_TIME,'yyyy-MM') = " + "'"+
                        time.replaceAll(" ","")+ "'"));
        //生成新总报告
        BuildingReportMouth mouth = new BuildingReportMouth();
        mouth.setName(titleName);
        mouth.setDataTime(timeDate);
        boolean b = reportMouthService.save(mouth);
        if (!b) {
            log.error("月份报告存储失败");
            return null;
        }

        //生成新用水用电报告
        ScadaDeviceArchiveManageDto manageDto = new ScadaDeviceArchiveManageDto();
        List<Date> dates = getDayOfMonth(timeDate);
        manageDto.setStart(dates.get(0));
        manageDto.setEnd(dates.get(1));
        manageDto.setStatisticsType(2);
        QueryWrapper<ScadaDeviceArchiveManageDto> wrapper = new QueryWrapper<>();
        wrapper.ge("s.START_TIME", manageDto.getStart())
                .le("s.END_TIME", manageDto.getEnd())
                .eq("a.TYPE", dictConfig.getMomentFlowAllIndex())
                .eq("s.STATISTICS_TYPE", manageDto.getStatisticsType())
                .ne("m.BUILDING_COLLEGE_ID",0L);
        //查询统计子集数据
        List<BuildReportVo> reportVos = this.baseMapper.selectScadaByBuildId(wrapper);
        if (CollectionUtils.isEmpty(reportVos)) {
            return null;
        }
        //查询统计总数据 分为校区和用水用电类型分组查询
//        List<BuildingReport> reports = this.baseMapper.selectScadaStatByBuildId(wrapper);
        List<BuildingReport> reports = this.baseMapper.selectScadaStatGroupDetectionType(wrapper);
        if (CollectionUtils.isEmpty(reports)) {
            return null;
        }
        //计算与上月相比和去年相比
        List<Date> mouths = getLastDayOfMonth(timeDate);
        QueryWrapper<ScadaDeviceArchiveManageDto> wrapperMouth = new QueryWrapper<>();
        wrapperMouth.ge("s.START_TIME", mouths.get(0))
                .le("s.END_TIME", mouths.get(1))
                .eq("a.TYPE", dictConfig.getMomentFlowAllIndex())
                .eq("s.STATISTICS_TYPE", manageDto.getStatisticsType())
                .ne("m.BUILDING_COLLEGE_ID",0L);
//        List<BuildingReport> mouthData = this.baseMapper.selectScadaStatByBuildId(wrapperMouth);
        List<BuildingReport> mouthData = this.baseMapper.selectScadaStatGroupDetectionType(wrapperMouth);
        Map<String, List<BuildingReport>> mouthDataMap = new HashMap<>();
        if (!CollectionUtils.isEmpty(mouthData)) {
//            mouthDataMap = mouthData.stream().collect(Collectors.groupingBy(o -> o.getArea() + "-" + o.getType()));
            mouthDataMap = mouthData.stream().collect(Collectors.groupingBy(BuildingReport::getType));
        }
        List<Date> year = getLastYear(timeDate);
        QueryWrapper<ScadaDeviceArchiveManageDto> wrapperYear = new QueryWrapper<>();
        wrapperYear.ge("s.START_TIME", year.get(0))
                .le("s.END_TIME", year.get(1))
                .eq("a.TYPE", dictConfig.getMomentFlowAllIndex())
                .eq("s.STATISTICS_TYPE", manageDto.getStatisticsType())
                .ne("m.BUILDING_COLLEGE_ID",0L);
//        List<BuildingReport> yearData = this.baseMapper.selectScadaStatByBuildId(wrapperYear);
        List<BuildingReport> yearData = this.baseMapper.selectScadaStatGroupDetectionType(wrapperYear);
        Map<String, List<BuildingReport>> yearDataMap = new HashMap<>();
        if (!CollectionUtils.isEmpty(yearData)) {
//            yearDataMap = yearData.stream().collect(Collectors.groupingBy(o -> o.getArea() + "-" + o.getType()));
            yearDataMap = yearData.stream().collect(Collectors.groupingBy(BuildingReport::getType));
        }

        Map<String, List<BuildReportVo>> listMap = reportVos.stream()
                .collect(Collectors.groupingBy(BuildReportVo::getType));
        for (BuildingReport report : reports) {
            report.setSendMail(sendTo);
            report.setLastMonth("0");
            report.setLastYear("0");
            report.setLeakage("0");
            String key = report.getType();
            if (listMap.get(key) != null) {
                List<BuildReportVo> vos = listMap.get(key);
                report.setBuildReportVos(vos);
                report.setContent(JSON.toJSONString(vos));
                report.setDataTime(manageDto.getStart());
                report.setName(titleName);
                report.setReportMouthId(mouth.getId());
            }
            //与上月相比
            if (mouthDataMap.get(key) != null) {
                List<BuildingReport> m = mouthDataMap.get(key);
                if (!CollectionUtils.isEmpty(m)) {
                    report.setLastMonth(calculateMouthOrYearData(report.getNowSum(), m.get(0).getNowSum()));
                }
            }
            //与去年同期相比
            if (yearDataMap.get(key) != null) {
                List<BuildingReport> y = yearDataMap.get(key);
                if (!CollectionUtils.isEmpty(y)) {
                    report.setLastYear(calculateMouthOrYearData(report.getNowSum(), y.get(0).getNowSum()));
                }
            }
            //漏损率/折合标煤（吨）
            if (key.contains("0")) {
                //用水
                String s = this.baseMapper.selectDmaStats(DateUtil.format(timeDate, "yyyy-MM-dd"));
                if (StringUtils.hasText(s)) {
                    report.setLeakage(s);
                }
            } else {
                //用电
                BigDecimal sum = new BigDecimal(report.getNowSum());
                BigDecimal equivalent = new BigDecimal("0.0035");
                BigDecimal multiply = sum.multiply(equivalent);
                report.setLeakage(String.valueOf(multiply));
            }
        }
        saveBatch(reports);
        return reports;
    }

    public String calculateMouthOrYearData(String nowStr, String lastStr) {
        BigDecimal now = new BigDecimal(nowStr);
        BigDecimal mb = new BigDecimal(lastStr);
        BigDecimal subtract = now.subtract(mb);
        BigDecimal divide = subtract.divide(now, 4, RoundingMode.HALF_UP);
        String flag;
        if (divide.compareTo(BigDecimal.ZERO) > 0) {
            flag = "+";
        } else {
            flag = "";
        }
        return flag + divide;
    }

    @Override
    public void getReport(String time, String sendMail) {
        sendMail = org.apache.commons.lang3.StringUtils.isNotBlank(sendMail) ? sendMail : sendTo;
        List<BuildingReport> reports = list(Wrappers.<BuildingReport>lambdaQuery()
                .apply("TO_CHAR(DATA_TIME,'yyyy-MM') = " + "'"+
                        time.replaceAll(" ","")+ "'").orderByAsc(BuildingReport::getType));
        if (CollectionUtils.isEmpty(reports)) {
            return;
        }
        String annex = reports.get(0).getAnnex();
        if (org.apache.commons.lang3.StringUtils.isNotEmpty(annex)) {
            FtpUtil.deleteFile(annex);
        }
        for (BuildingReport report : reports) {
            List<BuildReportVo> buildingReports = JSON.parseArray(report.getContent(), BuildReportVo.class);
            report.setBuildReportVos(buildingReports);
        }

        //数据 0水1电
        Map<String, Object> params = new HashMap<>();
        params.put("name", reports.get(0).getName());
        params.put("time", DateUtil.format(reports.get(0).getDataTime(), "yyyy年MM月"));

        params.put("water", reports.get(0).getNowSum());
        params.put("water_sum", reports.get(0).getNowSum());
        params.put("water_compare_mouth", reports.get(0).getLastMonth());
        params.put("water_compare_year", reports.get(0).getLastYear());
        params.put("water_lose", reports.get(0).getLeakage());
        if (reports.size() >= 2) {
            params.put("electricity", reports.get(1).getNowSum());
            params.put("electricity_sum", reports.get(1).getNowSum());

            params.put("electricity_compare_mouth", reports.get(1).getLastMonth());
            params.put("electricity_compare_year", reports.get(1).getLastYear());
            params.put("electricity_lose", reports.get(1).getLeakage());
        } else {
            params.put("electricity", "0");
            params.put("electricity_sum", "0");
            params.put("electricity_compare_mouth", "0%");
            params.put("electricity_compare_year", "0%");
            params.put("electricity_lose", "0");
        }
        //用水
        List<BuildReportVo> vos1 = reports.get(0).getBuildReportVos();
        for (int i = 1; i <= vos1.size(); i++) {
            vos1.get(i - 1).setIndex(i);
        }
        params.put("w_no1", vos1);
        //用电
        if (reports.size() >= 2) {
            List<BuildReportVo> vos2 = reports.get(1).getBuildReportVos();
            for (int i = 1; i <= vos2.size(); i++) {
                vos2.get(i - 1).setIndex(i);
            }
            params.put("e_no1", vos2);

        }
        String filePath = "/report/export/四川医专能耗报告.docx";
        //返回读取指定资源的输入流
        ClassPathResource resource = new ClassPathResource(filePath);
        InputStream inputStream = null;
        try {
            inputStream = resource.getInputStream();
        } catch (IOException e) {
            throw new RuntimeException(e);
        }

        // 获取模板文件流
        //poi-tl 配置
        LoopRowTableRenderPolicy policy = new LoopRowTableRenderPolicy();

        Configure config = Configure.builder()
                .bind("w_no1", policy).bind("e_no1", policy).build();


        XWPFTemplate template = XWPFTemplate.compile(Objects.requireNonNull(inputStream), config).render(params);
        //上传到ftp服务器
        String fileFtpPath = uploadFtpServer(template, reports);
        log.info("报告上传到ftp：{}", fileFtpPath);
        //修改总报告
        reportMouthService.update(new LambdaUpdateWrapper<BuildingReportMouth>()
                .set(BuildingReportMouth::getAnnex, fileFtpPath)
                .apply("TO_CHAR(DATA_TIME,'yyyy-MM') = " + "'"+
                        time.replaceAll(" ","")+ "'"));

        //发送邮件
        this.sendMailCustom(time, reports, sendMail);

        //输出网络流
        exportResponse(template);
    }

    @Override
    public void sendMailCustom(String timeStr, List<BuildingReport> reports, String sendMail) {
        List<String> collect = reports.stream().map(BuildingReport::getSendMail).filter(Objects::nonNull).distinct().collect(Collectors.toList());
        String[] collectStr = collect.toArray(new String[0]);
        if (org.apache.commons.lang3.StringUtils.isNotBlank(sendMail)) {
            collectStr = new String[]{sendMail};
        }
        if (!CollectionUtils.isEmpty(collect)) {
            ToEmail email = new ToEmail();
            email.setTos(collectStr);
            email.setSubject(timeStr + "能耗报告");
            email.setContent(timeStr + "能耗报告生成成功，请点击前往查看：https://wwww.baidu.com");
            customMailService.sendMailString(email);
            //修改总报告,邮件已发送
            reportMouthService.update(new LambdaUpdateWrapper<BuildingReportMouth>()
                    .set(BuildingReportMouth::getSend, "1")
                    .set(org.apache.commons.lang3.StringUtils.isNotBlank(sendMail),BuildingReportMouth::getSendMail,sendMail)
                    .apply("TO_CHAR(DATA_TIME,'yyyy-MM') = " + "'"+
                            timeStr.replaceAll(" ","")+ "'"));
            if (org.apache.commons.lang3.StringUtils.isNotBlank(sendMail)) {
                this.update(new LambdaUpdateWrapper<BuildingReport>()
                        .set(BuildingReport::getSendMail,sendMail)
                        .apply("TO_CHAR(DATA_TIME,'yyyy-MM') = " + "'"+
                                timeStr.replaceAll(" ","")+ "'"));
            }
        }
    }

    private void exportResponse(XWPFTemplate template) {
        HttpServletResponse response = HttpServletUtils.getServletResponse();
        OutputStream out = null;
        try {
            String fileName = "四川医专能耗报告";
            String type = ".docx";
            // 设置输出的格式
            String name = URLEncoder.encode(fileName + type, "UTF-8");
            //自定义的header
            response.setHeader("requestType", "file");
            // 自定义的header
            response.setHeader("requestFileName", name);
            //设置这个header 可见
            response.setHeader("Access-Control-Expose-Headers", "requestType,requestFileName");

            //设置响应类型为html，编码为utf-8，处理相应页面文本显示的乱码
            response.setContentType("application/octet-stream;charset=UTF-8");
            //设置文件头：最后一个参数是设置下载文件名
            response.setHeader("Content-disposition", "attachment;fileName=" + name);

            out = response.getOutputStream();
            BufferedOutputStream bos = new BufferedOutputStream(out);
            template.write(bos);
            bos.flush();
            out.flush();
            PoitlIOUtils.closeQuietlyMulti(template, bos, out);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 上传文件到ftp服务器
     *
     * @param template 报告文件
     */
    private String uploadFtpServer(XWPFTemplate template, List<BuildingReport> reports) {
        File file = null;
        try {
            file = new File("output.docx");
            template.write(Files.newOutputStream(file.toPath()));
            FileInputStream stream = new FileInputStream(file);
            String ftpPath = path + "/" + System.currentTimeMillis() + "output.docx";
            boolean b = FtpUtil.uploadFile2Ftp(stream, ftpPath);
            if (b) {
                for (BuildingReport report : reports) {
                    report.setAnnex(ftpPath);
                }
                updateBatchById(reports);
                return ftpPath;
            }
            return null;
        } catch (IOException e) {
            throw new RuntimeException(e);
        } finally {
            file.delete();
        }
    }

    @Override
    public List<BuildReportVo> selectScadaByBuildId(ScadaDeviceArchiveManageDto scadaDeviceArchiveManageDto) {
        QueryWrapper<ScadaDeviceArchiveManageDto> wrapper = getWrapper(scadaDeviceArchiveManageDto);
        return baseMapper.selectScadaByBuildId(wrapper);
    }


    @Override
    public List<BuildingReport> selectScadaStatByBuildId(ScadaDeviceArchiveManageDto scadaDeviceArchiveManageDto) {
        QueryWrapper<ScadaDeviceArchiveManageDto> wrapper = getWrapper(scadaDeviceArchiveManageDto);
        List<BuildingReport> reports = baseMapper.selectScadaStatByBuildId(wrapper);
        for (BuildingReport report : reports) {
            //漏损率/折合标煤（吨）
            if (report.getType() != null && report.getType().contains("0")) {
                //TODO 用水 漏损率不在此计算，移步到接口看实现/gis/customDisplay/getLeakageRate
//                if (scadaDeviceArchiveManageDto.getStart() != null) {
//                    String s = this.baseMapper.selectDmaStats(DateUtil.format(scadaDeviceArchiveManageDto.getStart(), "yyyy-MM-dd"));
//                    if (StringUtils.hasText(s)) {
//                        report.setLeakage(s + "%");
//                    }
//                }
            } else if (report.getType() != null && report.getType().contains("1")) {
                //用电
                BigDecimal sum = new BigDecimal(report.getNowSum());
                BigDecimal equivalent = new BigDecimal("0.0035");
                BigDecimal multiply = sum.multiply(equivalent);
                report.setLeakage(String.valueOf(multiply));
            } else {
                report.setLeakage("0");
            }
        }
        return reports;
    }

    private static QueryWrapper<ScadaDeviceArchiveManageDto> getWrapper(ScadaDeviceArchiveManageDto scadaDeviceArchiveManageDto) {
        QueryWrapper<ScadaDeviceArchiveManageDto> wrapper = new QueryWrapper<>();
        return wrapper.ge(scadaDeviceArchiveManageDto.getStart() != null, "s.START_TIME",
                        scadaDeviceArchiveManageDto.getStart())
                .le(scadaDeviceArchiveManageDto.getEnd() != null, "s.END_TIME",
                        scadaDeviceArchiveManageDto.getEnd())
                .in(!CollectionUtils.isEmpty(scadaDeviceArchiveManageDto.getBuildingCollegeIds()), "b.ID",
                        scadaDeviceArchiveManageDto.getBuildingCollegeIds())
                .eq(scadaDeviceArchiveManageDto.getStatisticsType() != null, "s.STATISTICS_TYPE",
                        scadaDeviceArchiveManageDto.getStatisticsType())
                .eq(scadaDeviceArchiveManageDto.getAllIndexType() != null, "a.TYPE",
                        scadaDeviceArchiveManageDto.getAllIndexType())
                .in(!CollectionUtils.isEmpty(scadaDeviceArchiveManageDto.getCodes()), "s.CODE",
                        scadaDeviceArchiveManageDto.getCodes())
                .eq(scadaDeviceArchiveManageDto.getAllIndexModel() != null, "a.MODEL",
                        scadaDeviceArchiveManageDto.getAllIndexModel())
                .eq(StringUtils.hasText(scadaDeviceArchiveManageDto.getDetectionType()), "m.DETECTION_TYPE",
                        scadaDeviceArchiveManageDto.getDetectionType())
                .eq(StringUtils.hasText(scadaDeviceArchiveManageDto.getBuildingType()), "m.BUILDING_TYPE",
                        scadaDeviceArchiveManageDto.getBuildingType())
                .like(org.apache.commons.lang3.StringUtils.isNotBlank(scadaDeviceArchiveManageDto.getName()), "m.NAME",
                        scadaDeviceArchiveManageDto.getName())
                .notLike(org.apache.commons.lang3.StringUtils.isBlank(scadaDeviceArchiveManageDto.getName()), "m.NAME", "总表");
    }

    public List<String> getTitleList(String title, int length) {
        List<String> stringList = new ArrayList<>();
        for (int i = 1; i <= length; i++) {
            stringList.add(title);
        }
        return stringList;
    }

    public List<Integer> getIndexList(int length) {
        List<Integer> integers = new ArrayList<>();
        for (int i = 1; i <= length; i++) {
            integers.add(i);
        }
        return integers;
    }


    /**
     * 获取某月的开始时间和结束时间
     */
    public static List<Date> getDayOfMonth(Date now) {
        //整个日历调整时间
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(now == null ? new Date() : now);
        //倒退一个月开始时间
        //calendar.add(Calendar.MONTH, -1);
        calendar.set(Calendar.DATE, 1);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        Date startTime = calendar.getTime();
        String start = DateUtil.format(calendar.getTime(), "yyyy-MM-dd HH:mm:ss");
        //倒退一个月最后一天时间
        calendar.set(Calendar.DATE, calendar.getActualMaximum(Calendar.DATE));
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        calendar.set(Calendar.MILLISECOND, 999);
        Date endTime = calendar.getTime();
        String end = DateUtil.format(calendar.getTime(), "yyyy-MM-dd HH:mm:ss");
        List<Date> dates = new ArrayList<>();
        dates.add(startTime);
        dates.add(endTime);
        return dates;
    }

    /**
     * 获取某月的开始时间和结束时间
     */
    public static List<Date> getLastDayOfMonth(Date now) {
        //整个日历调整时间
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(now == null ? new Date() : now);
        //倒退一个月开始时间
        calendar.add(Calendar.MONTH, -1);
        calendar.set(Calendar.DATE, 1);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        Date startTime = calendar.getTime();
        String start = DateUtil.format(calendar.getTime(), "yyyy-MM-dd HH:mm:ss");
        //倒退一个月最后一天时间
        calendar.set(Calendar.DATE, calendar.getActualMaximum(Calendar.DATE));
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        calendar.set(Calendar.MILLISECOND, 999);
        Date endTime = calendar.getTime();
        String end = DateUtil.format(calendar.getTime(), "yyyy-MM-dd HH:mm:ss");
        List<Date> dates = new ArrayList<>();
        dates.add(startTime);
        dates.add(endTime);
        return dates;
    }

    /**
     * 获取某年同期的开始时间和结束时间
     */
    public static List<Date> getLastYear(Date now) {
        //整个日历调整时间
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(now == null ? new Date() : now);
        //倒退一个月开始时间
        calendar.add(Calendar.YEAR, -1);
        calendar.set(Calendar.DATE, 1);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        Date startTime = calendar.getTime();
        String start = DateUtil.format(calendar.getTime(), "yyyy-MM-dd HH:mm:ss");
        //倒退一个月最后一天时间
        calendar.set(Calendar.DATE, calendar.getActualMaximum(Calendar.DATE));
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        calendar.set(Calendar.MILLISECOND, 999);
        Date endTime = calendar.getTime();
        String end = DateUtil.format(calendar.getTime(), "yyyy-MM-dd HH:mm:ss");
        List<Date> dates = new ArrayList<>();
        dates.add(startTime);
        dates.add(endTime);
        return dates;
    }

    @Override
    public String getEnergyData() {
        List<BuildingErcet> buildErcets = buildingErcetService.list(new QueryWrapper<BuildingErcet>().lambda()
                .eq(BuildingErcet::getPid, 0L));
        List<BuildingErcet> buildErcetAll = buildingErcetService.list();
        Calendar cal = Calendar.getInstance();
        cal.add(Calendar.DAY_OF_MONTH, -1);
        List<TimeValueVo> values = this.baseMapper.viewScadaDataGroupBuild(
                DateUtil.format(cal.getTime(), "yyyy-MM-dd") + " 00:00:00",
                DateUtil.format(cal.getTime(), "yyyy-MM-dd") + " 23:59:59",
                "2");
        return JSON.toJSONString(getRegionList(buildErcets, buildErcetAll, values));
    }

    /**
     *
     * @description 递归获取每个建筑数据及其子集数据
     */
    private List<Map<String, EnergyValueVo>> getRegionList(List<BuildingErcet> buildErcets, List<BuildingErcet> buildErcetAll, List<TimeValueVo> values) {
        List<Map<String, EnergyValueVo>> all = new ArrayList<>();
        if (com.baomidou.mybatisplus.core.toolkit.CollectionUtils.isNotEmpty(buildErcets)) {
            List<BuildingErcet> buildErcetsFinal = new ArrayList<>(buildErcets);
            for (BuildingErcet vo : buildErcetsFinal) {
                Long id = vo.getId();
                Map<String, EnergyValueVo> map = new HashMap<>();

                List<TimeValueVo> valuesFinal = values.stream().filter(e -> Objects.equals(id, e.getId())).collect(Collectors.toList());
                List<TimeValueVo> valuesTotal = getAllTimeValueVo(DateUtil.format(new Date(), "yyyy-MM-dd"), valuesFinal);
                EnergyValueVo energyValueVo = new EnergyValueVo();
                energyValueVo.setValues(valuesTotal.stream().map(TimeValueVo::getValue).collect(Collectors.toList()));

                List<BuildingErcet> buildErcetChildren =  buildErcetAll.stream().filter(e -> e.getPid().equals(id)).collect(Collectors.toList());
                if (CollUtil.isNotEmpty(buildErcets)) {
                    energyValueVo.setItems(getRegionList(buildErcetChildren, buildErcetAll, values));
                }
                map.put(vo.getName(), energyValueVo);
                all.add(map);
            }
        }
        return all;
    }

    /**
     *
     * @description 没数据时间点补零
     */
    private List<TimeValueVo> getAllTimeValueVo(String day, List<TimeValueVo> list) {
        List<TimeValueVo> res = new ArrayList<>();

        List<String> showedTimeList = null;
        if (CollUtil.isNotEmpty(list)) {
            showedTimeList = list.stream().map(TimeValueVo::getTime).collect(Collectors.toList());
        }
        String start = day + " 00:00:00";
        String end = day + " 23:59:59";
        Calendar cal = Calendar.getInstance();
        cal.setTime(DateUtil.parse(start, "yyyy-MM-dd HH:mm:ss"));
        List<String> dateList = new ArrayList<>();
        while (DateUtil.format(cal.getTime(), "yyyy-MM-dd HH:mm:ss").compareTo(end) < 0) {
            dateList.add(DateUtil.format(cal.getTime(), "yyyy-MM-dd HH:mm:ss"));
            cal.add(Calendar.HOUR, +1);
        }
        for (String date : dateList) {
            if (CollUtil.isNotEmpty(showedTimeList) && showedTimeList.contains(date)) {
                continue;
            }
            TimeValueVo timeValueVo = new TimeValueVo();
            timeValueVo.setTime(date);
            timeValueVo.setValue(0.0);
            res.add(timeValueVo);
        }

        return res;
    }

}

