package com.tofly.building.service.impl;

import com.tofly.building.entity.ToEmail;
import com.tofly.building.service.CustomMailService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.mail.MailException;
import org.springframework.mail.SimpleMailMessage;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.mail.javamail.MimeMessageHelper;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.mail.internet.MimeMessage;

/**
 * @author haiqiu
 * @date 2023/4/6
 */
@Service
public class CustomMailServiceImpl implements CustomMailService {

    @Value("${spring.mail.username}")
    private String from;

    @Resource
    private JavaMailSender mailSender;



    @Override
    public String sendMailString(ToEmail toEmail) {
        //创建简单邮件消息
        SimpleMailMessage message = new SimpleMailMessage();
        //谁发的
        message.setFrom(from);
        //谁要接收
        message.setTo(toEmail.getTos());
        //邮件标题
        message.setSubject(toEmail.getSubject());
        //邮件内容
        message.setText(toEmail.getContent());
        try {
            mailSender.send(message);
            return "发送普通邮件成功";
        } catch (MailException e) {
            e.printStackTrace();
            return "普通邮件发送失败";
        }
    }

    @Override
    public String sendMailHtml(ToEmail toEmail) {
        try {
            //创建一个MINE消息
            MimeMessage message = mailSender.createMimeMessage();
            MimeMessageHelper minehelper = new MimeMessageHelper(message, true);
            //谁发
            minehelper.setFrom(from);
            //谁要接收
            minehelper.setTo(toEmail.getTos());
            //邮件主题
            minehelper.setSubject(toEmail.getSubject());
            //邮件内容   true 表示带有附件或html
            minehelper.setText(toEmail.getContent(), true);
            mailSender.send(message);
            return "发送HTML邮件成功";
        } catch (Exception e) {
            e.printStackTrace();
            return "HTML邮件发送失败," + e.getMessage();
        }
    }


}
