package com.tofly.building.service.impl;


import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.building.entity.ScadaAllocation;
import com.tofly.building.entity.dto.ScadaDeviceArchiveManageDto;
import com.tofly.building.mapper.ScadaAllocationMapper;
import com.tofly.building.service.ScadaAllocationService;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.List;

/**
 * 指标配置(Allocation)表服务实现类
 *
 * @author HaiQiu
 * @since 2023-03-06 10:33:04
 */
@Service("allocationService")
public class ScadaAllocationServiceImpl extends ServiceImpl<ScadaAllocationMapper, ScadaAllocation> implements ScadaAllocationService {

    public static final String STATISTICS_SCADA = "STATISTICS_SCADA";

    public static final String SCADA_HISTORY = "SCADA_HISTORY";

    public static final String SCADA_MONITOR = "SCADA_MONITOR";

    public static final String TF_SCADA_NO_ALARM = "TF_SCADA_NO_ALARM";


    public static final String TF_SCADA_ALARM = "TF_SCADA_ALARM";


    @Override
    public List<ScadaAllocation> listByDevice(ScadaDeviceArchiveManageDto manageDto, String tableName) {
        return listByDevice(manageDto,tableName,true);
    }

    @Override
    public List<ScadaAllocation> listByDevice(ScadaDeviceArchiveManageDto manageDto, String tableName,boolean isAlarm) {
        QueryWrapper<ScadaDeviceArchiveManageDto> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(manageDto.getId() != null, "a.DEVICE_ID", manageDto.getId())
                .in(!CollectionUtils.isEmpty(manageDto.getScadaCode()), "s.code", manageDto.getCode())
                .eq(org.apache.commons.lang3.StringUtils.isNotBlank(manageDto.getAllIndexType()),"a.TYPE",manageDto.getAllIndexType())
                .in(CollUtil.isNotEmpty(manageDto.getAllIndexTypes()),"a.TYPE",manageDto.getAllIndexTypes());
        if (tableName.equals(STATISTICS_SCADA)) {
            queryWrapper.eq(manageDto.getStatisticsType() != null, "s.STATISTICS_TYPE", manageDto.getStatisticsType()).
                    ge(manageDto.getStart() != null, "s.START_TIME", manageDto.getStart())
                    .le(manageDto.getEnd() != null, "s.END_TIME", manageDto.getEnd());
        } else if (tableName.equals(TF_SCADA_ALARM) || tableName.equals(TF_SCADA_NO_ALARM)) {
            queryWrapper.eq(StringUtils.hasText(manageDto.getAlarmReport()),
                    "s.REPORT", manageDto.getAlarmReport());
            queryWrapper.between(manageDto.getStart() != null && manageDto.getEnd() != null,
                    "s.SCADA_TIME", manageDto.getStart(), manageDto.getEnd());
        } else {
            queryWrapper.between(manageDto.getStart() != null && manageDto.getEnd() != null,
                    "s.SCADA_TIME", manageDto.getStart(), manageDto.getEnd());
        }
        List<ScadaAllocation> allocationList = this.baseMapper.selectListData(queryWrapper, tableName);
        if (!CollectionUtils.isEmpty(allocationList) && tableName.equals(STATISTICS_SCADA) && isAlarm) {
            for (ScadaAllocation allocation : allocationList) {
                if (StringUtils.hasText(allocation.getCode())) {
                    long count = baseMapper.scadaAlarmCount(allocation.getCode(), manageDto.getStart(), manageDto.getEnd());
                    allocation.setScadaAlarmCount(count);
                }
            }
        }
        return allocationList;
    }
}

