/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.building.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.building.entity.*;
import com.tofly.building.entity.dto.ScadaDeviceArchiveManageDto;
import com.tofly.building.entity.vo.SafeElectricityVo;
import com.tofly.building.entity.vo.ScadaScadaDeviceArchiveManageVo;
import com.tofly.building.mapper.BuildingReportMapper;
import com.tofly.building.mapper.ScadaDeviceArchiveManageMapper;
import com.tofly.building.service.ScadaAllocationService;
import com.tofly.building.service.ScadaDeviceArchiveManageService;
import com.tofly.building.service.SysDictService;
import com.tofly.scada.entity.ScadaReport;
import com.tofly.scada.entity.vo.StatisticsScadaCalVo1;
import com.tofly.scada.mapper.ScadaMapper;
import com.tofly.scada.service.ScadaReportService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 设备档案管理
 *
 * @author admin
 * @date Fri Mar 11 00:00:00 CST 2022
 */
@Service("tfDeviceArchivesManageService")
public class ScadaDeviceArchiveManageServiceImpl extends ServiceImpl<ScadaDeviceArchiveManageMapper, ScadaDeviceArchiveManage> implements ScadaDeviceArchiveManageService {


    public static final String SCADA_ONLINE = "scada离线";

    public static final String SCADA_STATUS = "scada异常";
    public static final String zero = "0";
    @Autowired
    private ScadaAllocationService scadaAllocationService;

    @Autowired
    private ScadaMapper scadaMapper;

    @Autowired
    private SysDictService sysDictService;

    @Autowired
    private ScadaReportService scadaReportService;

    @Autowired
    private BuildingReportMapper buildingReportMapper;

    @Override
    public Page<ScadaScadaDeviceArchiveManageVo> pageList(Page page, ScadaDeviceArchiveManageDto manageDto) {
        ScadaDeviceArchiveManage manage = new ScadaDeviceArchiveManage();
        BeanUtils.copyProperties(manageDto, manage);
        Page data = page(page, Wrappers.lambdaQuery(manage)).addOrder(page.getOrders());
        if (data.getRecords() != null && !CollectionUtils.isEmpty(data.getRecords())) {
            List<ScadaDeviceArchiveManage> manages = data.getRecords();
            //设置参数状态
            getDeviceStatus(manages);
        }
        return data;
    }


    @Override
    public List<ScadaDeviceArchiveManage> allListMonitor(ScadaDeviceArchiveManageDto manageDto) {
        LambdaQueryWrapper<ScadaDeviceArchiveManage> wrapper = getWrapper(manageDto);
        List<ScadaDeviceArchiveManage> manages = list(wrapper);
        //设置参数状态
        getDeviceStatus(manages);
        List<ScadaDeviceArchiveManage> res = new ArrayList<>();
        //筛选楼栋类型
        if(org.apache.commons.lang3.StringUtils.isNotBlank(manageDto.getBuildingType())){
            for (ScadaDeviceArchiveManage manage : manages) {
                BuildingErcet buildingErcet = manage.getBuildingErcet();
                if (buildingErcet != null
                        && buildingErcet.getBuildingType() != null
                        && buildingErcet.getBuildingType().equals(manageDto.getBuildingType())) {
                    res.add(manage);
                }
            }
        } else {
            res = manages;
        }
        return res;
    }


    /**
     * 设置参数状态
     *
     * @param manages 设备数据
     */
    private void getDeviceStatus(List<ScadaDeviceArchiveManage> manages) {
        if (!CollectionUtils.isEmpty(manages)) {
            //查询字典配置离线和异常处理时间差
            List<SysDict> onLines = sysDictService.list(Wrappers.<SysDict>lambdaQuery().eq(SysDict::getName, SCADA_ONLINE));
            Map<String, SysDict> onLinesMaps = new HashMap<>();
            if (!CollectionUtils.isEmpty(onLines)) {
                onLinesMaps = onLines.stream().collect(Collectors.toMap(SysDict::getValue, o -> o));
            }
            List<SysDict> status = sysDictService.list(Wrappers.<SysDict>lambdaQuery().eq(SysDict::getName, SCADA_STATUS));
            Map<String, SysDict> statusMaps = new HashMap<>();
            if (!CollectionUtils.isEmpty(onLines)) {
                statusMaps = status.stream().collect(Collectors.toMap(SysDict::getName, o -> o));
            }
            for (ScadaDeviceArchiveManage manage : manages) {
                if (!CollectionUtils.isEmpty(manage.getScadaAllocationList())) {
                    for (ScadaAllocation allocation : manage.getScadaAllocationList()) {
                        //报警处理参数
                        handleReport(manage, allocation);
                        //离线处理
                        handleOnline(onLinesMaps, manage, allocation);
                        //异常处理
                        handleStatus(statusMaps, manage, allocation);
                        //缺省值处理
                        handleNvl(allocation);
                    }
                }
            }
        }
    }

    private void handleNvl(ScadaAllocation allocation) {
        if (!StringUtils.hasText(allocation.getValue())) {
            allocation.setValue("0");
        }
    }

    private static void handleReport(ScadaDeviceArchiveManage manage, ScadaAllocation allocation) {
        if (allocation != null && !CollectionUtils.isEmpty(allocation.getScadaReports()) && allocation.getScadaTime() != null) {
            for (ScadaReport report : allocation.getScadaReports()) {
                if (report.getReportLower() == null || report.getReportUpper() == null) {
                    continue;
                }
                BigDecimal lower = new BigDecimal(report.getReportLower());
                BigDecimal upper = new BigDecimal(report.getReportUpper());
                if (allocation.getValue() != null) {
                    String varStr = allocation.getValue();
                    BigDecimal var = new BigDecimal(varStr);
                    if (var.compareTo(lower) <= -1 || var.compareTo(upper) >= 1) {
                        allocation.setReport(true);
                        manage.setReport(true);
                    }
                }
            }
        }
    }

    private static void handleOnline(Map<String, SysDict> onLinesMaps, ScadaDeviceArchiveManage manage, ScadaAllocation allocation) {
        if (!CollectionUtils.isEmpty(onLinesMaps) && allocation != null && allocation.getScadaTime() != null) {
            for (Map.Entry<String, SysDict> entry : onLinesMaps.entrySet()) {
                SysDict d = entry.getValue();
                long time = new Date().getTime();
                long value = Long.parseLong(d.getValue());
                long differ = time - value;
                long scadaTime = allocation.getScadaTime().getTime();
                if (scadaTime <= differ) {
                    allocation.setOffline(true);
                    manage.setOffline(true);
                }
            }
        }
    }

    private static void handleStatus(Map<String, SysDict> statusMaps, ScadaDeviceArchiveManage manage, ScadaAllocation allocation) {
        if (!CollectionUtils.isEmpty(statusMaps) && allocation != null && allocation.getScadaTime() != null) {
            for (Map.Entry<String, SysDict> entry : statusMaps.entrySet()) {
                SysDict d = entry.getValue();
                long time = new Date().getTime();
                long value = Long.parseLong(d.getValue());
                long differ = time - value;
                long scadaTime = allocation.getScadaTime().getTime();
                if (scadaTime <= differ) {
                    allocation.setTitch(true);
                    manage.setHitch(true);
                }
            }
        }
    }

    private static LambdaQueryWrapper<ScadaDeviceArchiveManage> getWrapper(ScadaDeviceArchiveManageDto manageDto) {
        ScadaDeviceArchiveManage manage = new ScadaDeviceArchiveManage();
        BeanUtils.copyProperties(manageDto, manage);
        return Wrappers.lambdaQuery(manage)
                .in(!CollectionUtils.isEmpty(manageDto.getBuildingCollegeIds()), ScadaDeviceArchiveManage::getBuildingCollegeId, manageDto.getBuildingCollegeIds());
    }

    @Override
    public List<ScadaDeviceArchiveManage> allListStatistics(ScadaDeviceArchiveManageDto manageDto) {
        QueryWrapper<ScadaDeviceArchiveManageDto> wrapper = new QueryWrapper<>();
        if (org.apache.commons.lang3.StringUtils.isNotBlank(manageDto.getIsBinding()) && manageDto.getIsBinding().equals("1")) {
            wrapper.in(!CollectionUtils.isEmpty(manageDto.getBuildingCollegeIds()), "a.BUILDING_COLLEGE_ID", manageDto.getBuildingCollegeIds())
                    .ge(org.apache.commons.lang3.StringUtils.isNotBlank(manageDto.getBuildingType()), "b.BUILDING_TYPE", manageDto.getBuildingType())
                    .eq(org.apache.commons.lang3.StringUtils.isNotBlank(manageDto.getDetectionType()), "a.DETECTION_TYPE", manageDto.getDetectionType())
                    .ne("a.BUILDING_COLLEGE_ID",0L);
        } else if(org.apache.commons.lang3.StringUtils.isNotBlank(manageDto.getIsBinding()) && manageDto.getIsBinding().equals("0")) {
            wrapper.in(!CollectionUtils.isEmpty(manageDto.getBuildingCollegeIds()), "a.BUILDING_COLLEGE_ID", manageDto.getBuildingCollegeIds())
                    .ge(org.apache.commons.lang3.StringUtils.isNotBlank(manageDto.getBuildingType()), "b.BUILDING_TYPE", manageDto.getBuildingType())
                    .eq(org.apache.commons.lang3.StringUtils.isNotBlank(manageDto.getDetectionType()), "a.DETECTION_TYPE", manageDto.getDetectionType())
                    .eq("a.BUILDING_COLLEGE_ID",0L);
        } else {
            wrapper.in(!CollectionUtils.isEmpty(manageDto.getBuildingCollegeIds()), "a.BUILDING_COLLEGE_ID", manageDto.getBuildingCollegeIds())
                    .ge(org.apache.commons.lang3.StringUtils.isNotBlank(manageDto.getBuildingType()), "b.BUILDING_TYPE", manageDto.getBuildingType())
                    .eq(org.apache.commons.lang3.StringUtils.isNotBlank(manageDto.getDetectionType()), "a.DETECTION_TYPE", manageDto.getDetectionType());
        }

        List<ScadaDeviceArchiveManage> list = this.baseMapper.selectDeviceArchiveManageList(wrapper.orderByAsc("d.CODE"));
        if (CollectionUtils.isEmpty(list)) {
            return list;
        }
        List<ScadaAllocation> allocations = scadaAllocationService.listByDevice(manageDto, ScadaAllocationServiceImpl.STATISTICS_SCADA);
        for (ScadaDeviceArchiveManage manage : list) {
            List<ScadaAllocation> allocationsFinal = allocations.stream().filter(e -> Objects.equals(e.getDeviceId(), manage.getId())).collect(Collectors.toList());
            manage.setScadaAllocationList(allocationsFinal);
        }
        return list;
    }

    /**
     * 查询楼栋建筑绑定的指标设备scada数据
     * @param manageDto 查询参数
     * @param isAlarm 是否查询报警参数
     * @return
     */
    public List<ScadaDeviceArchiveManage> allListStatistics(ScadaDeviceArchiveManageDto manageDto, boolean isAlarm) {
        List<ScadaDeviceArchiveManage> list = list(getWrapper(manageDto));
        if (CollectionUtils.isEmpty(list)) {
            return list;
        }
        for (ScadaDeviceArchiveManage manage : list) {
            manageDto.setId(manage.getId());
            List<ScadaAllocation> allocations = scadaAllocationService.listByDevice(manageDto, ScadaAllocationServiceImpl.STATISTICS_SCADA, isAlarm);
            manage.setScadaAllocationList(allocations);
        }
        return list;
    }

    @Override
    public List<ScadaDeviceArchiveManage> allListHistory(ScadaDeviceArchiveManageDto manageDto) {
        List<ScadaDeviceArchiveManage> list = list(getWrapper(manageDto));
        if (CollectionUtils.isEmpty(list)) {
            return list;
        }
        for (ScadaDeviceArchiveManage manage : list) {
            manageDto.setId(manage.getId());
            List<ScadaAllocation> allocations = scadaAllocationService.listByDevice(manageDto, ScadaAllocationServiceImpl.SCADA_HISTORY);
            manage.setScadaAllocationList(allocations);
        }
        return list;
    }

    @Override
    public List<ScadaDeviceArchiveManage> allListAlarm(ScadaDeviceArchiveManageDto manageDto) {
        List<ScadaDeviceArchiveManage> list = list(getWrapper(manageDto));
        if (CollectionUtils.isEmpty(list)) {
            return list;
        }
        for (ScadaDeviceArchiveManage manage : list) {
            manageDto.setId(manage.getId());
            List<ScadaAllocation> allocations = scadaAllocationService.listByDevice(manageDto, ScadaAllocationServiceImpl.TF_SCADA_ALARM);
            for (ScadaAllocation allocation : allocations) {
                if (StringUtils.hasText(allocation.getReportId())) {
                    List<Long> reportIds = Arrays.stream(allocation.getReportId().split(",")).map(Long::new).collect(Collectors.toList());
                    List<ScadaReport> reportList = scadaReportService.list(Wrappers.<ScadaReport>lambdaQuery().in(ScadaReport::getId, reportIds));
                    allocation.setScadaReports(reportList);
                }
            }
            manage.setScadaAllocationList(allocations);
        }
        return list;
    }

    @Override
    public Page<ScadaDeviceArchiveManage> pageListHistory(Page page, ScadaDeviceArchiveManageDto manageDto) {
        Page<ScadaDeviceArchiveManage> pageList = this.baseMapper.pageList(page, getWrapper(manageDto));
        if (CollectionUtils.isEmpty(pageList.getRecords())) {
            return pageList;
        }
        for (ScadaDeviceArchiveManage manage : pageList.getRecords()) {
            manageDto.setId(manage.getId());
            List<ScadaAllocation> allocations = scadaAllocationService.listByDevice(manageDto, ScadaAllocationServiceImpl.SCADA_HISTORY);
            manage.setScadaAllocationList(allocations);
        }
        return pageList;
    }

    @Override
    public List<SafeElectricityVo> safeElectricity(ScadaDeviceArchiveManageDto manageDto) {
        List<ScadaDeviceArchiveManage> manages = allListStatistics(manageDto, false);
        if (CollectionUtils.isEmpty(manages)) {
            return null;
        }
        List<SafeElectricityVo> safeElectricityVos = new ArrayList<>();
        Map<String, ScadaDeviceArchiveManage> manageMap = manages.stream().filter(o -> o.getName() != null)
                .collect(Collectors.toMap(ScadaDeviceArchiveManage::getName, o->o,(o1, o2) -> o1));
        for (Map.Entry<String, ScadaDeviceArchiveManage> entry : manageMap.entrySet()) {
            SafeElectricityVo vo = new SafeElectricityVo();
            vo.setUnitName(entry.getKey());
            if (entry.getValue().getBelongToObject() != null) {
                vo.setCategoryName(entry.getValue().getBelongToObject().equals(zero) ? "学院" : "商户");
            }
            if (entry.getValue().getBuildingErcet() != null && entry.getValue().getBuildingErcet().getArea() != null) {
                vo.setBuildingArea(entry.getValue().getBuildingErcet().getArea());
            }
            List<ScadaAllocation> allocationList = entry.getValue().getScadaAllocationList();
            if (!CollectionUtils.isEmpty(allocationList)) {
                BigDecimal sum = allocationList.stream().map(ScadaAllocation::getDiff).filter(Objects::nonNull).map(BigDecimal::new).reduce(BigDecimal.ZERO, BigDecimal::add);
                vo.setElectricityCount(sum.toString());
                //TODO 报警次数
                getReportCount(manageDto, vo, allocationList);

            } else {
                vo.setElectricityCount(BigDecimal.ZERO.toString());
                vo.setElectricitySum(BigDecimal.ZERO.toString());
            }
            //单位面积用电量
            if (vo.getBuildingArea() == null) {
                vo.setBuildingArea(zero);
            }
            if (vo.getElectricityCount() == null) {
                vo.setElectricityCount(zero);
            }
            if (!vo.getElectricityCount().equals(zero) && !vo.getBuildingArea().equals(zero)) {
                BigDecimal divide = new BigDecimal(vo.getBuildingArea())
                        .divide(new BigDecimal(vo.getElectricityCount()),2, RoundingMode.CEILING);
                vo.setBuildingAreaToElectricity(divide.toString());
            } else {
                vo.setBuildingAreaToElectricity(zero);
            }

            safeElectricityVos.add(vo);
        }
        return safeElectricityVos;
    }

    @Override
    public Map<String, Object> analyse(ScadaDeviceArchiveManageDto manageDto) {
        Map<String, Object> map = new HashMap<>();
        List<String> allIndexTypes = new ArrayList<>();
        allIndexTypes.add("2");
        allIndexTypes.add("7");
        manageDto.setAllIndexTypes(allIndexTypes);
        //查询本月
        List<ScadaDeviceArchiveManage> manages = allListStatistics(manageDto);
        if (!CollectionUtils.isEmpty(manages)) {
            Map<String, List<ScadaDeviceArchiveManage>> listMap = manages.stream().filter(o -> StringUtils.hasText(o.getDetectionType())).collect(Collectors.groupingBy(ScadaDeviceArchiveManage::getDetectionType));
            map.put("manages_now", listMap);
        }
        //查询统计总数据 分为校区和用水用电类型分组查询
//        Map<String, List<BuildingReport>> sumData = getSumData(manageDto, true);
//        map.put("manages_now_sum", sumData);
        //查询上月
        if (!CollectionUtils.isEmpty(manageDto.getMouthTime())) {
            manageDto.setId(null);
            manageDto.setStart(manageDto.getMouthTime().get(0));
            manageDto.setEnd(manageDto.getMouthTime().get(1));
            List<ScadaDeviceArchiveManage> mouthManages = allListStatistics(manageDto);
            Map<String, List<ScadaDeviceArchiveManage>> listMap = mouthManages.stream().filter(o -> StringUtils.hasText(o.getDetectionType())).collect(Collectors.groupingBy(ScadaDeviceArchiveManage::getDetectionType));
            map.put("manages_mouth", listMap);
            //查询统计总数据 分为校区和用水用电类型分组查询
//            Map<String, List<BuildingReport>> sumDataMouth = getSumData(manageDto, false);
//            map.put("manages_mouth_sum", sumDataMouth);
        }
        //查询年
        if (!CollectionUtils.isEmpty(manageDto.getYearTime())) {
            manageDto.setId(null);
            manageDto.setStart(manageDto.getYearTime().get(0));
            manageDto.setEnd(manageDto.getYearTime().get(1));
            List<ScadaDeviceArchiveManage> yearManages = allListStatistics(manageDto);
            Map<String, List<ScadaDeviceArchiveManage>> listMap = yearManages.stream().filter(o -> StringUtils.hasText(o.getDetectionType())).collect(Collectors.groupingBy(ScadaDeviceArchiveManage::getDetectionType));
            map.put("manages_year", listMap);
            //查询统计总数据 分为校区和用水用电类型分组查询
//            Map<String, List<BuildingReport>> sumDataYear = getSumData(manageDto, false);
//            map.put("manages_year_sum", sumDataYear);
        }
        return map;
    }

    @Override
    public Object selectScadaStatsByTime(ScadaDeviceArchiveManageDto manageDto) {
        List<String> codes = baseMapper.selectCodeByManage(manageDto);
        if (CollectionUtils.isEmpty(codes)) {
            return null;
        }
        List<String> collect = codes.stream().filter(Objects::nonNull).collect(Collectors.toList());
        List<StatisticsScadaCalVo1> calVo1s = scadaMapper.selectHistoryMaxAndMinByScadaTime1(collect, manageDto.getStart(),
                manageDto.getEnd(), manageDto.getStatisticsType(), manageDto.getAllIndexType());
        if (CollectionUtils.isEmpty(calVo1s)) {
            return null;
        }
        List<ScadaDeviceArchiveManage> list = list(getWrapper(manageDto));
        if (CollectionUtils.isEmpty(list)) {
            return list;
        }
        for (ScadaDeviceArchiveManage manage : list) {
            manageDto.setId(manage.getId());
            List<ScadaAllocation> allocations = scadaAllocationService.listByDevice(manageDto, ScadaAllocationServiceImpl.SCADA_MONITOR);
            for (ScadaAllocation allocation : allocations) {
                //获取实时指标统计数据
                if (StringUtils.hasText(allocation.getCode())) {
                    for (StatisticsScadaCalVo1 vo1 : calVo1s) {
                        if (vo1.getCode().equals(allocation.getCode())) {
                            allocation.setMax(vo1.getMaxValue());
                            allocation.setValue(vo1.getValue());
                            allocation.setAve(vo1.getAvgValue());
                            allocation.setDateTimeStr(vo1.getDateTimeStr());
                            allocation.setCode(vo1.getCode());
                            allocation.setMin(vo1.getMinValue());
                        }
                    }
                }
            }
            manage.setScadaAllocationList(allocations);
        }
        return list;
    }

    @Override
    public Object selectScadaCodes(ScadaDeviceArchiveManageDto scadaDeviceArchiveManageDto){
        QueryWrapper<ScadaDeviceArchiveManageDto> wrapper = new QueryWrapper<>();
        wrapper.like(StringUtils.hasText(scadaDeviceArchiveManageDto.getAllIndexName()),"a.DISPLAY_NAME",
                        scadaDeviceArchiveManageDto.getAllIndexName())
                .in(!CollectionUtils.isEmpty(scadaDeviceArchiveManageDto.getBuildingCollegeIds()), "b.ID",
                        scadaDeviceArchiveManageDto.getBuildingCollegeIds())
                .eq(scadaDeviceArchiveManageDto.getAllIndexType() != null, "a.TYPE",
                        scadaDeviceArchiveManageDto.getAllIndexType())
                .in(!CollectionUtils.isEmpty(scadaDeviceArchiveManageDto.getCodes()), "a.VARIABLE_CODE",
                        scadaDeviceArchiveManageDto.getCodes())
                .eq(scadaDeviceArchiveManageDto.getAllIndexModel() != null, "a.MODEL",
                        scadaDeviceArchiveManageDto.getAllIndexModel())
                .eq(StringUtils.hasText(scadaDeviceArchiveManageDto.getDetectionType()), "d.DETECTION_TYPE",
                        scadaDeviceArchiveManageDto.getDetectionType())
                .eq(StringUtils.hasText(scadaDeviceArchiveManageDto.getType()), "d.TYPE",
                        scadaDeviceArchiveManageDto.getType())
                .eq(StringUtils.hasText(scadaDeviceArchiveManageDto.getBelongToCollege()), "d.BELONG_TO_COLLEGE",
                        scadaDeviceArchiveManageDto.getBelongToCollege())
                .eq(StringUtils.hasText(scadaDeviceArchiveManageDto.getBelongToObject()), "d.BELONG_TO_OBJECT",
                        scadaDeviceArchiveManageDto.getBelongToObject())
                .eq(StringUtils.hasText(scadaDeviceArchiveManageDto.getBuildingType()), "b.BUILDING_TYPE",
                        scadaDeviceArchiveManageDto.getBuildingType());

       return baseMapper.selectCodesByManageAndBuilding(wrapper);
    }

    /**
     * 获取统计数据
     *
     * @param manageDto 查询参数
     * @param isNow     是否查询为当前时间节点的数据
     * @return
     */
    private Map<String, List<BuildingReport>> getSumData(ScadaDeviceArchiveManageDto manageDto, boolean isNow) {
        if (manageDto != null) {
            if (!isNow) {
                if (!CollectionUtils.isEmpty(manageDto.getMouthTime())) {
                    manageDto.setStart(manageDto.getMouthTime().get(0));
                    manageDto.setEnd(manageDto.getMouthTime().get(1));
                }
                if (!CollectionUtils.isEmpty(manageDto.getYearTime())) {
                    manageDto.setStart(manageDto.getYearTime().get(0));
                    manageDto.setEnd(manageDto.getYearTime().get(1));
                }
            }
        }

        QueryWrapper<ScadaDeviceArchiveManageDto> wrapper = new QueryWrapper<>(manageDto);
        assert manageDto != null;
        wrapper.ge(manageDto.getStart() != null, "s.START_TIME", manageDto.getStart())
                .le(manageDto.getEnd() != null, "s.END_TIME", manageDto.getEnd())
                .in(!CollectionUtils.isEmpty(manageDto.getBuildingCollegeIds()), "b.ID", manageDto.getBuildingCollegeIds())
                .eq(manageDto.getStatisticsType() != null, "s.STATISTICS_TYPE", manageDto.getStatisticsType())
                .eq(StringUtils.hasText(manageDto.getDetectionType()), "m.DETECTION_TYPE", manageDto.getDetectionType());
        List<BuildingReport> reportsOrg = buildingReportMapper.selectScadaStatByBuildId(wrapper);
        List<BuildingReport> reports = new ArrayList<>();
        if (manageDto.getIsBinding() != null) {
            for (BuildingReport buildingReport : reportsOrg) {
                if (manageDto.getIsBinding().equals("1")
                        && buildingReport.getBuildingCollegeId() != null
                        && buildingReport.getBuildingCollegeId() != 0) {
                    if (manageDto.getBuildingType() != null
                            && buildingReport.getBuildingType() != null
                            && buildingReport.getBuildingType().equals(manageDto.getBuildingType())) {
                        reports.add(buildingReport);
                        continue;
                    } else if (manageDto.getBuildingType() == null) {
                        reports.add(buildingReport);
                        continue;
                    }
                } else if(manageDto.getIsBinding().equals("0")
                        && (buildingReport.getBuildingCollegeId() == null
                        || buildingReport.getBuildingCollegeId() == 0)) {
                    if (manageDto.getBuildingType() != null
                            && buildingReport.getBuildingType() != null
                            && buildingReport.getBuildingType().equals(manageDto.getBuildingType())) {
                        reports.add(buildingReport);
                        continue;
                    } else if (manageDto.getBuildingType() == null) {
                        reports.add(buildingReport);
                        continue;
                    }
                } else if(manageDto.getIsBinding().equals("1")
                        && (buildingReport.getBuildingCollegeId() == null
                        || buildingReport.getBuildingCollegeId() == 0)) {
                    if (manageDto.getBuildingType() != null
                            && buildingReport.getBuildingType() != null
                            && buildingReport.getBuildingType().equals(manageDto.getBuildingType())) {
                        reports.add(buildingReport);
                        continue;
                    } else if (manageDto.getBuildingType() == null) {
                        reports.add(buildingReport);
                        continue;
                    }
                } else if(manageDto.getIsBinding().equals("0")
                        && buildingReport.getBuildingCollegeId() != null
                        && buildingReport.getBuildingCollegeId() != 0) {
                    if (manageDto.getBuildingType() != null
                            && buildingReport.getBuildingType() != null
                            && buildingReport.getBuildingType().equals(manageDto.getBuildingType())) {
                        reports.add(buildingReport);
                        continue;
                    } else if (manageDto.getBuildingType() == null) {
                        reports.add(buildingReport);
                        continue;
                    }
                }
                reports.add(buildingReport);
            }
        } else {
            if (manageDto.getBuildingType() != null) {
                for (BuildingReport buildingReport : reportsOrg) {
                    if (buildingReport.getBuildingType() != null
                            && buildingReport.getBuildingType().equals(manageDto.getBuildingType())) {
                        reports.add(buildingReport);
                    } else if (manageDto.getBuildingType() == null) {
                        reports.add(buildingReport);
                    }
                }
            } else {
                reports = reportsOrg;
            }
        }
        if (!CollectionUtils.isEmpty(reports)) {
            Map<String, List<BuildingReport>> listMap = reports.stream().collect(Collectors.groupingBy(BuildingReport::getTypeName));
            return listMap;
        } else {
            List<BuildingReport> data = new ArrayList<>();
            BuildingReport report = new BuildingReport();
            report.setLeakage("0");
            report.setNowSum("0");
            data.add(report);
            Map<String, List<BuildingReport>> map = new HashMap<>();
            map.put("用水", data);
            map.put("用电", data);
            return map;
        }
    }


    private void getReportCount(ScadaDeviceArchiveManageDto manageDto, SafeElectricityVo vo, List<ScadaAllocation> allocationList) {
        Set<String> codes = allocationList.stream().filter(o -> StringUtils.hasText(o.getCode())).map(ScadaAllocation::getCode).collect(Collectors.toSet());
        if (!CollectionUtils.isEmpty(codes)) {
            int count = scadaReportService.count(Wrappers.<ScadaReport>lambdaQuery().in(!CollectionUtils.isEmpty(codes), ScadaReport::getAllocationId, codes).between(manageDto.getStart() != null && manageDto.getEnd() != null, ScadaReport::getCreateTime, manageDto.getStart(), manageDto.getEnd()));
            vo.setElectricitySum(String.valueOf(count));
        } else {
            vo.setElectricitySum(zero);
        }
    }

}
