package com.tofly.building.service.impl;


import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.haiqiu.tools.excel.utils.ExcelUtils;
import org.springframework.util.CollectionUtils;
import org.springframework.beans.BeanUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.building.mapper.SyschildcodeDMapper;
import com.tofly.building.entity.SyschildcodeD;
import com.tofly.building.entity.dto.SyschildcodeDDto;
import com.tofly.building.service.SyschildcodeDService;
import org.springframework.stereotype.Service;

import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.io.Serializable;

/**
 * 系统通用码表子级表(SyschildcodeD)表服务实现类
 *
 * @author HaiQiu
 * @since 2023-03-10 11:21:49
 */
@Service("syschildcodeDService")
public class SyschildcodeDServiceImpl extends ServiceImpl<SyschildcodeDMapper, SyschildcodeD> implements SyschildcodeDService {


    /**
     * 根据ID查询
     *
     * @param id id
     * @return 数据
     */
    @Override
    public SyschildcodeD get(Serializable id) {
        return getById(id);
    }


    /**
     * 新增
     *
     * @param syschildcodeDDto 实体
     */
    @Override
    public void insert(SyschildcodeDDto syschildcodeDDto) {
        SyschildcodeD syschildcodeD = convert(syschildcodeDDto);
        if (!save(syschildcodeD)) {
            throw new RuntimeException("新增失败");
        }
    }

    /**
     * 修改
     *
     * @param syschildcodeDDto 实体
     */
    @Override
    public void edit(SyschildcodeDDto syschildcodeDDto) {
        SyschildcodeD syschildcodeD = convert(syschildcodeDDto);
        if (!updateById(syschildcodeD)) {
            throw new RuntimeException("修改失败");
        }
    }

    /**
     * 实体转换
     *
     * @param syschildcodeDDto dto实体
     */
    private SyschildcodeD convert(SyschildcodeDDto syschildcodeDDto) {
        SyschildcodeD syschildcodeD = new SyschildcodeD();
        BeanUtils.copyProperties(syschildcodeDDto, syschildcodeD);
        return syschildcodeD;
    }

    /**
     * 批量删除
     *
     * @param idList 数据ID集合
     */
    @Override
    public void delete(List<Long> idList) {
        if (!removeByIds(idList)) {
            throw new RuntimeException("删除失败");
        }
    }

    /**
     * 生成模板下载
     */
    @Override
    public void template() {
        SyschildcodeD syschildcodeD = new SyschildcodeD();
        ExcelUtils.createTemplate("系统通用码表子级表", "系统通用码表子级表",
                "系统通用码表子级表", SyschildcodeD.class, Collections.singletonList(syschildcodeD));
    }

    /**
     * 数据导入
     *
     * @param file 文件
     */
    @Override
    public void importData(MultipartFile file) {
        List<SyschildcodeD> excelToList = ExcelUtils.getExcelToList("系统通用码表子级表", file, SyschildcodeD.class);
        if (CollectionUtils.isEmpty(excelToList)) {
            throw new RuntimeException("数据解析为空");
        }
        saveBatch(excelToList);
    }

    /**
     * 分页查询
     *
     * @param page          分页
     * @param syschildcodeD 查询wrapper
     * @return 数据集合
     */
    @Override
    public Page<SyschildcodeD> pageList(Page page, SyschildcodeD syschildcodeD) {

        return page(page, Wrappers.lambdaQuery(syschildcodeD)).addOrder(page.getOrders());
    }

    /**
     * 数据导出
     *
     * @param page          分页
     * @param syschildcodeD 查询模型
     * @param isAll         是否导出全部数据，true是，false否（默认否）
     */
    @Override
    public void export(Page page, SyschildcodeD syschildcodeD, boolean isAll) {
        Page<SyschildcodeD> pageData = new Page<>();
        if (!isAll) {
            pageData = page(page, Wrappers.lambdaQuery(syschildcodeD));
        } else {
            pageData.setRecords(list(Wrappers.lambdaQuery(syschildcodeD)));
        }
        //导出数据
        ExcelUtils.getListToExcel(SyschildcodeD.class, pageData.getRecords(), "系统通用码表子级表");
    }

    /**
     * 查询所有数据
     *
     * @param syschildcodeD 查询条件
     * @return 查询所有数据
     */
    @Override
    public List<SyschildcodeD> allList(SyschildcodeD syschildcodeD) {
        return list(Wrappers.lambdaQuery(syschildcodeD));
    }
}

