package com.tofly.building.utils;

import io.swagger.annotations.ApiModelProperty;
import org.springframework.util.CollectionUtils;

import java.io.Serializable;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @author HaiQiu
 * @since 2023/3/23
 * 配合mybatis-plus的排序工具实现
 */
public class OrderItem implements Serializable {

    /**
     * 是否正序排序，true正序，false倒序，默认true
     */
    @ApiModelProperty(value = "是否正序排序，true正序，false倒序，默认true")
    private boolean asc = true;

    /**
     * 是否正序排序，true正序，false倒序，默认true
     */
    @ApiModelProperty(value = "排序字段")
    private List<String> columns;

    /**
     * 是否驼峰字段转为下划线，默认true
     */
    @ApiModelProperty(value = "是否驼峰字段转为下划线，默认true")
    private boolean hump = true;


    public boolean isAsc() {
        return asc;
    }

    public void setAsc(boolean asc) {
        this.asc = asc;
    }

    public List<String> getColumns() {
        return columns;
    }

    public void setColumns(List<String> columns) {
        if (!CollectionUtils.isEmpty(columns) && this.hump) {
            columns.replaceAll(HumpUtil::humpToUnderline);
        }
        this.columns = columns;
    }

    public boolean isHump() {
        return hump;
    }

    public void setHump(boolean hump) {
        this.hump = hump;
    }

    public String getOrderBySql(){
        StringBuilder orderBySql = new StringBuilder("order by ");
        String ASC = "asc";
        if (!this.asc){
            ASC = "desc";
        }
        if (CollectionUtils.isEmpty(this.columns)){
            return null;
        }
        for (int i = 0; i < this.columns.size(); i++) {
            String column = this.columns.get(i);
            if (i == this.columns.size()-1){
                orderBySql.append(column).append(" ").append(ASC);
            }else {
                orderBySql.append(column).append(" ").append(ASC).append(",");
            }
        }
        return orderBySql.toString();
    }

    /**
     * 驼峰转换工具类
     */
    public static class HumpUtil {
        /**
         * 驼峰转下划线
         *
         * @param str 目标字符串
         * @return: java.lang.String
         */
        public static String humpToUnderline(String str) {
            String regex = "([A-Z])";
            Matcher matcher = Pattern.compile(regex).matcher(str);
            while (matcher.find()) {
                String target = matcher.group();
                str = str.replaceAll(target, "_" + target.toLowerCase());
            }
            return str;
        }

        /**
         * 下划线转驼峰
         *
         * @param str 目标字符串
         * @return: java.lang.String
         */
        public static String underlineToHump(String str) {
            String regex = "_(.)";
            Matcher matcher = Pattern.compile(regex).matcher(str);
            while (matcher.find()) {
                String target = matcher.group(1);
                str = str.replaceAll("_" + target, target.toUpperCase());
            }
            return str;
        }

    }
}
