package com.tofly.mqtt;

import javax.crypto.Cipher;
import java.security.*;
import java.security.interfaces.ECPrivateKey;
import java.security.interfaces.ECPublicKey;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.X509EncodedKeySpec;

/**
 * ECC算法工具类
 */
public class ECCUtil {

    static {
        Security.addProvider(new org.bouncycastle.jce.provider.BouncyCastleProvider());
    }

    /**
     *
     * 生成秘钥对
     */
    public static KeyPair getKeyPair() throws Exception {
        KeyPairGenerator keyPairGenerator = KeyPairGenerator.getInstance("EC", "BC");
        keyPairGenerator.initialize(256, new SecureRandom());
        return keyPairGenerator.generateKeyPair();
    }

    /**
     *
     * 取得私钥
     */
    public static String getPrivateKey(KeyPair keyPair) {
        ECPrivateKey privateKey = (ECPrivateKey) keyPair.getPrivate();
        return Base64Util.encryptBASE64(privateKey.getEncoded());
    }

    /**
     *
     * 取得公钥
     */
    public static String getPublicKey(KeyPair keyPair) {
        ECPublicKey publicKey = (ECPublicKey) keyPair.getPublic();
        return Base64Util.encryptBASE64(publicKey.getEncoded());
    }

    /**
     *
     * 将Base64编码后的公钥转换成PublicKey对象
     */
    public static ECPublicKey string2PublicKey(String pubStr) throws Exception {
        byte[] keyBytes = Base64Util.decryptBASE64(pubStr);
        X509EncodedKeySpec keySpec = new X509EncodedKeySpec(keyBytes);
        KeyFactory keyFactory = KeyFactory.getInstance("EC", "BC");
        return (ECPublicKey) keyFactory.generatePublic(keySpec);
    }

    /**
     *
     * 将Base64编码后的私钥转换成PrivateKey对象
     */
    public static ECPrivateKey string2PrivateKey(String priStr) throws Exception {
        byte[] keyBytes = Base64Util.decryptBASE64(priStr);
        PKCS8EncodedKeySpec keySpec = new PKCS8EncodedKeySpec(keyBytes);
        KeyFactory keyFactory = KeyFactory.getInstance("EC", "BC");
        return (ECPrivateKey) keyFactory.generatePrivate(keySpec);
    }

    /**
     *
     * 公钥加密为byte字节
     */
    public static byte[] publicEncrypt(byte[] content, PublicKey publicKey) throws Exception {
        Cipher cipher = Cipher.getInstance("ECIES", "BC");
        cipher.init(Cipher.ENCRYPT_MODE, publicKey);
        return cipher.doFinal(content);
    }

    /**
     *
     * 公钥加密转为字符串
     */
    public static String publicEncrypt(String content, PublicKey publicKey) throws Exception {
        byte[] contentBytes = Base64Util.stringToByteArray(content);
        return Base64Util.encryptBASE64(publicEncrypt(contentBytes, publicKey));
    }

    /**
     *
     * 私钥解密为byte字节
     */
    public static byte[] privateDecrypt(byte[] content, PrivateKey privateKey) throws Exception {
        Cipher cipher = Cipher.getInstance("ECIES", "BC");
        cipher.init(Cipher.DECRYPT_MODE, privateKey);
        return cipher.doFinal(content);
    }

    /**
     *
     * 私钥解密为字符串
     */
    public static String privateDecrypt(String content, PrivateKey privateKey) throws Exception {
        byte[] contentBytes = Base64Util.decryptBASE64(content);
        return new String(privateDecrypt(contentBytes, privateKey));
    }

    /**
     *
     * base64编码公钥加密为字符串
     */
    public static String publicEncrypt(String content, String publicKeyBase64Str) throws Exception {
        PublicKey publicKey = string2PublicKey(publicKeyBase64Str);
        byte[] contentBytes = Base64Util.stringToByteArray(content);
        return Base64Util.encryptBASE64(publicEncrypt(contentBytes, publicKey));
    }

    /**
     *
     * base64编码私钥解密为字符串
     */
    public static String privateDecrypt(String content, String privateKeyBase64Str) throws Exception {
        PrivateKey privateKey=string2PrivateKey(privateKeyBase64Str);
        byte[] contentBytes = Base64Util.decryptBASE64(content);
        return new String(privateDecrypt(contentBytes, privateKey));
    }

}