package com.tofly.scada.config;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.rabbitmq.client.Channel;
import com.tofly.scada.common.FlowConstant;
import com.tofly.scada.common.JsonItemConst;
import com.tofly.scada.entity.*;
import com.tofly.scada.service.*;
import com.tofly.scada.service.impl.ScadaServiceImpl;
import com.tofly.scada.util.StrUtils;
import groovy.transform.Synchronized;
import lombok.extern.slf4j.Slf4j;
import org.checkerframework.checker.units.qual.A;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.LinkedBlockingDeque;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 物联网监听数据配置
 */
@Component
@Slf4j
public class ReceiveHandler {

    public static final String YYYY_MM_DD_HH_MM_SS = "yyyy-MM-dd HH:mm:ss";

    @Autowired
    private ScadaDeviceDataService deviceDataService;

    @Autowired
    private ScadaService scadaService;

    @Autowired
    private ScadaHistoryService scadaHistoryService;

    @Autowired
    private DeviceArchiveManageService deviceArchiveManageService;

    @Autowired
    private AllocationService allocationService;

    @Autowired
    private DictConfig dictConfig;

    //监听scada队列
    @RabbitListener(queues = {RabbitmqConfig.QUEUE_INFORM_SCADA}, concurrency =  "4")
    public void receive_scada(Message message) {
        try {
            String encode = Base64.getEncoder().encodeToString(message.getBody());
            byte[] decode = Base64.getDecoder().decode(encode);
            String s = new String(decode);

            ScadaDeviceData scadaDeviceData = JSON.parseObject(StrUtils.underlineToHump(s), ScadaDeviceData.class);
            if (!ObjectUtils.isEmpty(scadaDeviceData)) {
                scadaDeviceData.setJsonData(s);
                deviceDataService.save(scadaDeviceData);
    //            //保存设备指标
    //            saveScadaData(scadaDeviceData);
    //            //保存数据到历史表
    //            saveScadaHistoryData(scadaDeviceData);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 保存scada数据
     *
     * @param scadaDeviceData
     */
    @Transactional(rollbackFor = Exception.class)
    public void saveScadaData(ScadaDeviceData scadaDeviceData) {
        //保存设备
        if (StringUtils.hasText(scadaDeviceData.getDeviceName())) {
            saveDevices(scadaDeviceData);
        }
    }

    void saveScadaHistoryData(ScadaDeviceData scadaDeviceData) {
        //获取时间
        Date generationTime = StringUtils.hasText(scadaDeviceData.getGenerationTime()) ?
                DateUtil.parse(scadaDeviceData.getGenerationTime(), YYYY_MM_DD_HH_MM_SS) : new Date();
        List<ScadaHistory> scadaHistories = new ArrayList<>();
        //保存指标：压力
        if (StringUtils.hasText(scadaDeviceData.getPressure())) {
            ScadaHistory history = new ScadaHistory();
            history.setCode(scadaDeviceData.getDeviceName() + "#" + JsonItemConst.g_pressure);
            history.setValue(matcherVal(scadaDeviceData.getPressure()));
            history.setScadaTime(generationTime);
            history.setUpdateTime(new Date());
            scadaHistories.add(history);
        }
        //保存指标：正向累计流量
        if (StringUtils.hasText(scadaDeviceData.getCumPosFlows())) {
            ScadaHistory history = new ScadaHistory();
            history.setCode(scadaDeviceData.getDeviceName() + "#" + JsonItemConst.g_cum_pos_flows);
            history.setValue(matcherVal(scadaDeviceData.getCumPosFlows()));
            history.setScadaTime(generationTime);
            history.setUpdateTime(new Date());
            scadaHistories.add(history);
        }
        //保存指标：逆向累计流量
        if (StringUtils.hasText(scadaDeviceData.getCumNegFlows())) {
            ScadaHistory history = new ScadaHistory();
            history.setCode(scadaDeviceData.getDeviceName() + "#" + JsonItemConst.g_cum_neg_flows);
            history.setValue(matcherVal(scadaDeviceData.getCumNegFlows()));
            history.setScadaTime(generationTime);
            history.setUpdateTime(new Date());
            scadaHistories.add(history);
        }
        //保存指标：瞬时流量
        if (StringUtils.hasText(scadaDeviceData.getInstantaneousFlow())) {
            ScadaHistory history = new ScadaHistory();
            history.setCode(scadaDeviceData.getDeviceName() + "#" + JsonItemConst.g_instantaneous_flow);
            history.setValue(matcherVal(scadaDeviceData.getInstantaneousFlow()));
            history.setScadaTime(generationTime);
            history.setUpdateTime(new Date());
            scadaHistories.add(history);
        }
        //保存指标：信号质量
        if (StringUtils.hasText(scadaDeviceData.getSignalQuality())) {
            ScadaHistory history = new ScadaHistory();
            history.setCode(scadaDeviceData.getDeviceName() + "#" + JsonItemConst.g_signal_quality);
            history.setValue(matcherVal(scadaDeviceData.getSignalQuality()));
            history.setScadaTime(generationTime);
            history.setUpdateTime(new Date());
            scadaHistories.add(history);
        }
        //保存指标：电池电压
        if (StringUtils.hasText(scadaDeviceData.getBatteryVoltage())) {
            ScadaHistory history = new ScadaHistory();
            history.setCode(scadaDeviceData.getDeviceName() + "#" + JsonItemConst.g_battery_voltage);
            history.setValue(matcherVal(scadaDeviceData.getBatteryVoltage()));
            history.setScadaTime(generationTime);
            history.setUpdateTime(new Date());
            scadaHistories.add(history);
        }
        if (!CollectionUtils.isEmpty(scadaHistories)){
            //                if (CollUtil.isNotEmpty(scadaHistoryService.list(new QueryWrapper<ScadaHistory>()
            //                        .eq("code", scadaHistory.getCode())
            //                        .eq("scada_time", scadaHistory.getScadaTime())))) {
            //                    continue;
            //                }
            List<ScadaHistory> scadaHistoryList = new ArrayList<>(new HashSet<>(scadaHistories));
            scadaHistoryService.saveBatch(scadaHistoryList);
        }
    }

    /**
     * 保存设备指标数据
     *
     * @param scadaDeviceData
     */
    private void saveDevices(ScadaDeviceData scadaDeviceData) {
        List<DeviceArchiveManage> manages = deviceArchiveManageService.list(Wrappers.<DeviceArchiveManage>lambdaQuery()
                .eq(DeviceArchiveManage::getCode, scadaDeviceData.getDeviceName()));
        if (CollectionUtils.isEmpty(manages)) {
            DeviceArchiveManage deviceArchiveManage = new DeviceArchiveManage();
            deviceArchiveManage.setCode(scadaDeviceData.getDeviceName());
            deviceArchiveManage.setName(scadaDeviceData.getDeviceType() + "#" + scadaDeviceData.getDeviceType());
            deviceArchiveManage.setDescribe(scadaDeviceData.getDeviceType() + "#" + scadaDeviceData.getDeviceType());
            boolean b = deviceArchiveManageService.save(deviceArchiveManage);
            //保存指标
            if (b) {
                saveAllocations(scadaDeviceData, deviceArchiveManage);
            }
        } else {
            //保存指标
            saveAllocations(scadaDeviceData, manages.get(0));
        }
    }

    private void saveAllocations(ScadaDeviceData scadaDeviceData, DeviceArchiveManage deviceArchiveManage) {
        List<Allocation> allocations = new ArrayList<>();
        //保存指标：压力
        int pressure = allocationService.count(Wrappers.<Allocation>lambdaQuery()
                .eq(Allocation::getVariableCode, scadaDeviceData.getDeviceName() + "#" + JsonItemConst.g_pressure)
                .eq(Allocation::getDeviceId, deviceArchiveManage.getId()));
        if (pressure == 0) {
            Allocation allocation = new Allocation();
            allocation.setDisplayName("压力");
            allocation.setVariableCode(scadaDeviceData.getDeviceName() + "#" + JsonItemConst.g_pressure);
            allocation.setVariableName(scadaDeviceData.getDeviceName() + "#" + JsonItemConst.g_pressure);
            allocation.setDeviceId(deviceArchiveManage.getId());
            allocation.setUnit("pa");
            allocation.setModel("4");
            allocation.setIsAlarm("0");
            allocations.add(allocation);
        }
        //保存指标：正向累计流
        int cumPosFlows = allocationService.count(Wrappers.<Allocation>lambdaQuery()
                .eq(Allocation::getVariableCode, scadaDeviceData.getDeviceName() + "#" + JsonItemConst.g_cum_pos_flows)
                .eq(Allocation::getDeviceId, deviceArchiveManage.getId()));
        if (cumPosFlows == 0) {
            Allocation allocation = new Allocation();
            allocation.setDisplayName("正向累计流");
            allocation.setVariableCode(scadaDeviceData.getDeviceName() + "#" + JsonItemConst.g_cum_pos_flows);
            allocation.setVariableName(scadaDeviceData.getDeviceName() + "#" + JsonItemConst.g_cum_pos_flows);
            allocation.setDeviceId(deviceArchiveManage.getId());
            allocation.setUnit("m³");
            allocation.setModel("4");
            allocation.setIsAlarm("0");
            allocation.setType(dictConfig.getNightFlowAllIndex());
            allocations.add(allocation);
        }
        //保存指标：逆向累计流量
        int cumNegFlows = allocationService.count(Wrappers.<Allocation>lambdaQuery()
                .eq(Allocation::getVariableCode, scadaDeviceData.getDeviceName() + "#" + JsonItemConst.g_cum_neg_flows)
                .eq(Allocation::getDeviceId, deviceArchiveManage.getId()));
        if (cumNegFlows == 0) {
            Allocation allocation = new Allocation();
            allocation.setDisplayName("逆向累计流量");
            allocation.setVariableCode(scadaDeviceData.getDeviceName() + "#" + JsonItemConst.g_cum_neg_flows);
            allocation.setVariableName(scadaDeviceData.getDeviceName() + "#" + JsonItemConst.g_cum_neg_flows);
            allocation.setDeviceId(deviceArchiveManage.getId());
            allocation.setUnit("m³");
            allocation.setModel("4");
            allocation.setIsAlarm("0");
            allocation.setType(dictConfig.getNightFlowAllIndex());
            allocations.add(allocation);
        }
        //保存指标：瞬时流量
        int instantaneousFlow = allocationService.count(Wrappers.<Allocation>lambdaQuery()
                .eq(Allocation::getVariableCode, scadaDeviceData.getDeviceName() + "#" + JsonItemConst.g_instantaneous_flow)
                .eq(Allocation::getDeviceId, deviceArchiveManage.getId()));
        if (instantaneousFlow == 0) {
            Allocation allocation = new Allocation();
            allocation.setDisplayName("瞬时流量");
            allocation.setVariableCode(scadaDeviceData.getDeviceName() + "#" + JsonItemConst.g_instantaneous_flow);
            allocation.setVariableName(scadaDeviceData.getDeviceName() + "#" + JsonItemConst.g_instantaneous_flow);
            allocation.setDeviceId(deviceArchiveManage.getId());
            allocation.setUnit("m³");
            allocation.setModel("4");
            allocation.setIsAlarm("0");
            allocation.setType(dictConfig.getBriefFlowAllIndex());
            allocations.add(allocation);
        }
        //保存指标：信号质量
        int signalQuality = allocationService.count(Wrappers.<Allocation>lambdaQuery()
                .eq(Allocation::getVariableCode, scadaDeviceData.getDeviceName() + "#" + JsonItemConst.g_signal_quality)
                .eq(Allocation::getDeviceId, deviceArchiveManage.getId()));
        if (signalQuality == 0) {
            Allocation allocation = new Allocation();
            allocation.setDisplayName("信号质量");
            allocation.setVariableCode(scadaDeviceData.getDeviceName() + "#" + JsonItemConst.g_signal_quality);
            allocation.setVariableName(scadaDeviceData.getDeviceName() + "#" + JsonItemConst.g_signal_quality);
            allocation.setDeviceId(deviceArchiveManage.getId());
            allocation.setUnit("db");
            allocation.setModel("4");
            allocation.setIsAlarm("0");
            allocation.setType(dictConfig.getSignalQuality());
            allocations.add(allocation);
        }
        //保存指标：电池电压
        int batteryVoltage = allocationService.count(Wrappers.<Allocation>lambdaQuery()
                .eq(Allocation::getVariableCode, scadaDeviceData.getDeviceName() + "#" + JsonItemConst.g_battery_voltage)
                .eq(Allocation::getDeviceId, deviceArchiveManage.getId()));
        if (batteryVoltage == 0) {
            Allocation allocation = new Allocation();
            allocation.setDisplayName("电池电压");
            allocation.setVariableCode(scadaDeviceData.getDeviceName() + "#" + JsonItemConst.g_battery_voltage);
            allocation.setVariableName(scadaDeviceData.getDeviceName() + "#" + JsonItemConst.g_battery_voltage);
            allocation.setDeviceId(deviceArchiveManage.getId());
            allocation.setUnit("v");
            allocation.setModel("4");
            allocation.setIsAlarm("0");
            allocation.setType(dictConfig.getVoltageFlowAllIndex());
            allocations.add(allocation);
        }
        if (!CollectionUtils.isEmpty(allocations)){
            for (Allocation allocation : allocations) {
                allocationService.save(allocation);
            }
        }
    }


    /**
     * 正则匹配数据值
     *
     * @param val
     * @return
     */
    private String matcherVal(String val) {
        String regex = "[0-9].*";
        if (!val.matches(regex)) {
            return "0" + val;
        }
        return val;
    }

    //监听sms队列
    @RabbitListener(queues = {RabbitmqConfig.QUEUE_INFORM_SMS})
    public void receive_sms(Object msg, Message message, Channel channel) {
    }
}
