package com.tofly.scada.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.scada.entity.ScadaAlarm;
import com.tofly.scada.entity.dto.ScadaAlarmDto;
import com.tofly.scada.service.ScadaAlarmService;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import com.tofly.common.core.entity.ResultRespone;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import com.tofly.common.log.annotation.ToFlyAppLog;

import javax.annotation.Resource;
import java.io.Serializable;
import java.util.List;

/**
 * 报警scada列表(ScadaAlarm)表控制层
 *
 * @author HaiQiu
 * @since 2023-03-29 17:49:29
 */
@Api(tags = "报警scada列表")
@RestController
@RequestMapping("scadaAlarm")
public class ScadaAlarmController {
    /**
     * 服务对象
     */
    @Resource
    private ScadaAlarmService scadaAlarmService;

    /**
     * 分页查询数据
     *
     * @param page       分页对象
     * @param scadaAlarm 查询实体
     * @return 所有数据
     */
    @ToFlyAppLog(title = "报警scada列表-分页查询数据")
    @ApiOperation(value = "分页查询数据")
    @GetMapping("/page")
    public ResultRespone<Page<ScadaAlarm>> pageList(Page page, @ModelAttribute ScadaAlarm scadaAlarm) {
        return ResultRespone.success(scadaAlarmService.pageList(page, scadaAlarm), "查询成功");
    }

    /**
     * 查询所有数据
     *
     * @param scadaAlarm 查询实体
     * @return 所有数据
     */
    @ToFlyAppLog(title = "报警scada列表-查询所有数据")
    @ApiOperation(value = "查询所有数据")
    @GetMapping("/list")
    public ResultRespone<List<ScadaAlarm>> allList(@ModelAttribute ScadaAlarm scadaAlarm) {
        return ResultRespone.success(scadaAlarmService.allList(scadaAlarm), "查询成功");
    }

    /**
     * 通过主键查询单条数据
     *
     * @param id 主键
     * @return 单条数据
     */
    @ToFlyAppLog(title = "报警scada列表-通过主键查询单条数据")
    @ApiOperation(value = "通过主键查询单条数据")
    @ApiImplicitParam(name = "id", value = "数据ID")
    @GetMapping("{id}")
    public ResultRespone<ScadaAlarm> get(@PathVariable Serializable id) {
        return ResultRespone.success(scadaAlarmService.get(id), "查询成功");
    }

    /**
     * 新增数据
     *
     * @param scadaAlarmDto 实体
     * @return 新增结果
     */
    @ToFlyAppLog(title = "报警scada列表-新增数据")
    @ApiOperation(value = "新增数据")
    @PostMapping
    public ResultRespone insert(@Validated @RequestBody ScadaAlarmDto scadaAlarmDto) {
        scadaAlarmService.insert(scadaAlarmDto);
        return ResultRespone.success(null, "新增成功");
    }

    /**
     * 修改数据
     *
     * @param scadaAlarmDto 实体
     * @return 修改结果
     */
    @ToFlyAppLog(title = "报警scada列表-修改数据")
    @ApiOperation(value = "修改数据")
    @PutMapping
    public ResultRespone edit(@Validated @RequestBody ScadaAlarmDto scadaAlarmDto) {
        scadaAlarmService.edit(scadaAlarmDto);
        return ResultRespone.success(null, "修改成功");
    }

    /**
     * 删除数据
     *
     * @param idList 主键结合
     * @return 删除结果
     */
    @ToFlyAppLog(title = "报警scada列表-删除数据")
    @ApiOperation(value = "删除数据")
    @ApiImplicitParam(name = "idList", value = "数据ID数组集合")
    @DeleteMapping
    public ResultRespone delete(@RequestParam("idList") List<Long> idList) {
        scadaAlarmService.delete(idList);
        return ResultRespone.success(null, "删除成功");
    }

    /**
     * 数据导入Excel模板下载
     */
    @ToFlyAppLog(title = "报警scada列表-模版下载")
    @ApiOperation(value = "模版下载")
    @GetMapping("/template")
    public void template() {
        scadaAlarmService.template();
    }

    /**
     * Excel数据导入
     *
     * @param file 导入的Excel数据文件
     * @return 导入结果
     */
    @ToFlyAppLog(title = "报警scada列表-Excel数据导入")
    @ApiOperation(value = "数据导入")
    @ApiImplicitParam(name = "file", value = "Excel文件", required = true)
    @PostMapping("/import")
    public ResultRespone importData(@RequestParam(name = "file") MultipartFile file) {
        scadaAlarmService.importData(file);
        return ResultRespone.success(null, "导入成功");
    }


    /**
     * 数据导出
     *
     * @param page       分页
     * @param scadaAlarm 查询模型
     * @param whole      是否导出全部数据，true是，false否
     */
    @ToFlyAppLog(title = "报警scada列表-数据导出")
    @ApiOperation(value = "数据导出")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "whole", value = "是否导出全部数据，true是，false否（默认否）"),
    })
    @GetMapping("/export")
    public void export(Page page, ScadaAlarm scadaAlarm, boolean whole) {
        scadaAlarmService.export(page, scadaAlarm, whole);
    }
}

