package com.tofly.scada.controller;

import com.alibaba.fastjson.JSON;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.scada.config.RabbitmqConfig;
import com.tofly.scada.config.ReceiveHandler;
import com.tofly.scada.entity.ScadaDeviceData;
import com.tofly.scada.util.StrUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * @author HaiQiu
 * @since 2023/3/27
 */
@Api(tags = "消息队列测试")
@RestController
@RequestMapping("/rabbit")
public class TestMqController {

    @Autowired
    private RabbitTemplate rabbitTemplate;

    @Autowired
    private ReceiveHandler receiveHandler;

    @ApiOperation(value = "测试消息推送")
    @GetMapping("/get")
    public ResultRespone get(@ApiParam(value = "发送的消息测试文本") @RequestParam String msg){
        rabbitTemplate.convertAndSend(RabbitmqConfig.EXCHANGE_TOPICS_INFORM,"inform.scada",msg);
        return ResultRespone.success();
    }

    @ApiOperation(value = "模拟设备推送数据接口")
    @GetMapping("/push")
    public ResultRespone push(@ApiParam(value = "推送的数据json文本") @RequestParam String msg){
        String s1 = StrUtils.underlineToHump(msg);
        ScadaDeviceData scadaDeviceData = JSON.parseObject(s1, ScadaDeviceData.class);
        if (!ObjectUtils.isEmpty(scadaDeviceData)) {
            receiveHandler.saveScadaData(scadaDeviceData);
        }
        return ResultRespone.success(scadaDeviceData);
    }

}
