package com.tofly.scada.entity;

import com.baomidou.mybatisplus.annotation.*;
import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.tofly.building.entity.ScadaDeviceArchiveManage;
import com.tofly.mybatisplus.config.ToflySqlCondition;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;
import org.codehaus.jackson.map.annotate.JsonSerialize;
import org.codehaus.jackson.map.ser.std.ToStringSerializer;
import org.springframework.format.annotation.DateTimeFormat;

import java.io.Serializable;
import java.util.Date;

/**
 * 设备档案管理(DeviceArchiveManage)表实体类
 *
 * @author HaiQiu
 * @since 2023-03-02 09:22:03
 */
@Data
@Accessors(chain = true)
@NoArgsConstructor
@EqualsAndHashCode(callSuper = false)
@KeySequence("TF_DEVICE_ARCHIVE_MANAGE")
@TableName(value = "TF_DEVICE_ARCHIVE_MANAGE", resultMap = "BaseResultMap")
@ApiModel(value = "DeviceArchiveManage对象", description = "设备档案管理")
public class DeviceArchiveManage extends Model<ScadaDeviceArchiveManage> implements Serializable {

    private static final long serialVersionUID = -36333121798689184L;

    /**
     * 所属水厂ID
     */
    @ApiModelProperty(value = "所属水厂ID")
    private Long waterId;
    @ApiModelProperty(hidden = true)
    @TableField(exist = false)
    private String waterName;
    /**
     * 设备编号
     */
    @ApiModelProperty(value = "设备编号")
    private String code;
    /**
     * 设备名称
     */
    @ApiModelProperty(value = "设备名称")
    private String name;
    /**
     * 类型（1水厂,2管网）
     */
    @ApiModelProperty(value = "类型（1水厂,2管网）")
    private String type;
    @ApiModelProperty(value = "类型转义（1水厂,2管网）")
    @TableField(exist = false)
    private String typeName;
    /**
     * 制造日期
     */
    @ApiModelProperty(value = "制造日期")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date makeDate;
    /**
     * 设备类型 53400
     */
    @ApiModelProperty(value = "设备类型 53400")
    private String deviceType;
    @ApiModelProperty(value = "设备类型名称")
    @TableField(exist = false)
    private String deviceTypeName;
    /**
     * 制造厂家全称
     */
    @ApiModelProperty(value = "制造厂家全称")
    private String manufacturerDesc;
    /**
     * 制造厂家
     */
    @ApiModelProperty(value = "制造厂家")
    @TableField(condition = ToflySqlCondition.LIKE)
    private String manufacturer;
    /**
     * 型号规格
     */
    @ApiModelProperty(value = "型号规格")
    private String modelSpecification;
    /**
     * 状态（0故障,1运行中,2空闲,3检修中）
     */
    @ApiModelProperty(value = "状态（0故障,1运行中,2空闲,3检修中）")
    private String status;
    @ApiModelProperty(value = "状态转义（0故障,1运行中,2空闲,3检修中）")
    @TableField(exist = false)
    private String statusName;
    /**
     * 经度
     */
    @ApiModelProperty(value = "经度")
    private Double longitude;
    /**
     * 纬度
     */
    @ApiModelProperty(value = "纬度")
    private Double latitude;
    /**
     * 负责人
     */
    @ApiModelProperty(value = "负责人")
    private Long leader;
    @ApiModelProperty(value = "负责人名称")
    @TableField(exist = false)
    private String leaderName;
    /**
     * 联系电话
     */
    @ApiModelProperty(value = "联系电话")
    private String phone;
    /**
     * 描述
     */
    @ApiModelProperty(value = "描述")
    private String describe;

    @TableField(value = "CREATE_USER",insertStrategy = FieldStrategy.NOT_EMPTY)
    private Long createUser;
    @TableField(exist = false)
    @ApiModelProperty(value = "${column.comment}")
    private String createUserName;
    @ApiModelProperty(value = "$column.comments")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(value = "CREATE_TIME", fill = FieldFill.INSERT)
    private Date createTime;
    @TableField(exist = false)
    @ApiModelProperty(value = "多个水厂,管线查询 以,分隔")
    private String waterIds;

    @TableId(type = IdType.INPUT)
    private Long id;
    /**
     * 地址
     */
    @ApiModelProperty(value = "地址")
    private String address;

    @TableField(exist = false)
    @ApiModelProperty("设备监测状态")
    private String deviceMonitorStatus;
    @TableField(exist = false)
    @ApiModelProperty("设备监测状态名称")
    private String deviceMonitorStatusName;
    /**
     * 视点Json对象
     */
    @ApiModelProperty(value = "视点Json对象")
    private String viewJson;
    /**
     * 口径(mm),15,20,25,40,50,80,100,200
     */
    @ApiModelProperty(value = "口径(mm),15,20,25,40,50,80,100,200")
    private String bore;
    /**
     * 压力设备id(归属)
     */
    @ApiModelProperty(value = "压力设备id(归属)")
    private Long pressureDeviceId;
    /**
     * 属于区域,字典码表pcid54000
     */
    @ApiModelProperty(value = "属于区域,字典码表pcid54000")
    private String areaId;

    @ApiModelProperty(value = "区域名称")
    @TableField(exist = false)
    private String areaName;
    /**
     * 检测类型
     */
    @ApiModelProperty(value = "检测类型")
    private String detectionType;
    /**
     * 出厂编码
     */
    @ApiModelProperty(value = "出厂编码")
    private String factoryCode;
    /**
     * 巡检养护周期
     */
    @ApiModelProperty(value = "巡检养护周期")
    private String nursingTime;
    /**
     * 使用寿命
     */
    @ApiModelProperty(value = "使用寿命")
    private String lifeSpan;
    /**
     * 购买价格
     */
    @ApiModelProperty(value = "购买价格")
    private String purchasePrice;
    /**
     * 采购价格
     */
    @ApiModelProperty(value = "采购价格")
    private String procurePrice;
    /**
     * 安装时间
     */
    @ApiModelProperty(value = "安装时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date installTime;
    /**
     * 购买时间
     */
    @ApiModelProperty(value = "购买时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date purchaseTime;
    /**
     * 所属院区(0南校区，1北校区) or 字典获取
     */
    @ApiModelProperty(value = "所属院区(0南校区，1北校区) or 字典获取")
    private String belongToCollege;
    /**
     * 所属对象(0学院，1商户) or 字典获取
     */
    @ApiModelProperty(value = "所属对象(0学院，1商户) or 字典获取")
    private String belongToObject;
    /**
     * 功能用途
     */
    @ApiModelProperty(value = "功能用途")
    private String description;
    /**
     * 所在楼栋ID
     */
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @JsonSerialize(using = ToStringSerializer.class)
    @ApiModelProperty(value = "所在楼栋ID")
    private Long buildingCollegeId;

    @TableField(exist = false)
    @ApiModelProperty(value = "楼栋建筑名称")
    private String buildingCollegeName;

    /**
     * 父级设备ID
     */
    @ApiModelProperty(value = "父级设备ID")
    private Long parentDeviceId;

    /**
     * 所属线路ID
     */
    @ApiModelProperty(value = "所属线路ID")
    private Long belongLineId;

}