package com.tofly.scada.entity;


import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.baomidou.mybatisplus.annotation.*;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiModel;
import com.fasterxml.jackson.annotation.JsonFormat;
import org.codehaus.jackson.map.annotate.JsonSerialize;
import org.codehaus.jackson.map.ser.std.ToStringSerializer;
import org.springframework.format.annotation.DateTimeFormat;
import lombok.Data;

import java.util.*;

import org.springframework.web.multipart.MultipartFile;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;

import java.io.Serializable;

import lombok.EqualsAndHashCode;
import com.haiqiu.tools.excel.annotation.Excel;

/**
 * 物联网接入数据原始表(ScadaDeviceData)表实体类
 *
 * @author HaiQiu
 * @since 2023-04-03 11:15:38
 */
@Data
@Accessors(chain = true)
@NoArgsConstructor
@EqualsAndHashCode(callSuper = false)
@TableName(value = "SCADA_DEVICE_DATA", resultMap = "ScadaDeviceDataMap")
@ApiModel(value = "ScadaDeviceData对象", description = "物联网接入数据原始表")
public class ScadaDeviceData extends Model<ScadaDeviceData> implements Serializable {

    private static final long serialVersionUID = 987420524080627719L;

    /**
     * ID
     */
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @JsonSerialize(using = ToStringSerializer.class)
    @Excel(name = "ID")
    @ApiModelProperty(value = "ID")
    @TableId(type = IdType.ASSIGN_ID)
    private Long id;
    /**
     * 设备唯一编号(必要)
     */
    @Excel(name = "设备唯一编号(必要)")
    @ApiModelProperty(value = "设备唯一编号(必要)")
    private String deviceId;
    /**
     * 数据接收时间(必要)
     */
    @Excel(name = "数据接收时间(必要)")
    @ApiModelProperty(value = "数据接收时间(必要)")
    private String collectTime;
    /**
     * 数据产生时间
     */
    @Excel(name = "数据产生时间")
    @ApiModelProperty(value = "数据产生时间")
    private String generationTime;
    /**
     * 甲烷浓度
     */
    @Excel(name = "甲烷浓度")
    @ApiModelProperty(value = "甲烷浓度")
    private String ch4Concentration;
    /**
     * 湿度
     */
    @Excel(name = "湿度")
    @ApiModelProperty(value = "湿度")
    private String humidity;
    /**
     * 压力
     */
    @Excel(name = "压力")
    @ApiModelProperty(value = "压力")
    private String pressure;
    /**
     * 信号强度
     */
    @Excel(name = "信号强度")
    @ApiModelProperty(value = "信号强度")
    private String signalStrength;
    /**
     * 环境温度/温度
     */
    @Excel(name = "环境温度/温度")
    @ApiModelProperty(value = "环境温度/温度")
    private String ambientTemperature;
    /**
     * 电池电压
     */
    @Excel(name = "电池电压")
    @ApiModelProperty(value = "电池电压")
    private String batteryVoltage;
    /**
     * 空高
     */
    @Excel(name = "空高")
    @ApiModelProperty(value = "空高")
    private String ullage;
    /**
     * 累计流量
     */
    @Excel(name = "累计流量")
    @ApiModelProperty(value = "累计流量")
    private String cumFlows;
    /**
     * 累计流量次数
     */
    @Excel(name = "累计流量次数")
    @ApiModelProperty(value = "累计流量次数")
    private String cumFlowsNum;
    /**
     * 横滚角
     */
    @Excel(name = "横滚角")
    @ApiModelProperty(value = "横滚角")
    private String rollAngle;
    /**
     * 俯仰角
     */
    @Excel(name = "俯仰角")
    @ApiModelProperty(value = "俯仰角")
    private String angleOfPitch;
    /**
     * 瞬时流量
     */
    @Excel(name = "瞬时流量")
    @ApiModelProperty(value = "瞬时流量")
    private String instantaneousFlow;
    /**
     * 水位
     */
    @Excel(name = "水位")
    @ApiModelProperty(value = "水位")
    private String waterLevels;
    /**
     * 频率
     */
    @Excel(name = "频率")
    @ApiModelProperty(value = "频率")
    private String frequency;
    /**
     * 表面流速
     */
    @Excel(name = "表面流速")
    @ApiModelProperty(value = "表面流速")
    private String surfaceVelocity;
    /**
     * 瞬时流速
     */
    @Excel(name = "瞬时流速")
    @ApiModelProperty(value = "瞬时流速")
    private String instantaneousVelocity;
    /**
     * 断面流速
     */
    @Excel(name = "断面流速")
    @ApiModelProperty(value = "断面流速")
    private String crossSectionVelocity;
    /**
     * 过水面积
     */
    @Excel(name = "过水面积")
    @ApiModelProperty(value = "过水面积")
    private String crossSectionArea;
    /**
     * 电池电量
     */
    @Excel(name = "电池电量")
    @ApiModelProperty(value = "电池电量")
    private String batteryLevel;
    /**
     * 电流
     */
    @Excel(name = "电流")
    @ApiModelProperty(value = "电流")
    private String current;
    /**
     * 电压
     */
    @Excel(name = "电压")
    @ApiModelProperty(value = "电压")
    private String voltage;
    /**
     * 功率
     */
    @Excel(name = "功率")
    @ApiModelProperty(value = "功率")
    private String power;
    /**
     * 浊度
     */
    @Excel(name = "浊度")
    @ApiModelProperty(value = "浊度")
    private String turbidity;
    /**
     * 液位
     */
    @Excel(name = "液位")
    @ApiModelProperty(value = "液位")
    private String liquidLevel;
    /**
     * 余氯
     */
    @Excel(name = "余氯")
    @ApiModelProperty(value = "余氯")
    private String residualChlorine;
    /**
     * PH值
     */
    @Excel(name = "PH值")
    @ApiModelProperty(value = "PH值")
    private String phValue;
    /**
     * 阀门开度
     */
    @Excel(name = "阀门开度")
    @ApiModelProperty(value = "阀门开度")
    private String openingOfValve;
    /**
     * 积水深度
     */
    @Excel(name = "积水深度")
    @ApiModelProperty(value = "积水深度")
    private String pondingDepth;
    /**
     * 水深
     */
    @Excel(name = "水深")
    @ApiModelProperty(value = "水深")
    private String depthOfWater;
    /**
     * 材质温度
     */
    @Excel(name = "材质温度")
    @ApiModelProperty(value = "材质温度")
    private String materialTemperature;
    /**
     * 冻结累计流量
     */
    @Excel(name = "冻结累计流量")
    @ApiModelProperty(value = "冻结累计流量")
    private String freCumFlows;
    /**
     * 冻结累计逆流量
     */
    @Excel(name = "冻结累计逆流量")
    @ApiModelProperty(value = "冻结累计逆流量")
    private String freCumNegFlows;
    /**
     * 冻结累计正流量
     */
    @Excel(name = "冻结累计正流量")
    @ApiModelProperty(value = "冻结累计正流量")
    private String freCumPosFlows;
    /**
     * 本周期累计流量
     */
    @Excel(name = "本周期累计流量")
    @ApiModelProperty(value = "本周期累计流量")
    private String cycCumFlows;
    /**
     * 本周期累计逆流量
     */
    @Excel(name = "本周期累计逆流量")
    @ApiModelProperty(value = "本周期累计逆流量")
    private String cycCumNegFlows;
    /**
     * 本周期累计正流量
     */
    @Excel(name = "本周期累计正流量")
    @ApiModelProperty(value = "本周期累计正流量")
    private String cycCumPosFlows;
    /**
     * CCID号
     */
    @Excel(name = "CCID号")
    @ApiModelProperty(value = "CCID号")
    private String ccid;
    /**
     * IMEI号
     */
    @Excel(name = "IMEI号")
    @ApiModelProperty(value = "IMEI号")
    private String imei;
    /**
     * 程序版本号
     */
    @Excel(name = "程序版本号")
    @ApiModelProperty(value = "程序版本号")
    private String programVersion;
    /**
     * 硬件版本号
     */
    @Excel(name = "硬件版本号")
    @ApiModelProperty(value = "硬件版本号")
    private String deviceVersion;
    /**
     * 逆向累计流量
     */
    @Excel(name = "逆向累计流量")
    @ApiModelProperty(value = "逆向累计流量")
    private String cumNegFlows;
    /**
     * 正向累计流
     */
    @Excel(name = "正向累计流")
    @ApiModelProperty(value = "正向累计流")
    private String cumPosFlows;
    /**
     * 逆向累计流量次数
     */
    @Excel(name = "逆向累计流量次数")
    @ApiModelProperty(value = "逆向累计流量次数")
    private String cumNegFlowsNum;
    /**
     * 正向累计流次数
     */
    @Excel(name = "正向累计流次数")
    @ApiModelProperty(value = "正向累计流次数")
    private String cumPosFlowsNum;
    /**
     * 信号质量
     */
    @Excel(name = "信号质量")
    @ApiModelProperty(value = "信号质量")
    private String signalQuality;
    /**
     * 信噪比
     */
    @Excel(name = "信噪比")
    @ApiModelProperty(value = "信噪比")
    private String signalNoiseRatio;
    /**
     * 覆盖等级
     */
    @Excel(name = "覆盖等级")
    @ApiModelProperty(value = "覆盖等级")
    private String coverageLevel;
    /**
     * 地址
     */
    @Excel(name = "地址")
    @ApiModelProperty(value = "地址")
    private String address;
    /**
     * 时钟
     */
    @Excel(name = "时钟")
    @ApiModelProperty(value = "时钟")
    private String clock;
    /**
     * 终端序列号
     */
    @Excel(name = "终端序列号")
    @ApiModelProperty(value = "终端序列号")
    private String terminalNumber;
    /**
     * 1分钟雨量，单位：毫米（MM）
     */
    @Excel(name = "1分钟雨量，单位：毫米（MM）")
    @ApiModelProperty(value = "1分钟雨量，单位：毫米（MM）")
    private String rainfallMinute01;
    /**
     * 5分钟雨量，单位：毫米（MM）
     */
    @Excel(name = "5分钟雨量，单位：毫米（MM）")
    @ApiModelProperty(value = "5分钟雨量，单位：毫米（MM）")
    private String rainfallMinute05;
    /**
     * 10分钟雨量，单位：毫米（MM）
     */
    @Excel(name = "10分钟雨量，单位：毫米（MM）")
    @ApiModelProperty(value = "10分钟雨量，单位：毫米（MM）")
    private String rainfallMinute10;
    /**
     * 30分钟雨量，单位：毫米（MM）
     */
    @Excel(name = "30分钟雨量，单位：毫米（MM）")
    @ApiModelProperty(value = "30分钟雨量，单位：毫米（MM）")
    private String rainfallMinute30;
    /**
     * 1小时雨量，单位：毫米（MM）
     */
    @Excel(name = "1小时雨量，单位：毫米（MM）")
    @ApiModelProperty(value = "1小时雨量，单位：毫米（MM）")
    private String rainfallHour01;
    /**
     * 2小时雨量，单位：毫米（MM）
     */
    @Excel(name = "2小时雨量，单位：毫米（MM）")
    @ApiModelProperty(value = "2小时雨量，单位：毫米（MM）")
    private String rainfallHour02;
    /**
     * 3小时雨量，单位：毫米（MM）
     */
    @Excel(name = "3小时雨量，单位：毫米（MM）")
    @ApiModelProperty(value = "3小时雨量，单位：毫米（MM）")
    private String rainfallHour03;
    /**
     * 6小时雨量，单位：毫米（MM）
     */
    @Excel(name = "6小时雨量，单位：毫米（MM）")
    @ApiModelProperty(value = "6小时雨量，单位：毫米（MM）")
    private String rainfallHour06;
    /**
     * 12小时雨量，单位：毫米（MM）
     */
    @Excel(name = "12小时雨量，单位：毫米（MM）")
    @ApiModelProperty(value = "12小时雨量，单位：毫米（MM）")
    private String rainfallHour12;
    /**
     * 溶解氧，单位：毫克每升（MG/L）
     */
    @Excel(name = "溶解氧，单位：毫克每升（MG/L）")
    @ApiModelProperty(value = "溶解氧，单位：毫克每升（MG/L）")
    private String dissolvedOxygen;
    /**
     * 水温，单位：摄氏度（℃）
     */
    @Excel(name = "水温，单位：摄氏度（℃）")
    @ApiModelProperty(value = "水温，单位：摄氏度（℃）")
    private String waterTemperature;
    /**
     * 电导率，单位：微西门/CM（ΜS/CM）
     */
    @Excel(name = "电导率，单位：微西门/CM（ΜS/CM）")
    @ApiModelProperty(value = "电导率，单位：微西门/CM（ΜS/CM）")
    private String electricConductivity;
    /**
     * 高锰酸盐指数，单位：毫克每升（MG/L）
     */
    @Excel(name = "高锰酸盐指数，单位：毫克每升（MG/L）")
    @ApiModelProperty(value = "高锰酸盐指数，单位：毫克每升（MG/L）")
    private String permanganateIndex;
    /**
     * 氨氮，单位：毫克每升（MG/L）
     */
    @Excel(name = "氨氮，单位：毫克每升（MG/L）")
    @ApiModelProperty(value = "氨氮，单位：毫克每升（MG/L）")
    private String ammoniaNitrogen;
    /**
     * 总磷，单位：毫克每升（MG/L）
     */
    @Excel(name = "总磷，单位：毫克每升（MG/L）")
    @ApiModelProperty(value = "总磷，单位：毫克每升（MG/L）")
    private String totalPhosphorus;
    /**
     * 总氮，单位：毫克每升（MG/L）
     */
    @Excel(name = "总氮，单位：毫克每升（MG/L）")
    @ApiModelProperty(value = "总氮，单位：毫克每升（MG/L）")
    private String totalNitrogen;
    /**
     * 叶绿素A，单位：微克每升(ΜG/L)
     */
    @Excel(name = "叶绿素A，单位：微克每升(ΜG/L)")
    @ApiModelProperty(value = "叶绿素A，单位：微克每升(ΜG/L)")
    private String chlorophyllA;
    /**
     * 闸门开度，单位：米（M）
     */
    @Excel(name = "闸门开度，单位：米（M）")
    @ApiModelProperty(value = "闸门开度，单位：米（M）")
    private String gatage;
    /**
     * 设备型号
     */
    @Excel(name = "设备型号")
    @ApiModelProperty(value = "设备型号")
    private String deviceModel;
    /**
     * 最高水位，单位：米（M）
     */
    @Excel(name = "最高水位，单位：米（M）")
    @ApiModelProperty(value = "最高水位，单位：米（M）")
    private String highestWaterLevels;
    /**
     * 最低水位，单位：米（M）
     */
    @Excel(name = "最低水位，单位：米（M）")
    @ApiModelProperty(value = "最低水位，单位：米（M）")
    private String lowestWaterLevels;
    /**
     * 最高水位对应的时间
     */
    @Excel(name = "最高水位对应的时间")
    @ApiModelProperty(value = "最高水位对应的时间")
    private String highestWaterLevelsTime;
    /**
     * 最低水位对应的时间
     */
    @Excel(name = "最低水位对应的时间")
    @ApiModelProperty(value = "最低水位对应的时间")
    private String lowestWaterLevelsTime;
    /**
     * 每小时降雨量，单位：毫米（MM）
     */
    @Excel(name = "每小时降雨量，单位：毫米（MM）")
    @ApiModelProperty(value = "每小时降雨量，单位：毫米（MM）")
    private String hourlyRainfall;
    /**
     * 暴雨加报降雨量，单位：毫米（MM）
     */
    @Excel(name = "暴雨加报降雨量，单位：毫米（MM）")
    @ApiModelProperty(value = "暴雨加报降雨量，单位：毫米（MM）")
    private String rainstormRainfall;
    /**
     * 累计雨量，单位：毫米（MM）
     */
    @Excel(name = "累计雨量，单位：毫米（MM）")
    @ApiModelProperty(value = "累计雨量，单位：毫米（MM）")
    private String cumRainfall;
    /**
     * 创建时间
     */
    @Excel(name = "创建时间")
    @TableField(fill = FieldFill.INSERT)
    @ApiModelProperty(value = "创建时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date createTime;
    /**
     * 更新时间
     */
    @Excel(name = "更新时间")
    @TableField(fill = FieldFill.INSERT_UPDATE)
    @ApiModelProperty(value = "更新时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date updateTime;
    /**
     * 设备唯一编号:新加
     */
    @Excel(name = "设备唯一编号:新加")
    @ApiModelProperty(value = "设备唯一编号:新加")
    private String deviceIdNew;
    /**
     * 设备唯一配置编号:新加
     */
    @Excel(name = "设备唯一配置编号:新加")
    @ApiModelProperty(value = "设备唯一配置编号:新加")
    private String deviceProfileId;
    /**
     * 设备名称:新加
     */
    @Excel(name = "设备名称:新加")
    @ApiModelProperty(value = "设备名称:新加")
    private String deviceName;
    /**
     * 设备类型:新加
     */
    @Excel(name = "设备类型:新加")
    @ApiModelProperty(value = "设备类型:新加")
    private String deviceType;
    /**
     * 源数据json
     */
    @Excel(name = "源数据json")
    @ApiModelProperty(value = "源数据json")
    private String jsonData;
}

