package com.tofly.scada.entity.dto;


import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.baomidou.mybatisplus.annotation.*;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiModel;
import com.fasterxml.jackson.annotation.JsonFormat;
import org.codehaus.jackson.map.annotate.JsonSerialize;
import org.codehaus.jackson.map.ser.std.ToStringSerializer;
import org.springframework.format.annotation.DateTimeFormat;
import lombok.Data;

import java.util.*;

import org.springframework.web.multipart.MultipartFile;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;

import java.io.Serializable;

import lombok.EqualsAndHashCode;

import javax.validation.constraints.*;

/**
 * 物联网接入数据原始表(ScadaDeviceData)表实体类
 *
 * @author HaiQiu
 * @since 2023-04-03 11:15:39
 */
@Data
@Accessors(chain = true)
@NoArgsConstructor
@EqualsAndHashCode(callSuper = false)
@ApiModel(value = "ScadaDeviceData对象", description = "物联网接入数据原始表")
public class ScadaDeviceDataDto implements Serializable {

    private static final long serialVersionUID = 833664071045173492L;

    /**
     * ID
     */
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @JsonSerialize(using = ToStringSerializer.class)
    @Digits(integer = 20, fraction = 0, message = "ID长度不能超过20")
    @ApiModelProperty(value = "ID")
    @TableId(type = IdType.ASSIGN_ID)
    private Long id;

    /**
     * 设备唯一编号(必要)
     */
    @Size(max = 85, message = "设备唯一编号(必要)长度不能超过85")
    @ApiModelProperty(value = "设备唯一编号(必要)")
    private String deviceId;

    /**
     * 数据接收时间(必要)
     */
    @Size(max = 85, message = "数据接收时间(必要)长度不能超过85")
    @ApiModelProperty(value = "数据接收时间(必要)")
    private String collectTime;

    /**
     * 数据产生时间
     */
    @Size(max = 85, message = "数据产生时间长度不能超过85")
    @ApiModelProperty(value = "数据产生时间")
    private String generationTime;

    /**
     * 甲烷浓度
     */
    @Size(max = 85, message = "甲烷浓度长度不能超过85")
    @ApiModelProperty(value = "甲烷浓度")
    private String ch4Concentration;

    /**
     * 湿度
     */
    @Size(max = 85, message = "湿度长度不能超过85")
    @ApiModelProperty(value = "湿度")
    private String humidity;

    /**
     * 压力
     */
    @Size(max = 85, message = "压力长度不能超过85")
    @ApiModelProperty(value = "压力")
    private String pressure;

    /**
     * 信号强度
     */
    @Size(max = 85, message = "信号强度长度不能超过85")
    @ApiModelProperty(value = "信号强度")
    private String signalStrength;

    /**
     * 环境温度/温度
     */
    @Size(max = 85, message = "环境温度/温度长度不能超过85")
    @ApiModelProperty(value = "环境温度/温度")
    private String ambientTemperature;

    /**
     * 电池电压
     */
    @Size(max = 85, message = "电池电压长度不能超过85")
    @ApiModelProperty(value = "电池电压")
    private String batteryVoltage;

    /**
     * 空高
     */
    @Size(max = 85, message = "空高长度不能超过85")
    @ApiModelProperty(value = "空高")
    private String ullage;

    /**
     * 累计流量
     */
    @Size(max = 85, message = "累计流量长度不能超过85")
    @ApiModelProperty(value = "累计流量")
    private String cumFlows;

    /**
     * 累计流量次数
     */
    @Size(max = 85, message = "累计流量次数长度不能超过85")
    @ApiModelProperty(value = "累计流量次数")
    private String cumFlowsNum;

    /**
     * 横滚角
     */
    @Size(max = 85, message = "横滚角长度不能超过85")
    @ApiModelProperty(value = "横滚角")
    private String rollAngle;

    /**
     * 俯仰角
     */
    @Size(max = 85, message = "俯仰角长度不能超过85")
    @ApiModelProperty(value = "俯仰角")
    private String angleOfPitch;

    /**
     * 瞬时流量
     */
    @Size(max = 85, message = "瞬时流量长度不能超过85")
    @ApiModelProperty(value = "瞬时流量")
    private String instantaneousFlow;

    /**
     * 水位
     */
    @Size(max = 85, message = "水位长度不能超过85")
    @ApiModelProperty(value = "水位")
    private String waterLevels;

    /**
     * 频率
     */
    @Size(max = 85, message = "频率长度不能超过85")
    @ApiModelProperty(value = "频率")
    private String frequency;

    /**
     * 表面流速
     */
    @Size(max = 85, message = "表面流速长度不能超过85")
    @ApiModelProperty(value = "表面流速")
    private String surfaceVelocity;

    /**
     * 瞬时流速
     */
    @Size(max = 85, message = "瞬时流速长度不能超过85")
    @ApiModelProperty(value = "瞬时流速")
    private String instantaneousVelocity;

    /**
     * 断面流速
     */
    @Size(max = 85, message = "断面流速长度不能超过85")
    @ApiModelProperty(value = "断面流速")
    private String crossSectionVelocity;

    /**
     * 过水面积
     */
    @Size(max = 85, message = "过水面积长度不能超过85")
    @ApiModelProperty(value = "过水面积")
    private String crossSectionArea;

    /**
     * 电池电量
     */
    @Size(max = 85, message = "电池电量长度不能超过85")
    @ApiModelProperty(value = "电池电量")
    private String batteryLevel;

    /**
     * 电流
     */
    @Size(max = 85, message = "电流长度不能超过85")
    @ApiModelProperty(value = "电流")
    private String current;

    /**
     * 电压
     */
    @Size(max = 85, message = "电压长度不能超过85")
    @ApiModelProperty(value = "电压")
    private String voltage;

    /**
     * 功率
     */
    @Size(max = 85, message = "功率长度不能超过85")
    @ApiModelProperty(value = "功率")
    private String power;

    /**
     * 浊度
     */
    @Size(max = 85, message = "浊度长度不能超过85")
    @ApiModelProperty(value = "浊度")
    private String turbidity;

    /**
     * 液位
     */
    @Size(max = 85, message = "液位长度不能超过85")
    @ApiModelProperty(value = "液位")
    private String liquidLevel;

    /**
     * 余氯
     */
    @Size(max = 85, message = "余氯长度不能超过85")
    @ApiModelProperty(value = "余氯")
    private String residualChlorine;

    /**
     * PH值
     */
    @Size(max = 85, message = "PH值长度不能超过85")
    @ApiModelProperty(value = "PH值")
    private String phValue;

    /**
     * 阀门开度
     */
    @Size(max = 85, message = "阀门开度长度不能超过85")
    @ApiModelProperty(value = "阀门开度")
    private String openingOfValve;

    /**
     * 积水深度
     */
    @Size(max = 85, message = "积水深度长度不能超过85")
    @ApiModelProperty(value = "积水深度")
    private String pondingDepth;

    /**
     * 水深
     */
    @Size(max = 85, message = "水深长度不能超过85")
    @ApiModelProperty(value = "水深")
    private String depthOfWater;

    /**
     * 材质温度
     */
    @Size(max = 85, message = "材质温度长度不能超过85")
    @ApiModelProperty(value = "材质温度")
    private String materialTemperature;

    /**
     * 冻结累计流量
     */
    @Size(max = 85, message = "冻结累计流量长度不能超过85")
    @ApiModelProperty(value = "冻结累计流量")
    private String freCumFlows;

    /**
     * 冻结累计逆流量
     */
    @Size(max = 85, message = "冻结累计逆流量长度不能超过85")
    @ApiModelProperty(value = "冻结累计逆流量")
    private String freCumNegFlows;

    /**
     * 冻结累计正流量
     */
    @Size(max = 85, message = "冻结累计正流量长度不能超过85")
    @ApiModelProperty(value = "冻结累计正流量")
    private String freCumPosFlows;

    /**
     * 本周期累计流量
     */
    @Size(max = 85, message = "本周期累计流量长度不能超过85")
    @ApiModelProperty(value = "本周期累计流量")
    private String cycCumFlows;

    /**
     * 本周期累计逆流量
     */
    @Size(max = 85, message = "本周期累计逆流量长度不能超过85")
    @ApiModelProperty(value = "本周期累计逆流量")
    private String cycCumNegFlows;

    /**
     * 本周期累计正流量
     */
    @Size(max = 85, message = "本周期累计正流量长度不能超过85")
    @ApiModelProperty(value = "本周期累计正流量")
    private String cycCumPosFlows;

    /**
     * CCID号
     */
    @Size(max = 85, message = "CCID号长度不能超过85")
    @ApiModelProperty(value = "CCID号")
    private String ccid;

    /**
     * IMEI号
     */
    @Size(max = 85, message = "IMEI号长度不能超过85")
    @ApiModelProperty(value = "IMEI号")
    private String imei;

    /**
     * 程序版本号
     */
    @Size(max = 85, message = "程序版本号长度不能超过85")
    @ApiModelProperty(value = "程序版本号")
    private String programVersion;

    /**
     * 硬件版本号
     */
    @Size(max = 85, message = "硬件版本号长度不能超过85")
    @ApiModelProperty(value = "硬件版本号")
    private String deviceVersion;

    /**
     * 逆向累计流量
     */
    @Size(max = 85, message = "逆向累计流量长度不能超过85")
    @ApiModelProperty(value = "逆向累计流量")
    private String cumNegFlows;

    /**
     * 正向累计流
     */
    @Size(max = 85, message = "正向累计流长度不能超过85")
    @ApiModelProperty(value = "正向累计流")
    private String cumPosFlows;

    /**
     * 逆向累计流量次数
     */
    @Size(max = 85, message = "逆向累计流量次数长度不能超过85")
    @ApiModelProperty(value = "逆向累计流量次数")
    private String cumNegFlowsNum;

    /**
     * 正向累计流次数
     */
    @Size(max = 85, message = "正向累计流次数长度不能超过85")
    @ApiModelProperty(value = "正向累计流次数")
    private String cumPosFlowsNum;

    /**
     * 信号质量
     */
    @Size(max = 85, message = "信号质量长度不能超过85")
    @ApiModelProperty(value = "信号质量")
    private String signalQuality;

    /**
     * 信噪比
     */
    @Size(max = 85, message = "信噪比长度不能超过85")
    @ApiModelProperty(value = "信噪比")
    private String signalNoiseRatio;

    /**
     * 覆盖等级
     */
    @Size(max = 85, message = "覆盖等级长度不能超过85")
    @ApiModelProperty(value = "覆盖等级")
    private String coverageLevel;

    /**
     * 地址
     */
    @Size(max = 85, message = "地址长度不能超过85")
    @ApiModelProperty(value = "地址")
    private String address;

    /**
     * 时钟
     */
    @Size(max = 85, message = "时钟长度不能超过85")
    @ApiModelProperty(value = "时钟")
    private String clock;

    /**
     * 终端序列号
     */
    @Size(max = 85, message = "终端序列号长度不能超过85")
    @ApiModelProperty(value = "终端序列号")
    private String terminalNumber;

    /**
     * 1分钟雨量，单位：毫米（MM）
     */
    @Size(max = 85, message = "1分钟雨量，单位：毫米（MM）长度不能超过85")
    @ApiModelProperty(value = "1分钟雨量，单位：毫米（MM）")
    private String rainfallMinute01;

    /**
     * 5分钟雨量，单位：毫米（MM）
     */
    @Size(max = 85, message = "5分钟雨量，单位：毫米（MM）长度不能超过85")
    @ApiModelProperty(value = "5分钟雨量，单位：毫米（MM）")
    private String rainfallMinute05;

    /**
     * 10分钟雨量，单位：毫米（MM）
     */
    @Size(max = 85, message = "10分钟雨量，单位：毫米（MM）长度不能超过85")
    @ApiModelProperty(value = "10分钟雨量，单位：毫米（MM）")
    private String rainfallMinute10;

    /**
     * 30分钟雨量，单位：毫米（MM）
     */
    @Size(max = 85, message = "30分钟雨量，单位：毫米（MM）长度不能超过85")
    @ApiModelProperty(value = "30分钟雨量，单位：毫米（MM）")
    private String rainfallMinute30;

    /**
     * 1小时雨量，单位：毫米（MM）
     */
    @Size(max = 85, message = "1小时雨量，单位：毫米（MM）长度不能超过85")
    @ApiModelProperty(value = "1小时雨量，单位：毫米（MM）")
    private String rainfallHour01;

    /**
     * 2小时雨量，单位：毫米（MM）
     */
    @Size(max = 85, message = "2小时雨量，单位：毫米（MM）长度不能超过85")
    @ApiModelProperty(value = "2小时雨量，单位：毫米（MM）")
    private String rainfallHour02;

    /**
     * 3小时雨量，单位：毫米（MM）
     */
    @Size(max = 85, message = "3小时雨量，单位：毫米（MM）长度不能超过85")
    @ApiModelProperty(value = "3小时雨量，单位：毫米（MM）")
    private String rainfallHour03;

    /**
     * 6小时雨量，单位：毫米（MM）
     */
    @Size(max = 85, message = "6小时雨量，单位：毫米（MM）长度不能超过85")
    @ApiModelProperty(value = "6小时雨量，单位：毫米（MM）")
    private String rainfallHour06;

    /**
     * 12小时雨量，单位：毫米（MM）
     */
    @Size(max = 85, message = "12小时雨量，单位：毫米（MM）长度不能超过85")
    @ApiModelProperty(value = "12小时雨量，单位：毫米（MM）")
    private String rainfallHour12;

    /**
     * 溶解氧，单位：毫克每升（MG/L）
     */
    @Size(max = 85, message = "溶解氧，单位：毫克每升（MG/L）长度不能超过85")
    @ApiModelProperty(value = "溶解氧，单位：毫克每升（MG/L）")
    private String dissolvedOxygen;

    /**
     * 水温，单位：摄氏度（℃）
     */
    @Size(max = 85, message = "水温，单位：摄氏度（℃）长度不能超过85")
    @ApiModelProperty(value = "水温，单位：摄氏度（℃）")
    private String waterTemperature;

    /**
     * 电导率，单位：微西门/CM（ΜS/CM）
     */
    @Size(max = 85, message = "电导率，单位：微西门/CM（ΜS/CM）长度不能超过85")
    @ApiModelProperty(value = "电导率，单位：微西门/CM（ΜS/CM）")
    private String electricConductivity;

    /**
     * 高锰酸盐指数，单位：毫克每升（MG/L）
     */
    @Size(max = 85, message = "高锰酸盐指数，单位：毫克每升（MG/L）长度不能超过85")
    @ApiModelProperty(value = "高锰酸盐指数，单位：毫克每升（MG/L）")
    private String permanganateIndex;

    /**
     * 氨氮，单位：毫克每升（MG/L）
     */
    @Size(max = 85, message = "氨氮，单位：毫克每升（MG/L）长度不能超过85")
    @ApiModelProperty(value = "氨氮，单位：毫克每升（MG/L）")
    private String ammoniaNitrogen;

    /**
     * 总磷，单位：毫克每升（MG/L）
     */
    @Size(max = 85, message = "总磷，单位：毫克每升（MG/L）长度不能超过85")
    @ApiModelProperty(value = "总磷，单位：毫克每升（MG/L）")
    private String totalPhosphorus;

    /**
     * 总氮，单位：毫克每升（MG/L）
     */
    @Size(max = 85, message = "总氮，单位：毫克每升（MG/L）长度不能超过85")
    @ApiModelProperty(value = "总氮，单位：毫克每升（MG/L）")
    private String totalNitrogen;

    /**
     * 叶绿素A，单位：微克每升(ΜG/L)
     */
    @Size(max = 85, message = "叶绿素A，单位：微克每升(ΜG/L)长度不能超过85")
    @ApiModelProperty(value = "叶绿素A，单位：微克每升(ΜG/L)")
    private String chlorophyllA;

    /**
     * 闸门开度，单位：米（M）
     */
    @Size(max = 85, message = "闸门开度，单位：米（M）长度不能超过85")
    @ApiModelProperty(value = "闸门开度，单位：米（M）")
    private String gatage;

    /**
     * 设备型号
     */
    @Size(max = 85, message = "设备型号长度不能超过85")
    @ApiModelProperty(value = "设备型号")
    private String deviceModel;

    /**
     * 最高水位，单位：米（M）
     */
    @Size(max = 85, message = "最高水位，单位：米（M）长度不能超过85")
    @ApiModelProperty(value = "最高水位，单位：米（M）")
    private String highestWaterLevels;

    /**
     * 最低水位，单位：米（M）
     */
    @Size(max = 85, message = "最低水位，单位：米（M）长度不能超过85")
    @ApiModelProperty(value = "最低水位，单位：米（M）")
    private String lowestWaterLevels;

    /**
     * 最高水位对应的时间
     */
    @Size(max = 85, message = "最高水位对应的时间长度不能超过85")
    @ApiModelProperty(value = "最高水位对应的时间")
    private String highestWaterLevelsTime;

    /**
     * 最低水位对应的时间
     */
    @Size(max = 85, message = "最低水位对应的时间长度不能超过85")
    @ApiModelProperty(value = "最低水位对应的时间")
    private String lowestWaterLevelsTime;

    /**
     * 每小时降雨量，单位：毫米（MM）
     */
    @Size(max = 85, message = "每小时降雨量，单位：毫米（MM）长度不能超过85")
    @ApiModelProperty(value = "每小时降雨量，单位：毫米（MM）")
    private String hourlyRainfall;

    /**
     * 暴雨加报降雨量，单位：毫米（MM）
     */
    @Size(max = 85, message = "暴雨加报降雨量，单位：毫米（MM）长度不能超过85")
    @ApiModelProperty(value = "暴雨加报降雨量，单位：毫米（MM）")
    private String rainstormRainfall;

    /**
     * 累计雨量，单位：毫米（MM）
     */
    @Size(max = 85, message = "累计雨量，单位：毫米（MM）长度不能超过85")
    @ApiModelProperty(value = "累计雨量，单位：毫米（MM）")
    private String cumRainfall;

    /**
     * 创建时间
     */
    @ApiModelProperty(value = "创建时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date createTime;

    /**
     * 更新时间
     */
    @ApiModelProperty(value = "更新时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date updateTime;

    /**
     * 设备唯一编号:新加
     */
    @Size(max = 85, message = "设备唯一编号:新加长度不能超过85")
    @ApiModelProperty(value = "设备唯一编号:新加")
    private String deviceIdNew;

    /**
     * 设备唯一配置编号:新加
     */
    @Size(max = 85, message = "设备唯一配置编号:新加长度不能超过85")
    @ApiModelProperty(value = "设备唯一配置编号:新加")
    private String deviceProfileId;

    /**
     * 设备名称:新加
     */
    @Size(max = 85, message = "设备名称:新加长度不能超过85")
    @ApiModelProperty(value = "设备名称:新加")
    private String deviceName;

    /**
     * 设备类型:新加
     */
    @Size(max = 85, message = "设备类型:新加长度不能超过85")
    @ApiModelProperty(value = "设备类型:新加")
    private String deviceType;

    /**
     * 源数据json
     */
    @ApiModelProperty(value = "源数据json")
    private String jsonData;

}

