package com.tofly.scada.entity.dto;


import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.baomidou.mybatisplus.annotation.*;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiModel;
import com.fasterxml.jackson.annotation.JsonFormat;
import org.codehaus.jackson.map.annotate.JsonSerialize;
import org.codehaus.jackson.map.ser.std.ToStringSerializer;
import org.springframework.format.annotation.DateTimeFormat;
import lombok.Data;

import java.util.*;

import org.springframework.web.multipart.MultipartFile;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;

import java.io.Serializable;

import lombok.EqualsAndHashCode;

import javax.validation.constraints.*;

/**
 * 报警scada列表(ScadaReport)表实体类
 *
 * @author HaiQiu
 * @since 2023-03-29 17:41:01
 */
@Data
@Accessors(chain = true)
@NoArgsConstructor
@EqualsAndHashCode(callSuper = false)
@ApiModel(value = "ScadaReport对象", description = "报警scada列表")
public class ScadaReportDto implements Serializable {

    private static final long serialVersionUID = 827265149346691219L;

    /**
     * ID
     */
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @JsonSerialize(using = ToStringSerializer.class)
    @Digits(integer = 20, fraction = 0, message = "ID长度不能超过20")
    @ApiModelProperty(value = "ID")
    @TableId(type = IdType.ASSIGN_ID)
    private Long id;

    /**
     * 指标ID
     */
    @Size(max = 85, message = "指标ID长度不能超过85")
    @NotBlank(message = "指标ID不能为空")
    @ApiModelProperty(value = "指标ID")
    private String allocationId;

    /**
     * 报警等级
     */
    @Size(max = 85, message = "报警等级长度不能超过85")
    @ApiModelProperty(value = "报警等级")
    private String reportLevel;

    /**
     * 报警上限
     */
    @Size(max = 6, message = "报警上限长度不能超过6")
    @ApiModelProperty(value = "报警上限")
    private String reportUpper;

    /**
     * 报警下限
     */
    @Size(max = 6, message = "报警下限长度不能超过6")
    @ApiModelProperty(value = "报警下限")
    private String reportLower;

    /**
     * 报警联系人
     */
    @Digits(integer = 15, fraction = 0, message = "报警联系人长度不能超过15")
    @ApiModelProperty(value = "报警联系人")
    private Long liaisonPeople;

    /**
     * 联系人电话
     */
    @Size(max = 85, message = "联系人电话长度不能超过85")
    @ApiModelProperty(value = "联系人电话")
    private String liaisonPhone;

    /**
     * 负责人
     */
    @Digits(integer = 15, fraction = 0, message = "负责人长度不能超过15")
    @ApiModelProperty(value = "负责人")
    private Long leaderPeople;

    /**
     * 负责人电话
     */
    @Size(max = 33, message = "负责人电话长度不能超过33")
    @ApiModelProperty(value = "负责人电话")
    private String leaderPhone;

    /**
     * 详细描述
     */
    @Size(max = 166, message = "详细描述长度不能超过166")
    @ApiModelProperty(value = "详细描述")
    private String detail;

    /**
     * 消息通知模版
     */
    @Size(max = 666, message = "消息通知模版长度不能超过666")
    @ApiModelProperty(value = "消息通知模版")
    private String msgTemplate;

    /**
     * 排序号
     */
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @JsonSerialize(using = ToStringSerializer.class)
    @Digits(integer = 2147483646, fraction = 0, message = "排序号长度不能超过2147483646")
    @ApiModelProperty(value = "排序号")
    private Long sort;

    /**
     * 是否报警,0否，1是
     */
    @Digits(integer = 2, fraction = 0, message = "是否报警,0否，1是长度不能超过2")
    @ApiModelProperty(value = "是否报警,0否，1是")
    private Integer report;

    @Digits(integer = 15, fraction = 0, message = "${column.comment}长度不能超过15")
    private Long createUser;

    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date createTime;

    @Digits(integer = 15, fraction = 0, message = "${column.comment}长度不能超过15")
    private Long updateUser;

    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date updateTime;

    /**
     * 是否删除1删除；0未删除
     */
    @ApiModelProperty(value = "是否删除1删除；0未删除")
    private Boolean isDel;

}

