package com.tofly.scada.mapper;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.toolkit.Constants;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.scada.entity.Scada;
import com.tofly.scada.entity.ScadaHistory;
import com.tofly.scada.entity.dto.ScadaQuery;
import com.tofly.scada.entity.dto.WisdomDisplayDto;
import com.tofly.scada.entity.vo.ScadaVo;
import com.tofly.scada.entity.vo.StationIndexVo;
import com.tofly.scada.entity.vo.StatisticsScadaCalVo1;
import com.tofly.scada.entity.vo.StatisticsScadaVo;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

import java.util.Date;
import java.util.List;
import java.util.Set;

/**
 * @author HaiQiu
 * @date 2022/4/25
 */
@Mapper
public interface ScadaMapper extends BaseMapper<Scada> {

    /**
     * 临时接口，查询历史数据
     *
     * @return
     */
    @Select("select * " +
            "  from (select t1.*, " +
            "               row_number() over(partition by t1.CODE order by SCADA_TIME DESC) rn " +
            "          from SCADA_HISTORY t1) t1 " +
            " where t1.rn = 1")
    List<Scada> getListData();


    /**
     * 临时接口，查询历史数据,支持code
     *
     * @param code code
     * @return
     */
    @Select("select * " +
            "  from (select t1.*, " +
            "               row_number() over(partition by t1.CODE order by SCADA_TIME DESC) rn " +
            "          from SCADA_HISTORY t1 where CODE = #{cdoe} ) t1 " +
            " where t1.rn = 1")
    List<Scada> getListDataByCode(String code);


    /**
     * 创建表
     *
     * @param tableName 表名
     */
    void createTableMonths(String tableName);

    /**
     * 判断表是否存在
     *
     * @param tableName 表名
     * @return 条数大于1为存在
     */
    int verifyTableMonthsIsExits(String tableName);

    /**
     * 根据时间段查询历史scada数据
     *
     * @param tableName 表名
     * @param start     开始时间
     * @param end       结束时间
     * @return 历史数据列表
     */
    @Select("select  * from ${tableName} " +
            "where SCADA_TIME between to_date(#{start},'yyyy-mm-dd hh24:mi:ss') " +
            "and to_date(#{end},'yyyy-mm-dd hh24:mi:ss')")
    List<Scada> selectHistoryByTime(@Param("tableName") String tableName,
                                    @Param("start") String start, @Param("end") String end);


    /**
     * 根据ID查询历史scada数据
     *
     * @param tableName 表名
     * @param id        ID
     * @return 历史数据列表
     */
    @Select("select  * from ${tableName} " +
            "where ID = #{id}")
    List<Scada> selectHistoryById(@Param("tableName") String tableName,
                                  @Param("id") Long id);


    @Select("select  * from ${tableName}")
    List<Scada> selectMonitoring(@Param("tableName") String tableName);

    /**
     * 添加月份数据
     *
     * @param scadas    数据
     * @param tableName 表名
     * @return 影响条数
     */
    int insertMouth(@Param("scadas") List<Scada> scadas, @Param("tableName") String tableName);


    /**
     * 添加月份数据
     *
     * @param scada     数据
     * @param tableName 表名
     * @return 影响条数
     */
    int insertByOne(@Param("item") Scada scada, @Param("tableName") String tableName);

    /**
     * 根据表名和时间段删除表数据
     *
     * @param tableName 表名
     * @param start     开始时间
     * @param end       结束时间
     * @return 成功条数
     */
    int deleteFromTableName(@Param("tableName") String tableName,
                            @Param("start") String start, @Param("end") String end);


    /**
     * 查询SCADA数据
     *
     * @param queryWrapper 条件
     * @return
     */
    @Select("SELECT hours.* FROM ${tableName} hours " +
            "LEFT JOIN TF_DEVICE_ARCHIVE_MANAGE manage ON manage.CODE = hours.CODE " +
            "LEFT JOIN TF_YWPN_SCADASTAITEM_W staitem ON staitem.ITCD =  hours.CODE " +
            "${ew.customSqlSegment} ")
    List<ScadaVo> list(@Param(Constants.WRAPPER) QueryWrapper<ScadaQuery> queryWrapper,
                       @Param("tableName") String tableName);


    /**
     * 查询SCADA数据
     *
     * @param page         分页
     * @param queryWrapper 条件
     * @return
     */
    @Select("SELECT hours.*,manage.NAME name,manage.MAKE_DATE makeDate,manage.TYPE type," +
            "manage.MANUFACTURER manufacturer,manage.LEADER leader,manage.PHONE phone,staitem.STATION_ID stationId " +
            " FROM ${tableName} hours " +
            "LEFT JOIN TF_DEVICE_ARCHIVE_MANAGE manage ON manage.CODE = hours.CODE " +
            "LEFT JOIN TF_YWPN_SCADASTAITEM_W staitem ON staitem.ITCD =  hours.CODE " +
            "${ew.customSqlSegment} ")
    Page<ScadaVo> pageList(Page page, @Param(Constants.WRAPPER) QueryWrapper<ScadaQuery> queryWrapper,
                           @Param("tableName") String tableName);

    Scada selectDataByMonitoring(String code);

    List<Scada> selectDataByMonitorList();

    int insertMonitoring(@Param("scada") Scada scada);

    int updateMonitoring(@Param("scada") Scada scada);

    void insertBatchTempSet(@Param("tempList") Set<Scada> tempList);

    void insertBatchTemp(@Param("tempList") List<Scada> tempList);

    void updateRealBatchTemp(@Param("tempList") List<Scada> tempList);

    void updateRealTemp(@Param("temp") Scada temp);

    Page<StatisticsScadaVo> selectHistoryByMouthsTables(@Param("mouthTables") List<String> mouthTables,
                                                        Page page,
                                                        @Param(Constants.WRAPPER) QueryWrapper<ScadaQuery> queryWrapper);

    List<StatisticsScadaVo> selectHistoryByMouthsTablesNoPage(@Param("mouthTables") List<String> mouthTables,
                                                              @Param(Constants.WRAPPER) QueryWrapper<ScadaQuery> queryWrapper);


    /**
     * 根据时间段查询历史scada数据最大值最小值
     *
     * @param start 开始时间
     * @param end   结束时间
     * @return 历史数据列表
     */
    List<StatisticsScadaCalVo1> selectHistoryMaxAndMinByScadaTime1(@Param("codes") List<String> codes,
                                                                   @Param("start") Date start,
                                                                   @Param("end") Date end,
                                                                   @Param("dataType") Integer dataType,
                                                                   @Param("allIndexType") String allIndexType);

    List<Scada> selectHistoryData(@Param("start") String start, @Param("end") String end);

    void deleteHistoryData(@Param("start") String start, @Param("end") String end);

    List<StationIndexVo> viewStationIndex(@Param("wisdomDisplayDto") WisdomDisplayDto wisdomDisplayDto);
}
