package com.tofly.scada.service;


import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import com.tofly.scada.entity.ScadaAlarm;
import com.tofly.scada.entity.dto.ScadaAlarmDto;

import org.springframework.web.multipart.MultipartFile;

import java.io.Serializable;
import java.util.List;

/**
 * 报警scada列表(ScadaAlarm)表服务接口
 *
 * @author HaiQiu
 * @since 2023-03-29 17:49:29
 */
public interface ScadaAlarmService extends IService<ScadaAlarm> {

    /**
     * 根据ID查询
     *
     * @param id id
     * @return 数据
     */
    ScadaAlarm get(Serializable id);

    /**
     * 新增
     *
     * @param scadaAlarmDto 实体
     */
    void insert(ScadaAlarmDto scadaAlarmDto);

    /**
     * 修改
     *
     * @param scadaAlarmDto 实体
     */
    void edit(ScadaAlarmDto scadaAlarmDto);

    /**
     * 批量删除
     *
     * @param idList 数据ID集合
     */
    void delete(List<Long> idList);

    /**
     * 生成模板下载
     */
    void template();

    /**
     * 数据导入
     *
     * @param file 文件
     */
    void importData(MultipartFile file);

    /**
     * 分页查询
     *
     * @param page       分页
     * @param scadaAlarm 查询wrapper
     * @return 数据集合
     */
    Page<ScadaAlarm> pageList(Page page, ScadaAlarm scadaAlarm);

    /**
     * 数据导出
     *
     * @param page       分页
     * @param scadaAlarm 查询模型
     * @param whole      是否导出全部数据，true是，false否（默认否）
     */
    void export(Page page, ScadaAlarm scadaAlarm, boolean whole);

    /**
     * 查询所有数据
     *
     * @param scadaAlarm 查询条件
     * @return 查询所有数据
     */
    List<ScadaAlarm> allList(ScadaAlarm scadaAlarm);
}

