package com.tofly.scada.service;


import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import com.tofly.scada.entity.ScadaDeviceData;
import com.tofly.scada.entity.dto.ScadaDeviceDataDto;

import org.springframework.web.multipart.MultipartFile;

import java.io.Serializable;
import java.util.List;

/**
 * 物联网接入数据原始表(ScadaDeviceData)表服务接口
 *
 * @author HaiQiu
 * @since 2023-04-03 11:15:39
 */
public interface ScadaDeviceDataService extends IService<ScadaDeviceData> {

    /**
     * 根据ID查询
     *
     * @param id id
     * @return 数据
     */
    ScadaDeviceData get(Serializable id);

    /**
     * 新增
     *
     * @param scadaDeviceDataDto 实体
     */
    void insert(ScadaDeviceDataDto scadaDeviceDataDto);

    /**
     * 修改
     *
     * @param scadaDeviceDataDto 实体
     */
    void edit(ScadaDeviceDataDto scadaDeviceDataDto);

    /**
     * 批量删除
     *
     * @param idList 数据ID集合
     */
    void delete(List<Long> idList);

    /**
     * 生成模板下载
     */
    void template();

    /**
     * 数据导入
     *
     * @param file 文件
     */
    void importData(MultipartFile file);

    /**
     * 分页查询
     *
     * @param page            分页
     * @param scadaDeviceData 查询wrapper
     * @return 数据集合
     */
    Page<ScadaDeviceData> pageList(Page page, ScadaDeviceData scadaDeviceData);

    /**
     * 数据导出
     *
     * @param page            分页
     * @param scadaDeviceData 查询模型
     * @param whole           是否导出全部数据，true是，false否（默认否）
     */
    void export(Page page, ScadaDeviceData scadaDeviceData, boolean whole);

    /**
     * 查询所有数据
     *
     * @param scadaDeviceData 查询条件
     * @return 查询所有数据
     */
    List<ScadaDeviceData> allList(ScadaDeviceData scadaDeviceData);
}

