package com.tofly.scada.service;


import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import com.tofly.scada.entity.ScadaHistory;
import com.tofly.scada.entity.dto.ScadaHistoryDto;

import org.springframework.web.multipart.MultipartFile;

import java.io.Serializable;
import java.util.List;

/**
 * 存储历史值(ScadaHistory)表服务接口
 *
 * @author HaiQiu
 * @since 2023-04-03 11:25:00
 */
public interface ScadaHistoryService extends IService<ScadaHistory> {

    /**
     * 根据ID查询
     *
     * @param id id
     * @return 数据
     */
    ScadaHistory get(Serializable id);

    /**
     * 新增
     *
     * @param scadaHistoryDto 实体
     */
    void insert(ScadaHistoryDto scadaHistoryDto);

    /**
     * 修改
     *
     * @param scadaHistoryDto 实体
     */
    void edit(ScadaHistoryDto scadaHistoryDto);

    /**
     * 批量删除
     *
     * @param idList 数据ID集合
     */
    void delete(List<Long> idList);

    /**
     * 生成模板下载
     */
    void template();

    /**
     * 数据导入
     *
     * @param file 文件
     */
    void importData(MultipartFile file);

    /**
     * 分页查询
     *
     * @param page         分页
     * @param scadaHistory 查询wrapper
     * @return 数据集合
     */
    Page<ScadaHistory> pageList(Page page, ScadaHistory scadaHistory);

    /**
     * 数据导出
     *
     * @param page         分页
     * @param scadaHistory 查询模型
     * @param whole        是否导出全部数据，true是，false否（默认否）
     */
    void export(Page page, ScadaHistory scadaHistory, boolean whole);

    /**
     * 查询所有数据
     *
     * @param scadaHistory 查询条件
     * @return 查询所有数据
     */
    List<ScadaHistory> allList(ScadaHistory scadaHistory);
}

