package com.tofly.scada.service;


import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import com.tofly.scada.entity.ScadaNoAlarm;
import com.tofly.scada.entity.dto.ScadaNoAlarmDto;

import org.springframework.web.multipart.MultipartFile;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

/**
 * 不报警scada列表(ScadaNoAlarm)表服务接口
 *
 * @author HaiQiu
 * @since 2023-03-29 17:49:59
 */
public interface ScadaNoAlarmService extends IService<ScadaNoAlarm> {

    /**
     * 根据ID查询
     *
     * @param id id
     * @return 数据
     */
    ScadaNoAlarm get(Serializable id);

    /**
     * 新增
     *
     * @param scadaNoAlarmDto 实体
     */
    void insert(ScadaNoAlarmDto scadaNoAlarmDto);

    /**
     * 修改
     *
     * @param scadaNoAlarmDto 实体
     */
    void edit(ScadaNoAlarmDto scadaNoAlarmDto);

    /**
     * 批量删除
     *
     * @param idList 数据ID集合
     */
    void delete(List<Long> idList);

    /**
     * 生成模板下载
     */
    void template();

    /**
     * 数据导入
     *
     * @param file 文件
     */
    void importData(MultipartFile file);

    /**
     * 分页查询
     *
     * @param page         分页
     * @param scadaNoAlarm 查询wrapper
     * @return 数据集合
     */
    Page<ScadaNoAlarm> pageList(Page page, ScadaNoAlarm scadaNoAlarm);

    /**
     * 数据导出
     *
     * @param page         分页
     * @param scadaNoAlarm 查询模型
     * @param whole        是否导出全部数据，true是，false否（默认否）
     */
    void export(Page page, ScadaNoAlarm scadaNoAlarm, boolean whole);

    /**
     * 查询所有数据
     *
     * @param scadaNoAlarm 查询条件
     * @return 查询所有数据
     */
    List<ScadaNoAlarm> allList(ScadaNoAlarm scadaNoAlarm);


    /**
     * 设备报警次数统计
     *
     * @param start 开始时间
     * @param end   结束时间
     * @param type 统计方式,0小时,1天,2月
     * @param code 统计设备code编码
     * @return 设备报警次数统计
     */
    Object selectScadaNoAlarm(Date start, Date end, int type, String code);


    /**
     * 设备报警次数统计
     *
     * @param start 开始时间
     * @param end   结束时间
     * @param type 统计方式,0小时,1天,2月
     * @param code 统计设备code编码
     * @return 设备报警次数统计
     */
    Object selectScadaAlarm(Date start, Date end, int type, String code);
}
