package com.tofly.scada.service.impl;


import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.haiqiu.tools.excel.utils.ExcelUtils;
import org.springframework.util.CollectionUtils;
import org.springframework.beans.BeanUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.scada.mapper.ScadaAlarmMapper;
import com.tofly.scada.entity.ScadaAlarm;
import com.tofly.scada.entity.dto.ScadaAlarmDto;
import com.tofly.scada.service.ScadaAlarmService;
import org.springframework.stereotype.Service;

import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.io.Serializable;

/**
 * 报警scada列表(ScadaAlarm)表服务实现类
 *
 * @author HaiQiu
 * @since 2023-03-29 17:49:29
 */
@Service("scadaAlarmService")
public class ScadaAlarmServiceImpl extends ServiceImpl<ScadaAlarmMapper, ScadaAlarm> implements ScadaAlarmService {


    /**
     * 根据ID查询
     *
     * @param id id
     * @return 数据
     */
    @Override
    public ScadaAlarm get(Serializable id) {
        return getById(id);
    }


    /**
     * 新增
     *
     * @param scadaAlarmDto 实体
     */
    @Override
    public void insert(ScadaAlarmDto scadaAlarmDto) {
        ScadaAlarm scadaAlarm = convert(scadaAlarmDto);
        if (!save(scadaAlarm)) {
            throw new RuntimeException("新增失败");
        }
    }

    /**
     * 修改
     *
     * @param scadaAlarmDto 实体
     */
    @Override
    public void edit(ScadaAlarmDto scadaAlarmDto) {
        ScadaAlarm scadaAlarm = convert(scadaAlarmDto);
        if (!updateById(scadaAlarm)) {
            throw new RuntimeException("修改失败");
        }
    }

    /**
     * 实体转换
     *
     * @param scadaAlarmDto dto实体
     */
    private ScadaAlarm convert(ScadaAlarmDto scadaAlarmDto) {
        ScadaAlarm scadaAlarm = new ScadaAlarm();
        BeanUtils.copyProperties(scadaAlarmDto, scadaAlarm);
        return scadaAlarm;
    }

    /**
     * 批量删除
     *
     * @param idList 数据ID集合
     */
    @Override
    public void delete(List<Long> idList) {
        if (!removeByIds(idList)) {
            throw new RuntimeException("删除失败");
        }
    }

    /**
     * 生成模板下载
     */
    @Override
    public void template() {
        ScadaAlarm scadaAlarm = new ScadaAlarm();
        ExcelUtils.createTemplate("报警scada列表", "报警scada列表",
                "报警scada列表", ScadaAlarm.class, Collections.singletonList(scadaAlarm));
    }

    /**
     * 数据导入
     *
     * @param file 文件
     */
    @Override
    public void importData(MultipartFile file) {
        List<ScadaAlarm> excelToList = ExcelUtils.getExcelToList("报警scada列表", file, ScadaAlarm.class);
        if (CollectionUtils.isEmpty(excelToList)) {
            throw new RuntimeException("数据解析为空");
        }
        saveBatch(excelToList);
    }

    /**
     * 分页查询
     *
     * @param page       分页
     * @param scadaAlarm 查询wrapper
     * @return 数据集合
     */
    @Override
    public Page<ScadaAlarm> pageList(Page page, ScadaAlarm scadaAlarm) {
        return page(page, Wrappers.lambdaQuery(scadaAlarm)).addOrder(page.getOrders());
    }

    /**
     * 数据导出
     *
     * @param page       分页
     * @param scadaAlarm 查询模型
     * @param whole      是否导出全部数据，true是，false否（默认否）
     */
    @Override
    public void export(Page page, ScadaAlarm scadaAlarm, boolean whole) {
        Page<ScadaAlarm> pageData = new Page<>();
        if (!whole) {
            pageData = page(page, Wrappers.lambdaQuery(scadaAlarm));
        } else {
            pageData.setRecords(list(Wrappers.lambdaQuery(scadaAlarm)));
        }
        //导出数据
        ExcelUtils.getListToExcel(ScadaAlarm.class, pageData.getRecords(), "报警scada列表");
    }

    /**
     * 查询所有数据
     *
     * @param scadaAlarm 查询条件
     * @return 查询所有数据
     */
    @Override
    public List<ScadaAlarm> allList(ScadaAlarm scadaAlarm) {
        return list(Wrappers.lambdaQuery(scadaAlarm));
    }
}

