package com.tofly.scada.service.impl;


import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.haiqiu.tools.excel.utils.ExcelUtils;
import org.springframework.util.CollectionUtils;
import org.springframework.beans.BeanUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.scada.mapper.ScadaDeviceDataMapper;
import com.tofly.scada.entity.ScadaDeviceData;
import com.tofly.scada.entity.dto.ScadaDeviceDataDto;
import com.tofly.scada.service.ScadaDeviceDataService;
import org.springframework.stereotype.Service;

import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.io.Serializable;

/**
 * 物联网接入数据原始表(ScadaDeviceData)表服务实现类
 *
 * @author HaiQiu
 * @since 2023-04-03 11:15:39
 */
@Service("scadaDeviceDataService")
public class ScadaDeviceDataServiceImpl extends ServiceImpl<ScadaDeviceDataMapper, ScadaDeviceData> implements ScadaDeviceDataService {


    /**
     * 根据ID查询
     *
     * @param id id
     * @return 数据
     */
    @Override
    public ScadaDeviceData get(Serializable id) {
        return getById(id);
    }


    /**
     * 新增
     *
     * @param scadaDeviceDataDto 实体
     */
    @Override
    public void insert(ScadaDeviceDataDto scadaDeviceDataDto) {
        ScadaDeviceData scadaDeviceData = convert(scadaDeviceDataDto);
        if (!save(scadaDeviceData)) {
            throw new RuntimeException("新增失败");
        }
    }

    /**
     * 修改
     *
     * @param scadaDeviceDataDto 实体
     */
    @Override
    public void edit(ScadaDeviceDataDto scadaDeviceDataDto) {
        ScadaDeviceData scadaDeviceData = convert(scadaDeviceDataDto);
        if (!updateById(scadaDeviceData)) {
            throw new RuntimeException("修改失败");
        }
    }

    /**
     * 实体转换
     *
     * @param scadaDeviceDataDto dto实体
     */
    private ScadaDeviceData convert(ScadaDeviceDataDto scadaDeviceDataDto) {
        ScadaDeviceData scadaDeviceData = new ScadaDeviceData();
        BeanUtils.copyProperties(scadaDeviceDataDto, scadaDeviceData);
        return scadaDeviceData;
    }

    /**
     * 批量删除
     *
     * @param idList 数据ID集合
     */
    @Override
    public void delete(List<Long> idList) {
        if (!removeByIds(idList)) {
            throw new RuntimeException("删除失败");
        }
    }

    /**
     * 生成模板下载
     */
    @Override
    public void template() {
        ScadaDeviceData scadaDeviceData = new ScadaDeviceData();
        ExcelUtils.createTemplate("物联网接入数据原始表", "物联网接入数据原始表",
                "物联网接入数据原始表", ScadaDeviceData.class, Collections.singletonList(scadaDeviceData));
    }

    /**
     * 数据导入
     *
     * @param file 文件
     */
    @Override
    public void importData(MultipartFile file) {
        List<ScadaDeviceData> excelToList = ExcelUtils.getExcelToList("物联网接入数据原始表", file, ScadaDeviceData.class);
        if (CollectionUtils.isEmpty(excelToList)) {
            throw new RuntimeException("数据解析为空");
        }
        saveBatch(excelToList);
    }

    /**
     * 分页查询
     *
     * @param page            分页
     * @param scadaDeviceData 查询wrapper
     * @return 数据集合
     */
    @Override
    public Page<ScadaDeviceData> pageList(Page page, ScadaDeviceData scadaDeviceData) {
        return page(page, Wrappers.lambdaQuery(scadaDeviceData)).addOrder(page.getOrders());
    }

    /**
     * 数据导出
     *
     * @param page            分页
     * @param scadaDeviceData 查询模型
     * @param whole           是否导出全部数据，true是，false否（默认否）
     */
    @Override
    public void export(Page page, ScadaDeviceData scadaDeviceData, boolean whole) {
        Page<ScadaDeviceData> pageData = new Page<>();
        if (!whole) {
            pageData = page(page, Wrappers.lambdaQuery(scadaDeviceData));
        } else {
            pageData.setRecords(list(Wrappers.lambdaQuery(scadaDeviceData)));
        }
        //导出数据
        ExcelUtils.getListToExcel(ScadaDeviceData.class, pageData.getRecords(), "物联网接入数据原始表");
    }

    /**
     * 查询所有数据
     *
     * @param scadaDeviceData 查询条件
     * @return 查询所有数据
     */
    @Override
    public List<ScadaDeviceData> allList(ScadaDeviceData scadaDeviceData) {
        return list(Wrappers.lambdaQuery(scadaDeviceData));
    }
}

