package com.tofly.scada.service.impl;


import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.haiqiu.tools.excel.utils.ExcelUtils;
import org.springframework.util.CollectionUtils;
import org.springframework.beans.BeanUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.scada.mapper.ScadaHistoryMapper;
import com.tofly.scada.entity.ScadaHistory;
import com.tofly.scada.entity.dto.ScadaHistoryDto;
import com.tofly.scada.service.ScadaHistoryService;
import org.springframework.stereotype.Service;

import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.io.Serializable;

/**
 * 存储历史值(ScadaHistory)表服务实现类
 *
 * @author HaiQiu
 * @since 2023-04-03 11:25:00
 */
@Service("scadaHistoryService")
public class ScadaHistoryServiceImpl extends ServiceImpl<ScadaHistoryMapper, ScadaHistory> implements ScadaHistoryService {


    /**
     * 根据ID查询
     *
     * @param id id
     * @return 数据
     */
    @Override
    public ScadaHistory get(Serializable id) {
        return getById(id);
    }


    /**
     * 新增
     *
     * @param scadaHistoryDto 实体
     */
    @Override
    public void insert(ScadaHistoryDto scadaHistoryDto) {
        ScadaHistory scadaHistory = convert(scadaHistoryDto);
        if (!save(scadaHistory)) {
            throw new RuntimeException("新增失败");
        }
    }

    /**
     * 修改
     *
     * @param scadaHistoryDto 实体
     */
    @Override
    public void edit(ScadaHistoryDto scadaHistoryDto) {
        ScadaHistory scadaHistory = convert(scadaHistoryDto);
        if (!updateById(scadaHistory)) {
            throw new RuntimeException("修改失败");
        }
    }

    /**
     * 实体转换
     *
     * @param scadaHistoryDto dto实体
     */
    private ScadaHistory convert(ScadaHistoryDto scadaHistoryDto) {
        ScadaHistory scadaHistory = new ScadaHistory();
        BeanUtils.copyProperties(scadaHistoryDto, scadaHistory);
        return scadaHistory;
    }

    /**
     * 批量删除
     *
     * @param idList 数据ID集合
     */
    @Override
    public void delete(List<Long> idList) {
        if (!removeByIds(idList)) {
            throw new RuntimeException("删除失败");
        }
    }

    /**
     * 生成模板下载
     */
    @Override
    public void template() {
        ScadaHistory scadaHistory = new ScadaHistory();
        ExcelUtils.createTemplate("存储历史值", "存储历史值",
                "存储历史值", ScadaHistory.class, Collections.singletonList(scadaHistory));
    }

    /**
     * 数据导入
     *
     * @param file 文件
     */
    @Override
    public void importData(MultipartFile file) {
        List<ScadaHistory> excelToList = ExcelUtils.getExcelToList("存储历史值", file, ScadaHistory.class);
        if (CollectionUtils.isEmpty(excelToList)) {
            throw new RuntimeException("数据解析为空");
        }
        saveBatch(excelToList);
    }

    /**
     * 分页查询
     *
     * @param page         分页
     * @param scadaHistory 查询wrapper
     * @return 数据集合
     */
    @Override
    public Page<ScadaHistory> pageList(Page page, ScadaHistory scadaHistory) {
        return page(page, Wrappers.lambdaQuery(scadaHistory)).addOrder(page.getOrders());
    }

    /**
     * 数据导出
     *
     * @param page         分页
     * @param scadaHistory 查询模型
     * @param whole        是否导出全部数据，true是，false否（默认否）
     */
    @Override
    public void export(Page page, ScadaHistory scadaHistory, boolean whole) {
        Page<ScadaHistory> pageData = new Page<>();
        if (!whole) {
            pageData = page(page, Wrappers.lambdaQuery(scadaHistory));
        } else {
            pageData.setRecords(list(Wrappers.lambdaQuery(scadaHistory)));
        }
        //导出数据
        ExcelUtils.getListToExcel(ScadaHistory.class, pageData.getRecords(), "存储历史值");
    }

    /**
     * 查询所有数据
     *
     * @param scadaHistory 查询条件
     * @return 查询所有数据
     */
    @Override
    public List<ScadaHistory> allList(ScadaHistory scadaHistory) {
        return list(Wrappers.lambdaQuery(scadaHistory));
    }
}

