package com.tofly.scada.service.impl;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.haiqiu.tools.excel.utils.ExcelUtils;
import com.tofly.scada.entity.ScadaNoAlarm;
import com.tofly.scada.entity.dto.ScadaNoAlarmDto;
import com.tofly.scada.mapper.ScadaNoAlarmMapper;
import com.tofly.scada.service.ScadaNoAlarmService;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.Serializable;
import java.util.*;

/**
 * 不报警scada列表(ScadaNoAlarm)表服务实现类
 *
 * @author HaiQiu
 * @since 2023/3/24
 */
@Service("scadaNoAlarmService")
public class ScadaNoAlarmServiceImpl extends ServiceImpl<ScadaNoAlarmMapper, ScadaNoAlarm> implements ScadaNoAlarmService {


    public static final String YYYY_MM_DD = "yyyy-MM-dd";

    public static final String YYYY_MM_DD_mm_HH_ss = "yyyy-MM-dd HH:mm:ss";

    /**
     * 根据ID查询
     *
     * @param id id
     * @return 数据
     */
    @Override
    public ScadaNoAlarm get(Serializable id) {
        return getById(id);
    }


    /**
     * 新增
     *
     * @param scadaNoAlarmDto 实体
     */
    @Override
    public void insert(ScadaNoAlarmDto scadaNoAlarmDto) {
        ScadaNoAlarm scadaNoAlarm = convert(scadaNoAlarmDto);
        if (!save(scadaNoAlarm)) {
            throw new RuntimeException("新增失败");
        }
    }

    /**
     * 修改
     *
     * @param scadaNoAlarmDto 实体
     */
    @Override
    public void edit(ScadaNoAlarmDto scadaNoAlarmDto) {
        ScadaNoAlarm scadaNoAlarm = convert(scadaNoAlarmDto);
        if (!updateById(scadaNoAlarm)) {
            throw new RuntimeException("修改失败");
        }
    }

    /**
     * 实体转换
     *
     * @param scadaNoAlarmDto dto实体
     */
    private ScadaNoAlarm convert(ScadaNoAlarmDto scadaNoAlarmDto) {
        ScadaNoAlarm scadaNoAlarm = new ScadaNoAlarm();
        BeanUtils.copyProperties(scadaNoAlarmDto, scadaNoAlarm);
        return scadaNoAlarm;
    }

    /**
     * 批量删除
     *
     * @param idList 数据ID集合
     */
    @Override
    public void delete(List<Long> idList) {
        if (!removeByIds(idList)) {
            throw new RuntimeException("删除失败");
        }
    }

    /**
     * 生成模板下载
     */
    @Override
    public void template() {
        ScadaNoAlarm scadaNoAlarm = new ScadaNoAlarm();
        ExcelUtils.createTemplate("不报警scada列表", "不报警scada列表",
                "不报警scada列表", ScadaNoAlarm.class, Collections.singletonList(scadaNoAlarm));
    }

    /**
     * 数据导入
     *
     * @param file 文件
     */
    @Override
    public void importData(MultipartFile file) {
        List<ScadaNoAlarm> excelToList = ExcelUtils.getExcelToList("不报警scada列表", file, ScadaNoAlarm.class);
        if (CollectionUtils.isEmpty(excelToList)) {
            throw new RuntimeException("数据解析为空");
        }
        saveBatch(excelToList);
    }

    /**
     * 分页查询
     *
     * @param page         分页
     * @param scadaNoAlarm 查询wrapper
     * @return 数据集合
     */
    @Override
    public Page<ScadaNoAlarm> pageList(Page page, ScadaNoAlarm scadaNoAlarm) {
        return page(page, Wrappers.lambdaQuery(scadaNoAlarm)).addOrder(page.getOrders());
    }

    /**
     * 数据导出
     *
     * @param page         分页
     * @param scadaNoAlarm 查询模型
     * @param whole        是否导出全部数据，true是，false否（默认否）
     */
    @Override
    public void export(Page page, ScadaNoAlarm scadaNoAlarm, boolean whole) {
        Page<ScadaNoAlarm> pageData = new Page<>();
        if (!whole) {
            pageData = page(page, Wrappers.lambdaQuery(scadaNoAlarm));
        } else {
            pageData.setRecords(list(Wrappers.lambdaQuery(scadaNoAlarm)));
        }
        //导出数据
        ExcelUtils.getListToExcel(ScadaNoAlarm.class, pageData.getRecords(), "不报警scada列表");
    }

    /**
     * 查询所有数据
     *
     * @param scadaNoAlarm 查询条件
     * @return 查询所有数据
     */
    @Override
    public List<ScadaNoAlarm> allList(ScadaNoAlarm scadaNoAlarm) {
        return list(Wrappers.lambdaQuery(scadaNoAlarm));
    }

    @Override
    public Object selectScadaNoAlarm(Date start, Date end, int type, String code) {
        List<Map<String, Object>> maps = this.baseMapper.selectGroupByTimeCount(start, end);
        return maps;
    }


    @Override
    public Object selectScadaAlarm(Date start, Date end, int type, String code) {
        Map<String, Object> maps = new HashMap<>();
        Object yes = null;
        Object no = null;
        if (type == 0) {
            yes = this.baseMapper.selectGroupByTimeHour(DateUtil.format(start, YYYY_MM_DD_mm_HH_ss),
                    DateUtil.format(end, YYYY_MM_DD_mm_HH_ss), code);
            maps.put("alarm", yes);
            no = this.baseMapper.selectGroupNoByTimeHour(DateUtil.format(start, YYYY_MM_DD_mm_HH_ss),
                    DateUtil.format(end, YYYY_MM_DD_mm_HH_ss), code);
            maps.put("noAlarm", no);
        } else if (type == 1) {
            yes = this.baseMapper.selectGroupByTimeDay(DateUtil.format(start, YYYY_MM_DD_mm_HH_ss),
                    DateUtil.format(end, YYYY_MM_DD_mm_HH_ss), code);
            maps.put("alarm", yes);
            no = this.baseMapper.selectGroupNoByTimeDay(DateUtil.format(start, YYYY_MM_DD_mm_HH_ss),
                    DateUtil.format(end, YYYY_MM_DD_mm_HH_ss), code);
            maps.put("noAlarm", no);
        } else {
            yes = this.baseMapper.selectGroupByTimeMouth(DateUtil.format(start, YYYY_MM_DD_mm_HH_ss),
                    DateUtil.format(end, YYYY_MM_DD_mm_HH_ss), code);
            maps.put("alarm", yes);
            no = this.baseMapper.selectGroupNoByTimeMouth(DateUtil.format(start, YYYY_MM_DD_mm_HH_ss),
                    DateUtil.format(end, YYYY_MM_DD_mm_HH_ss), code);
            maps.put("noAlarm", no);
        }
        return maps;
    }

}
