package com.tofly.scada.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Snowflake;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.scada.common.FlowConstant;
import com.tofly.scada.common.JsonItemConst;
import com.tofly.scada.common.MessageTopicEnum;
import com.tofly.scada.config.DictConfig;
import com.tofly.scada.entity.*;
import com.tofly.scada.entity.dto.DeviceQuery;
import com.tofly.scada.entity.dto.ScadaQuery;
import com.tofly.scada.entity.dto.WisdomDisplayDto;
import com.tofly.scada.entity.vo.*;
import com.tofly.scada.mapper.AllocationMapper;
import com.tofly.scada.mapper.DeviceArchiveManageMapper;
import com.tofly.scada.mapper.ScadaMapper;
import com.tofly.scada.mapper.ScadaReportMapper;
import com.tofly.scada.service.*;
import com.tofly.scada.util.DateTimeUtil;
import com.tofly.scada.util.SendTidingsUtil;
import com.tofly.scada.util.SnowFlake;
import com.tofly.scada.util.TimeUtils;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * @author HaiQiu
 * @date 2022/4/25
 */
@Configuration
@Slf4j
public class ScadaServiceImpl extends ServiceImpl<ScadaMapper, Scada> implements ScadaService {

    private final Logger logger = LoggerFactory.getLogger(ScadaServiceImpl.class);

    private static final Pattern patternStr = Pattern.compile("[0-9.-]+");

    @Autowired
    private ScadaMapper scadaMapper;

    @Autowired
    private StatisticsScadaService statisticsScadaService;

    @Autowired
    private AllocationMapper allocationMapper;

    @Autowired
    private DeviceArchiveManageMapper deviceArchiveManageMapper;

    @Autowired
    private ScadaReportMapper reportMapper;

    @Autowired
    private SendTidingsUtil sendTidingsUtil;

    @Autowired
    private ScadaReportServiceImpl scadaReportService;

    @Autowired
    private ScadaLjFlowService scadaLjFlowService;

    @Autowired
    private ScadaSynHisService scadaSynHisService;

    @Autowired
    private ScadaNoAlarmService scadaNoAlarmService;

    @Autowired
    private ScadaAlarmService scadaAlarmService;

    public static final String YYYY_MM_DD_HH_MM_SS = "yyyy-MM-dd HH:mm:ss";

    @Autowired
    private ScadaDeviceDataService deviceDataService;

    @Autowired
    private DictConfig dictConfig;
    @Autowired
    private DeviceArchiveManageService deviceArchiveManageService;
    @Autowired
    private AllocationService allocationService;

    /**
     * 根据当前时间动态自动创建月份表，创建完成之后返回表名
     * <p>
     * 往后往前推迟月份，1即创建下一月的表，-1表示创建上个月表
     *
     * @return 月份表名
     */
    //每月最后28日凌晨2点执行创建下个月的数据表
    public void createMonths() {
        Calendar cal = Calendar.getInstance();
        cal.setTime(new Date());
        cal.add(Calendar.MONTH, 1);
        SimpleDateFormat format = new SimpleDateFormat("yyyy_MM");
        String date = format.format(cal.getTime());
        String tableName = "SCADA_MONTH_" + date;
        if (scadaMapper.verifyTableMonthsIsExits(tableName) <= 0) {
            logger.info("Scada月份表状态：不存在，开始创建");
            scadaMapper.createTableMonths(tableName);
            logger.info("Scada月份表状态：创建完成");
        }
    }

    public String createMonths(Date time) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(time == null ? new Date() : time);
        SimpleDateFormat format = new SimpleDateFormat("yyyy_MM");
        String date = format.format(cal.getTime());
        String tableName = "SCADA_MONTH_" + date;
        if (scadaMapper.verifyTableMonthsIsExits(tableName) <= 0) {
            scadaMapper.createTableMonths(tableName);
        }
        return tableName;
    }


    /**
     * 同步24小时数据，支持自定义时间添加
     */

    // 每日 7点 19点执行
    public void insert24H() {
        // 当天开始时间-1 结束时间(当前时间)
        Date startDate = DateTimeUtil.convertLDTToDate(DateTimeUtil.getTodayBeginTime().minusDays(1));
        Date endDate = new Date();

        List<Date> dates = findDates(startDate, endDate, 0);
        ThreadPoolExecutor threadPoolExecutor = new ThreadPoolExecutor(
                12,
                24,
                120,
                TimeUnit.SECONDS,
                new ArrayBlockingQueue<>(48),
                (r)->{
                    Thread t = new Thread(r);
                    t.setName("thread-" + ((int)(Math.random()*1000)));
                    return t;},
                new ThreadPoolExecutor.CallerRunsPolicy());
        dates.forEach(date -> {
                    //整个日历调整时间
                    Calendar calendar = Calendar.getInstance();
                    calendar.setTime(date);
                    calendar.set(Calendar.MINUTE, 0);
                    calendar.set(Calendar.SECOND, 0);
                    calendar.set(Calendar.MILLISECOND, 0);
                    Date startTime = calendar.getTime();
                    //拉长时间到59分59秒
                    calendar.set(Calendar.MINUTE, 59);
                    calendar.set(Calendar.SECOND, 59);
                    calendar.set(Calendar.MILLISECOND, 999);
                    Date endTime = calendar.getTime();
                    List<Allocation> codes = getCodes(null);
                    //保存数据
                    threadPoolExecutor.execute(() -> saveStatisticsCode(startTime, endTime, codes, FlowConstant.INTEGER_ZERO));
                }
        );


    }

    @Async
    public void insert24H(String code, Date time) {
        // 获取

        //整个日历调整时间
        Calendar calendar = Calendar.getInstance();
        Date date = time == null ? new Date() : time;
        calendar.setTime(date);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        Date startTime = calendar.getTime();
        //拉长时间到59分59秒
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        calendar.set(Calendar.MILLISECOND, 999);
        Date endTime = calendar.getTime();
        List<Allocation> codes = getCodes(code);
        //保存数据
        saveStatisticsCode(startTime, endTime, codes, FlowConstant.INTEGER_ZERO);
    }


    /**
     * 同步每天数据，每天6半同步昨天的数据
     */
    public void syncDaysData() {
        //整个日历调整时间
        Calendar calendar = Calendar.getInstance();
        Date date = new Date();
        calendar.setTime(date);
        //延迟一天
        calendar.add(Calendar.DAY_OF_MONTH, -1);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        Date startTime = calendar.getTime();
        //拉长时间到59分59秒
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        calendar.set(Calendar.MILLISECOND, 999);
        Date endTime = calendar.getTime();
        List<Allocation> codes = getCodes(null);
        //保存数据
        saveStatisticsCode(startTime, endTime, codes, FlowConstant.INTEGER_ONE);

    }


    @Async
    public void syncDaysData(String code, Date time) {
        //整个日历调整时间
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(time == null ? new Date() : time);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        Date startTime = calendar.getTime();
        //拉长时间到59分59秒
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        calendar.set(Calendar.MILLISECOND, 999);
        Date endTime = calendar.getTime();
        List<Allocation> codes = getCodes(code);
        //保存数据
        saveStatisticsCode(startTime, endTime, codes, FlowConstant.INTEGER_ONE);

    }


    /**
     * 每月1号凌晨1点同步更新历史数据与上个月表数据，支持自定义时间添加
     */
    public void syncDataIntoMonths() {
        //整个日历调整时间
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        //倒退一个月开始时间
        calendar.add(Calendar.MONTH, -1);
        calendar.set(Calendar.DATE, 1);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        Date startTime = calendar.getTime();
        //倒退一个月最后一天时间
        calendar.set(Calendar.DATE, calendar.getActualMaximum(Calendar.DATE));
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        calendar.set(Calendar.MILLISECOND, 999);
        Date endTime = calendar.getTime();
        List<Allocation> codes = getCodes(null);
        //保存数据
        saveStatisticsCode(startTime, endTime, codes, FlowConstant.INTEGER_TWO);
    }

    /**
     * 每天凌晨0点同步更新这个月表数据
     */
    public void syncDataMonthEverday() {
        //整个日历调整时间
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        //当月开始时间
        calendar.set(Calendar.DATE, 1);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        Date startTime = calendar.getTime();
        //当月最后一天时间
        calendar.set(Calendar.DATE, calendar.getActualMaximum(Calendar.DATE));
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        Date endTime = calendar.getTime();
        List<Allocation> codes = getCodes(null);
        //保存数据
        saveStatisticsCode(startTime, endTime, codes, FlowConstant.INTEGER_TWO);
    }

    @Async
    public void syncDataIntoMonths(String code, Date time) {
        //整个日历调整时间
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(time == null ? new Date() : time);
        calendar.set(Calendar.DATE, 1);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        Date startTime = calendar.getTime();
        String start = DateUtil.format(calendar.getTime(), "yyyy-MM-dd HH:mm:ss");
        //倒退一个月最后一天时间
        calendar.set(Calendar.DATE, calendar.getActualMaximum(Calendar.DATE));
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        calendar.set(Calendar.MILLISECOND, 999);
        Date endTime = calendar.getTime();
        List<Allocation> codes = getCodes(code);
        //保存数据
        saveStatisticsCode(startTime, endTime, codes, FlowConstant.INTEGER_TWO);
    }


//    /**
//     * 普通进程跑
//     *
//     * @param startTime
//     * @param endTime
//     * @param codes
//     * @param DateType
//     */
//    public void saveStatisticsByCode(Date startTime, Date endTime, List<String> codes, Integer DateType) {
//        if (!CollectionUtils.isEmpty(codes)) {
//            logger.info("同步scada历史数据到表：正在进行数据分组");
//            //有数据就分组
//            logger.info("同步scada历史数据到表：正在进行分组循环计算");
//            //分组循环计算添加
//            for (String code : codes) {
//                //当前线程名称
//                String name = Thread.currentThread().getName();
//                logger.info("当前线程：{}----统计添加数据：{}中。。。。", name, code);
//                QueryWrapper<Scada> scadaQueryWrapper = new QueryWrapper<>();
//                scadaQueryWrapper.between("SCADA_TIME", startTime, endTime)
//                        .in("CODE", code);
//                List<Scada> scadaList = scadaMapper.selectList(scadaQueryWrapper);
//                if (CollectionUtils.isEmpty(scadaList)) {
//                    return;
//                }
//
//                String jlj = ".JLJ";
//                String defaultValue = "0";
//
//                BigDecimal sum = BigDecimal.ZERO;
//                BigDecimal average = BigDecimal.ZERO;
//                BigDecimal maxVal = new BigDecimal(scadaList.get(0).getValue());
//                BigDecimal minVal = new BigDecimal(scadaList.get(0).getValue());
//
//                if (scadaList.get(0).getCode().contains(jlj)) {
//                    Iterator<Scada> iterator = scadaList.iterator();
//                    while (iterator.hasNext()) {
//                        Scada next = iterator.next();
//                        if (next.getCode().contains(jlj) && !next.getValue().equals(defaultValue)) {
//                            maxVal = new BigDecimal(next.getValue());
//                            minVal = maxVal;
//                            break;
//                        }
//                    }
//                }
//
//                for (Scada scada : scadaList) {
//                    if (scada.getCode().contains(jlj) && scada.getValue().equals(defaultValue)) {
//                        continue;
//                    }
//                    BigDecimal decimal = new BigDecimal(scada.getValue());
//                    sum = decimal.add(sum);
//                    if (decimal.compareTo(maxVal) > 0) {
//                        maxVal = decimal;
//                    }
//                    if (decimal.compareTo(minVal) < 0) {
//                        minVal = decimal;
//                    }
//                }
//                average = sum.divide(BigDecimal.valueOf(scadaList.size()), RoundingMode.HALF_UP);
//
//                StatisticsScada statisticsScada = new StatisticsScada();
//                statisticsScada.setCode(scadaList.get(0).getCode());
//                statisticsScada.setValue(scadaList.get(scadaList.size() - 1).getValue());
//                statisticsScada.setAve(average.toString());
//                statisticsScada.setMax(maxVal.toString());
//                statisticsScada.setMin(minVal.toString());
//                statisticsScada.setFold(sum.toString());
//                statisticsScada.setStartTime(startTime);
//                statisticsScada.setEndTime(endTime);
//                statisticsScada.setStatisticsType(DateType);
//
//                //查询指标报警上下限计算合格率相关
//                QueryWrapper<Allocation> allocationQueryWrapper = new QueryWrapper<>();
//                allocationQueryWrapper.eq("VARIABLE_CODE", scadaList.get(0).getCode());
//                List<Allocation> allocations = allocationMapper.selectList(allocationQueryWrapper);
//                if (!CollectionUtils.isEmpty(allocations) && StringUtils.hasText(allocations.get(0).getType())) {
//                    Set<Long> reportIdSets = getReportIdSets(allocations);
//                    if (!CollectionUtils.isEmpty(reportIdSets)) {
//                        logger.info("计算指标报警上下限计算合格率相关....");
//                        QueryWrapper<ScadaReport> scadaReportQueryWrapper = new QueryWrapper<>();
//                        scadaReportQueryWrapper.in("ID", reportIdSets);
//                        List<ScadaReport> scadaReports = reportMapper.selectList(scadaReportQueryWrapper);
//                        if (!CollectionUtils.isEmpty(scadaReports)) {
//                            //超标次数
//                            int errorCount = 0;
//                            //不合格超标时长
//                            long errorTimeTotal = 0;
//                            //超标总时长
//                            long timeTotal = (scadaList.get(scadaList.size() - 1).getScadaTime().getTime()
//                                    - scadaList.get(0).getScadaTime().getTime()) / 1000;
//                            //错误时间
//                            Date errorTime = null;
//                            for (Scada scada : scadaList) {
//                                if (scada.getCode().contains(".JLJ") && scada.getCode().equals("0")) {
//                                    continue;
//                                }
//                                for (ScadaReport report : scadaReports) {
//                                    if (Double.parseDouble(scada.getValue()) < Double.parseDouble(report.getReportLower())
//                                            || Double.parseDouble(scada.getValue()) > Double.parseDouble(report.getReportUpper())) {
//                                        errorCount = errorCount + 1;
//                                        errorTime = scada.getScadaTime();
//                                        break;
//                                    } else if (errorTime != null) {
//                                        errorTimeTotal = errorTimeTotal +
//                                                (scada.getScadaTime().getTime() - errorTime.getTime()) / 1000;
//                                    }
//                                }
//                            }
//                            //合格率
//                            float pass = 0;
//                            if (timeTotal != 0) {
//                                BigDecimal num1 = new BigDecimal(String.valueOf(errorTimeTotal));
//                                BigDecimal num2 = new BigDecimal(String.valueOf(timeTotal));
//                                pass = num1.divide(num2, RoundingMode.HALF_UP).floatValue();
//                            }
//
//                            statisticsScada.setErrorCount(String.valueOf(errorCount));
//                            statisticsScada.setErrorTimeTotal(String.valueOf(errorTimeTotal == 0 ? timeTotal : errorTimeTotal));
//                            statisticsScada.setPass(String.valueOf(pass));
//                            statisticsScada.setTimeTotal(String.valueOf(timeTotal));
//                        }
//                    }
//                    statisticsScada.setDataType(String.valueOf(allocations.get(0).getType()));
//                }
//
//                //删除旧数据
//
//                int delete = statisticsScadaService.delete(DateType, statisticsScada.getCode(),
//                        DateUtil.format(statisticsScada.getStartTime(), "yyyy-MM-dd HH:mm:ss"),
//                        DateUtil.format(statisticsScada.getEndTime(), "yyyy-MM-dd HH:mm:ss"));
//                logger.info("成功删除{}条旧数据", delete);
//                //添加数据
//                logger.info("统计添加数据.....：{}", statisticsScada);
//                try {
//                    statisticsScadaService.save(statisticsScada);
//                    logger.info("统计添加数据完成：{}", statisticsScada);
//                } catch (Exception e) {
//                    e.printStackTrace();
//                    log.error("统计数据添加出现错误----数据：{}---错误:{}", statisticsScada, e.getMessage());
//                }
//                logger.info("统计添加数据完成：{}", statisticsScada);
//            }
//        } else {
//            logger.info("获取历史数据为空");
//        }
//    }


//    /**
//     * 设置统计数据
//     *
//     * @param startTime 开始时间
//     * @param endTime   结束时间
//     * @param scadas    数据集合
//     */
//    @Async
//    public void saveStatisticsData(Date startTime, Date endTime, List<Scada> scadas, Integer DateType) {
//        if (!CollectionUtils.isEmpty(scadas)) {
//            logger.info("同步scada历史数据到表：正在进行数据分组");
//            //有数据就分组
//            Map<String, List<Scada>> listMap = scadas.stream().filter(scada -> scada.getCode() != null && scada.getValue() != null)
//                    .collect(Collectors.groupingBy(Scada::getCode));
//            logger.info("同步scada历史数据到表：正在进行分组循环计算");
//            ExecutorService executorService = Executors.newFixedThreadPool(listMap.entrySet().size());
//            CountDownLatch countDownLatch = new CountDownLatch(listMap.entrySet().size());
//            //分组循环计算添加
//            for (Map.Entry<String, List<Scada>> entity : listMap.entrySet()) {
//                //开启多线程
//                executorService.submit(new Runnable() {
//                    @Override
//                    public void run() {
//                        //当前线程名称
//                        String name = Thread.currentThread().getName();
//                        logger.info("当前线程：{}----统计添加数据中。。。。", name);
//                        List<Scada> scadaList = entity.getValue();
//
////                double average = scadaList.stream().mapToLong(value -> Long.parseLong(value.getValue())).average().getAsDouble();
////                double max = scadaList.stream().mapToLong(value -> Long.parseLong(value.getValue())).max().getAsLong();
////                double min = scadaList.stream().mapToLong(value -> Long.parseLong(value.getValue())).min().getAsLong();
////                double sum = scadaList.stream().mapToLong(value -> Long.parseLong(value.getValue())).sum();
//                        BigDecimal sum = BigDecimal.ZERO;
//                        BigDecimal average = BigDecimal.ZERO;
//                        BigDecimal maxVal = new BigDecimal(scadaList.get(0).getValue());
//                        BigDecimal minVal = new BigDecimal(scadaList.get(0).getValue());
//                        for (Scada scada : scadaList) {
//                            BigDecimal decimal = new BigDecimal(scada.getValue());
//                            sum = decimal.add(sum);
//                            if (decimal.compareTo(maxVal) > 0) {
//                                maxVal = decimal;
//                            }
//                            if (decimal.compareTo(minVal) < 0) {
//                                minVal = decimal;
//                            }
//                        }
//                        average = sum.divide(BigDecimal.valueOf(scadaList.size()), RoundingMode.HALF_UP);
//
//                        StatisticsScada statisticsScada = new StatisticsScada();
//                        statisticsScada.setCode(scadaList.get(0).getCode());
//                        statisticsScada.setValue(scadaList.get(scadaList.size() - 1).getValue());
//                        statisticsScada.setAve(average.toString());
//                        statisticsScada.setMax(maxVal.toString());
//                        statisticsScada.setMin(minVal.toString());
//                        statisticsScada.setFold(sum.toString());
//                        statisticsScada.setStartTime(startTime);
//                        statisticsScada.setEndTime(endTime);
//                        statisticsScada.setStatisticsType(DateType);
//
//                        //查询指标报警上下限计算合格率相关
//                        QueryWrapper<Allocation> allocationQueryWrapper = new QueryWrapper<>();
//                        allocationQueryWrapper.eq("VARIABLE_CODE", scadaList.get(0).getCode());
//                        List<Allocation> allocations = allocationMapper.selectList(allocationQueryWrapper);
//                        if (!CollectionUtils.isEmpty(allocations) && StringUtils.hasText(allocations.get(0).getType())) {
//                            Set<Long> reportIdSets = getReportIdSets(allocations);
//                            if (!CollectionUtils.isEmpty(reportIdSets)) {
//                                logger.info("计算指标报警上下限计算合格率相关....");
//                                QueryWrapper<ScadaReport> scadaReportQueryWrapper = new QueryWrapper<>();
//                                scadaReportQueryWrapper.in("ID", reportIdSets);
//                                List<ScadaReport> scadaReports = reportMapper.selectList(scadaReportQueryWrapper);
//                                if (!CollectionUtils.isEmpty(scadaReports)) {
//                                    //超标次数
//                                    int errorCount = 0;
//                                    //不合格超标时长
//                                    long errorTimeTotal = 0;
//                                    //超标总时长
//                                    long timeTotal = (scadaList.get(scadaList.size() - 1).getScadaTime().getTime()
//                                            - scadaList.get(0).getScadaTime().getTime()) / 1000;
//                                    //错误时间
//                                    Date errorTime = null;
//                                    for (Scada scada : scadaList) {
//                                        for (ScadaReport report : scadaReports) {
//                                            if (Double.parseDouble(scada.getValue()) < Double.parseDouble(report.getReportLower())
//                                                    || Double.parseDouble(scada.getValue()) > Double.parseDouble(report.getReportUpper())) {
//                                                errorCount = errorCount + 1;
//                                                errorTime = scada.getScadaTime();
//                                                break;
//                                            } else if (errorTime != null) {
//                                                errorTimeTotal = errorTimeTotal +
//                                                        (scada.getScadaTime().getTime() - errorTime.getTime()) / 1000;
//                                            }
//                                        }
//                                    }
//                                    //合格率
//                                    float pass = 0;
//                                    if (timeTotal != 0) {
//                                        BigDecimal num1 = new BigDecimal(String.valueOf(errorTimeTotal));
//                                        BigDecimal num2 = new BigDecimal(String.valueOf(timeTotal));
//                                        pass = num1.divide(num2, RoundingMode.HALF_UP).floatValue();
//                                    }
//
//                                    statisticsScada.setErrorCount(String.valueOf(errorCount));
//                                    statisticsScada.setErrorTimeTotal(String.valueOf(errorTimeTotal == 0 ? timeTotal : errorTimeTotal));
//                                    statisticsScada.setPass(String.valueOf(pass));
//                                    statisticsScada.setTimeTotal(String.valueOf(timeTotal));
//                                }
//                            }
//                            statisticsScada.setDataType(String.valueOf(allocations.get(0).getType()));
//                        }
//
//                        //删除旧数据
//
//                        int delete = statisticsScadaService.delete(DateType, statisticsScada.getCode(),
//                                DateUtil.format(statisticsScada.getStartTime(), "yyyy-MM-dd HH:mm:ss"),
//                                DateUtil.format(statisticsScada.getEndTime(), "yyyy-MM-dd HH:mm:ss"));
//                        logger.info("成功删除{}条旧数据", delete);
//                        //添加数据
//                        logger.info("统计添加数据完成：{}", statisticsScada);
//                        try {
//                            statisticsScadaService.save(statisticsScada);
//                        } catch (Exception e) {
//                            e.printStackTrace();
//                            log.error("统计数据添加出现错误----数据：{}---错误:{}", statisticsScada, e.getMessage());
//                        }
//                        logger.info("统计添加数据完成：{}", statisticsScada);
//                        // 闭锁
//                        countDownLatch.countDown();
//                    }
//                });
//            }
//        } else {
//            logger.info("获取历史数据为空");
//        }
//    }


//    /**
//     * 截取数组长度批量插入
//     *
//     * @param scadas    数据集合
//     * @param tableName 表名
//     * @return 成功条数
//     */
//    public int insertMouthBatch(List<Scada> scadas, String tableName) {
//        int size = scadas.size();
//        int limit = 1000;
//        int startIndex = 0;
//        int endIndex;
//        if (size <= limit) {
//            //添加新数据
//            int insertMouth = scadaMapper.insertMouth(scadas, tableName);
//            log.info("新增数据：{}条", insertMouth);
//        } else {
//            while (startIndex < size) {
//                //结束索引范围
//                endIndex = startIndex + limit;
//                //最后索引不能超过本身长度
//                endIndex = endIndex > size ? size : endIndex;
//                //截取数据
//                List<Scada> scadaList = scadas.subList(startIndex, endIndex);
//                //下次索引位置
//                startIndex = endIndex;
//                //添加新数据
//                int insertMouth = scadaMapper.insertMouth(scadaList, tableName);
//                log.info("新增数据：{}条", insertMouth);
//            }
//        }
//        return size;
//    }


    /**
     * 分code进行多线程存储
     *
     * @param startTime
     * @param endTime
     * @param codes
     * @param dateType
     */
    public void saveStatisticsCode(Date startTime, Date endTime, List<Allocation> codes, Integer dateType) {
        if (CollectionUtils.isEmpty(codes)) {
            return;
        }
        //有数据就分组
        List<StatisticsScada> statisticsScadaList = new ArrayList<>();

        List<Scada> scadaListElecLast = null;
        List<Scada> lastHourListAll = null;
        if (dateType == 0) {
            Calendar calendarStart = Calendar.getInstance();
            calendarStart.setTime(startTime);
            calendarStart.add(Calendar.HOUR, -1);
            Calendar calendarEnd = Calendar.getInstance();
            calendarEnd.setTime(endTime);
            calendarEnd.add(Calendar.HOUR, -1);
            QueryWrapper<Scada> wrapper1 = new QueryWrapper<>();
            wrapper1.between("SCADA_TIME", DateUtil.parse(DateUtil.format(calendarStart.getTime(),"yyyy-MM-dd HH:mm:ss"),"yyyy-MM-dd HH:mm:ss"),
                    DateUtil.parse(DateUtil.format(calendarEnd.getTime(),"yyyy-MM-dd HH:mm:ss"),"yyyy-MM-dd HH:mm:ss"));
            lastHourListAll = scadaMapper.selectList(wrapper1);

            calendarStart.add(Calendar.DATE, -3);
            calendarStart.set(Calendar.HOUR, 0);
            QueryWrapper<Scada> wrapper = new QueryWrapper<>();
            wrapper.between("SCADA_TIME", DateUtil.parse(DateUtil.format(calendarStart.getTime(),"yyyy-MM-dd HH:mm:ss"),"yyyy-MM-dd HH:mm:ss"),
                    DateUtil.parse(DateUtil.format(calendarEnd.getTime(),"yyyy-MM-dd HH:mm:ss"),"yyyy-MM-dd HH:mm:ss"));
            wrapper.like("CODE", "czruixin");
            scadaListElecLast = scadaMapper.selectList(wrapper);
        }
        QueryWrapper<Scada> scadaQueryWrapper = new QueryWrapper<>();
        scadaQueryWrapper.between("SCADA_TIME", startTime, endTime);
        List<Scada> scadaListAll = scadaMapper.selectList(scadaQueryWrapper);
        if (CollectionUtils.isEmpty(scadaListAll)) {
            return;
        }
        List<Allocation> allocationsAll = allocationMapper.selectList(new QueryWrapper<>());
        List<ScadaReport> scadaReportsAll = reportMapper.selectList(new QueryWrapper<>());
        //分组循环计算添加
        for (Allocation code : codes) {
            List<Scada> scadaList;
            List<Scada> lastHourList = null;
            if (CollUtil.isNotEmpty(lastHourListAll)) {
                lastHourList = lastHourListAll.stream().filter(e -> e.getCode().equals(code.getVariableCode())).collect(Collectors.toList());
            }
            if (org.apache.commons.lang3.StringUtils.isNotBlank(code.getType())
                    && code.getType().equals(dictConfig.getElectricAllIndex())
                    && CollUtil.isNotEmpty(scadaListElecLast)) {
                lastHourList = scadaListElecLast.stream().filter(e -> e.getCode().equals(code.getVariableCode())).collect(Collectors.toList());
            }
            scadaList = scadaListAll.stream().filter(e -> e.getCode().equals(code.getVariableCode())).collect(Collectors.toList());
            if (CollectionUtils.isEmpty(scadaList)) {
                continue;
            }
//            //todo 使用统计天的数据来进行弥补月份延迟数据新增
//            if (DateType.equals(FlowConstant.INTEGER_ONE)){
//                //存储到分表月份
//                String tableName = createMonths(startTime);
//                log.info("创建月份表成功：-----{}-----", tableName);
//                //删除旧数据
//                int deleteMouth = scadaMapper.deleteFromTableName(tableName,
//                        DateUtil.format(startTime, "yyyy-MM-dd HH:mm:ss"),
//                        DateUtil.format(endTime, "yyyy-MM-dd HH:mm:ss"));
//                log.info("成功删除月份旧数据：{}条", deleteMouth);
//
//                log.info("历史月份：{}不为空，正在新增数据code:{}", DateUtil.format(startTime, "yyyy-MM-dd HH:mm:ss"), code);
//                //添加新数据
//                int insertMouth = insertMouthBatch(scadaList, tableName);
//                log.info("月份新增合计数据：{}条", insertMouth);
//            }
            String jlj = ".JLJ";
            String defaultValue = "0";

            BigDecimal sum = BigDecimal.ZERO;
            BigDecimal average;
            BigDecimal maxVal = new BigDecimal(scadaList.get(0).getValue());
            BigDecimal minVal = new BigDecimal(scadaList.get(0).getValue());

            if (scadaList.get(0).getCode().contains(jlj)) {
                for (Scada next : scadaList) {
                    if (next.getCode().contains(jlj) && !next.getValue().equals(defaultValue)) {
                        maxVal = new BigDecimal(next.getValue());
                        minVal = maxVal;
                        break;
                    }
                }
            }

            for (Scada scada : scadaList) {
                if (scada.getCode().contains(jlj) && scada.getValue().equals(defaultValue)) {
                    continue;
                }
                BigDecimal decimal = new BigDecimal(scada.getValue());
                sum = decimal.add(sum);
                if (decimal.compareTo(maxVal) > 0) {
                    maxVal = decimal;
                }
                if (!decimal.equals(BigDecimal.ZERO) && decimal.compareTo(minVal) < 0) {
                    minVal = decimal;
                }
            }
            average = sum.divide(BigDecimal.valueOf(scadaList.size()), RoundingMode.HALF_UP);

            StatisticsScada statisticsScada = new StatisticsScada();
            statisticsScada.setCode(scadaList.get(0).getCode());
            statisticsScada.setValue(scadaList.get(scadaList.size() - 1).getValue());
            statisticsScada.setAve(average.toString());
            statisticsScada.setMax(maxVal.toString());
            statisticsScada.setId(SnowFlake.nextId());
            if (CollUtil.isNotEmpty(lastHourList)) {
                List<Double> indexValList = lastHourList.stream().map(s -> Double.parseDouble(s.getValue())).collect(Collectors.toList());
                statisticsScada.setMin(String.valueOf(Collections.max(indexValList)));
            } else {
                statisticsScada.setMin(minVal.toString());
            }
            statisticsScada.setFold(sum.toString());
            statisticsScada.setStartTime(startTime);
            statisticsScada.setEndTime(endTime);
            statisticsScada.setStatisticsType(dateType);

            List<Allocation> allocations = allocationsAll.stream().filter(e -> e.getVariableCode().equals(code.getVariableCode())).collect(Collectors.toList());
            statisticsScada.setDataType(String.valueOf(allocations.get(0).getType()));
            if (CollUtil.isEmpty(scadaReportsAll)) {
                statisticsScadaList.add(statisticsScada);
                continue;
            }
            Set<Long> reportIdSets = getReportIdSets(allocations);
            if (CollUtil.isEmpty(reportIdSets)) {
                statisticsScadaList.add(statisticsScada);
                continue;
            }
            List<ScadaReport> scadaReports = scadaReportsAll.stream().filter(e -> reportIdSets.contains(e.getId())).collect(Collectors.toList());
            if (CollUtil.isEmpty(scadaReports)) {
                statisticsScadaList.add(statisticsScada);
                continue;
            }
            //超标次数
            int errorCount = 0;
            //不合格超标时长
            long errorTimeTotal = 0;
            //超标总时长
            long timeTotal = (scadaList.get(scadaList.size() - 1).getScadaTime().getTime()
                    - scadaList.get(0).getScadaTime().getTime()) / 1000;
            //错误时间
            Date errorTime = null;
            for (Scada scada : scadaList) {
                if (scada.getCode().contains(".JLJ") && scada.getCode().equals("0")) {
                    continue;
                }
                for (ScadaReport report : scadaReports) {
                    if (Double.parseDouble(scada.getValue()) < Double.parseDouble(report.getReportLower())
                            || Double.parseDouble(scada.getValue()) > Double.parseDouble(report.getReportUpper())) {
                        errorCount = errorCount + 1;
                        errorTime = scada.getScadaTime();
                        break;
                    } else if (errorTime != null) {
                        errorTimeTotal = errorTimeTotal +
                                (scada.getScadaTime().getTime() - errorTime.getTime()) / 1000;
                    }
                }
            }
            //合格率
            float pass = 0;
            if (timeTotal != 0) {
                BigDecimal num1 = new BigDecimal(String.valueOf(errorTimeTotal));
                BigDecimal num2 = new BigDecimal(String.valueOf(timeTotal));
                pass = num1.divide(num2, RoundingMode.HALF_UP).floatValue();
            }

            statisticsScada.setErrorCount(String.valueOf(errorCount));
            statisticsScada.setErrorTimeTotal(String.valueOf(errorTimeTotal == 0 ? timeTotal : errorTimeTotal));
            statisticsScada.setPass(String.valueOf(pass));
            statisticsScada.setTimeTotal(String.valueOf(timeTotal));

            statisticsScadaList.add(statisticsScada);
        }
        if (CollUtil.isNotEmpty(statisticsScadaList)) {
            statisticsScadaService.delete(dateType, null,
                    DateUtil.format(startTime, "yyyy-MM-dd HH:mm:ss"),
                    DateUtil.format(endTime, "yyyy-MM-dd HH:mm:ss"));
            statisticsScadaService.saveBatch(statisticsScadaList);
        }
    }

    /**
     * 获取历史数据
     *
     * @param start 开始时间
     * @param end   结束时间
     * @return 历史数据
     */
    public List<Scada> getScadasDatas(Allocation code, Date start, Date end) {
        List<Allocation> allocations = allocationMapper.selectList(Wrappers.<Allocation>lambdaQuery()
                .eq(StringUtils.hasText(code.getVariableCode()), Allocation::getVariableCode, code));
        List<String> codes = allocations.stream().filter(allocation -> allocation.getVariableCode() != null)
                .map(Allocation::getVariableCode).collect(Collectors.toList());

        List<Scada> scadas = new ArrayList<>();
        if (!CollectionUtils.isEmpty(codes)) {
            QueryWrapper<Scada> scadaQueryWrapper = new QueryWrapper<>();
            scadaQueryWrapper.between("SCADA_TIME", start, end)
                    .in("CODE", codes);
            scadas = scadaMapper.selectList(scadaQueryWrapper);
        }
        return scadas;
    }

    public List<Allocation> getCodes(String code) {
        if (StringUtils.hasText(code) && code.contains(",")) {
            String[] codeList = code.split(",");
            List<Allocation> allocations = allocationMapper.selectList(Wrappers.<Allocation>lambdaQuery()
                    .in(!CollectionUtils.isEmpty(Arrays.asList(codeList)), Allocation::getVariableCode, codeList));
            List<Allocation> codes = allocations.stream().filter(allocation -> allocation.getVariableCode() != null).collect(Collectors.toList());

            if (!CollectionUtils.isEmpty(codes)) {
                return codes;
            }
        }
        List<Allocation> allocations = allocationMapper.selectList(Wrappers.<Allocation>lambdaQuery()
                .eq(StringUtils.hasText(code), Allocation::getVariableCode, code));
        List<Allocation> codes = allocations.stream().filter(allocation -> allocation.getVariableCode() != null).collect(Collectors.toList());

        if (!CollectionUtils.isEmpty(codes)) {
            return codes;
        }
        return null;
    }

    @Async
    public void insertMouth(List<Scada> scadas, String tableName) {
        int size = scadas.size();
        int i = 0;
        while (size > 600) {
            scadaMapper.insertMouth(scadas.subList(i, i + 600), tableName);
            i = i + 600;
            size = size - 600;
        }
        if (size > 0) {
            scadaMapper.insertMouth(scadas.subList(i, i + size), tableName);
        }
    }


    @Override
    public List<Scada> getListData() {
        List<Scada> scadaList = scadaMapper.selectMonitoring("SCADA_MONITOR");
        for (Scada scada : scadaList) {
            if (StringUtils.hasText(scada.getValue()) && scada.getValue().startsWith(".")) {
                scada.setValue("0" + scada.getValue());
            }
        }
        return scadaList;
    }

    @Override
    public boolean fetch24HoursData(String start, String end) {
        List<Date> dates = findDates(DateUtil.parse(start, "yyyy-MM-dd HH:mm:ss"), DateUtil.parse(end, "yyyy-MM-dd HH:mm:ss"), 0);
        ThreadPoolExecutor threadPoolExecutor = new ThreadPoolExecutor(
                12,
                24,
                120,
                TimeUnit.SECONDS,
                new ArrayBlockingQueue<>(48),
                (r)->{
                    Thread t = new Thread(r);
                    t.setName("thread-" + ((int)(Math.random()*1000)));
                    return t;},
                new ThreadPoolExecutor.CallerRunsPolicy());
        if (!CollectionUtils.isEmpty(dates)) {
            for (Date date : dates) {
                threadPoolExecutor.execute(() -> insert24H(null, date));
            }
        }
        return true;
    }

    @Override
    public void fetchMonthsData(String syncTime) {
        Date date = null;
        if (StringUtils.isEmpty(syncTime)) {
            date = new Date();
            Calendar cal = Calendar.getInstance();
            cal.setTime(date);
            cal.set(Calendar.MINUTE, 0);
            cal.set(Calendar.SECOND, 0);
        } else {
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
            try {
                Date parse = format.parse(syncTime);
                Date now = new Date();
                Calendar cal = Calendar.getInstance();
                cal.setTime(now);
                if (parse.getTime() > now.getTime()) {
                    throw new RuntimeException("时间参数不允许大于当前时间：" + format.format(now));
                }
                date = parse;
            } catch (ParseException e) {
                throw new RuntimeException("时间参数格式传入错误：" + e);
            }
        }
        syncDataIntoMonths(null, date);
    }

    @Override
    public void fetchDaysData(String syncTime) {
        Date date = null;
        if (StringUtils.isEmpty(syncTime)) {
            date = new Date();
            Calendar cal = Calendar.getInstance();
            cal.setTime(date);
            cal.set(Calendar.MINUTE, 0);
            cal.set(Calendar.SECOND, 0);
        } else {
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
            try {
                Date parse = format.parse(syncTime);
                Date now = new Date();
                Calendar cal = Calendar.getInstance();
                cal.setTime(now);
                if (parse.getTime() > now.getTime()) {
                    throw new RuntimeException("时间参数不允许大于当前时间：" + format.format(now));
                }
                date = parse;
            } catch (ParseException e) {
                throw new RuntimeException("时间参数格式传入错误：" + e);
            }
        }
        syncDaysData(null, date);
    }

    @Override
    public Page<ScadaVo> get24HoursData(Page page, ScadaQuery scadaQuery) {
        QueryWrapper<ScadaQuery> queryWrapper = new QueryWrapper<>();
        if (StringUtils.hasText(scadaQuery.getCode())) {
            queryWrapper.like("hours.CODE", scadaQuery.getCode());
        }
        if (StringUtils.hasText(scadaQuery.getValue())) {
            queryWrapper.like("hours.VALUE", scadaQuery.getValue());
        }
        if (scadaQuery.getStart() != null && scadaQuery.getEnd() != null) {
            queryWrapper.between("hours.SCADA_TIME", scadaQuery.getStart(), scadaQuery.getEnd());
        }
        queryWrapper.orderByDesc("hours.SCADA_TIME");
        return scadaMapper.pageList(page, queryWrapper, "SCADA_HOURS");
    }


    @Override
    public Page getMonthsData(Page page, ScadaQuery scadaQuery) {
        QueryWrapper<ScadaQuery> queryWrapper = new QueryWrapper<>();
        queryWrapper.orderByDesc("hours.SCADA_TIME");
        if (StringUtils.hasText(scadaQuery.getCode())) {
            queryWrapper.like("hours.CODE", scadaQuery.getCode());
        }
        if (StringUtils.hasText(scadaQuery.getValue())) {
            queryWrapper.like("hours.VALUE", scadaQuery.getValue());
        }
        if (scadaQuery.getStart() != null && scadaQuery.getEnd() != null) {

            if (scadaQuery.getStart().getYear() == scadaQuery.getEnd().getYear() &&
                    scadaQuery.getStart().getMonth() == scadaQuery.getEnd().getMonth()) {
                List<Date> startAndEndByDate = TimeUtils.getMonthsToStartAndEndByDate(scadaQuery.getStart());
                queryWrapper.between("hours.SCADA_TIME", scadaQuery.getStart(), scadaQuery.getEnd());

                Calendar cal = Calendar.getInstance();
                cal.setTime(startAndEndByDate.get(0));
                SimpleDateFormat format = new SimpleDateFormat("yyyy_MM");
                String date = format.format(cal.getTime());
                String tableName = "SCADA_MONTH_" + date;
                return scadaMapper.pageList(page, queryWrapper, tableName);
            }
        }
        return scadaMapper.pageList(page, queryWrapper, "SCADA_HISTORY");
    }

    @Override
    public List<DeviceArchiveManageVo> getAppData(DeviceQuery deviceQuery) {
        QueryWrapper<DeviceArchiveManage> queryWrapper = getQueryWrapper(deviceQuery);
        List<DeviceArchiveManage> deviceArchiveManages = deviceArchiveManageMapper.pageList(queryWrapper);
        if (CollectionUtils.isEmpty(deviceArchiveManages)) {
            return null;
        }
        //查询所有指标
        List<Allocation> allocations = allocationMapper.selectList(null);
        //查询所有报警
        List<ScadaReport> scadaReports = reportMapper.selectList(null);
        //查询最新scada数据
        List<Scada> scadas = getListData();

        List<DeviceArchiveManageVo> manageVos = deviceArchiveManages.stream().map(deviceArchiveManage -> {
            DeviceArchiveManageVo vo = new DeviceArchiveManageVo();
            BeanUtils.copyProperties(deviceArchiveManage, vo);
            //查询绑定的指标
            if (!CollectionUtils.isEmpty(allocations)) {
                //获取符合设备绑定的指标
                List<AllocationVo> allocationVos = new HashSet<>(allocations).stream().filter(allocation -> allocation.getDeviceId() != null &&
                        allocation.getDeviceId().equals(vo.getId())).map(allocation -> {
                    AllocationVo allocationVo = new AllocationVo();
                    BeanUtils.copyProperties(allocation, allocationVo);
                    //查询scada数据
                    if (!CollectionUtils.isEmpty(scadas)) {
                        List<Scada> scadaList = scadas.stream()
                                .filter(scada -> scada.getCode() != null && scada.getCode().equals(allocation.getVariableCode()))
                                .collect(Collectors.toList());
                        if (!CollectionUtils.isEmpty(scadaList)) {
                            allocationVo.setScada(scadaList.get(0));
                        }
                    }
                    //设置报警相关信息
                    Set<Long> reportIdSets = getReportIdSets(Collections.singletonList(allocation));
                    if (!CollectionUtils.isEmpty(scadaReports) && !CollectionUtils.isEmpty(reportIdSets)
                            && allocationVo.getIsAlarm() != null
                            && allocationVo.getIsAlarm().equals("1")) {
                        //筛选出报警信息列表
                        List<ScadaReport> reports = scadaReports.stream().filter(scadaReport -> scadaReport.getId() != null
                                && reportIdSets.contains(scadaReport.getId())).collect(Collectors.toList());
                        allocationVo.setScadaReports(reports);
                    }
                    return allocationVo;
                }).collect(Collectors.toList());
                vo.setAllocations(allocationVos);
            }
            return vo;
        }).collect(Collectors.toList());
        return manageVos;
    }


    /**
     * 获取关联指标
     *
     * @param vo
     */
    private void getAllocation(DeviceArchiveManageVo vo) {
        QueryWrapper<Allocation> allocationQueryWrapper = new QueryWrapper<>();
        allocationQueryWrapper.eq("DEVICE_ID", vo.getId());
        List<Allocation> allocations = allocationMapper.selectList(allocationQueryWrapper);
        if (!CollectionUtils.isEmpty(allocations)) {
            List<AllocationVo> allocationVos = allocations.stream().map(allocation -> {
                AllocationVo allocationVo = new AllocationVo();
                BeanUtils.copyProperties(allocation, allocationVo);
                //查询scada数据
                List<Scada> dataByCode = scadaMapper.getListDataByCode(allocation.getVariableCode());
                if (!CollectionUtils.isEmpty(dataByCode)) {
                    allocationVo.setScada(dataByCode.get(0));
                }
                //查询绑定的报警列表
                Set<Long> reportIdSets = getReportIdSets(Collections.singletonList(allocation));
                if (!CollectionUtils.isEmpty(reportIdSets)) {
                    QueryWrapper<ScadaReport> scadaReportQueryWrapper = new QueryWrapper<>();
                    scadaReportQueryWrapper.in("ID", reportIdSets);
                    List<ScadaReport> scadaReports = reportMapper.selectList(scadaReportQueryWrapper);
                    if (!CollectionUtils.isEmpty(scadaReports)) {
                        allocationVo.setScadaReports(scadaReports);
                    }
                }
                //返回数据
                return allocationVo;
            }).collect(Collectors.toList());
            vo.setAllocations(allocationVos);
        }
    }


    /**
     * 设置查询条件
     *
     * @param deviceQuery
     * @return
     */
    private QueryWrapper<DeviceArchiveManage> getQueryWrapper(DeviceQuery deviceQuery) {
        QueryWrapper<DeviceArchiveManage> queryWrapper = new QueryWrapper<>();
        if (!ObjectUtils.isEmpty(deviceQuery)) {
            if (StringUtils.hasText(deviceQuery.getType())) {
                queryWrapper.eq("manage.TYPE", deviceQuery.getType());
            }
            if (StringUtils.hasText(deviceQuery.getDeviceType())) {
                queryWrapper.eq("manage.DEVICE_TYPE", deviceQuery.getDeviceType());
            }
            if (StringUtils.hasText(deviceQuery.getStatus())) {
                queryWrapper.eq("manage.STATUS", deviceQuery.getStatus());
            }
            if (StringUtils.hasText(deviceQuery.getQuery())) {
                queryWrapper.and(q -> {
                    q.like("manage.ADDRESS", deviceQuery.getQuery())
                            .or().like("manage.NAME", deviceQuery.getQuery())
                            .or().like("manage.DESCRIBE", deviceQuery.getQuery());
                });

            }
            if (StringUtils.hasText(deviceQuery.getName())) {
                queryWrapper.like("manage.NAME", deviceQuery.getName());
            }
            if (StringUtils.hasText(deviceQuery.getCode())) {
                queryWrapper.like("allocation.VARIABLE_CODE", deviceQuery.getCode());
            }
            if (deviceQuery.getDeviceId() != null) {
                queryWrapper.eq("manage.ID", deviceQuery.getDeviceId());
            }
            if (deviceQuery.getLatitude() != null && deviceQuery.getLongitude() != null) {
                queryWrapper.eq("manage.LATITUDE", deviceQuery.getLatitude())
                        .eq("manage.LONGITUDE", deviceQuery.getLongitude());
            }
        }
        return queryWrapper;
    }


    @Override
    public void insertData(Scada scada) {
        if (ObjectUtils.isEmpty(scada)) {
            throw new ToflyDeniedException("数据为空");
        }
        if (!ObjectUtils.isEmpty(scada)) {
            //存储到分表月份
            String tableName = createMonths(scada.getScadaTime() == null ? new Date() : scada.getScadaTime());
            //检测数据是否存在，不存在就插入
            if (CollectionUtils.isEmpty(scadaMapper.selectHistoryById(tableName, scada.getId()))) {
                scadaMapper.insertByOne(scada, tableName);
            }
            //检测报警信息
            sendNotice(scada);
        }
    }

    /**
     * 定时任务，每分钟10执行一次
     */
    @Override
    public void timingInsertData(String start, String end) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date startTime = null;
        try {
            startTime = sdf.parse(start);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        Date endTime = null;
        try {
            endTime = sdf.parse(end);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        List<Scada> scadas = scadaMapper.selectList(new QueryWrapper<Scada>().lambda()
                .between(Scada::getScadaTime, startTime, endTime));
        List<Scada> insertScadas = new ArrayList<>();
        List<Scada> updateScadas = new ArrayList<>();
        List<String> monitorCodeList = scadaMapper.selectDataByMonitorList().stream().map(Scada::getCode).collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(scadas)) {
            for (Scada scada : scadas) {
                //实时监控表
                if (scada.getValue().startsWith(".")) {
                    scada.setValue("0" + scada.getValue());
                }
                scada.setUpdateTime(new Date());
                if (!monitorCodeList.contains(scada.getCode())) {
                    if (!insertScadas.stream().map(Scada::getCode).collect(Collectors.toList())
                            .contains(scada.getCode())) {
                        insertScadas.add(scada);
                    }
                } else {
                    if (!updateScadas.stream().map(Scada::getCode).collect(Collectors.toList())
                            .contains(scada.getCode())) {
                        updateScadas.add(scada);
                    }

                }
                //月份表-警告通知
//                insertData(scada);
            }
            if (CollUtil.isNotEmpty(insertScadas)) {
                scadaMapper.insertBatchTemp(insertScadas);
            }
            if (CollUtil.isNotEmpty(updateScadas)) {
                ExecutorService executorService = Executors.newSingleThreadExecutor();
                executorService.execute(() -> {
                    //为了防止SQL语句超出长度出错，分成几次插入
                    if (updateScadas.size() <= 2000) {
                        scadaMapper.updateRealBatchTemp(updateScadas);
                    } else {
                        int times = updateScadas.size() / 2000;
                        for (int i = 0; i <= times; i++) {
                            scadaMapper.updateRealBatchTemp(updateScadas.subList(i * 2000, Math.min((i + 1) * 2000, updateScadas.size())));
                        }
                    }
                });
            }
        }
    }

    @Override
    public void fetchData(String code, String start, String end, Integer type) {
        Date startTime = null;
        Date endTime = null;
        try {
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
            startTime = format.parse(start);
            endTime = format.parse(end);
        } catch (Exception e) {
            throw new RuntimeException("时间格式有误");
        }
        switch (type) {
            case 0: {
                List<Date> dates = findDates(startTime, endTime, 0);
                if (!CollectionUtils.isEmpty(dates)) {
                    for (Date date : dates) {
                        insert24H(code, date);
                    }
                }
                break;
            }
            case 1: {
                List<Date> dates = findDates(startTime, endTime, 1);
                if (!CollectionUtils.isEmpty(dates)) {
                    for (Date date : dates) {
                        syncDaysData(code, date);
                    }
                }
                break;
            }
            case 2: {
                List<Date> dates = findDates(startTime, endTime, 2);
                if (!CollectionUtils.isEmpty(dates)) {
                    for (Date date : dates) {
                        syncDataIntoMonths(code, date);
                    }
                }
                break;
            }
            case 3: {
                //TODO 暂时不同步年数据
//                List<Date> dates = findDates(startTime, endTime,0);
//                if (!CollectionUtils.isEmpty(dates)){
//                    for (Date date : dates) {
//                        syncDaysData(date);
//                    }
//                }
                break;
            }
        }
    }

    @Override
    public Page<StatisticsScadaVo> selectHistoryByMouthsTables(List<String> mouthTables, Page page,
                                                               QueryWrapper<ScadaQuery> queryWrapper, boolean isPage) {
        List<String> stringList = new ArrayList<>();
        stringList.add("SCADA_HISTORY");
        mouthTables = stringList;
        if (isPage) {
            Page<StatisticsScadaVo> statisticsScadaVoPage = baseMapper.selectHistoryByMouthsTables(mouthTables, page, queryWrapper);
            //特殊值处理
            for (StatisticsScadaVo scadaVo : statisticsScadaVoPage.getRecords()) {
                if (scadaVo.getValue().startsWith(".")) {
                    String value = scadaVo.getValue();
                    scadaVo.setValue("0" + value);
                }
            }
            return statisticsScadaVoPage;
        } else {
            Page<StatisticsScadaVo> pageDb = new Page<>();
            List<StatisticsScadaVo> statisticsScadaVoPage = baseMapper.selectHistoryByMouthsTablesNoPage(mouthTables, queryWrapper);
            pageDb.setRecords(statisticsScadaVoPage);
            //特殊值处理
            for (StatisticsScadaVo scadaVo : pageDb.getRecords()) {
                if (scadaVo.getValue().startsWith(".")) {
                    String value = scadaVo.getValue();
                    scadaVo.setValue("0" + value);
                }
            }
            return pageDb;
        }
    }

    @Override
    public void fetchDataMouth(String code, String startStr, String endStr) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        Date end = DateUtil.parseDateTime(endStr);
        calendar.add(Calendar.MINUTE, -1);
        Date start = DateUtil.parseDateTime(startStr);

        List<Allocation> codes = getCodes(code);
        if (CollectionUtils.isEmpty(codes)) {
            return;
        }
        List<Scada> scadas = scadaMapper.selectList(new QueryWrapper<Scada>().lambda()
                .between(Scada::getScadaTime, start, end)
                .in(Scada::getCode, codes));
        if (!CollectionUtils.isEmpty(scadas)) {
            for (Scada scada : scadas) {
                //实时监控表
                if (scada.getValue().startsWith(".")) {
                    scada.setValue("0" + scada.getValue());
                }
                Scada monitoring = scadaMapper.selectDataByMonitoring(scada.getCode());
                if (ObjectUtils.isEmpty(monitoring)) {
                    scadaMapper.insertMonitoring(scada);
                } else {
                    scadaMapper.updateMonitoring(scada);
                }
                //月份表-警告通知
                insertData(scada);
            }
        }
    }

    /**
     * 获取日期列表
     *
     * @param start 开始时间:2022-01-01 00:00:00
     * @param end   结束时间:2022-10-01 14:00:00
     * @param type  统计类型0天，1月份，2年份
     * @return
     */
    public static List<Date> findDates(Date start, Date end, Integer type) {
        Calendar s = Calendar.getInstance();
        s.setTime(start);
        Calendar e = Calendar.getInstance();
        e.setTime(end);
        if (start.getYear() == end.getYear() && start.getMonth() == end.getMonth() && start.getDay() == end.getDay() && type != 0) {
            List<Date> dates = new ArrayList<>();
            dates.add(start);
            return dates;
        }
        if (start.getYear() == end.getYear() && start.getMonth() == end.getMonth() && type == 1) {
            List<Date> dates = new ArrayList<>();
            dates.add(start);
            return dates;
        }
        //结束时间在开始时间之后
        List<Date> dates = new ArrayList<>();
        dates.add(s.getTime());
        while (end.getTime() >= s.getTime().getTime()) {
            if (type == 0) {
                s.add(Calendar.HOUR_OF_DAY, 1);
            } else if (type == 1) {
                s.add(Calendar.DAY_OF_MONTH, 1);
            } else if (type == 2) {
                s.add(Calendar.MONTH, 1);
            }
            dates.add(s.getTime());
        }
        dates.remove(dates.size() - 1);
        return dates;
    }


    /**
     * 异步执行同步信息
     *
     * @param data
     */
    public void syncSave(Scada data) {
        if (ObjectUtils.isEmpty(data)) {
            throw new RuntimeException("参数为空");
        }

        ExecutorService executorService = Executors.newSingleThreadExecutor();
        executorService.submit(() -> {
            try {
                Thread.sleep(5000);
                syncScadaData(data);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        });
//        executorService.shutdown(); // 回收线程池

        try {
            Thread.sleep(2000);
        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        }
    }

    private void syncScadaData(Scada scada) {
//        Scada scada = null;
//        if (data.getId() != null) {
//            scada = scadaMapper.selectById(data.getId());
//        } else if (StringUtils.hasText(data.getCode())) {
//            Page<Scada> page = new Page<>(1, 1);
//            QueryWrapper<Scada> scadaQueryWrapper = new QueryWrapper<>();
//            scadaQueryWrapper.eq("CODE", data.getCode())
//                    .orderByDesc("SCADA_TIME");
//            Page<Scada> scadas = scadaMapper.selectPage(page, scadaQueryWrapper);
//            List<Scada> records = scadas.getRecords();
//            if (!CollectionUtils.isEmpty(records)) {
//                scada = records.get(0);
//            }
//        }

        if (!ObjectUtils.isEmpty(scada)) {
            //检测是否为系统创建的指标
            Integer selectCount = allocationMapper.selectCount(new QueryWrapper<Allocation>()
                    .eq("VARIABLE_CODE", scada.getCode()));
            if (selectCount == 0) {
                throw new RuntimeException("数据不存在指标绑定配置，无法进行同步更新存储");
            }
            //存储到历史表
            scadaMapper.insert(scada);
            //存储到分表月份
            String tableName = createMonths(scada.getScadaTime() == null ? new Date() : scada.getScadaTime());
            //检测数据是否存在，不存在就插入
            if (CollectionUtils.isEmpty(scadaMapper.selectHistoryById(tableName, scada.getId()))) {
                scadaMapper.insertByOne(scada, tableName);
            }
            //检测报警信息
            sendNotice(scada);
        }
    }

    /**
     * 发送报警信息
     *
     * @param scada scada数据
     */
    private void sendNotice(Scada scada) {
        QueryWrapper<Allocation> allocationQueryWrapper = new QueryWrapper<>();
        allocationQueryWrapper.eq("VARIABLE_CODE", scada.getCode())
                .eq("IS_ALARM", "1");
        List<Allocation> allocations = allocationMapper.selectList(allocationQueryWrapper);
        if (!CollectionUtils.isEmpty(allocations)) {
            for (Allocation allocation : allocations) {
                if (!StringUtils.hasText(allocation.getIsAlarm()) || allocation.getIsAlarm().equals("0")) {
                    break;
                }

                List<Long> reports = getReportIds(Collections.singletonList(allocation));
                QueryWrapper<ScadaReport> scadaReportQueryWrapper = new QueryWrapper<>();
                scadaReportQueryWrapper.eq("IS_DEL", FlowConstant.SHORT_ZERO)
                        .in("ID", reports);
                List<ScadaReport> scadaReports = reportMapper.selectList(scadaReportQueryWrapper);
                if (!CollectionUtils.isEmpty(scadaReports)) {
                    for (ScadaReport scadaReport : scadaReports) {
                        if (Double.parseDouble(scadaReport.getReportUpper()) <= Double.parseDouble(scada.getValue())
                                || Double.parseDouble(scadaReport.getReportLower()) >= Double.parseDouble(scada.getValue())) {
                            MessagepushW messagepush = new MessagepushW();
                            // TODO 报警模版后续处理
                            messagepush.setTopic(MessageTopicEnum.REPORT_MSG.getTopic());
                            messagepush.setMessage(createMsg(scadaReport.getMsgTemplate(), scadaReport));
                            messagepush.setType("8");
                            messagepush.setTableName(MessageTopicEnum.REPORT_MSG.getTableName());
                            messagepush.setCreateTime(scada.getScadaTime());
                            sendTidingsUtil.sendMsg(messagepush, scadaReport.getLiaisonPeople());
                            //保存报警信息
                            saveScadaAlarm(scada);
                        }
                    }
                } else {
                    saveScadaNoAlarm(scada);
                }
            }
        } else {
            saveScadaNoAlarm(scada);
        }
    }

    /**
     * 保存不报警列表信息
     * @param scada 数据
     */
    private void saveScadaNoAlarm(Scada scada) {
        ScadaNoAlarm noAlarm = new ScadaNoAlarm();
        noAlarm.setAllocationId(scada.getCode());
        noAlarm.setReport(0);
        noAlarm.setScadaTime(scada.getScadaTime());
        noAlarm.setCreateTime(new Date());
        noAlarm.setValue(scada.getValue());
        scadaNoAlarmService.save(noAlarm);
    }


    /**
     * 保存报警列表信息
     * @param scada 数据
     */
    private void saveScadaAlarm(Scada scada) {
        ScadaAlarm alarm = new ScadaAlarm();
        alarm.setAllocationId(scada.getCode());
        alarm.setReport(0);
        alarm.setScadaTime(scada.getScadaTime());
        alarm.setCreateTime(new Date());
        alarm.setValue(scada.getValue());
        scadaAlarmService.save(alarm);
    }


    /**
     * 构建信息内容模版
     *
     * @param template
     * @param scadaReport
     * @return
     */
    public String createMsg(String template, ScadaReport scadaReport) {
        if (org.apache.commons.lang3.StringUtils.isBlank(template)) {
            return null;
        }
        if (template.contains("#TIME#")) {
            String date = DateUtil.format(new Date(), "yyyy-MM-dd HH:mm:ss");
            template = template.replaceAll("#TIME#", date);
        }
        if (template.contains("#YHXM#")) {
            String user = scadaReportService.getUser(scadaReport.getLiaisonPeople());
            template = template.replace(user, "#YHXM#");
        }
        if (template.contains("#LXDH#")) {
            template = template.replace(scadaReport.getLiaisonPhone(), "#LXDH#");
        }
        return template;
    }

    /**
     * 获取报警ID集合
     *
     * @param allocations
     * @return
     */
    private List<Long> getReportIds(List<Allocation> allocations) {
        List<Long> reports = allocations.stream().filter(allocation -> StringUtils.hasText(allocation.getReportId()))
                .map(allocation -> {
                    if (allocation.getReportId().contains(",")) {
                        String[] reportIds = allocation.getReportId().split(",");
                        for (String reportId : reportIds) {
                            return Long.parseLong(reportId);
                        }
                    } else {
                        return Long.parseLong(allocation.getReportId());
                    }
                    return null;
                }).collect(Collectors.toList());
        return reports;
    }

    /**
     * 获取报警ID集合(去重)
     *
     * @param allocations
     * @return
     */
    public static Set<Long> getReportIdSets(List<Allocation> allocations) {
        if (CollUtil.isEmpty(allocations)) {
            return null;
        }
        Set<Long> reports = allocations.stream().filter(allocation -> StringUtils.hasText(allocation.getReportId()))
                .map(allocation -> {
                    if (allocation.getReportId().contains(",")) {
                        String[] reportIds = allocation.getReportId().split(",");
                        for (String reportId : reportIds) {
                            return Long.parseLong(reportId);
                        }
                    } else {
                        return Long.parseLong(allocation.getReportId());
                    }
                    return null;
                }).collect(Collectors.toSet());
        return reports;
    }

//    /**
//     * 累计流量差值处理(5分钟求值)
//     */
//    // 每日 6点 15点执行
//    @Transactional
//    public void sendCumulativeFlowDifference() {
//        String startDate = "";
//        String endDate = "";
//        Date startTime;
//        Date endTime;
//        // 时间判断 为空默认前一天天开始时间 结束时间(当前时间)
//        if (StringUtils.isEmpty(startDate) || StringUtils.isEmpty(endDate)) {
//            LocalDate currentDay = LocalDate.now().minusDays(1);
//            startTime = Date.from(LocalDateTime.of(currentDay, LocalTime.MIN).atZone(ZoneId.systemDefault()).toInstant());
//            endTime = new Date();
//        } else {
//            DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
//            startTime = Date.from(LocalDateTime.parse(startDate, df).atZone(ZoneId.systemDefault()).toInstant());
//            endTime = Date.from(LocalDateTime.parse(endDate, df).atZone(ZoneId.systemDefault()).toInstant());
//        }
//        // 获取当前时间各指标流量计5分钟差值
//        List<ScadaLjFlow> flowDifference = scadaLjFlowService.getSendCumulativeFlowDifference(startTime, endTime);
//        // 先清空范围内时间数据 再插入
//        scadaLjFlowService.remove(new QueryWrapper<ScadaLjFlow>().lambda().between(ScadaLjFlow::getScadaTime, startTime, endTime));
//
//        scadaLjFlowService.saveBatch(new HashSet<>(flowDifference));
//    }

    @Transactional(rollbackFor = Exception.class)
    public void sendCumulativeFlowDifference(String startTime, String endTime) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(DateUtil.parse(startTime,"yyyy-MM-dd HH:mm:ss"));
        cal.add(Calendar.MINUTE, -5);
        WisdomDisplayDto dto = new WisdomDisplayDto();
        dto.setStartDate(DateUtil.format(cal.getTime(), "yyyy-MM-dd HH:mm:ss"));
        dto.setEndDate(endTime);
        List<String> indexTypeList = new ArrayList<>();
        indexTypeList.add("2");
        dto.setIndexTypeList(indexTypeList);
        List<StationIndexVo> stationIndexVos = scadaMapper.viewStationIndex(dto);
        List<Allocation> codes = allocationMapper.selectList(new QueryWrapper<Allocation>().lambda()
                .eq(Allocation::getType, "2"));
        List<Map<String, String>> dates = findDiffFiveDates(DateUtil.parse(startTime, "yyyy-MM-dd HH:mm:ss"),
                DateUtil.parse(endTime, "yyyy-MM-dd HH:mm:ss"));
        List<ScadaLjFlow> scadaLjFlowList = new ArrayList<>();
        dates.forEach(info ->  {
            List<StationIndexVo> stationIndexVosF = stationIndexVos.stream()
                    .filter(e -> e.getScadaTime().compareTo(info.get("startTime")) >= 0 && e.getScadaTime().compareTo(info.get("endTime")) <= 0)
                    .collect(Collectors.toList());
            if (CollUtil.isEmpty(stationIndexVosF)) {
                return;
            }
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(DateUtil.parse(info.get("startTime"),"yyyy-MM-dd HH:mm:ss"));
            calendar.add(Calendar.MINUTE, -1);
            calendar.set(Calendar.SECOND, 59);
            Date endTimeLast = calendar.getTime();
            calendar.add(Calendar.MINUTE, -4);
            calendar.set(Calendar.SECOND, 0);
            Date startTimeLast = calendar.getTime();
            String start = DateUtil.format(startTimeLast,"yyyy-MM-dd HH:mm:ss");
            String end = DateUtil.format(endTimeLast,"yyyy-MM-dd HH:mm:ss");
            List<StationIndexVo> stationIndexVosLast = stationIndexVos.stream()
                    .filter(e -> e.getScadaTime().compareTo(start) >= 0 && e.getScadaTime().compareTo(end) <= 0)
                    .collect(Collectors.toList());
            codes.forEach(code -> {
                List<StationIndexVo> stationIndexVosFinal = stationIndexVosF.stream().filter(e -> e.getIndexCode().equals(code.getVariableCode())).collect(Collectors.toList());
                if (CollUtil.isEmpty(stationIndexVosFinal)) {
                    return;
                }
                double newVal = Double.parseDouble(stationIndexVosFinal.get(0).getValue());
                double oldVal = 0;
                if (CollUtil.isNotEmpty(stationIndexVosLast)) {
                    List<StationIndexVo> stationIndexVosLastFinal = stationIndexVosLast.stream().filter(e -> e.getIndexCode().equals(code.getVariableCode())).collect(Collectors.toList());
                    if (CollUtil.isNotEmpty(stationIndexVosLastFinal)) {
                        oldVal = Double.parseDouble(stationIndexVosLastFinal.get(0).getValue());
                    }
                }
                double difVal = oldVal != 0 ? new BigDecimal(newVal).subtract(new BigDecimal(oldVal)).doubleValue() : 0;
                ScadaLjFlow scadaLjFlow = new ScadaLjFlow();
                scadaLjFlow.setScadaTime(DateUtil.parse(info.get("startTime"), "yyyy-MM-dd HH:mm:ss"));
                scadaLjFlow.setId(SnowFlake.nextId());
                scadaLjFlow.setCode(code.getVariableCode());
                scadaLjFlow.setOldValue(BigDecimal.valueOf(oldVal));
                scadaLjFlow.setNewValue(BigDecimal.valueOf(newVal));
                scadaLjFlow.setDifferenceValue(BigDecimal.valueOf(difVal));
                scadaLjFlowList.add(scadaLjFlow);
            });
        });
        if (CollUtil.isEmpty(scadaLjFlowList)) {
            return;
        }
        // 先清空范围内时间数据 再插入
        scadaLjFlowService.remove(new QueryWrapper<ScadaLjFlow>().lambda()
                .between(ScadaLjFlow::getScadaTime, DateUtil.parse(startTime,"yyyy-MM-dd HH:mm:ss"),
                        DateUtil.parse(endTime,"yyyy-MM-dd HH:mm:ss")));
        scadaLjFlowService.saveBatch(scadaLjFlowList);
    }

    /**
     *
     * @param startDate 开始时间 yyyy-MM-dd HH:mm:ss
     * @param endDate   结束时间 yyyy-MM-dd HH:mm:ss
     * @description 获取五分钟间隔时间列表
     */
    public static List<Map<String, String>> findDiffFiveDates(Date startDate, Date endDate) {
        List<Map<String, String>> dates = new ArrayList<>();
        Calendar startTime = Calendar.getInstance();
        Calendar midTime = Calendar.getInstance();
        startTime.setTime(startDate);
        midTime.setTime(startDate);
        midTime.set(Calendar.MINUTE, 4);
        midTime.set(Calendar.SECOND, 59);
        while (midTime.getTime().compareTo(endDate) <= 0) {
            Map<String, String> timeMap = new HashMap<>();
            timeMap.put("startTime", DateUtil.format(startTime.getTime(), "yyyy-MM-dd HH:mm:ss"));
            timeMap.put("endTime", DateUtil.format(midTime.getTime(), "yyyy-MM-dd HH:mm:ss"));
            dates.add(timeMap);
            startTime.add(Calendar.MINUTE, +5);
            midTime.add(Calendar.MINUTE, +5);
        }
        return dates;
    }

//    @Override
//    @Transactional
//    public void sendCumulativeFlowDifference(String startDate, String endDate) {
//        Date startTime = null;
//        Date endTime = null;
//        // 时间判断 为空默认前一天天开始时间 结束时间
//        if (StringUtils.isEmpty(startDate) || StringUtils.isEmpty(endDate)) {
//            LocalDate currentDay = LocalDate.now().minusDays(1);
//            // 默认开始时间-6分钟 获取到前天末尾的数据 用于计算差值
//            startTime = Date.from(LocalDateTime.of(currentDay, LocalTime.MIN).atZone(ZoneId.systemDefault()).toInstant());
//            endTime = Date.from(LocalDateTime.of(currentDay, LocalTime.MAX).atZone(ZoneId.systemDefault()).toInstant());
//        } else {
//            DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
//            // 默认开始时间-6分钟 获取到前天末尾的数据 用于计算差值
//            startTime = Date.from(LocalDateTime.parse(startDate, df).atZone(ZoneId.systemDefault()).toInstant());
//            endTime = Date.from(LocalDateTime.parse(endDate, df).atZone(ZoneId.systemDefault()).toInstant());
//        }
//        // 获取当前时间各指标流量计5分钟差值
//        List<ScadaLjFlow> flowDifference = scadaLjFlowService.getSendCumulativeFlowDifference(startTime, endTime);
//        // 先清空范围内时间数据 再插入
//        scadaLjFlowService.remove(new QueryWrapper<ScadaLjFlow>().lambda().between(ScadaLjFlow::getScadaTime, startTime, endTime));
//
//        scadaLjFlowService.saveBatch(new HashSet<>(flowDifference));
//    }

    @Override
    public void origDataToScadaHistory(String startDate, String endDate, String timeColumn) {
        List<ScadaDeviceData> deviceDatas;
        if (timeColumn.equals("GENERATION_TIME")) {
            deviceDatas = deviceDataService.list(new QueryWrapper<ScadaDeviceData>()
                    .between(timeColumn, startDate, endDate));
        } else {
            deviceDatas = deviceDataService.list(new QueryWrapper<ScadaDeviceData>()
                    .between("to_char(" + timeColumn + ",'yyyy-mm-dd hh24:mi:ss')", startDate, endDate));
        }
        if (CollUtil.isEmpty(deviceDatas)) {
            return;
        }
        List<Scada> scadaHistoryList = new ArrayList<>();
        for (ScadaDeviceData scadaDeviceData : deviceDatas) {
            //获取时间
            Date generationTime = StringUtils.hasText(scadaDeviceData.getGenerationTime())
                    ? DateUtil.parse(scadaDeviceData.getGenerationTime(), YYYY_MM_DD_HH_MM_SS) : new Date();
            //保存指标：压力
            if (StringUtils.hasText(scadaDeviceData.getPressure())) {
                Scada history = new Scada();
                history.setId(SnowFlake.nextId());
                history.setCode(scadaDeviceData.getDeviceName() + "#" + JsonItemConst.g_pressure);
                history.setValue(matcherVal(scadaDeviceData.getPressure()));
                history.setScadaTime(generationTime);
                history.setUpdateTime(new Date());
                scadaHistoryList.add(history);
            }
            //保存指标：正向累计流量
            if (StringUtils.hasText(scadaDeviceData.getCumPosFlows())) {
                Scada history = new Scada();
                history.setId(SnowFlake.nextId());
                history.setCode(scadaDeviceData.getDeviceName() + "#" + JsonItemConst.g_cum_pos_flows);
                history.setValue(matcherVal(scadaDeviceData.getCumPosFlows()));
                history.setScadaTime(generationTime);
                history.setUpdateTime(new Date());
                scadaHistoryList.add(history);
            }
            //保存指标：逆向累计流量
            if (StringUtils.hasText(scadaDeviceData.getCumNegFlows())) {
                Scada history = new Scada();
                history.setId(SnowFlake.nextId());
                history.setCode(scadaDeviceData.getDeviceName() + "#" + JsonItemConst.g_cum_neg_flows);
                history.setValue(matcherVal(scadaDeviceData.getCumNegFlows()));
                history.setScadaTime(generationTime);
                history.setUpdateTime(new Date());
                scadaHistoryList.add(history);
            }
            //保存指标：瞬时流量
            if (StringUtils.hasText(scadaDeviceData.getInstantaneousFlow())) {
                Scada history = new Scada();
                history.setId(SnowFlake.nextId());
                history.setCode(scadaDeviceData.getDeviceName() + "#" + JsonItemConst.g_instantaneous_flow);
                history.setValue(matcherVal(scadaDeviceData.getInstantaneousFlow()));
                history.setScadaTime(generationTime);
                history.setUpdateTime(new Date());
                scadaHistoryList.add(history);
            }
            //保存指标：信号质量
            if (StringUtils.hasText(scadaDeviceData.getSignalQuality())) {
                Scada history = new Scada();
                history.setId(SnowFlake.nextId());
                history.setCode(scadaDeviceData.getDeviceName() + "#" + JsonItemConst.g_signal_quality);
                history.setValue(matcherVal(scadaDeviceData.getSignalQuality()));
                history.setScadaTime(generationTime);
                history.setUpdateTime(new Date());
                scadaHistoryList.add(history);
            }
            //保存指标：电池电压
            if (StringUtils.hasText(scadaDeviceData.getBatteryVoltage())) {
                Scada history = new Scada();
                history.setId(SnowFlake.nextId());
                history.setCode(scadaDeviceData.getDeviceName() + "#" + JsonItemConst.g_battery_voltage);
                history.setValue(matcherVal(scadaDeviceData.getBatteryVoltage()));
                history.setScadaTime(generationTime);
                history.setUpdateTime(new Date());
                scadaHistoryList.add(history);
            }
        }
        if (CollUtil.isEmpty(scadaHistoryList)) {
            return;
        }
        if (timeColumn.equals("GENERATION_TIME")) {
            scadaMapper.deleteHistoryData(startDate, endDate);
        }
        ThreadPoolExecutor threadPoolExecutor = new ThreadPoolExecutor(
                4,
                4,
                120,
                TimeUnit.SECONDS,
                new ArrayBlockingQueue<>(50),
                (r)->{
                    Thread t = new Thread(r);
                    t.setName("thread-" + ((int)(Math.random()*1000)));
                    return t;},
                new ThreadPoolExecutor.CallerRunsPolicy());
        //为了防止SQL语句超出长度出错，分成几次插入
        if (scadaHistoryList.size() <= 200) {
            threadPoolExecutor.execute(() -> scadaMapper.insertBatchTempSet(new HashSet<>(scadaHistoryList)));
        } else {
            int times = scadaHistoryList.size() / 200;
            for (int i = 0; i <= times; i++) {
                int finalI = i;
                threadPoolExecutor.execute(() ->
                        scadaMapper.insertBatchTempSet(new HashSet<>(scadaHistoryList.subList(finalI * 200, Math.min((finalI + 1) * 200, scadaHistoryList.size()))))
                );
            }
        }

        // 更新实时数据
        List<Scada> list = scadaHistoryList;
        list = list.stream()
                .sorted(Comparator.comparing(Scada::getScadaTime).reversed())
                .collect(Collectors.collectingAndThen(Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(Scada::getCode))), ArrayList::new));
        List<Scada> indexValNewList = new ArrayList<>();
        List<Scada> indexValNewUpdateList = new ArrayList<>();
        // 查询实时表所有指标
        List<String> monitorCodeList = scadaMapper.selectDataByMonitorList().stream().map(Scada::getCode).collect(Collectors.toList());
        for (Scada scada : list) {
            Scada indexVal = new Scada();
            indexVal.setCode(scada.getCode());
            indexVal.setValue(scada.getValue());
            indexVal.setScadaTime(scada.getScadaTime());
            indexVal.setUpdateTime(new Date());
            if (monitorCodeList.contains(scada.getCode())) {
                indexValNewUpdateList.add(indexVal);
            } else {
                indexVal.setId(SnowFlake.nextId());
                indexValNewList.add(indexVal);
            }
        }
        if (CollUtil.isNotEmpty(indexValNewList)) {
            scadaMapper.insertBatchTemp(indexValNewList);
        }
        if (CollUtil.isNotEmpty(indexValNewUpdateList)) {
            for (Scada scada : indexValNewUpdateList) {
                scadaMapper.updateRealTemp(scada);
            }
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void origDeviceAndIndex() {
        List<DeviceArchiveManage> manages = deviceArchiveManageService.list();
        // 存储设备和指标数据
        List<Allocation> parameterList = new ArrayList<>();
        List<Allocation> parameters = allocationService.list();
        for (DeviceArchiveManage deviceArchiveManage : manages) {
            List<Allocation> parametersFinal = new ArrayList<>();
            if (CollUtil.isNotEmpty(parameters)) {
                parametersFinal = parameters.stream()
                        .filter(e -> Objects.equals(e.getDeviceId(), deviceArchiveManage.getId()))
                        .collect(Collectors.toList());
            }
            parameterList.addAll(saveParameters(deviceArchiveManage, parametersFinal));
        }
        if (CollUtil.isNotEmpty(parameterList)) {
            parameterList = parameterList.stream().collect(Collectors.collectingAndThen(Collectors
                    .toCollection(() -> new TreeSet<>(Comparator.comparing(Allocation::getVariableCode))), ArrayList::new));
            parameterList.forEach(allocationService::save);
        }
    }

    private List<Allocation> saveParameters(DeviceArchiveManage deviceArchiveManage, List<Allocation> parameters) {
        List<Allocation> allocations = new ArrayList<>();
        // 保存指标：压力
        List<Allocation> pressureList = CollUtil.isNotEmpty(parameters) ? parameters.stream()
                .filter(e -> e.getVariableCode().equals(deviceArchiveManage.getCode() + "#" + JsonItemConst.g_pressure))
                .collect(Collectors.toList()):null;
        if (CollUtil.isEmpty(pressureList)) {
            Allocation allocation = new Allocation();
            allocation.setDisplayName("压力");
            allocation.setVariableCode(deviceArchiveManage.getCode() + "#" + JsonItemConst.g_pressure);
            allocation.setVariableName(deviceArchiveManage.getCode() + "#" + JsonItemConst.g_pressure);
            allocation.setDeviceId(deviceArchiveManage.getId());
            allocation.setUnit("Mpa");
            allocation.setType(dictConfig.getPressure());
            allocation.setAccuracy(1L);
            allocation.setModel("4");
            allocation.setSortNumber("4");
            allocation.setArchivesCode(deviceArchiveManage.getCode());
            allocations.add(allocation);
        }
        // 保存指标：逆向累计流量
        List<Allocation> cumNegFlowList = CollUtil.isNotEmpty(parameters) ? parameters.stream()
                .filter(e -> e.getVariableCode().equals(deviceArchiveManage.getCode() + "#" + JsonItemConst.g_cum_neg_flows))
                .collect(Collectors.toList()):null;
        if (CollUtil.isEmpty(cumNegFlowList)) {
            Allocation allocation = new Allocation();
            allocation.setDisplayName("逆向累计流量");
            allocation.setVariableCode(deviceArchiveManage.getCode() + "#" + JsonItemConst.g_cum_neg_flows);
            allocation.setVariableName(deviceArchiveManage.getCode() + "#" + JsonItemConst.g_cum_neg_flows);
            allocation.setDeviceId(deviceArchiveManage.getId());
            allocation.setUnit("m³");
            allocation.setType(dictConfig.getNegCumFlow());
            allocation.setAccuracy(1L);
            allocation.setModel("4");
            allocation.setSortNumber("3");
            allocation.setArchivesCode(deviceArchiveManage.getCode());
            allocations.add(allocation);
        }
        // 保存指标：正向累计流量
        List<Allocation> cumPosFlowList = CollUtil.isNotEmpty(parameters) ? parameters.stream()
                .filter(e -> e.getVariableCode().equals(deviceArchiveManage.getCode() + "#" + JsonItemConst.g_cum_pos_flows))
                .collect(Collectors.toList()):null;
        if (CollUtil.isEmpty(cumPosFlowList)) {
            Allocation allocation = new Allocation();
            allocation.setDisplayName("正向累计流量");
            allocation.setVariableCode(deviceArchiveManage.getCode() + "#" + JsonItemConst.g_cum_pos_flows);
            allocation.setVariableName(deviceArchiveManage.getCode() + "#" + JsonItemConst.g_cum_pos_flows);
            allocation.setDeviceId(deviceArchiveManage.getId());
            allocation.setUnit("m³");
            allocation.setType(dictConfig.getMomentFlowAllIndex());
            allocation.setAccuracy(1L);
            allocation.setModel("4");
            allocation.setSortNumber("2");
            allocation.setArchivesCode(deviceArchiveManage.getCode());
            allocations.add(allocation);
        }
        // 保存指标：瞬时流量
        List<Allocation> instantaneousFlowList = CollUtil.isNotEmpty(parameters) ? parameters.stream()
                .filter(e -> e.getVariableCode().equals(deviceArchiveManage.getCode() + "#" + JsonItemConst.g_instantaneous_flow))
                .collect(Collectors.toList()):null;
        if (CollUtil.isEmpty(instantaneousFlowList)) {
            Allocation allocation = new Allocation();
            allocation.setDisplayName("瞬时流量");
            allocation.setVariableCode(deviceArchiveManage.getCode() + "#" + JsonItemConst.g_instantaneous_flow);
            allocation.setVariableName(deviceArchiveManage.getCode() + "#" + JsonItemConst.g_instantaneous_flow);
            allocation.setDeviceId(deviceArchiveManage.getId());
            allocation.setUnit("m³/h");
            allocation.setType(dictConfig.getNightFlowAllIndex());
            allocation.setAccuracy(1L);
            allocation.setModel("4");
            allocation.setSortNumber("1");
            allocation.setArchivesCode(deviceArchiveManage.getCode());
            allocations.add(allocation);
        }
        // 保存指标：电池电压
        List<Allocation> voltageFlowAllIndexList = CollUtil.isNotEmpty(parameters) ? parameters.stream()
                .filter(e -> e.getVariableCode().equals(deviceArchiveManage.getCode() + "#" + JsonItemConst.g_battery_voltage))
                .collect(Collectors.toList()):null;
        if (CollUtil.isEmpty(voltageFlowAllIndexList)) {
            Allocation allocation = new Allocation();
            allocation.setDisplayName("电池电压");
            allocation.setVariableCode(deviceArchiveManage.getCode() + "#" + JsonItemConst.g_battery_voltage);
            allocation.setVariableName(deviceArchiveManage.getCode() + "#" + JsonItemConst.g_battery_voltage);
            allocation.setDeviceId(deviceArchiveManage.getId());
            allocation.setUnit("v");
            allocation.setType(dictConfig.getVoltageFlowAllIndex());
            allocation.setAccuracy(1L);
            allocation.setModel("4");
            allocation.setSortNumber("6");
            allocation.setArchivesCode(deviceArchiveManage.getCode());
            allocations.add(allocation);
        }
        // 保存指标：信号质量
        List<Allocation> signalQualityList = CollUtil.isNotEmpty(parameters) ? parameters.stream()
                .filter(e -> e.getVariableCode().equals(deviceArchiveManage.getCode() + "#" + JsonItemConst.g_signal_quality))
                .collect(Collectors.toList()):null;
        if (CollUtil.isEmpty(signalQualityList)) {
            Allocation allocation = new Allocation();
            allocation.setDisplayName("瞬时流量");
            allocation.setVariableCode(deviceArchiveManage.getCode() + "#" + JsonItemConst.g_signal_quality);
            allocation.setVariableName(deviceArchiveManage.getCode() + "#" + JsonItemConst.g_signal_quality);
            allocation.setDeviceId(deviceArchiveManage.getId());
            allocation.setType(dictConfig.getSignalQuality());
            allocation.setAccuracy(1L);
            allocation.setModel("4");
            allocation.setSortNumber("5");
            allocation.setArchivesCode(deviceArchiveManage.getCode());
            allocations.add(allocation);
        }
        return allocations;
    }

    @Override
    public void updateMonitorData(String startTime, String endTime) {
        // 更新实时数据
        List<Scada> list = scadaMapper.selectHistoryData(startTime, endTime);
        if (CollUtil.isEmpty(list)) {
            return;
        }
        list = list.stream()
                .sorted(Comparator.comparing(Scada::getScadaTime).reversed())
                .collect(Collectors.collectingAndThen(Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(Scada::getCode))), ArrayList::new));
        List<Scada> indexValNewList = new ArrayList<>();
        List<Scada> indexValNewUpdateList = new ArrayList<>();
        // 查询实时表所有指标
        List<String> monitorCodeList = scadaMapper.selectDataByMonitorList().stream().map(Scada::getCode).collect(Collectors.toList());
        for (Scada scada : list) {
            Scada indexVal = new Scada();
            indexVal.setCode(scada.getCode());
            indexVal.setValue(scada.getValue());
            indexVal.setScadaTime(scada.getScadaTime());
            indexVal.setUpdateTime(new Date());
            if (monitorCodeList.contains(scada.getCode())) {
                indexValNewUpdateList.add(indexVal);
            } else {
                indexVal.setId(SnowFlake.nextId());
                indexValNewList.add(indexVal);
            }
        }
        if (CollUtil.isNotEmpty(indexValNewList)) {
            scadaMapper.insertBatchTemp(indexValNewList);
        }
        if (CollUtil.isNotEmpty(indexValNewUpdateList)) {
            for (Scada scada : indexValNewUpdateList) {
                scadaMapper.updateRealTemp(scada);
            }
        }
    }

    @Override
    public void synScadaDiffFlow(String startTime, String endTime) {
        this.sendCumulativeFlowDifference(startTime, endTime);
    }

    // 每日 6点 15点执行
    public void automaticSynchronousThirdParty() {
        // 获取对应第三方同步指标列表
        List<String> synCodeList = scadaSynHisService.getSynCodeList("0");
        try {
            scadaSynHisService.synchronousThirdParty(synCodeList);
        } catch (Exception e) {
            log.error(e.getMessage());
            e.printStackTrace();
        }
    }

    /**
     * 正则匹配数据值
     *
     * @param val
     * @return
     */
    private String matcherVal(String val) {
        Matcher matcher = patternStr.matcher(val);
        StringBuilder str = new StringBuilder();
        while (matcher.find()) {
            str.append(matcher.group());
        }
        return str.toString();
    }
}
