package com.tofly.scada.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.scada.entity.Allocation;
import com.tofly.scada.entity.Scada;
import com.tofly.scada.entity.ScadaReport;
import com.tofly.scada.entity.StatisticsScada;
import com.tofly.scada.entity.dto.ScadaMoreQuery;
import com.tofly.scada.entity.dto.ScadaQuery;
import com.tofly.scada.entity.vo.ScadaVo;
import com.tofly.scada.entity.vo.StatisticsScadaVo;
import com.tofly.scada.mapper.AllocationMapper;
import com.tofly.scada.mapper.ScadaMapper;
import com.tofly.scada.mapper.ScadaReportMapper;
import com.tofly.scada.mapper.StatisticsScadaMapper;
import com.tofly.scada.service.StatisticsScadaService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @author haiqiu
 * @description 针对表【STATISTICS_SCADA】的数据库操作Service实现
 * @createDate 2022-05-10 15:55:35
 */
@Service
public class StatisticsScadaServiceImpl extends ServiceImpl<StatisticsScadaMapper, StatisticsScada> implements StatisticsScadaService {
    private final Logger logger = LoggerFactory.getLogger(StatisticsScadaServiceImpl.class);

    @Autowired
    private AllocationMapper allocationMapper;

    @Autowired
    private ScadaReportMapper reportMapper;

    @Autowired
    @Lazy
    private ScadaServiceImpl scadaService;

    @Autowired
    private ScadaMapper scadaMapper;

    /**
     * 根据开始结束时间获取月份表集合
     * @param start 开始时间
     * @param end 结束时间
     * @return 月份表集合
     */
    public List<String> getMouthTables(Date start, Date end) {
        List<Date> dates = ScadaServiceImpl.findDates(start, end, 1);
        if (!CollectionUtils.isEmpty(dates)) {
            List<String> tables = new ArrayList<>();
            for (Date date : dates) {
                //存储到分表月份
                String tableName = scadaService.createMonths(date == null ? new Date() : date);
                tables.add(tableName);
            }
            return tables;
        }
        return null;
    }

    @Override
    public Page<StatisticsScadaVo> getPageData(Page page, ScadaQuery scadaQuery, Boolean isPage) {
        if (scadaQuery.getIsStatisticsScada() == null) {
            scadaQuery.setIsStatisticsScada(false);
        }
        //isGeTrendData 是否为趋势分析查询条件,主要是更换查询条件，此处主要是更换表查询 SCADA_HISTORY  STATISTICS_SCADA
        QueryWrapper<ScadaQuery> queryWrapper = getQuery(scadaQuery, scadaQuery.getIsStatisticsScada());
        // 判断是否站点对比
        Boolean isSiteCompared = scadaQuery.getIsSiteCompared();
        QueryWrapper<ScadaQuery> queryWrapper1 = null;
        if(null!=isSiteCompared && isSiteCompared){
            // 重新赋值设备id,开始时间,结束时间
            ScadaQuery scadaQuery1 = JSON.parseObject(JSON.toJSONString(scadaQuery),ScadaQuery.class);
            scadaQuery1.setDeviceIds(scadaQuery.getSameDeviceIds());
            scadaQuery1.setStart(scadaQuery.getSameStart());
            scadaQuery1.setEnd(scadaQuery.getSameEnd());
            queryWrapper1 = getQuery(scadaQuery1, scadaQuery.getIsStatisticsScada());
        }
        if (null == isPage){
            isPage = true;
        }
        if (scadaQuery.getIsStatisticsScada()){
            // 站点对比(分页大小除以二)
            page.setSize(0==page.getSize()?5:page.getSize()/2);
            if(null!=isSiteCompared && isSiteCompared){

                Page<StatisticsScadaVo> scadaVoPage = baseMapper.pageList(page, queryWrapper, "STATISTICS_SCADA");
                List<StatisticsScadaVo> records = scadaVoPage.getRecords();
                Page<StatisticsScadaVo> scadaVoPage1 = baseMapper.pageList(page, queryWrapper1, "STATISTICS_SCADA");
                Page<StatisticsScadaVo> pageDb = new Page();
                // 填充
                scadaVoPage1.getRecords().addAll(records);
                pageDb.setRecords(scadaVoPage1.getRecords());
                pageDb.setSize(scadaVoPage1.getRecords().size());
                pageDb.setTotal(scadaVoPage.getTotal()+scadaVoPage1.getTotal());
                //特殊值处理
                for (StatisticsScadaVo scadaVo : pageDb.getRecords()) {
                    if (scadaVo.getValue().startsWith(".")) {
                        String value = scadaVo.getValue();
                        scadaVo.setValue("0" + value);
                    }
                }
                return pageDb;
            }else {
                if(isPage){
                    return baseMapper.pageList(page, queryWrapper,"STATISTICS_SCADA");
                }else {
                    Page<StatisticsScadaVo> pageDb = new Page<>();
                    List<StatisticsScadaVo> statisticsScada = baseMapper.selectListData(queryWrapper, "STATISTICS_SCADA");
                    pageDb.setRecords(statisticsScada);
                    //特殊值处理
                    for (StatisticsScadaVo scadaVo : pageDb.getRecords()) {
                        if (scadaVo.getValue().startsWith(".")) {
                            String value = scadaVo.getValue();
                            scadaVo.setValue("0" + value);
                        }
                    }
                    return pageDb;
                }

            }

        }else {

            if(null!=isSiteCompared && isSiteCompared){
                // 站点对比(分页大小除以二)
                page.setSize(0==page.getSize()?5:page.getSize()/2);
                Page<StatisticsScadaVo> scadaVoPage = scadaService.selectHistoryByMouthsTables(getMouthTables(scadaQuery.getStart(),
                        scadaQuery.getEnd()), page, queryWrapper, isPage);

                List<StatisticsScadaVo> records = scadaVoPage.getRecords();
                Page<StatisticsScadaVo> scadaVoPage1 = scadaService.selectHistoryByMouthsTables(getMouthTables(scadaQuery.getSameStart(),
                        scadaQuery.getSameEnd()), page, queryWrapper1, isPage);
                // 填充
                Page<StatisticsScadaVo> pageDb = new Page();
                if (!CollectionUtils.isEmpty(scadaVoPage1.getRecords())){
                    scadaVoPage1.getRecords().addAll(records);
                }else {
                    scadaVoPage1.setRecords(records);
                }
                pageDb.setRecords(scadaVoPage1.getRecords());
                pageDb.setSize(scadaVoPage1.getRecords().size());
                pageDb.setTotal(scadaVoPage.getTotal()+scadaVoPage1.getTotal());
                //特殊值处理
                for (StatisticsScadaVo scadaVo : pageDb.getRecords()) {
                    if (scadaVo.getValue().startsWith(".")) {
                        String value = scadaVo.getValue();
                        scadaVo.setValue("0" + value);
                    }
                }
                return pageDb;
            }else {
                return scadaService.selectHistoryByMouthsTables(getMouthTables(scadaQuery.getStart(),
                        scadaQuery.getEnd()),page,queryWrapper,isPage);
            }

        }
//        scadaService.selectHistoryByMouthsTables(getMouthTables(scadaQuery.getStart(),scadaQuery.getEnd()),page,queryWrapper);
//        return baseMapper.pageList(page, queryWrapper, scadaQuery.getIsStatisticsScada() ? "STATISTICS_SCADA" : "SCADA_HISTORY");
    }

    @Override
    public Page geTrendData(Page page, ScadaQuery scadaQuery, Boolean isPage) {
        QueryWrapper<ScadaQuery> queryWrapper = getQuery(scadaQuery, false);
        if (isPage) {
//            Page<ScadaVo> scadaHistory = baseMapper.pageListNotStatistics(page, queryWrapper, "SCADA_HISTORY");
            Page<ScadaVo> scadaHistory = baseMapper.selectHistoryByMouthsTables(getMouthTables(scadaQuery.getStart(),
                    scadaQuery.getEnd()), page, queryWrapper);
            scadaValueHandle(scadaHistory.getRecords());
            return scadaHistory;
        } else {
//            List<ScadaVo> scadaHistory = baseMapper.selectListDataNotStatistics(queryWrapper, "SCADA_HISTORY");
            List<ScadaVo> scadaHistory = baseMapper.selectHistoryByMouthsTablesNoPage(getMouthTables(scadaQuery.getStart(),
                    scadaQuery.getEnd()), queryWrapper);
            scadaValueHandle(scadaHistory);
            Page<StatisticsScadaVo> voPage = new Page<>();
            voPage.setRecords(calculationScadaData(scadaHistory, scadaQuery.getTimeTotal(), scadaQuery.getStart(), scadaQuery.getEnd()));
            return voPage;
        }
    }

    /**
     * 特殊值处理
     * @param scadaHistory
     */
    private void scadaValueHandle(List<ScadaVo> scadaHistory) {
        //特殊值处理
        for (ScadaVo scadaVo : scadaHistory) {
            if (scadaVo.getValue().startsWith(".")) {
                String value = scadaVo.getValue();
                scadaVo.setValue("0" + value);
            }
        }
    }

    @Override
    public int delete(Integer dateType, String code, String start, String end) {
        return baseMapper.deleteByTime(dateType, code, start, end);
    }


    private List<StatisticsScadaVo> calculationScadaData(List<ScadaVo> scadaHistory, Integer excess, Date start, Date end) {
        //有数据就分组
        if (!CollectionUtils.isEmpty(scadaHistory)) {
            Map<String, List<ScadaVo>> listMap = scadaHistory.stream().filter(scada -> scada.getCode() != null && scada.getValue() != null)
                    .collect(Collectors.groupingBy(ScadaVo::getCode));

            List<StatisticsScadaVo> statisticsScadaVos = new ArrayList<>();
            for (Map.Entry<String, List<ScadaVo>> entity : listMap.entrySet()) {
                StatisticsScadaVo statisticsScadaVo = new StatisticsScadaVo();
                List<ScadaVo> scadaList = entity.getValue();
                double average = scadaList.stream().map(scada -> Double.parseDouble(scada.getValue()))
                        .mapToDouble(Double::shortValue).average().getAsDouble();
                double max = scadaList.stream().map(scada -> Double.parseDouble(scada.getValue()))
                        .mapToDouble(Double::shortValue).max().getAsDouble();
                double min = scadaList.stream().map(scada -> Double.parseDouble(scada.getValue()))
                        .mapToDouble(Double::shortValue).min().getAsDouble();
                double sum = scadaList.stream().map(scada -> Double.parseDouble(scada.getValue()))
                        .mapToDouble(Double::shortValue).sum();

                statisticsScadaVo.setCode(scadaList.get(0).getCode());
                statisticsScadaVo.setValue(scadaList.get(scadaList.size() - 1).getValue());
                statisticsScadaVo.setAve(average);
                statisticsScadaVo.setMax(max);
                statisticsScadaVo.setMin(min);
                statisticsScadaVo.setFold(sum);
                statisticsScadaVo.setStartTime(start);
                statisticsScadaVo.setEndTime(end);
                statisticsScadaVo.setStatisticsType(null);
                //设置分组子数据
                statisticsScadaVo.setScadaVos(scadaList);

                //查询指标报警上下限计算合格率相关
                QueryWrapper<Allocation> allocationQueryWrapper = new QueryWrapper<>();
                allocationQueryWrapper.eq("VARIABLE_CODE", scadaList.get(0).getCode());
                List<Allocation> allocations = allocationMapper.selectList(allocationQueryWrapper);
                if (!CollectionUtils.isEmpty(allocations)) {
                    Set<Long> reportIdSets = ScadaServiceImpl.getReportIdSets(allocations);
                    if (!CollectionUtils.isEmpty(reportIdSets)) {
                        QueryWrapper<ScadaReport> scadaReportQueryWrapper = new QueryWrapper<>();
                        scadaReportQueryWrapper.in("ID", reportIdSets);
                        List<ScadaReport> scadaReports = reportMapper.selectList(scadaReportQueryWrapper);
                        if (!CollectionUtils.isEmpty(scadaReports)) {
                            //赋值报警列表
                            statisticsScadaVo.setScadaReports(scadaReports);
                            //超标次数
                            int errorCount = 0;
                            //不合格超标时长
                            long errorTimeTotal = 0;
                            //超标总时长
                            long timeTotal = (scadaList.get(scadaList.size() - 1).getScadaTime().getTime()
                                    - scadaList.get(0).getScadaTime().getTime()) / 1000;
                            //错误时间
                            Date errorTime = null;
                            for (ScadaVo scada : scadaList) {
                                for (ScadaReport report : scadaReports) {
                                    if (scada.getValue() == null || report.getReportLower() == null || report.getReportUpper() == null) {
                                        break;
                                    }
                                    if (Double.parseDouble(scada.getValue()) < Double.parseDouble(report.getReportLower())
                                            || Double.parseDouble(scada.getValue()) > Double.parseDouble(report.getReportUpper())) {
                                        errorCount = errorCount + 1;
                                        errorTime = scada.getScadaTime();
                                        break;
                                    } else if (errorTime != null) {
                                        errorTimeTotal = errorTimeTotal +
                                                (scada.getScadaTime().getTime() - errorTime.getTime()) / 1000;
                                    }
                                }
                            }
                            //合格率
                            float pass = (float) errorTimeTotal / (float) timeTotal;
                            statisticsScadaVo.setErrorCount(errorCount);
                            statisticsScadaVo.setErrorTimeTotal(errorTimeTotal == 0 ? timeTotal : errorTimeTotal);
                            statisticsScadaVo.setPass(pass);
                            statisticsScadaVo.setTimeTotal(timeTotal);
                        }
                    }
                    statisticsScadaVo.setDisplayName(allocations.get(0).getDisplayName());
                    statisticsScadaVo.setDataType(String.valueOf(allocations.get(0).getType()));
                }
                //TODO 暂时全部返回，后续调整
//                if (excess != null && statisticsScadaVo.getErrorTimeTotal()!=null && statisticsScadaVo.getErrorTimeTotal() >= excess) {
//                    statisticsScadaVos.add(statisticsScadaVo);
//                } else if (excess == null) {
                statisticsScadaVos.add(statisticsScadaVo);
//                }
            }
            return statisticsScadaVos;
        }
        return null;
    }


    /**
     * 构建查询条件
     *
     * @param scadaQuery    查询实体
     * @param isGeTrendData 是否为趋势分析查询条件
     * @return
     */
    public QueryWrapper<ScadaQuery> getQuery(ScadaQuery scadaQuery, Boolean isGeTrendData) {
        QueryWrapper<ScadaQuery> queryWrapper = new QueryWrapper<>();
        if (!CollectionUtils.isEmpty(scadaQuery.getCodes())) {
            queryWrapper.in("scada.CODE", scadaQuery.getCodes());
        }
        if (!StringUtils.isEmpty(scadaQuery.getCode())) {
            queryWrapper.eq("scada.CODE", scadaQuery.getCode());
        }
        if (!CollectionUtils.isEmpty(scadaQuery.getDeviceIds())) {
            queryWrapper.in("manage.ID", scadaQuery.getDeviceIds());
        }
        if (scadaQuery.getDataTypeIds() != null) {
            Set<String> typeIds = scadaQuery.getDataTypeIds().stream().filter(Objects::nonNull)
                    .map(String::valueOf).collect(Collectors.toSet());
            queryWrapper.in("allocation.TYPE", typeIds);
        }
        if (scadaQuery.getStart() != null && scadaQuery.getEnd() != null) {
            if (scadaQuery.getStart().getTime() > scadaQuery.getEnd().getTime()) {
                throw new RuntimeException("开始时间不能大于结束时间");
            }
            if (isGeTrendData) {
                if (scadaQuery.getStatisticsType() != null) {
                    queryWrapper.eq("scada.STATISTICS_TYPE", scadaQuery.getStatisticsType());
                }
                queryWrapper.ge("scada.START_TIME", scadaQuery.getStart())
                        .le("scada.END_TIME", scadaQuery.getEnd());
            } else {
                queryWrapper.between("scada.SCADA_TIME", scadaQuery.getStart(), scadaQuery.getEnd());
//                queryWrapper.orderByAsc("scada.SCADA_TIME");
            }
        }
        //如果是趋势分析查询
        if (isGeTrendData) {
            if (scadaQuery.getStart() == null && scadaQuery.getEnd() == null) {
                throw new RuntimeException("该接口时间范围为必填项");
            }
            //如果没有传入code就默认查询全部
            if (!StringUtils.hasText(scadaQuery.getCode()) && CollectionUtils.isEmpty(scadaQuery.getCodes())) {
                List<Allocation> allocations = allocationMapper.selectList(null);
                if (!CollectionUtils.isEmpty(allocations)) {
                    Set<String> codes = allocations.stream().filter(allocation -> allocation.getVariableCode() != null)
                            .map(Allocation::getVariableCode).collect(Collectors.toSet());
                    queryWrapper.in("scada.CODE", codes);

                }
            }
        }

        return queryWrapper;
    }

    @Transactional(rollbackFor = Exception.class)
    public void insertBacthHistoryTemp(ScadaMoreQuery scadaMoreQuery) {
        if (scadaMoreQuery == null) {
            return;
        }
        List<Scada> historyTempList = scadaMoreQuery.getHistoryTempList();
        if (historyTempList == null || historyTempList.size() == 0) {
            return;
        }
        ThreadPoolExecutor threadPoolExecutor = new ThreadPoolExecutor(
                4,
                8,
                120,
                TimeUnit.SECONDS,
                new ArrayBlockingQueue<>(100),
                (r)->{
                    Thread t = new Thread(r);
                    t.setName("thread-" + ((int)(Math.random()*1000)));
                    return t;
                    },
                new ThreadPoolExecutor.CallerRunsPolicy());
        //为了防止SQL语句超出长度出错，分成几次插入
        if (historyTempList.size() <= 2000) {
            threadPoolExecutor.execute(() -> baseMapper.insertBacthHistoryTemp(historyTempList));
        } else {
            int times = historyTempList.size() / 2000;
            for (int i = 0; i <= times; i++) {
                List<Scada> historyFinalList = historyTempList.subList(i * 2000, Math.min((i + 1) * 2000, historyTempList.size()));
                if (CollUtil.isEmpty(historyFinalList)) {
                    continue;
                }
                threadPoolExecutor.execute(() -> baseMapper.insertBacthHistoryTemp(historyFinalList));
            }
        }
    }
}
