package com.tofly.scada.xxljob.handle;

import cn.hutool.core.date.DateUtil;
import com.tofly.scada.service.ScadaService;
import com.tofly.scada.service.impl.ScadaServiceImpl;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.SneakyThrows;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @author HaiQiu
 * @since 2023/3/24
 */
@Component
public class ScadaHandle {

    @Autowired
    private ScadaServiceImpl scadaService;


    /**
     * 每月最后28日凌晨2点执行创建下个月的数据表
     * @return
     */
    @XxlJob("createMonths")
    public ReturnT<String> createMonths(){
        scadaService.createMonths();
        return ReturnT.SUCCESS;
    }

    /**
     * 同步24小时数据，支持自定义时间添加
     * @return
     */
    @XxlJob("syncHourData")
    public ReturnT<String> syncHourData(){
        scadaService.insert24H();
        return ReturnT.SUCCESS;
    }

    /**
     *  同步每天数据，每天6半同步昨天的数据
     * @return
     */

    @XxlJob("syncDayData")
    public ReturnT<String> syncDayData(){
        //每月最后28日凌晨2点执行创建下个月的数据表
        scadaService.syncDaysData();
        return ReturnT.SUCCESS;
    }

    /**
     *每天凌晨0点同步更新这个月表数据
     * @return
     */
    @XxlJob("syncMonthDataEverday")
    public ReturnT<String> syncMonthDataEverday(){
        scadaService.syncDataMonthEverday();
        return ReturnT.SUCCESS;
    }

    /**
     *每月1号凌晨1点同步更新历史数据与上个月表数据，支持自定义时间添加
     * @return
     */
    @XxlJob("syncMonthData")
    public ReturnT<String> syncMonthData(){
        scadaService.syncDataIntoMonths();
        return ReturnT.SUCCESS;
    }


    /**
     * 获取对应第三方同步指标列表
     * @return
     */
    @XxlJob("automaticSynchronousThirdParty")
    public ReturnT<String> automaticSynchronousThirdParty(){
        scadaService.automaticSynchronousThirdParty();
        return ReturnT.SUCCESS;
    }

    /**
     * 累计流量差值处理(5分钟求值)
     * @return
     */
    @XxlJob("sendCumulativeFlowDifference")
    public ReturnT<String> sendCumulativeFlowDifference(){
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        Date endTime = calendar.getTime();
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        Date startTime = calendar.getTime();
        scadaService.sendCumulativeFlowDifference(DateUtil.format(startTime,"yyyy-MM-dd HH:mm:ss"), DateUtil.format(endTime,"yyyy-MM-dd HH:mm:ss"));
        Calendar calendar1 = Calendar.getInstance();
        calendar1.add(Calendar.DATE, -1);
        calendar1.set(Calendar.HOUR_OF_DAY, 23);
        calendar1.set(Calendar.MINUTE, 59);
        calendar1.set(Calendar.SECOND, 59);
        Date endTime1 = calendar1.getTime();
        calendar1.set(Calendar.HOUR_OF_DAY, 0);
        calendar1.set(Calendar.MINUTE, 0);
        calendar1.set(Calendar.SECOND, 0);
        Date startTime1 = calendar1.getTime();
        scadaService.sendCumulativeFlowDifference(DateUtil.format(startTime1,"yyyy-MM-dd HH:mm:ss"), DateUtil.format(endTime1,"yyyy-MM-dd HH:mm:ss"));
        return ReturnT.SUCCESS;
    }

    /**
     * 	获取历史数据进行报警和监控表更新每分钟10执行一次
     * @return
     */
    @XxlJob("timingInsertData")
    public ReturnT<String> timingInsertData(){
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        Date end = calendar.getTime();
        calendar.add(Calendar.MINUTE, -10);
        Date start = calendar.getTime();
        scadaService.timingInsertData(sdf.format(start), sdf.format(end));
        return ReturnT.SUCCESS;
    }

    /**
     * 	同步原始数据到历史表
     * @return
     */
    @XxlJob("originToScadaHistoryData")
    public ReturnT<String> originToScadaHistoryData(){
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        calendar.add(Calendar.MINUTE, -1);
        calendar.set(Calendar.SECOND, 59);
        Date end = calendar.getTime();
        calendar.add(Calendar.MINUTE, -9);
        calendar.set(Calendar.SECOND, 0);
        Date start = calendar.getTime();
        String startDate = sdf.format(start);
        String endDate = sdf.format(end);
        scadaService.origDataToScadaHistory(startDate, endDate, "CREATE_TIME");
        return ReturnT.SUCCESS;
    }
}
